var getDataOrCache = function(sDay, eDay, startDate, endDate, appliance, apiKey, registrationToken, endPointing, isForecast) {

  // Parse
  var start = new Date(sDay.getTime());
  var end = new Date(eDay.getTime());

  if(appliance !== 'forecast') {
    start = new Date() < start ? new Date() : start;
    end = new Date() < end ? new Date() : end;
  }

  // Get parameters
  var cParam = {
    start: start.toISOString().split(":")[0] + ":00:00.000Z",
    end: end.toISOString().split(":")[0] + ":00:00.000Z",
    range: parseInt((end - start) / 3600000)
  }

  // Get initial and final data cached position
  cParam.cacheStart = gViz.shared.data.barChart.cached[appliance].findIndex(function(d) { return d.x == cParam.start; });
  cParam.cacheEnd   = gViz.shared.data.barChart.cached[appliance].findIndex(function(d) { return d.x == cParam.end; });

  if(cParam.cacheStart !== -1 && cParam.cacheEnd !== -1 && cParam.cacheEnd - cParam.cacheStart >= cParam.range) {

    // Get appliance data
    var aData = { appliance: appliance, consumption: { w: [], t: [] }, isCache: true };
    gViz.shared.data.barChart.cached[appliance].slice(cParam.cacheStart, cParam.cacheEnd+1).forEach(function(d) {
      aData.consumption.w.push(d.value);
      var date = `${d.date.getTime()}`;
      aData.consumption.t.push(date.substring(0,date.length-3));
    });;
    return window.gViz.shared.data.api.resolvedFetch(aData);

  } else {

    // Fetch data from api :: actual = "all" appliance
    return window.gViz.shared.data.api.fetch(appliance, startDate.substring(0,startDate.length-3), endDate.substring(0,endDate.length-3), "3600", 100000, null, apiKey, registrationToken, endPointing);

  }

}

// Store data in cache
var cacheData = function(rData) {

  // Apply cache only if the data wasn't fetched from cache
  if(rData.isCache == null || rData.isCache !== true) {

    // Cache data for future uses
    var keys = {};
    rData.consumption.t.forEach(function(d, i) {
      gViz.shared.data.barChart.cached[rData.appliance].push({
        x: (new Date(parseInt(`${d}000`))).toISOString(),
        date: new Date(parseInt(`${d}000`)),
        value: rData.consumption.w[i]
      });
    });

    // Unique keys
    var newCacheData = [], storedKeys = {};
    gViz.shared.data.barChart.cached[rData.appliance].reverse().forEach(function(d) {
      if(storedKeys[d.x] == null) {
        storedKeys[d.x] = true;
        newCacheData.push(d);
      }
    });

    // Sort new cache array and store data
    gViz.shared.data.barChart.cached[rData.appliance] = newCacheData.reverse().sort(function(a,b) { return d3.ascending(a.date.getTime(), b.date.getTime()); });

  }

}

// Get data considering interval, date point and direction
var getData = function(interval, x, direction, appliance, isForecast, apiKey, registrationToken, endPointing) {

  // Initialize data cache for appliance and forecast
  gViz.shared.data.barChart.cached[appliance] = gViz.shared.data.barChart.cached[appliance] || [];
  gViz.shared.data.barChart.cached["forecast"] = gViz.shared.data.barChart.cached["forecast"] || [];

  // Get interval
  if(interval == null) { interval = 'day'; }

  // Get data depending on cache
  var data = [];

  // Get other parameters
  if(direction == null) { direction = 'both'; }
  if(appliance == null) { appliance = 'all'; }
  if(isForecast == null) { isForecast = true; }

  // Get x
  if(x == null) { x = 'Today'; }
  else { x = d3.timeParse(interval === 'year' ? "%Y" : "%Y-%m-%d")(x); }

  switch(interval) {

    case 'week':

      // Set is now
      var lastMonday, sDay, eDay, startDate, endDate, fStartDate, fEndDate, sDate, eDate;

      // Set date format
      var dateOrder = gViz.shared.helpers.date.translated.DATEORDER['7DAYS'];

      // Get time for today
      if(x === 'Today' || direction === 'both') {

        sDate = x === 'Today' ? new Date() : new Date(x);
        eDate = x === 'Today' ? new Date() : new Date(x);

        // Get start date for actual and forecasted values
        sDay = gViz.shared.helpers.date.getLastMonday(sDate).addDays(-7);
        startDate =`${sDay.setHours(0,0,0,000)}`;
        fStartDate =`${sDay.setHours(0,0,0,000)}`;

        // Get end date for actual and forecasted values
        eDay = gViz.shared.helpers.date.getLastMonday(eDate).addDays(13);
        fEndDate = `${eDay.setHours(23,59,59,000)}`;
        endDate = `${eDate < new Date() ? eDay.getTime() : (new Date()).getTime()}`;

      // Get time for other periods
      } else {

        // Get start date for actual and forecasted values
        sDay = gViz.shared.helpers.date.getLastMonday((new Date(x)).addDays(direction === 'prev' ? -7 : 7));
        fStartDate = `${sDay.setHours(0,0,0,000)}`;
        startDate = `${sDay.setHours(0,0,0,000)}`;

        // Get end date for actual and forecasted values
        eDay = gViz.shared.helpers.date.getLastMonday((new Date(x)).addDays(direction === 'prev' ? -7 : 7)).addDays(6);
        fEndDate = `${eDay.setHours(23,59,59,999)}`;
        endDate = `${eDay.setHours(23,59,59,999)}`;

      }

      var parseWeekData = function(fData, aData) {

        // Initialize parents structures
        var parents = {}, itemNames = {}, dateValues = {};

        // Parse new data from api
        [fData, aData].forEach(function(rData) {

          // Store data in cache
          cacheData(rData);

          rData.consumption.t.forEach(function(d, i) {

            // Get date from data
            var days = d3.timeFormat("%a")(new Date(parseInt(`${d}000`)));
            var date = d3.timeFormat("%Y-%m-%d")(new Date(parseInt(`${d}000`)));
            var index = d3.timeFormat("%u")(new Date(parseInt(`${d}000`)));
            var weekStart = new Date(parseInt(`${d}000`)).addDays(1 - index);
            var isCurrentYear = weekStart.getFullYear() === (new Date()).getFullYear();

            // Fill dateValues with month date and aggregated values
            dateValues[rData.appliance] = dateValues[rData.appliance] || {};
            if(dateValues[rData.appliance][date] == null) { dateValues[rData.appliance][date] = { value: 0, weekStart: weekStart, days: days, index: index }; }
            dateValues[rData.appliance][date].value += +rData.consumption.w[i];

            // Initialize parent structure for date if not initialized
            if(parents[d3.timeFormat("%Y-%m-%d")(weekStart)] == null) {
              parents[d3.timeFormat("%Y-%m-%d")(weekStart)] = {
                "x": d3.timeFormat("%Y-%m-%d")(weekStart),
                "name": d3.timeFormat(dateOrder === 'default' ? ("%b %d" + (isCurrentYear ? "" : ", %Y")) : dateOrder)(weekStart),
                "values": [
                ]
              };
            }

            // Store item and parent item names
            if(itemNames[d3.timeFormat("%Y-%m-%d")(weekStart)] == null) { itemNames[d3.timeFormat("%Y-%m-%d")(weekStart)] = {} };

          });
        });

        Object.keys(dateValues).forEach(function(app) {
          Object.keys(dateValues[app]).sort(function(a,b) { return d3.ascending(d3.timeParse("%Y-%m-%d")(a),d3.timeParse("%Y-%m-%d")(b)); }).forEach(function(k) {

            // Store value on parent week
            if(dateValues[app][k].value !== 0) {
              parents[d3.timeFormat("%Y-%m-%d")(dateValues[app][k].weekStart)].values.push({
                "id": app,
                "x": dateValues[app][k].days,
                "name": dateValues[app][k].days,
                "date": d3.timeParse("%Y-%m-%d")(k),
                "y": dateValues[app][k].value,
                "index": dateValues[app][k].index,
                "color": app === 'forecast' ? "#ddd" : "#2566bc",
                "secondaryColor": app === 'forecast' ? "#233a71" : "rgba(36,121,219,0.7)",
                "opacity": 1
              });

              // Store item names
              itemNames[d3.timeFormat("%Y-%m-%d")(dateValues[app][k].weekStart)][dateValues[app][k].index.toString()] = true;
            }
          });
        });

        // Set missing months to 0 just to fit axis
        Object.keys(itemNames).forEach(function(k) {
          d3.range(7).forEach(function(d, i) {
            var date = d3.timeParse("%Y-%m-%d")(k);
            var index = d3.timeFormat("%u")(date.addDays(i)).toString();
            var days = d3.timeFormat("%a")(date);
            if(itemNames[k][index] == null) {
              parents[k].values.push({
                "id": isForecast ? "forecast" : "actual",
                "x": days.toString(),
                "name": days.toString(),
                "y": 0,
                "index": index,
                "color": "transparent",
                "secondaryColor": "transparent",
                "opacity": 0
              });
            }
          });
        });

        Object.keys(parents).sort(function(a,b) { return d3.ascending(a,b); }).forEach(function(k) {
          parents[k].values = parents[k].values.sort(function(a,b) {
            return d3.ascending(+a.index,+b.index) || d3.descending(a.id,b.id);
          });
          data.push(parents[k]);
        });

        // Store data to be used and also in cache
        this._data = data;

      };

      var req2 = getDataOrCache(sDay, eDay, startDate, endDate, appliance, apiKey, registrationToken, endPointing, isForecast);
      var req  = getDataOrCache(sDay, eDay, fStartDate, fEndDate, "forecast", apiKey, registrationToken, endPointing, isForecast);

      // Add or remove days depending on direction
      if(direction === 'prev') { x.addDays(-1); }
      else if(direction === 'next') { x.addDays(1); }

      return $.when(req, req2).done(parseWeekData);

      break;

    case 'month':

      // Set is now
      var now = new Date(), xDate = new Date(x);
      var sDay, eDay, startDate, endDate, fStartDate, fEndDate, eDate;

      // Set date format
      var dateOrder = gViz.shared.helpers.date.translated.DATEORDER['30DAYS'];

      // Get time for today
      if(x === 'Today' || direction === 'both') {

        xDate = x === 'Today' ? new Date() : new Date(x);

        // Get start date for actual and forecasted values
        sDay = (new Date(xDate.getFullYear(), xDate.getMonth(), 1)).addMonths(-1);
        fStartDate =`${sDay.setHours(0,0,0,000)}`;
        startDate =`${sDay.setHours(0,0,0,000)}`;

        // Get end date for actual and forecasted values
        eDate = (new Date(xDate.getFullYear(), xDate.getMonth(), 1)).addMonths(2).addDays(-1);
        eDay = eDate;
        fEndDate = `${eDay.setHours(23,59,59,000)}`;
        endDate = `${eDate < new Date() ? eDate : (new Date()).getTime()}`;

      // Get time for other periods
      } else {

        // Add or remove months from date, depending on direction
        xDate.addMonths(direction === 'prev' ? -1 : 1);

        // Get start date for actual and forecasted values
        sDay = new Date(xDate.getFullYear(), xDate.getMonth(), 1);
        fStartDate =`${sDay.setHours(0,0,0,000)}`;
        startDate =`${sDay.setHours(0,0,0,000)}`;

        // Get end date for actual and forecasted values
        eDay = new Date(xDate.getFullYear(), xDate.getMonth(), 1).addMonths(1).addDays(-1);
        fEndDate = `${eDay.setHours(23,59,59,000)}`;
        endDate = `${eDay.setHours(23,59,59,000)}`;

        // Fix start and end date not to go over now
        if(new Date(parseInt(startDate)) > new Date) { startDate = `${(new Date()).getTime()}`; }
        if(new Date(parseInt(endDate)) > new Date) { endDate = `${(new Date()).getTime()}`; }

      }

      var parseMonthData = function(fData, aData) {

        // Initialize parents structures
        var parents = {}, itemNames = {}, dateValues = {};

        // Parse new data from api
        [fData, aData].forEach(function(rData) {

          // Store data in cache
          cacheData(rData);

          rData.consumption.t.forEach(function(d, i) {

            // Get date from data
            var days = d3.timeFormat("%e")(new Date(parseInt(`${d}000`))).replace(' ','');
            var date = d3.timeFormat("%Y-%m-%d")(new Date(parseInt(`${d}000`)));
            var monthStart = new Date((new Date(parseInt(`${d}000`))).setDate(1));
            var isCurrentYear = monthStart.getFullYear() === (new Date()).getFullYear();

            // Fill dateValues with month date and aggregated values
            dateValues[rData.appliance] = dateValues[rData.appliance] || {};
            if(dateValues[rData.appliance][date] == null) { dateValues[rData.appliance][date] = { value: 0, monthStart: monthStart, days: days, index: parseInt(days) }; }
            dateValues[rData.appliance][date].value += +rData.consumption.w[i];

            // Initialize parent structure for date if not initialized
            if(parents[d3.timeFormat("%Y-%m-%d")(monthStart)] == null) {
              parents[d3.timeFormat("%Y-%m-%d")(monthStart)] = {
                "id": d3.timeFormat("%Y-%m-%d")(monthStart),
                "dataType": 'month',
                "x": d3.timeFormat("%Y-%m-%d")(monthStart),
                "name": d3.timeFormat(dateOrder === 'default' ? ("%b %d" + (isCurrentYear ? "" : ", %Y")) : dateOrder)(monthStart),
                "values": [
                ]
              };
            }

            // Store parent item names
            if(itemNames[d3.timeFormat("%Y-%m-%d")(monthStart)] == null) { itemNames[d3.timeFormat("%Y-%m-%d")(monthStart)] = {} };

          });
        });

        // Parse and store values
        Object.keys(dateValues).forEach(function(app) {
          Object.keys(dateValues[app]).sort(function(a,b) { return d3.ascending(d3.timeParse("%Y-%m-%d")(a),d3.timeParse("%Y-%m-%d")(b)); }).forEach(function(k) {

            // Store value on parent month
            if(dateValues[app][k].value !== 0) {
              parents[d3.timeFormat("%Y-%m-%d")(dateValues[app][k].monthStart)].values.push({
                "id": app,
                "x": dateValues[app][k].days,
                "name": dateValues[app][k].days,
                "date": d3.timeParse("%Y-%m-%d")(k),
                "y": dateValues[app][k].value,
                "index": dateValues[app][k].index,
                "color": app === 'forecast' ? "#ddd" : "#2566bc",
                "secondaryColor": app === 'forecast' ? "#233a71" : "rgba(36,121,219,0.7)",
                "opacity": 1
              });

              // Store item names
              itemNames[d3.timeFormat("%Y-%m-%d")(dateValues[app][k].monthStart)][dateValues[app][k].days.toString()] = true;
            }

          });
        });

        // Set missing months to 0 just to fit axis
        Object.keys(itemNames).forEach(function(k) {
          d3.range(31).forEach(function(d, i) {
            if(itemNames[k][(i+1).toString()] == null) {
              parents[k].values.push({
                "id": isForecast ? "forecast" : "actual",
                "x": (i+1).toString(),
                "name": (i+1).toString(),
                "y": 0,
                "index": i+1,
                "color": "transparent",
                "secondaryColor": "transparent",
                "opacity": 0
              });
            }
          });
        });

        Object.keys(parents).sort(function(a,b) { return d3.ascending(a,b); }).forEach(function(k) {
          parents[k].values = parents[k].values.sort(function(a,b) {
            return d3.ascending(+a.index,+b.index) || d3.descending(a.id,b.id);
          });
          data.push(parents[k]);
        });

        // Store data to be used and also in cache
        this._data = data;

      }

      // Add or remove days depending on direction
      if(direction === 'prev') { x.addDays(-1); }
      else if(direction === 'next') { x.addDays(1); }

      var req2 = getDataOrCache(sDay, eDay, startDate, endDate, appliance, apiKey, registrationToken, endPointing, isForecast);
      var req  = getDataOrCache(sDay, eDay, fStartDate, fEndDate, "forecast", apiKey, registrationToken, endPointing, isForecast);

      return $.when(req, req2).done(parseMonthData);

      break;

    case 'year':

      // Set is now
      var now = new Date(), xDate = new Date(x);
      var sDay, eDay, startDate, endDate, fStartDate, fEndDate, months = [];

      // Set date format
      var dateOrder = gViz.shared.helpers.date.translated.DATEORDER['THISYEAR'];

      // Get year
      var year = x === 'Today' ? now.getFullYear() : (xDate.getFullYear() + (direction === 'prev' ? - 1 : 1));

      sDay = new Date(year + (x === 'Today' ? -1 : 0), 0, 1);
      eDay = new Date(year + (x === 'Today' ? 1 : 0), 11, 31);
      fsDay = new Date(year + (x === 'Today' ? -1 : 0), 0, 1);
      feDay = new Date(year + (x === 'Today' ? 1 : 0), 11, 31);
      fStartDate = `${fsDay.setHours(0,0,0,0)}`;
      fEndDate = `${feDay.setHours(23,59,59,59)}`;
      startDate = `${sDay.setHours(0,0,0,0)}`;
      endDate = `${eDay.setHours(23,59,59,59)}`;

      var parseYearData = function(fData, aData) {

        // Initialize parents structures
        var parents = {}, itemNames = {}, monthValues = {};

        // Parse new data from api
        [aData, fData].forEach(function(rData) {

          // Store data in cache
          cacheData(rData);

          rData.consumption.t.forEach(function(d, i) {

            // Get date from data
            var date = new Date((new Date(parseInt(`${d}000`))).setDate(1));
            var months = d3.timeFormat("%b")(date);
            var isCurrentYear = date.getFullYear() === (new Date()).getFullYear();
            var year = date.getFullYear();

            // Only add the element if its on the searched year
            if(`${date.getFullYear()}` === `${year}`) {

              // Fill monthValues with month date and aggregated values
              monthValues[rData.appliance] = monthValues[rData.appliance] || {};
              monthValues[rData.appliance][year] = monthValues[rData.appliance][year] || {};
              if(monthValues[rData.appliance][year][months] == null) { monthValues[rData.appliance][year][months] = { value: 0, date: date, year: year }; }
              monthValues[rData.appliance][year][months].value += +rData.consumption.w[i];

              // Initialize parent structure for date if not initialized
              if(parents[d3.timeFormat("%Y")(date)] == null) {
                parents[d3.timeFormat("%Y")(date)] = {
                  "x": d3.timeFormat("%Y")(date),
                  "name": d3.timeFormat(dateOrder === 'default' ? "%Y" : dateOrder)(date),
                  "values": [
                  ]
                };
              }

              // Store parent item names
              if(itemNames[d3.timeFormat("%Y")(date)] == null) { itemNames[d3.timeFormat("%Y")(date)] = {}; }

            }
          });
        });

        // Push all month values to the year parent
        Object.keys(monthValues).forEach(function(rApp) {
          Object.keys(monthValues[rApp]).forEach(function(year) {
            Object.keys(monthValues[rApp][year]).forEach(function(months) {

              // Get attributes
              var date = monthValues[rApp][year][months].date;
              var value = monthValues[rApp][year][months].value;

              // Push month object
              if(value !== 0) {
                parents[d3.timeFormat("%Y")(date)].values.push({
                  "id": rApp,
                  "x": months,
                  "date": date,
                  "name": months,
                  "index": d3.timeFormat("%m")(date),
                  "y": value,
                  "color": rApp === 'forecast' ? "#ddd" : "#2566bc",
                  "secondaryColor": rApp === 'forecast' ? "#233a71" : "rgba(36,121,219,0.7)",
                  "opacity": 1
                });

                // Store item names
                itemNames[year][d3.timeFormat("%Y-%m-%d")(date)] = true;
              }
            });
          });
        });

        // Set missing months to 0 just to fit axis
        Object.keys(itemNames).forEach(function(k) {
          d3.range(12).forEach(function(d, i) {
            var date = new Date(parseInt(k), i, 1);
            var months = d3.timeFormat("%b")(date);
            if(itemNames[k][d3.timeFormat("%Y-%m-%d")(date)] == null) {
              parents[d3.timeFormat("%Y")(date)].values.push({
                "id": isForecast ? "forecast" : "actual",
                "x": months,
                "date": date,
                "name": months,
                "index": parseInt(d3.timeFormat("%m")(date)),
                "y": 0,
                "color": "transparent",
                "secondaryColor": "transparent",
                "opacity": 0
              });

            }
          });
        });

        Object.keys(parents).sort(function(a,b) { return d3.ascending(a,b); }).forEach(function(k) {
          parents[k].values = parents[k].values.sort(function(a,b) {
            return d3.ascending(+a.index,+b.index) || d3.descending(a.id,b.id);
          });
          data.push(parents[k]);
        });

        // Store data to be used
        this._data = data;

      }

      var req2 = getDataOrCache(sDay, eDay, startDate, endDate, appliance, apiKey, registrationToken, endPointing, isForecast);
      var req  = getDataOrCache(fsDay, feDay, fStartDate, fEndDate, "forecast", apiKey, registrationToken, endPointing, isForecast);

      return $.when(req, req2).done(parseYearData);

      break;

    default:

      // Set is now
      var now = new Date();
      var sDay, eDay, startDate, endDate, fStartDate, fEndDate, sDate, eDate;

      // Set date format
      var dateOrder = gViz.shared.helpers.date.translated.DATEORDER['TODAY'];

      // Get time for today
      if(x === 'Today' || direction === 'both') {

        sDate = x === 'Today' ? new Date() : new Date(x);
        eDate = x === 'Today' ? new Date() : new Date(x);

        // Get start date for actual and forecasted values
        sDay = (sDate).addDays(-1);
        startDate = `${sDay.setHours(0,0,0,0)}`;
        fStartDate = `${sDay.setHours(0,0,0,0)}`;

        // Get end date for actual and forecasted values
        eDay = (eDate).addDays(1);
        fEndDate = `${eDay.setHours(23,59,59,59)}`;
        endDate = `${eDay < new Date() ? eDay : (new Date()).getTime()}`;

      // Get time for other periods
      } else {

        // Get start date for actual and forecasted values
        sDay = (new Date(x)).addDays(direction === 'prev' ? -1 : 1);
        startDate = `${sDay.setHours(0,0,0,0)}`;
        fStartDate = `${sDay.setHours(0,0,0,0)}`;

        // Get end date for actual and forecasted values
        eDay = (new Date(x)).addDays(direction === 'prev' ? -1 : 1);
        fEndDate = `${eDay.setHours(23,59,59,59)}`;
        endDate = `${(direction === 'prev' ? eDay.setHours(23,59,59,59) : eDay.setHours(now.getHours(), now.getMinutes(), now.getSeconds(), now.getMilliseconds()))}`;

      }

      var parseTodayData = function(fData, aData) {

        // Store data in cache
        cacheData(fData);
        cacheData(aData);

        // Initialize parents structures
        var parents = {}, itemNames = {};

        // Parse new data from api
        fData.consumption.t.forEach(function(d, i) {

          // Get date from data
          var hours = (new Date(parseInt(`${d}000`))).getHours() + 1;
          var date = (new Date(parseInt(`${d}000`)));
          var isCurrentYear = date.getFullYear() === (new Date()).getFullYear();

          // Initialize parent structure for date if not initialized
          if(parents[d3.timeFormat("%Y-%m-%d")(date)] == null) {
            parents[d3.timeFormat("%Y-%m-%d")(date)] = {
              "x": d3.timeFormat("%Y-%m-%d")(date),
              "name": d3.timeFormat(dateOrder === 'default' ? ("%A, %b %d" + (isCurrentYear ? "" : ", %Y")) : dateOrder)(date),
              "values": [
              ]
            };
          }
          if(itemNames[d3.timeFormat("%Y-%m-%d")(date)] == null) { itemNames[d3.timeFormat("%Y-%m-%d")(date)] = {} };

          if(fData.consumption.w[i] !== 0) {
            parents[d3.timeFormat("%Y-%m-%d")(date)].values.push({
              "id": "forecast",
              "x": hours === 0 ? 24 : hours,
              "name": hours === 0 ? 24 : hours,
              "date": date,
              "y": fData.consumption.w[i],
              "index": hours === 0 ? 24 : hours,
              "color": "#ddd",
              "secondaryColor": "#233a71",
              "opacity": 1
            });

            // Store item and parent item names
            itemNames[d3.timeFormat("%Y-%m-%d")(date)][(hours === 0 ? 24 : hours).toString()] = true;
          }
        });

        // Parse new data from api
        aData.consumption.t.forEach(function(d, i) {

          // Get date from data
          var hours = (new Date(parseInt(`${d}000`))).getHours() + 1;
          var date = (new Date(parseInt(`${d}000`)));
          var isCurrentYear = date.getFullYear() === (new Date()).getFullYear();

          // Initialize parent structure for date if not initialized
          if(parents[d3.timeFormat("%Y-%m-%d")(date)] == null) {
            parents[d3.timeFormat("%Y-%m-%d")(date)] = {
              "x": d3.timeFormat("%Y-%m-%d")(date),
              "name": d3.timeFormat(dateOrder === 'default' ? ("%A, %b %d" + (isCurrentYear ? "" : ", %Y")) : dateOrder)(date),
              "values": [
              ]
            };
          }
          if(itemNames[d3.timeFormat("%Y-%m-%d")(date)] == null) { itemNames[d3.timeFormat("%Y-%m-%d")(date)] = {} };

          if(aData.consumption.w[i] !== 0) {
            parents[d3.timeFormat("%Y-%m-%d")(date)].values.push({
              "id": "actual",
              "x": hours === 0 ? 24 : hours,
              "name": hours === 0 ? 24 : hours,
              "date": date,
              "y": aData.consumption.w[i],
              "index": hours === 0 ? 24 : hours,
              "color": "#2566bc",
              "secondaryColor": "rgba(36,121,219,0.7)",
              "opacity": 1
            });

            // Store item and parent item names
            itemNames[d3.timeFormat("%Y-%m-%d")(date)][(hours === 0 ? 24 : hours).toString()] = true;
          }
        });

        // Set missing months to 0 just to fit axis
        Object.keys(itemNames).forEach(function(k) {
          d3.range(23).forEach(function(d, i) {
            if(itemNames[k][(i+1).toString()] == null) {
              parents[k].values.push({
                "id": isForecast ? "forecast" : "actual",
                "x": (i+1).toString(),
                "name": (i+1).toString(),
                "y": 0,
                "index": i,
                "color": "transparent",
                "secondaryColor": "transparent",
                "opacity": 0
              });
            }
          });
        });

        Object.keys(parents).sort(function(a,b) { return d3.ascending(a,b); }).forEach(function(k) {
          parents[k].values = parents[k].values.sort(function(a,b) {
            return d3.ascending(a.index,b.index) || d3.descending(a.id,b.id);
          });
          data.push(parents[k]);
        });

        // Store data to be used and also in cache
        this._data = data;

      }

      var req2 = getDataOrCache(sDay, eDay, startDate, endDate, appliance, apiKey, registrationToken, endPointing, isForecast);
      var req  = getDataOrCache(sDay, eDay, fStartDate, fEndDate, "forecast", apiKey, registrationToken, endPointing, isForecast);

      // Add or remove days depending on direction
      if(direction === 'prev') { x.addDays(-1); }
      else if(direction === 'next') { x.addDays(1); }

      return $.when(req, req2).done(parseTodayData);

      break;

  }

}

// Initialize the helpers class¬
gViz.shared.data.barChart = {

  // Store cached data for initial values
  cached: {},
  cachedKeys: {},

  // Get data
  fetch: function(interval, x, direction, appliance, isForecast, apiKey, registrationToken, endPointing) {
    return getData(interval, x, direction, appliance, isForecast, apiKey, registrationToken, endPointing);
  },

  // Replace text variables
  format: function() {

    return {
      "title": "",
      "legend": {
        "isVisible": false,
      },
      "tooltip": {
        "body": [ "<span class='metric'>{{y}}</span>" ]
      },
      "y": {
        "title": "",
        "prefix": "",
        "sufix": "Wh",
        "format": "",
        "min": 0
      },
      "x": {
        "title": "",
        "type": "text",
        "prefix": "",
        "sufix": "",
        "format": ""
      },
      "attrs": {
        "barWidth": 25
      },
      "colors": {
        "forecast": { "fill": "#ddd", "gradient": "#ddd", "stroke": "#ddd" },
        "actual": { "fill": "#3a6cc8", "gradient": "#3a6cc8", "stroke": "#3a6cc8" }
      },
      "data": []
    };
  },

  // Replace text variables
  getData: getData,

  // Set unit
  setUnit: function(data, unit) {
    if(unit != null) { data.y.sufix = unit; }
  }

}


