var bubbleFakeData = [
  {
    "id": 'radiator',
    "name": 'Radiator',
    "value": 50,
    "img": "/img/event/HeatenB-01.png"
  },
  {
    "id": 'kitchen',
    "name": 'Kitchen',
    "value": 30,
    "img": "/img/event/Kitchenb-01.png"
  },
  {
    "id": 'ironing',
    "name": 'Ironing',
    "value": 10,
    "img": "/img/event/4.png"
  },
  {
    "id": 'freezer',
    "name": 'Freezer',
    "value": 60,
    "img": "/img/event/FridgeB-01.png"
  },
  {
    "id": 'loundry',
    "name": 'Loundry',
    "value": 60,
    "img": "/img/event/ACb-01.png"
  },
  {
    "id": 'Power',
    "name": 'Power',
    "value": 20,
    "img": "/img/event/powerb-01.png"
  }
];

// Get data considering interval, date point and direction
var getBubbleData = function(interval, x, direction, appliance, isForecast, apiKey, registrationToken, endPointing) {
    d3.timeFormatLocale({
  "dateTime": "%A, le %e %B %Y, %X",
  "date": "%d/%m/%Y",
  "time": "%H:%M:%S",
  "periods": ["AM", "PM"],
  "days": ["dimanche", "lundi", "mardi", "mercredi", "jeudi", "vendredi", "samedi"],
  "shortDays": ["dim.", "lun.", "mar.", "mer.", "jeu.", "ven.", "sam."],
  "months": ["janvier", "février", "mars", "avril", "mai", "juin", "juillet", "août", "septembre", "octobre", "novembre", "décembre"],
  "shortMonths": ["janv.", "févr.", "mars", "avr.", "mai", "juin", "juil.", "août", "sept.", "oct.", "nov.", "déc."]
});


  // Get interval
  if(interval == null) { interval = 'day'; }
  if(direction == null) { direction = 'both'; }
  if(appliance == null) { appliance = 'all'; }
  if(isForecast == null) { isForecast = true; }

  // Get x
  if(x == null) { x = 'Today'; }
  else { x = d3.timeParse(interval === 'year' ? "%Y" : "%Y-%m-%d")(x); }

  var data = [];

  switch(interval) {

    case 'week':

      // Set is now
      var lastMonday, sDay, eDay, startDate, endDate, fStartDate, fEndDate;

      // Get time for today
      if(x === 'Today') {

        // Get start date for actual and forecasted values
        sDay = gViz.shared.helpers.date.getLastMonday(new Date()).addDays(-6);
        startDate =`${sDay.setHours(0,0,1,000)}`;
        fStartDate =`${sDay.setHours(0,0,1,000)}`;

        // Get end date for actual and forecasted values
        eDay = gViz.shared.helpers.date.getLastMonday(new Date()).addDays(14);
        fEndDate = `${eDay.setHours(23,59,59,999)}`;
        endDate = `${(new Date()).getTime()}`;

      // Get time for other periods
      } else {

        // Get start date for actual and forecasted values
        sDay = gViz.shared.helpers.date.getLastMonday((new Date(x)).addDays(direction === 'prev' ? -7 : 7)).addDays(1);
        fStartDate = `${sDay.setHours(0,0,1,000)}`;
        startDate = `${sDay.setHours(0,0,1,000)}`;

        // Get end date for actual and forecasted values
        eDay = gViz.shared.helpers.date.getLastMonday((new Date(x)).addDays(direction === 'prev' ? -7 : 7)).addDays(6);
        fEndDate = `${eDay.setHours(23,59,59,999)}`;
        endDate = `${eDay.setHours(23,59,59,999)}`;

        // Fix start and end date not to go over now
        if(new Date(parseInt(startDate)) > new Date) { startDate = `${(new Date()).getTime()}`; }
        if(new Date(parseInt(endDate)) > new Date) { endDate = `${(new Date()).getTime()}`; }

      }

      // Remove isForecast
      if(isForecast != null && isForecast === false) { fStartDate = fEndDate = `${(new Date()).getTime()}`; }

      // Fetch data from api :: forecast = "forecast" appliance (change when API is done)
      var fReq = window.gViz.shared.data.api.fetch("forecast", fStartDate.substring(0,fStartDate.length-3), fEndDate.substring(0,fEndDate.length-3), "86400", 100000, null, apiKey, registrationToken, endPointing);

      // Fetch data from api :: actual = "all" appliance
      var aReq = window.gViz.shared.data.api.fetch(appliance, startDate.substring(0,startDate.length-3), endDate.substring(0,endDate.length-3), "86400", 100000, null, apiKey, registrationToken, endPointing);

      // Add or remove days depending on direction
      if(direction === 'prev') { x.addDays(-1); }
      else if(direction === 'next') { x.addDays(1); }

      return $.when(fReq, aReq).done(function(fData, aData) {

        // console.log("Fetched forecast new data", `${fData.consumption.t.length} points`);
        // console.log("Fetched actual new data", `${aData.consumption.t.length} points`);

        // Initialize parents structures
        var parents = {}, itemNames = {};

        // Parse new data from api
        fData.consumption.t.forEach(function(d, i) {

          // Get date from data
          var days = d3.timeFormat("%a")(new Date(parseInt(`${d}000`)));
          var index = d3.timeFormat("%u")(new Date(parseInt(`${d}000`)));
          var date = new Date(parseInt(`${d}000`)).addDays(1 - index);
          var isCurrentYear = date.getYear() === (new Date()).getYear();

          // Initialize parent structure for date if not initialized
          if(parents[d3.timeFormat("%Y-%m-%d")(date)] == null) {
            parents[d3.timeFormat("%Y-%m-%d")(date)] = {
              "x": d3.timeFormat("%Y-%m-%d")(date),
              "name": d3.timeFormat("%b %d" + (isCurrentYear ? "" : ", %Y"))(date),
              "values": [
              ]
            };
          }

          parents[d3.timeFormat("%Y-%m-%d")(date)].values.push({
            "id": "forecast",
            "x": days,
            "name": days,
            "date": date,
            "y": fData.consumption.w[i],
            "index": index,
            "color": "#ddd",
            "secondaryColor": "#233a71",
            "opacity": 1
          });

          // Store item and parent item names
          if(itemNames[d3.timeFormat("%Y-%m-%d")(date)] == null) { itemNames[d3.timeFormat("%Y-%m-%d")(date)] = {} };
          itemNames[d3.timeFormat("%Y-%m-%d")(date)][days.toString()] = true;
        });

        // Parse new data from api
        aData.consumption.t.forEach(function(d, i) {

          // Get date from data
          var days = d3.timeFormat("%a")(new Date(parseInt(`${d}000`)));
          var index = d3.timeFormat("%u")(new Date(parseInt(`${d}000`)));
          var date = new Date(parseInt(`${d}000`)).addDays(1 - index);
          var isCurrentYear = date.getYear() === (new Date()).getYear();

          // Initialize parent structure for date if not initialized
          if(parents[d3.timeFormat("%Y-%m-%d")(date)] == null) {
            parents[d3.timeFormat("%Y-%m-%d")(date)] = {
              "x": d3.timeFormat("%Y-%m-%d")(date),
              "name": d3.timeFormat("%b %d" + (isCurrentYear ? "" : ", %Y"))(date),
              "values": [
              ]
            };
          }

          parents[d3.timeFormat("%Y-%m-%d")(date)].values.push({
            "id": "actual",
            "x": days,
            "name": days,
            "date": date,
            "y": aData.consumption.w[i],
            "index": index,
            "color": "#2566bc",
            "secondaryColor": "rgba(36,121,219,0.7)",
            "opacity": 1
          });

          // Store item and parent item names
          if(itemNames[d3.timeFormat("%Y-%m-%d")(date)] == null) { itemNames[d3.timeFormat("%Y-%m-%d")(date)] = {} };
          itemNames[d3.timeFormat("%Y-%m-%d")(date)][days.toString()] = true;
        });

        // Set missing months to 0 just to fit axis
        Object.keys(itemNames).forEach(function(k) {
          d3.range(7).forEach(function(d, i) {
            var days = d3.timeFormat("%a")((new Date()).setDate(i+1));
            var index = d3.timeFormat("%u")((new Date()).setDate(i+1));
            if(itemNames[k][days.toString()] == null) {
              parents[k].values.push({
                "id": isForecast ? "forecast" : "actual",
                "x": days.toString(),
                "name": days.toString(),
                "y": 0,
                "index": index,
                "color": "transparent",
                "secondaryColor": "transparent",
                "opacity": 0
              });
            }
          });
        });

        Object.keys(parents).sort(function(a,b) { return d3.ascending(a,b); }).forEach(function(k) {
          parents[k].values = parents[k].values.sort(function(a,b) {
            return d3.descending(a.id,b.id) || d3.ascending(+a.index,+b.index);
          });
          data.push(parents[k]);
        });

        this._data = data;
        this._data = bubbleFakeData.slice(0,2);

      });

      break;

    case 'month':

      // Set is now
      var now = new Date(), xDate = new Date(x);
      var sDay, eDay, startDate, endDate, fStartDate, fEndDate;

      // Get time for today
      if(x === 'Today') {

        // Get start date for actual and forecasted values
        sDay = (new Date(now.getFullYear(), now.getMonth(), 2)).addMonths(-1);
        fStartDate =`${sDay.setHours(0,0,1,000)}`;
        startDate =`${sDay.setHours(0,0,1,000)}`;

        // Get end date for actual and forecasted values
        eDay = (new Date((new Date()).getFullYear(), now.getMonth() + 1, 1)).addMonths(+1);
        fEndDate = `${eDay.setHours(23,59,59,999)}`;
        endDate = `${(new Date()).getTime()}`;

      // Get time for other periods
      } else {

        // Add or remove months from date, depending on direction
        xDate.addMonths(direction === 'prev' ? -1 : 1);

        // Get start date for actual and forecasted values
        sDay = new Date(xDate.getFullYear(), xDate.getMonth(), 2);
        fStartDate =`${sDay.setHours(0,0,1,000)}`;
        startDate =`${sDay.setHours(0,0,1,000)}`;

        // Get end date for actual and forecasted values
        eDay = new Date(xDate.getFullYear(), xDate.getMonth() + 1, 1);
        fEndDate = `${eDay.setHours(0,59,59,999)}`;
        endDate = `${eDay.setHours(0,59,59,999)}`;

        // Fix start and end date not to go over now
        if(new Date(parseInt(startDate)) > new Date) { startDate = `${(new Date()).getTime()}`; }
        if(new Date(parseInt(endDate)) > new Date) { endDate = `${(new Date()).getTime()}`; }

      }

      // Remove isForecast
      if(isForecast != null && isForecast === false) { fStartDate = fEndDate = `${(new Date()).getTime()}`; }

      // Fetch data from api :: forecast = "forecast" appliance (change when API is done)
      var fReq = window.gViz.shared.data.api.fetch("forecast", fStartDate.substring(0,fStartDate.length-3), fEndDate.substring(0,fEndDate.length-3), "86400", 100000, null, apiKey, registrationToken, endPointing);

      // Fetch data from api :: actual = "all" appliance
      var aReq = window.gViz.shared.data.api.fetch(appliance, startDate.substring(0,startDate.length-3), endDate.substring(0,endDate.length-3), "86400", 100000, null, apiKey, registrationToken, endPointing);

      // Add or remove days depending on direction
      if(direction === 'prev') { x.addDays(-1); }
      else if(direction === 'next') { x.addDays(1); }

      return $.when(fReq, aReq).done(function(fData, aData) {

        // console.log("Fetched forecast new data", `${fData.consumption.t.length} points`);
        // console.log("Fetched actual new data", `${aData.consumption.t.length} points`);

        // Initialize parents structures
        var parents = {}, itemNames = {};

        // Parse new data from api
        fData.consumption.t.forEach(function(d, i) {

          // Get date from data
          var days = d3.timeFormat("%e")(new Date(parseInt(`${d}000`))).replace(' ','');
          var date = new Date((new Date(parseInt(`${d}000`))).setDate(1));
          var isCurrentYear = date.getYear() === (new Date()).getYear();

          // Initialize parent structure for date if not initialized
          if(parents[d3.timeFormat("%Y-%m-%d")(date)] == null) {
            parents[d3.timeFormat("%Y-%m-%d")(date)] = {
              "x": d3.timeFormat("%Y-%m-%d")(date),
              "name": d3.timeFormat("%B" + (isCurrentYear ? "" : ", %Y"))(date),
              "values": [
              ]
            };
          }

          parents[d3.timeFormat("%Y-%m-%d")(date)].values.push({
            "id": "forecast",
            "x": days,
            "name": days,
            "date": date,
            "y": fData.consumption.w[i],
            "index": +days,
            "color": "#ddd",
            "secondaryColor": "#233a71",
            "opacity": 1
          });

          // Store item and parent item names
          if(itemNames[d3.timeFormat("%Y-%m-%d")(date)] == null) { itemNames[d3.timeFormat("%Y-%m-%d")(date)] = {} };
          itemNames[d3.timeFormat("%Y-%m-%d")(date)][days.toString()] = true;
        });

        // Parse new data from api
        aData.consumption.t.forEach(function(d, i) {

          // Get date from data
          var days = d3.timeFormat("%e")(new Date(parseInt(`${d}000`))).replace(' ','');
          var date = new Date((new Date(parseInt(`${d}000`))).setDate(1));
          var isCurrentYear = date.getYear() === (new Date()).getYear();

          // Initialize parent structure for date if not initialized
          if(parents[d3.timeFormat("%Y-%m-%d")(date)] == null) {
            parents[d3.timeFormat("%Y-%m-%d")(date)] = {
              "x": d3.timeFormat("%Y-%m-%d")(date),
              "name": d3.timeFormat("%B" + (isCurrentYear ? "" : ", %Y"))(date),
              "values": [
              ]
            };
          }

          parents[d3.timeFormat("%Y-%m-%d")(date)].values.push({
            "id": "actual",
            "x": days,
            "name": days,
            "date": date,
            "y": aData.consumption.w[i],
            "index": +days,
            "color": "#2566bc",
            "secondaryColor": "rgba(36,121,219,0.7)",
            "opacity": 1
          });

          // Store item and parent item names
          if(itemNames[d3.timeFormat("%Y-%m-%d")(date)] == null) { itemNames[d3.timeFormat("%Y-%m-%d")(date)] = {} };
          itemNames[d3.timeFormat("%Y-%m-%d")(date)][days.toString()] = true;
        });

        // Set missing months to 0 just to fit axis
        Object.keys(itemNames).forEach(function(k) {
          d3.range(31).forEach(function(d, i) {
            if(itemNames[k][(i+1).toString()] == null) {
              parents[k].values.push({
                "id": isForecast ? "forecast" : "actual",
                "x": (i+1).toString(),
                "name": (i+1).toString(),
                "y": 0,
                "index": i+1,
                "color": "transparent",
                "secondaryColor": "transparent",
                "opacity": 0
              });
            }
          });
        });

        Object.keys(parents).sort(function(a,b) { return d3.ascending(a,b); }).forEach(function(k) {
          parents[k].values = parents[k].values.sort(function(a,b) {
            return d3.descending(a.id,b.id) || d3.ascending(+a.index,+b.index);
          });
          data.push(parents[k]);
        });

        this._data = data;
        this._data = bubbleFakeData.slice(0,3);

      });

      break;

    case 'year':

      // Set is now
      var now = new Date(), xDate = new Date(x);
      var sDay, eDay, startDate, endDate, fStartDate, fEndDate, months = [];

      // Get year
      var year = x === 'Today' ? now.getFullYear() : (xDate.getFullYear() + (direction === 'prev' ? - 1 : 1));

      // Iterate over months to join an year
      d3.range(12).forEach(function(d, i) {
        sDay = new Date(year, i, 2);
        eDay = new Date(year, i + 1, 1);
        fStartDate = `${sDay.setHours(0,0,1,0)}`;
        fEndDate = `${eDay.setHours(23,59,59,999)}`;
        startDate = `${sDay.setHours(0,0,1,0)}`;
        endDate = `${eDay.setHours(23,59,59,999)}`;

        // Remove isForecast
        if(isForecast != null && isForecast === false) { fStartDate = fEndDate = `${(new Date()).getTime()}`; }

        // Fetch data from api :: forecast = "forecast" appliance (change when API is done)
        months.push(window.gViz.shared.data.api.fetch("forecast", fStartDate.substring(0,fStartDate.length-3), fEndDate.substring(0,fEndDate.length-3),"86400", 100000, null, apiKey, registrationToken, endPointing));

        if((now.getMonth() >= i && now.getFullYear() === year) || year < now.getFullYear()) {

          // Reset endDate to now
          if(i === now.getMonth() && now.getFullYear() === year) { endDate = `${now.getTime()}`; }

          // Fetch data from api :: Actual = "all" appliance (change when API is done)
          months.push(window.gViz.shared.data.api.fetch(appliance, startDate.substring(0,startDate.length-3), endDate.substring(0,endDate.length-3),"86400", 100000, null, apiKey, registrationToken, endPointing));

        }

      });

      return $.when.apply($, months).done(function() {

        // Initialize parents structures
        var parents = {};

        // console.log("Fetched new data from year");

        // Iterate over responses of promises
        Array.from(arguments).forEach(function(data) {

          if(data.consumption.w.length !== 0) {

            // Parse new data from api
            var date = d3.min(data.consumption.t, function(d) { return new Date(parseInt(`${d}000`)); })
            var months = d3.timeFormat("%b")(date);
            var dataSum = d3.sum(data.consumption.w, function(d) { return d; })

            // Initialize parent structure for date if not initialized
            if(parents[d3.timeFormat("%Y")(date)] == null) {
              parents[d3.timeFormat("%Y")(date)] = {
                "x": d3.timeFormat("%Y")(date),
                "name": d3.timeFormat("%Y")(date),
                "values": [
                ]
              };
            }

            parents[d3.timeFormat("%Y")(date)].values.push({
              "id": data.appliance,
              "x": months,
              "date": date,
              "name": months,
              "y": dataSum,
              "color": data.appliance === 'forecast' ? "#ddd" : "#2566bc",
              "secondaryColor": data.appliance === 'forecast' ? "#233a71" : "rgba(36,121,219,0.7)",
              "opacity": 1
            });

          }

        });

        Object.keys(parents)
          .sort(function(a,b) { return d3.ascending(a,b); })
          .forEach(function(k) {
            parents[k].values = parents[k].values.sort(function(a,b) {
              return d3.ascending(a.date, b.date) || d3.descending(a.id, b.id);
            });
            data.push(parents[k]);
          });

        this._data = data;
        this._data = bubbleFakeData.slice(0,4);

      });

      break;

    default:

      // Set is now
      var now = new Date();
      var sDay, eDay, startDate, endDate, fStartDate, fEndDate;

      // Get time for today
      if(x === 'Today') {

        // Get start date for actual and forecasted values
        sDay = (new Date()).addDays(-1);
        startDate = `${sDay.setHours(1,0,0,0)}`;
        fStartDate = `${sDay.setHours(1,0,0,0)}`;

        // Get end date for actual and forecasted values
        eDay = (new Date()).addDays(2);
        fEndDate = `${eDay.setHours(1,0,0,0)}`;
        endDate = `${(new Date()).getTime()}`;

      // Get time for other periods
      } else {

        // Get start date for actual and forecasted values
        sDay = (new Date(x)).addDays(direction === 'prev' ? -1 : 1);
        startDate = `${sDay.setHours(1,0,0,0)}`;
        fStartDate = `${sDay.setHours(1,0,0,0)}`;

        // Get end date for actual and forecasted values
        eDay = (new Date(x)).addDays(direction === 'prev' ? 0 : 2);
        fEndDate = `${eDay.setHours(0,59,59,59)}`;
        endDate = `${direction === 'prev' ? eDay.setHours(0,59,59,59) : eDay.setHours(now.getHours(), now.getMinutes(), now.getSeconds(), now.getMilliseconds())}`;

        // Fix start and end date not to go over now
        if(new Date(parseInt(startDate)) > new Date) { startDate = `${(new Date()).getTime()}`; }
        if(new Date(parseInt(endDate)) > new Date) { endDate = `${(new Date()).getTime()}`; }

      }

      // Remove isForecast
      if(isForecast != null && isForecast === false) { fStartDate = fEndDate = `${(new Date()).getTime()}`; }

      // Fetch data from api :: forecast = "forecast" appliance (change when API is done)
      var fReq = window.gViz.shared.data.api.fetch("forecast", fStartDate.substring(0,fStartDate.length-3), fEndDate.substring(0,fEndDate.length-3), "3600", 100000, null, apiKey, registrationToken, endPointing);

      // Fetch data from api :: actual = "all" appliance
      var aReq = window.gViz.shared.data.api.fetch(appliance, startDate.substring(0,startDate.length-3), endDate.substring(0,endDate.length-3), "3600", 100000, null, apiKey, registrationToken, endPointing);

      // Add or remove days depending on direction
      if(direction === 'prev') { x.addDays(-1); }
      else if(direction === 'next') { x.addDays(1); }

      return $.when(fReq, aReq).done(function(fData, aData) {

        // console.log("Fetched forecast new data", `${fData.consumption.t.length} points`);
        // console.log("Fetched actual new data", `${aData.consumption.t.length} points`);

        // Initialize parents structures
        var parents = {}, itemNames = {};

        // Parse new data from api
        fData.consumption.t.forEach(function(d, i) {

          // Get date from data
          var hours = (new Date(parseInt(`${d}000`))).getHours();
          var date = hours === 0 ? (new Date(parseInt(`${d}000`))).addDays(-1) : (new Date(parseInt(`${d}000`)));
          var isCurrentYear = date.getYear() === (new Date()).getYear();

          // Initialize parent structure for date if not initialized
          if(parents[d3.timeFormat("%Y-%m-%d")(date)] == null) {
            parents[d3.timeFormat("%Y-%m-%d")(date)] = {
              "x": d3.timeFormat("%Y-%m-%d")(date),
              "name": d3.timeFormat("%A, %b %d" + (isCurrentYear ? "" : ", %Y"))(date),
              "values": [
              ]
            };
          }

          parents[d3.timeFormat("%Y-%m-%d")(date)].values.push({
            "id": "forecast",
            "x": hours === 0 ? 24 : hours,
            "name": hours === 0 ? 24 : hours,
            "date": date,
            "y": fData.consumption.w[i],
            "index": hours === 0 ? 24 : hours,
            "color": "#ddd",
            "secondaryColor": "#233a71",
            "opacity": 1
          });

          // Store item and parent item names
          if(itemNames[d3.timeFormat("%Y-%m-%d")(date)] == null) { itemNames[d3.timeFormat("%Y-%m-%d")(date)] = {} };
          itemNames[d3.timeFormat("%Y-%m-%d")(date)][(hours === 0 ? 24 : hours).toString()] = true;
        });

        // Parse new data from api
        aData.consumption.t.forEach(function(d, i) {

          // Get date from data
          var hours = (new Date(parseInt(`${d}000`))).getHours();
          var date = hours === 0 ? (new Date(parseInt(`${d}000`))).addDays(-1) : (new Date(parseInt(`${d}000`)));
          var isCurrentYear = date.getYear() === (new Date()).getYear();

          // Initialize parent structure for date if not initialized
          if(parents[d3.timeFormat("%Y-%m-%d")(date)] == null) {
            parents[d3.timeFormat("%Y-%m-%d")(date)] = {
              "x": d3.timeFormat("%Y-%m-%d")(date),
              "name": d3.timeFormat("%A, %b %d" + (isCurrentYear ? "" : ", %Y"))(date),
              "values": [
              ]
            };
          }

          parents[d3.timeFormat("%Y-%m-%d")(date)].values.push({
            "id": "actual",
            "x": hours === 0 ? 24 : hours,
            "name": hours === 0 ? 24 : hours,
            "date": date,
            "y": aData.consumption.w[i],
            "index": hours === 0 ? 24 : hours,
            "color": "#2566bc",
            "secondaryColor": "rgba(36,121,219,0.7)",
            "opacity": 1
          });

          // Store item and parent item names
          if(itemNames[d3.timeFormat("%Y-%m-%d")(date)] == null) { itemNames[d3.timeFormat("%Y-%m-%d")(date)] = {} };
          itemNames[d3.timeFormat("%Y-%m-%d")(date)][(hours === 0 ? 24 : hours).toString()] = true;
        });

        // Set missing months to 0 just to fit axis
        Object.keys(itemNames).forEach(function(k) {
          d3.range(23).forEach(function(d, i) {
            if(itemNames[k][(i+1).toString()] == null) {
              parents[k].values.push({
                "id": isForecast ? "forecast" : "actual",
                "x": (i+1).toString(),
                "name": (i+1).toString(),
                "y": 0,
                "index": i,
                "color": "transparent",
                "secondaryColor": "transparent",
                "opacity": 0
              });
            }
          });
        });

        Object.keys(parents).sort(function(a,b) { return d3.ascending(a,b); }).forEach(function(k) {
          parents[k].values = parents[k].values.sort(function(a,b) { return d3.descending(a.id,b.id) || d3.ascending(a.index,b.index); });
          data.push(parents[k]);
        });

        this._data = data;
        this._data = bubbleFakeData;

      });

      break;

  }

  return data;

}

// Initialize the helpers class¬
gViz.shared.data.bubbleChart = {

  // Get mock data
  fetch: function(interval, x, direction, appliance, isForecast, apiKey, registrationToken, endPointing) {
    return getBubbleData(interval, x, direction, appliance, isForecast, apiKey, registrationToken, endPointing);
  },

  // Replace text variables
  format: function() {

    return {
      "title": "",
      "size": {
        "title": "",
        "prefix": "",
        "sufix": "W",
        "format": ""
      },
      "data": []
    };
  },

  // Replace text variables
  getData: getBubbleData

}
