// Initialize helpers array
gViz.shared.data.lineChart = {};

gViz.shared.data.lineChart.getDataOrCache = function(sDay, eDay, appliance, apiKey, registrationToken, endPointing, zoomLevel) {

  // Initialize dates
  var startDate = `${sDay.getTime()}`, endDate = `${eDay.getTime()}`;

  // Parse
  var start = new Date(sDay.getTime());
  var end = new Date(eDay.getTime());

  // Get cache interval
  var cacheInt = gViz.shared.data.lineChart.getCacheInterval(zoomLevel);

  // Fix start and end dates
  start = new Date() < start ? new Date() : start;
  end = new Date() < end ? new Date() : end;

  // Get parameters
  var cParam = {
    start: start.toISOString().split(".")[0] + ".000Z",
    end: end.toISOString().split(".")[0] + ".000Z",
    range: parseInt((end - start) / (cacheInt * 1000))
  }

  // Get initial and final data cached position
  cParam.cacheStart = gViz.shared.data.lineChart.cached[cacheInt][appliance].findIndex(function(d) { return d.x == cParam.start; });
  cParam.cacheEnd   = gViz.shared.data.lineChart.cached[cacheInt][appliance].findIndex(function(d) { return d.x == cParam.end; });

  if(cParam.cacheStart !== -1 && cParam.cacheEnd !== -1 && Math.abs(cParam.cacheEnd - cParam.cacheStart) >= cParam.range) {

    // Get appliance data
    var aData = { appliance: appliance, consumption: { w: [], t: [] }, isCache: true };
    gViz.shared.data.lineChart.cached[cacheInt][appliance].slice(cParam.cacheStart, cParam.cacheEnd+1).forEach(function(d) {
      aData.consumption.w.push(d.value);
      var date = `${d.date.getTime()}`;
      aData.consumption.t.push(date.substring(0,date.length-3));
    });

    return window.gViz.shared.data.api.resolvedFetch(aData);

  } else {

    // // Dev only
    //var aData = { appliance: appliance, consumption: { w: [], t: [] }, isCache: true };
    //return window.gViz.shared.data.api.resolvedFetch(aData);

    // // Fetch data from api :: actual = "all" appliance
    return window.gViz.shared.data.api.fetch(appliance, startDate.substring(0,startDate.length-3), endDate.substring(0,endDate.length-3), cacheInt, 100000, null, apiKey, registrationToken, endPointing);

  }

}

// Store data in cache
gViz.shared.data.lineChart.cacheData = function(rData, zoomLevel) {

  // Apply cache only if the data wasn't fetched from cache
  if(rData.isCache == null || rData.isCache !== true) {

    // Get cache interval
    var cacheInt = gViz.shared.data.lineChart.getCacheInterval(zoomLevel);

    // Cache data for future uses
    var keys = {};
    rData.consumption.t.forEach(function(d, i) {
      gViz.shared.data.lineChart.cached[cacheInt][rData.appliance].push({
        x: (new Date(parseInt(`${d}000`))).toISOString(),
        date: new Date(parseInt(`${d}000`)),
        value: rData.consumption.w[i]
      });
    });

    // Unique keys
    var newCacheData = [], storedKeys = {};
    gViz.shared.data.lineChart.cached[cacheInt][rData.appliance].forEach(function(d) {
      if(storedKeys[d.x] == null) {
        storedKeys[d.x] = true;
        newCacheData.push(d);
      }
    });

    // Sort new cache array and store data
    gViz.shared.data.lineChart.cached[cacheInt][rData.appliance] = newCacheData.sort(function(a,b) { return d3.ascending(a.date.getTime(), b.date.getTime()); });

  }

}

// Get data considering interval, date point and direction
gViz.shared.data.lineChart.getData = function(start, end, appliance, zoomLevel, apiKey, registrationToken, endPointing) {

  // Get cache interval
  var cacheInt = gViz.shared.data.lineChart.getCacheInterval(zoomLevel);

  // Initialize data cache for appliance and forecast
  gViz.shared.data.lineChart.cached[cacheInt] = gViz.shared.data.lineChart.cached[cacheInt] || {};
  gViz.shared.data.lineChart.cached[cacheInt][appliance] = gViz.shared.data.lineChart.cached[cacheInt][appliance] || [];

  // Get data depending on cache
  var data = [], req = null, mod = 0;

  if(appliance == null) { appliance = 'all'; }
  if(zoomLevel == null) { zoomLevel = 1; }

  if(zoomLevel >= 1) {
    if(start == null && end != null) {
      end = gViz.shared.data.lineChart.dateAdjust(end, zoomLevel);
      start = (new Date(end.getTime())).addDays(-30);
    } else if (start != null && end == null) {
      start = gViz.shared.data.lineChart.dateAdjust(start, zoomLevel);
      end = (new Date(start.getTime())).addDays(30);
      end = end > new Date() ? new Date() : end;
      end = gViz.shared.data.lineChart.dateAdjust(end, zoomLevel);
    } else if (start != null && end != null) {
      start = gViz.shared.data.lineChart.dateAdjust(start, zoomLevel);
      end = gViz.shared.data.lineChart.dateAdjust(end, zoomLevel);
    }
  }

  // Fetch data for adjusted dates
  req = gViz.shared.data.lineChart.getDataOrCache(start, end, appliance, apiKey, registrationToken, endPointing, zoomLevel);

  var parseData = function(rData) {

    // Fill rData consumption arrays with 0s if there is no data
    if(rData.consumption.t.length === 0) {
      var zoomInfo = gViz.shared.data.lineChart.zoomLevelInfo(zoomLevel);
      var cacheInt = gViz.shared.data.lineChart.getCacheInterval(zoomLevel);
      var cDate = new Date(start.getTime()), cDateString;
      while(cDate <= end) {
        cDateString = `${cDate.getTime()}`;
        rData.consumption.t.push(cDateString.substring(0, cDateString.length-3));
        rData.consumption.w.push(0);
        cDate.addSeconds(cacheInt);
      }
    }
    if(parseInt(`${rData.consumption.t[rData.consumption.t.length-1]}000`) !== end.getTime()) {
        cDateString = `${end.getTime()}`;
        rData.consumption.t.push(cDateString.substring(0, cDateString.length-3));
        rData.consumption.w.push(0);
    }

    // Store data in cache
    gViz.shared.data.lineChart.cacheData(rData, zoomLevel);

    // Group data depending on the zoom level
    rData = gViz.shared.data.lineChart.groupData(rData, zoomLevel);

    // Store data to be used and also in cache
    this._data = rData;

  }

  return $.when(req).done(parseData);

}

// Group data depending on zoom level
gViz.shared.data.lineChart.groupData = function(rData, zoomLevel) {

  // Don't group for default level
  if(zoomLevel == 1 || zoomLevel == 5) { return rData; }

  // Get initial information to group data
  var zoomInfo = gViz.shared.data.lineChart.zoomLevelInfo(zoomLevel);
  var cacheInt = gViz.shared.data.lineChart.getCacheInterval(zoomLevel);
  var consT = [], consW = [], consValue = 0, consInterval = zoomInfo.seconds / cacheInt;

  // Group data
  rData.consumption.t.forEach(function(d, i) {
    if(i === 0) {
      consT.push(d);
      consW.push(rData.consumption.w[0]);
    } else {
      consValue += rData.consumption.w[i];
      if(i % consInterval === 0) {
        consT.push(d);
        consW.push(consValue);
        consValue = 0;
      }
    }
  });

  // Store new grouped values on rData
  rData.consumption.t = consT;
  rData.consumption.w = consW;

  return rData;
}

// Replace text variables
gViz.shared.data.lineChart.format = function(data) {

  // Set date format
  var dateOrder = gViz.shared.helpers.date.translated.DATEORDER['NOW'];

  var json = {
    "title": "",
    "legend": {
      "isVisible": false,
    },
    "tooltip": {
      "backgroundColor": "#FFF",
      "borderColor": "transparent",
      "title": [ "This is a title example" ],
      "body": [ "This is a body example" ]
    },
    "y": {
      "title": "",
      "prefix": "",
      "sufix": "W",
      "format": "",
      "min": 0,
      "grid": false
    },
    "x": {
      "title": "",
      "type": "time",
      "prefix": "",
      "sufix": "",
      "format": "",
      "inFormat": "%Y-%m-%d %H:%M:%S",
      "outFormat": dateOrder,
      "tickSpace": 100,
      "grid": false
    },
    "data": [
      {
        "id": "actual",
        "name": "Mês Atual",
        "color": "#3d62cc",
        "pointColor": "#267c56",
        "hasArea": true,
        "areaColor": "#e0e9f4",
        "strokeWidth": 2,
        "pointSize": 0,
        "values": []
      }
    ]
  };

  // Get in and Out format to parse dates
  var inFmt = function(d) { return new Date(d); };
  var outFmt = d3.timeFormat("%Y-%m-%d %H:%M:%S");

  // Sort elements and store it on the json
  data.sort(function(a,b) { return d3.ascending(+a.x, +b.x); }).forEach(function(d) {
    d.x = outFmt(inFmt(d.x));
    d.y = d.y;
    json.data[0].values.push(d);
  });

  return json;

};

gViz.shared.data.lineChart.parse =  function(data) {

  var _data = [];

  // Get in and out format to parse dates
  var inFmt = function(d) { return new Date(d); };
  var outFmt = d3.timeFormat("%Y-%m-%d %H:%M:%S");

  // sort elements and store it on the json
  data.sort(function(a,b) { return d3.ascending(+a.x, +b.x); }).forEach(function(d) {
    d.x = outFmt(inFmt(d.x));
    d.y = d.y;
    _data.push(d);
  });

  return _data;

};

// Store cached data for initial values
gViz.shared.data.lineChart.cached = {};

// Adjust data point based on zoom level
gViz.shared.data.lineChart.dateAdjust = function(date, zoomLevel) {
  var aDate, mod;
  if(zoomLevel <= 1) {
    aDate = new Date(date);
    mod = aDate.getMinutes() % 3;
    return new Date(aDate.setHours(aDate.getHours(), aDate.getMinutes() - mod, 0, 000));
  } else {
    aDate = new Date(date);
    mod = aDate.getSeconds() % 3;
    return new Date(aDate.setHours(aDate.getHours(), aDate.getMinutes(), aDate.getSeconds() - mod, 000));
  }
}

gViz.shared.data.lineChart.getCacheInterval = function(zoomLevel) {
  if(zoomLevel <= 1) { return 180; }
  else { return 3; }
}

gViz.shared.data.lineChart.zoomLevelInfo = function(zoomLevel) {
  switch(zoomLevel) {
    case -2:
      return { interval: 43200, max: 61, num: 20, seconds: 2160, zoom: 0.08333333333333333 };
    case -1:
      return { interval: 21600, max: 61, num: 20, seconds: 1080, zoom: 0.16666666666666666 };
    case 0:
      return { interval: 7200, max: 61, num: 20, seconds: 360, zoom: 0.5 };
    case 1:
      return { interval: 3600, max: 61, num: 20, seconds: 180, zoom: 1 };
    case 2:
      return { interval: 1800, max: 61, num: 20, seconds: 90, zoom: 2 };
    case 3:
      return { interval: 900, max: 61, num: 20, seconds: 45, zoom: 4 };
    case 4:
      return { interval: 300, max: 61, num: 20, seconds: 15, zoom: 12 };
    case 5:
      return { interval: 60, max: 61, num: 20, seconds: 3, zoom: 60 };
    default:
      return { interval: 3600, max: 61, num: 20, seconds: 180, zoom: 1 };
  }
}

// 12h, 24 points
// 6h, 24points
// 2h, 16 points
// 1h, start screen, 20 points
// 30min 30pts
// 15min 30pts
// 5min, 20pts
// 1min, 24pts

// Get data
gViz.shared.data.lineChart.fetch = function(start, end, appliance, zoomLevel, apiKey, registrationToken, endPointing) {
  return gViz.shared.data.lineChart.getData(start, end, appliance, zoomLevel, apiKey, registrationToken, endPointing);
}

// Set unit
gViz.shared.data.lineChart.setUnit = function(data, unit) {
  if(unit != null) { data.y.sufix = unit; }
}
