Date.prototype.monthDays = function(){
    var d= new Date(this.getFullYear(), this.getMonth()+1, 0);
    return d.getDate();
}

// Extend date object to increase number of years
Date.prototype.addYears = function(years) {
  this.setFullYear(this.getFullYear() + parseInt(years));
  return this;
};

// Extend date object to increase number of months
Date.prototype.addMonths = function(months) {
  this.setMonth(this.getMonth() + parseInt(months));
  return this;
};

// Extend date object to increase number of days
Date.prototype.addDays = function(days) {
  this.setDate(this.getDate() + parseInt(days));
  return this;
};

// Extend date object to increase number of minutes
Date.prototype.addMinutes = function(minutes) {
  this.setTime(this.getTime() + minutes*60000);
  return this;
}

// Extend date object to increase number of seconds
Date.prototype.addSeconds = function(seconds) {
  this.setTime(this.getTime() + seconds*1000);
  return this;
}

// Initialize the helpers class¬
gViz.shared.helpers.date = function() {

  // Multi Format
  var formatMillisecond = d3.timeFormat(".%L"),
      formatSecond = d3.timeFormat(":%S"),
      formatMinute = d3.timeFormat("%H:%M"),
      formatHour = d3.timeFormat("%H"),
      formatDay = d3.timeFormat("%a %d"),
      formatWeek = d3.timeFormat("%b %d"),
      formatMonth = d3.timeFormat("%B"),
      formatYear = d3.timeFormat("%Y");

  return {

    print: function(date) {
      return d3.timeFormat("%d/%m/%Y %H:%M:%S")(date);
    },

    translated: {
    },

    // Set locale from d3 time format
    setLocale: function(language) {
      d3.json("https://unpkg.com/d3-time-format@2/locale/"+language+".json", function(error, locale) {
        d3.timeFormatDefaultLocale(locale);
      });
    },

    getWeekDayNumberFromMonday: function(d) {
      if(d === 'Mon') { return 0; }
      else if(d === 'Tue') { return 1; }
      else if(d === 'Wed') { return 2; }
      else if(d === 'Thu') { return 3; }
      else if(d === 'Fri') { return 4; }
      else if(d === 'Sat') { return 5; }
      else if(d === 'Sun') { return 6; }
      else { return 0; }
    },

    getLastMonday: function (d) {
      var t = new Date(d);
      var day = t.getDay();
      t.setDate(t.getDate() - day);
      t.setHours(0, 0, 0);
      return t.addDays(day === 0 ? -6 : 1);
    },

    // Helpers
    helpers: {
      daysInMonth: function(month,year) {
        return new Date(year, month, 0).getDate();
      },
      dayDiff: function(to, from) {
        var n = 86400000;
        return Math.round(Math.abs((to - from)) / n);
      },
      diff: function(to, from, interval) {
        var n = 86400000;
        switch(interval) {
          case 'quarter': return Math.ceil((to.getMonth() - from.getMonth() + (12 * (to.getFullYear() - from.getFullYear())) + 1)/3); break;
          case 'month': return to.getMonth() - from.getMonth() + (12 * (to.getFullYear() - from.getFullYear())) + 1; break;
          case 'week': n = 60 * 60 * 1000 * 24 * 7; break;
          case 'day': n = 60 * 60 * 1000 * 24; break;
          case 'hour': n = 60 * 60 * 1000; break;
          case 'minute': n = 60 * 1000; break;
          case 'second': n = 1000; break;
          default: n = 86400000; break;
        }
        return Math.round(Math.abs((to - from)) / n) + 1;
      },
    },

    // Formats
    format: {
      hms: {
        parse:  function(d) { return d3.timeParse("%H:%M")(d); },
        format: function(d) { return d3.timeFormat("%H:%M")(d); },
      },
      en: {
        parse:  function(d) { return d3.timeParse("%Y-%m-%d")(d); },
        format: function(d) { return d3.timeFormat("%Y-%m-%d")(d); },
      },
      milliseconds: {
        parse:  function(d) { return d3.timeParse("%Y-%m-%d %H:%M:%S.%L")(d); },
        format: function(d) { return d3.timeFormat("%Y-%m-%d %H:%M:%S.%L")(d); },
      },
    },

    // Multi Formats
    multiFormat: function(date) {
      return (d3.timeSecond(date) < date ? formatMillisecond
          : d3.timeMinute(date) < date ? formatSecond
          : d3.timeHour(date) < date ? formatMinute
          : d3.timeDay(date) < date ? formatHour
          : d3.timeMonth(date) < date ? (d3.timeWeek(date) < date ? formatDay : formatWeek)
          : d3.timeYear(date) < date ? formatMonth
          : formatYear)(date);
    },

    // Get format for axis
    parseFormat: function(axis) {

      // Get axis format with prefix and suffix
      if(axis != null) {

        // Set prefix and suffix
        var prefix = axis.prefix != null ? axis.prefix : "";
        var suffix  = axis.suffix != null ? axis.suffix : (axis.sufix != null ? axis.sufix : "");

        // Get input format
        var inFmt = d3.timeParse("%Y-%m-%d");
        if(axis.inFormat != null && axis.inFormat != "") { inFmt = d3.timeParse(axis.inFormat); }

        // Get input format
        var outFmt = d3.timeFormat("%Y-%m-%d");
        if(axis.outFormat != null && axis.outFormat != "") { outFmt = d3.timeFormat(axis.outFormat); }

      } else {
        var prefix = "", suffix = "", inFmt = d3.timeParse("%Y-%m-%d"), outFmt = d3.timeFormat("%Y-%m-%d");
      }

      // Return format parsed
      return function(d) { return prefix + outFmt(inFmt(d)) + suffix; };

    }
  }
}();
