// Module declaration
gViz.viz.barChart = function () {
  "use strict";

  // Get attributes values
  var _id = `vis-bar-${Math.floor(Math.random() * ((1000000000 - 5) + 1)) + 5}`;
  var _var = null;
  var action = 'build';
  var animation = 900;
  var container = null;
  var colors = { main: gViz.shared.helpers.colors.main };
  var data = [];
  var margin = { top: 22, right: 0, bottom: 45, left: 0 };
  var hover = null;
  var zoom = {};
  var priceFunction = null;;
  var xInDomain = null;;

  // Event bindings
  var onHover = function(d) { console.log(d); };
  var onClick = function(d) { console.log(d); };

  // Validate attributes
  var validate = function (step) {
    switch (step) {
      case 'build':      return (container != null) && (d3.selectAll(container).size() !== 0 || d3.select(container).size() !== 0);
      case 'initialize': return true;
      case 'axis':       return data != null && data.data != null && data.data.length > 0;
      case 'create':     return data != null && data.data != null && data.data.length > 0;
      case 'elements':   return data != null && data.data != null && data.data.length > 0;
      case 'misc':       return true;
      case 'style':      return true;
      case 'xScale':     return data != null && data.data != null && data.data.length > 0;
      case 'yScale':     return data != null && data.data != null && data.data.length > 0;
      case 'yScaleSize': return data != null && data.data != null && data.data.length > 0;
      case 'zoom':       return data != null && data.data != null && data.data.length > 0;
      default:           return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'build':

          main('initialize');
          main('style');
          main('misc');
          main('yScale');
          main('yScaleSize');
          main('xScale');
          main('yScaleSize');
          main('create');
          main('axis');
          main('zoom');
          main('elements');
          main('zoom');
          break;

        // Initialize visualization variable
        case 'initialize':

          // Initializing
          if (!_var) { _var = {};  }
          _var = gViz.viz.barChart.initialize()
            ._var(_var)
            ._id((_var._id != null) ? _var._id : _id)
            .animation(animation)
            .container(container)
            .colors(colors)
            .data(data)
            .hover(hover)
            .margin(margin)
            .zoom(zoom)
            .priceFunction(priceFunction)
            .onHover(onHover)
            .onClick(onClick)
            .xInDomain(xInDomain)
            .run();

          break;

        // Setup style functions
        case 'style':

          // Styling
          _var = gViz.viz.barChart.style()
            ._var(_var)
            .run();
          break;

        // Create initial elements
        case 'create':

          // Creating wrappers
          _var = gViz.viz.barChart.create()
            ._var(_var)
            .run();
          break;

        // Setup x scale
        case 'xScale':

          // Creating
          _var = gViz.viz.barChart.xScale()
            ._var(_var)
            .data(_var.data.data)
            .run();
          break;

        // Setup y scale
        case 'yScale':

          // Creating
          _var = gViz.viz.barChart.yScale()
            ._var(_var)
            .data(_var.data.data)
            .run();
          break;

        // Setup y scale size
        case 'yScaleSize':

          // Creating
          _var = gViz.viz.barChart.yScaleSize()
            ._var(_var)
            .data(_var.data.data)
            .run();
          break;

        // Setup axis elements
        case 'axis':

          // Running
          _var = gViz.viz.barChart.axis()
            ._var(_var)
            .action('create')
            .run();
          break;

        // Setup chart elements
        case 'elements':

          // Running
          _var = gViz.viz.barChart.elements()
            ._var(_var)
            .components(gViz.viz.barChart)
            .data(_var.data.data)
            .run();
          break;

        // Set zoom case
        case 'zoom':

          // Creating wrappers
          _var = gViz.viz.barChart.zoom()
            ._var(_var)
            .components(gViz.viz.barChart)
            .run();
          break;

        // Show misc
        case 'misc':

          // Running
          _var = gViz.viz.barChart.misc()
            ._var(_var)
            .components(gViz.viz.barChart)
            .run();
          break;

      }
    }

    // Getters
    _var.get = {

      // Get most left point
      start: function() {
        var start = null;
        _var.data.data.forEach(function(d) {
          if(_var.x(d.x) <= -_var.zoom.transform.x && _var.x(d.x) + _var.x.bandwidth() > -_var.zoom.transform.x) {
            start = d.values[0].date;
            var x = _var.x(d.x);
            d.values.forEach(function(v) {
              var xIn = _var.xIn(v.x);
              if(x + xIn <= -_var.zoom.transform.x && x + xIn + _var.xIn.bandwidth() > -_var.zoom.transform.x) {
                start = v.date;
              }
            });
          }
        });
        return start;
      },

      // Get most right point
      end: function() {
        var end = null;
        _var.data.data.forEach(function(d) {
          if(_var.x(d.x) <= -_var.zoom.transform.x+_var.width && _var.x(d.x)+_var.x.bandwidth() > -_var.zoom.transform.x+_var.width) {
            end = d.values[0].date;
            var x = _var.x(d.x);
            d.values.forEach(function(v) {
              var xIn = _var.xIn(v.x);
              if(x + xIn <= -_var.zoom.transform.x+_var.width && x + xIn + _var.xIn.bandwidth() > -_var.zoom.transform.x+_var.width) {
                end = v.date;
              }
            });
          }
        });
        return end;
      },

      // Get min y value
      min: function() { return _var.yBounds[0]; },

      // Get max y value
      max: function() { return _var.yBounds[1]; }

    }

    return _var;
  };

  // Expose global variables
  ['_id','_var','action','animation','container','colors','data','margin','hover','zoom','onHover','onClick','priceFunction','xInDomain'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return main;
    };
  });

  // Secondary functions
  main.build = function (_) { return main("build"); };

  // Execute the specific called function
  main.run = function (_) { return main(_); };

  return main;

}
