// Initialize the visualization class
gViz.viz.barChart.bars = function() {
  "use strict";

  // Get attributes values
  var _var       = null;
  var animation  = 900;
  var components = {};
  var nodeObj    = null;
  var nodeIndex  = 0;
  var node       = null;

  // Validate attributes
  var validate = function(step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function(step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Get parent selection
          var nodeSel = d3.select(nodeObj);

          // Clean wrap gradients
          gViz.shared.visualComponents.gradient()
            .action('clean')
            .run();

          // Draw grouping bar
          var wrapperBars = nodeSel.selectAll(".wrapper-bar").data([node], function(d) { return d.x; });
          wrapperBars.exit().remove();
          wrapperBars = wrapperBars.enter().append('rect').attr("class", "wrapper-bar").merge(wrapperBars);
          wrapperBars
            .style('fill', function(d) {

              // Find first node with color, if its available
              var colorIndex = _var.data.data.findIndex(function(d) { return d.color != null; });

              // Get color for group and set color
              if(d.color != null) { return d.color; }
              else if(_var.data.data.length === 1) { d.color = "transparent"; }
              else if(colorIndex === -1) { d.color = "#eaeaea"; }
              else {
                if(Math.abs(nodeIndex - colorIndex) % 2 === 0) { d.color = _var.data.data[colorIndex].color; }
                else { d.color = _var.data.data[colorIndex].color === "transparent" ? "#eaeaea" : "transparent"; }
              }

              return d.color;
            })
            .attr("x", 0)
            .attr('width', _var.x.bandwidth())
            .attr('y', -_var.margin.top)
            .attr('height',  _var.height + _var.margin.bottom + _var.margin.top)

          // Set bar width
          _var.barWidth = Math.min(_var.xIn.bandwidth(), _var.barWidth);

          // // Draw bars
          var bars = nodeSel.selectAll(".bar-rect").data(node.values, function(d) { return node.x + "--" + d.x + "--" + d.id; });
          bars.exit().remove();
          bars = bars.enter().append('path').attr("class", "bar-rect").merge(bars);
          bars
            .attr('opacity', _var.getBarOpacity)
            .style('fill', function(d) { return _var.getBarColor(node, d); })
            .transition().duration(600)
              .attr('rx', _var.getRadius)
              .attr('ry', _var.getRadius)
              .attr('x', _var.getXIn)
              .attr('y', _var.getY)
              .attr('width', _var.barWidth.toFixed(2))
              .attr('height', function(d) { return (_var.getHeight(d) + _var.getRadius(d)).toFixed(2); })
              .attr('stroke', "transparent")
              .attr('stroke-width', "60px")
              .each(function(d) {
                var sel = d3.select(this);
                setTimeout(function() {
                  sel.style('stroke-dasharray', function(d) { return _var.getStrokeDasharray(d); })
                }, 100);
              });

          // Get values for inside texts
          var textValuesObj = {}, index = -1;
          var textValues = node.values.filter(function(d, i) {
            var flag = textValuesObj[d.x] == null;
            textValuesObj[d.x] = true;
            return flag;
          }).filter(function(d) {
            index += 1;
            return  _var.xIn.bandwidth() >= 20 || (index%2 === 1);
          });

          // Draw X Inside Texts
          var texts = nodeSel.selectAll("text.x-in-text").data(textValues, function(d) { return d.x; });
          texts.exit().remove();
          texts = texts.enter().append('text').attr("class", "x-in-text").merge(texts);
          texts
            .attr("x", function(d) { return _var.xIn(d.x) + _var.xIn.bandwidth()/2; })
            .attr('y', _var.height + 17)
            .attr('text-anchor', 'middle')
            .text(function(d) { return d.name; })
            .each(function(d) { gViz.shared.helpers.text.wrapBySize(d3.select(this), 2*_var.xIn.bandwidth(), _var.margin.bottom, _var.xMaxLines); })

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','animation','components','nodeObj','nodeIndex','node'].forEach(function(key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};
