// Initialize the visualization class
gViz.viz.barChart.elements = function () {
  "use strict";

  // Get attributes values
  var _var       = null;
  var components = null;
  var data       = null;

  // Validate attributes
  var validate = function (step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Set data array
          var _data = (data == null ? _var.data.data : data);

          // Element canvas
          var elements = _var.g.selectAll(".chart-elements").data(["chart-elements"], function(d) { return d; });
          elements.exit().remove();
          elements = elements.enter().insert("g", ":first-child").attr("class", "chart-elements").merge(elements);

          // Create groups
          var groups = elements.selectAll(".element-group").data(_data, function (d) { return d.x; });
          groups.exit().remove();
          groups = groups.enter().append("g").attr("class", "element-group").merge(groups);

          // For each element in group
          groups
            .attr('data-id', function(d) { return d.x; })
            .attr("transform", function (d) { return `translate(${_var.x(d.x)},0)`; })
            .each(function (e, i) {

              // Mouseover event
              components.bars()
                ._var(_var)
                .components(components)
                .nodeIndex(i)
                .nodeObj(this)
                .node(e)
                .run();

            });

          // Event bindings
          elements.selectAll('.bar-rect').on('mouseover', function(e) {

            // Set hovered node
            _var.hovered = e;

            // Mouseover event
            components.events()
              ._var(_var)
              .action("mouseover")
              .components(components)
              .node(e)
              .run();

          }).on('mouseout', function(e) {

            // Reset hovered node
            _var.hovered = null;

            // Mouseout event
            components.events()
              ._var(_var)
              .action("mouseout")
              .components(components)
              .run();

          });

          // Element canvas
          var tElements = _var.g.selectAll(".chart-t-elements").data(["chart-t-elements"], function(d) { return d; });
          tElements.exit().remove();
          tElements = tElements.enter().insert("g", ":first-child").attr("class", "chart-t-elements").merge(tElements);

          // Create tGroups
          var tGroups = tElements.selectAll(".element-t-group").data(_data, function (d) { return d.x; });
          tGroups.exit().remove();
          tGroups = tGroups.enter().append("g").attr("class", "element-t-group").merge(tGroups);

          // For each element in group
          tGroups
            .attr('data-id', function(d) { return d.x; })
            .attr("transform", function (d) { return `translate(${_var.x(d.x)},0)`; })
            .each(function (e, i) {

              // Get month days for month parse
              var monthDays = 31;
              if(e.dataType === 'month') { monthDays = d3.timeParse("%Y-%m-%d")(e.id).monthDays(); }

              // Get values for inside texts
              var textValuesObj = {}, index = -1;
              var textValues = e.values.filter(function(d, i) {
                var flag = textValuesObj[d.x] == null;
                textValuesObj[d.x] = true;
                if(e.dataType === 'month') { flag = flag && parseInt(d.x) <= monthDays; }
                return flag;
              }).filter(function(d) {
                index += 1;
                return  _var.xIn.bandwidth() >= 20 || (index%2 === 1);
              });

              // Draw X Inside Texts
              var texts = d3.select(this).selectAll("text.x-in-text").data(textValues, function(d) { return d.x; });
              texts.exit().remove();
              texts = texts.enter().append('text').attr("class", "x-in-text").merge(texts);
              texts
                .attr("x", function(d) {
                  return _var.xIn(d.x) + _var.xIn.bandwidth()/2;
                })
                .attr('y', _var.height + 17)
                .attr('text-anchor', 'middle')
                .text(function(d) { return d.name; })
                .each(function(d) { gViz.shared.helpers.text.wrapBySize(d3.select(this), 2*_var.xIn.bandwidth(), _var.margin.bottom, _var.xMaxLines); })

            });

          // Draw Background clip Path
          _var.bgClip = _var.defs.selectAll(".bg-clip").data(["bg-clip"]);
          _var.bgClip.exit().remove();
          _var.bgClip = _var.bgClip.enter().insert('clipPath', ':first-child').attr("class", "bg-clip").merge(_var.bgClip);
          _var.bgClip
            .attr("id", "clip-path-"+_var._id)
            .each(function() {

              // Draw Background rect
              _var.bgClipRect = d3.select(this).selectAll("rect.bg-rect").data(["bg-rect"]);
              _var.bgClipRect.exit().remove();
              _var.bgClipRect = _var.bgClipRect.enter().insert('rect', ':first-child').attr("class", "bg-rect").style('fill', 'transparent').merge(_var.bgClipRect);
              _var.bgClipRect
                .style('fill', 'transparent')
                .attr("x", 0)
                .attr('y', 0)
                .attr('width', _var.width + _var.margin.right + 1)
                .attr("height", _var.height + _var.margin.bottom + _var.margin.top);

            });

          // Draw Background clip Path
          _var.bgBarsClip = _var.defs.selectAll(".bars-bg-clip").data(["bg-clip"]);
          _var.bgBarsClip.exit().remove();
          _var.bgBarsClip = _var.bgBarsClip.enter().insert('clipPath', ':first-child').attr("class", "bars-bg-clip").merge(_var.bgBarsClip);
          _var.bgBarsClip
            .attr("id", "bars-clip-path-"+_var._id)
            .each(function() {

              // Draw Background rect
              _var.bgBarsClipRect = d3.select(this).selectAll("rect.bg-rect").data(["bg-rect"]);
              _var.bgBarsClipRect.exit().remove();
              _var.bgBarsClipRect = _var.bgBarsClipRect.enter().insert('rect', ':first-child').attr("class", "bg-rect").style('fill', 'transparent').merge(_var.bgBarsClipRect);
              _var.bgBarsClipRect
                .style('fill', 'transparent')
                .attr("x", 0)
                .attr('y', -_var.margin.top)
                .attr('width', _var.width + _var.margin.right + 1)
                .attr("height", _var.height + _var.margin.top);

            });


          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','components','data'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};
