// Initialize the visualization class
gViz.viz.barChart.style = function() {
  "use strict";

  // Get attributes values
  var _var      = undefined;
  var components = {};

  // Validate attributes
  var validate = function(step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function(step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Is wrapper function
          _var.hasWrapper = function(wrap) {
            return ['sum','median','mean','max','min'].indexOf(wrap) !== -1 || (wrap != null && !isNaN(+wrap));
          }

          // Is wrapper function
          _var.wrapperType = function(wrap) {
            if(['sum','median','mean','max','min'].indexOf(wrap) !== -1) { return 'metric'; }
            else { return 'number'; };
          }

          // Get color function
          _var.getBarColor = function(parent, d, attr="fill") {
            var otherValues = parent.values != null ? parent.values.filter(function(f) { return f.x == d.x && f.id != d.id; }) : [];
            if(otherValues.length === 0) { return _var.getColor(d); }
            else {
              var other = otherValues[0];
              if(d.id === 'forecast' && +d.y > +other.y) { return _var.getColor(d); }
              else if(d.id === 'forecast' && +d.y <= +other.y) { return d.secondaryColor != null && d.secondaryColor !== '' ? d.secondaryColor : _var.getColor(d); }
              else if(d.id !== 'forecast' && +d.y < +other.y) { return _var.getColor(d); }
              else if(d.id !== 'forecast' && +d.y >= +other.y) { return d.secondaryColor != null && d.secondaryColor !== '' ? d.secondaryColor : _var.getColor(d); }
            }
          }

          // Get color function
          _var.getColor = function(d, attr="fill") {
            if(d.color != null && attr != 'stroke') { return d.color; }
            else if(_var.data.colors[d.x] == null) { return attr === 'stroke' ? "#333" : "#999"; }
            else if(+d.y >= 0 && _var.data.colors[d.x][attr] != null) { return _var.data.colors[d.x][attr]; }
            else if(+d.y >= 0 && _var.data.colors[d.x][attr+"-neg"] != null) { return _var.data.colors[d.x][attr+"-neg"]; }
            else if(+d.y < 0  && _var.data.colors[d.x][attr+"-neg"] != null) { return _var.data.colors[d.x][attr+"-neg"]; }
            else if(+d.y < 0  && _var.data.colors[d.x][attr] != null) { return _var.data.colors[d.x][attr]; }
            else { return attr === 'stroke' ? "#333" : "#999"; }
          }

          // Get bar opacity
          _var.getOpacity = function(d) {
            return d.opacity != null && !isNaN(+d.opacity) && +d.opacity > 0 && +d.opacity <= 1 ? +d.opacity : 1;
          }

          // Get stroke dasharray
          _var.getStrokeDasharray = function(d) {
            var height = _var.getHeight(d) + _var.getRadius(d);
            var width = _var.barWidth;
            return parseInt(width) + ' , ' + parseInt(2 * height);
          }


          // Get Y function
          _var.getStrokeY = function(d) {
            if(_var.yBounds[0] >= 0) { return _var.y(+d.y); }
            else if (_var.yBounds[1] < 0) { return _var.y(_var.yBounds[1]); }
            else { return _var.y(+d.y); }
          }

          // Get Height function
          _var.getHeight = function(d) {
            var value = 0;
            if(_var.yBounds[0] >= 0) { value = _var.height - _var.y(+d.y); }
            else if (_var.yBounds[1] < 0) { value = _var.y(+d.y); }
            else { value = +d.y >= 0 ? (_var.y(0) - _var.y(+d.y)) : (_var.y(+d.y) - _var.y(0)); }
            return value < 0 ? 0 : value;
          }

          // Get Y function
          _var.getY = function(d) {
            if(_var.yBounds[0] >= 0) { return _var.y(+d.y); }
            else if (_var.yBounds[1] < 0) { return _var.y(_var.yBounds[1]); }
            else { return +d.y >= 0 ? _var.y(+d.y) : _var.y(0); }
          }

          // Get X function
          _var.getXIn = function(d) {
            return _var.xIn(d.x) + _var.xIn.bandwidth()/2 - _var.barWidth/2;
          }


          // Get path for rounded corners
          _var.getRadius = function(d) {
            var radiusWidth = Math.min(16, _var.barWidth*0.7);
            var value = _var.getHeight(d) < radiusWidth ? _var.getHeight(d)/2 : radiusWidth/2;
            return value < 0 ? 0 : value;
          }

          // Get path for rounded corners
          _var.getPath = function(d) {

            // Get parameters
            var x = _var.getXIn(d);
            var y = _var.getY(d);
            var width = _var.barWidth;
            var height = _var.getHeight(d);
            var radiusWidth = Math.min(16, _var.barWidth*0.7);
            var radius = height < radiusWidth ? height/2 : radiusWidth/2;

            return "M" + x + "," + (y + height)
                 + "v-" + (height - 2 * radius)
                 + "a" + radius + "," + radius + " 0 0 1 " + radius + ", -" + radius
                 + "h" + (width - 2 * radius)
                 + "a" + radius + "," + radius + " 0 0 1 " + radius + "," + radius
                 + "v" + (height - 2 * radius)
                 + "z";
          };

          // Get x padding from scale
          _var.getXPadding = function() {
            return ((_var.calcWidth / Object.keys(_var.xDomain).length) - _var.x.bandwidth());
          }


          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','components'].forEach(function(key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};
