// Initialize the visualization class
gViz.viz.barChart.xScale = function () {
  "use strict";

  // Get attributes values
  var _var = undefined;
  var data = [];

  // Validate attributes
  var validate = function (step) {

    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Set format
          _var.xIsDate = (_var.data.x != null && _var.data.x.type === 'time' && _var.data.x.inFormat != null && _var.data.x.outFormat != null);
          _var.xIsNumber = (_var.data.x != null && _var.data.x.type === 'number' && _var.data.x.format != null);
          var xFmt = _var.xIsDate ? 'date' : (_var.xIsNumber ? 'number' : 'text');
          _var.xFormat = gViz.shared.helpers[xFmt].parseFormat(_var.data == null ? null : _var.data.x);

          // Initialize domains
          _var.xDomain = {};
          _var.xInDomain = {};

          // Initialize text values objects for labels resizing
          var textValuesObj = { x: {}, xIn: {} };

          // Get domains
          data.forEach(function(d) {

            // Parse date value
            if(_var.xIsDate) {
              d.parsedName = d3.timeParse(_var.data.x.inFormat)(d.name);
              if(d.parsedName != null) { d.name = _var.xFormat(d.name); }
            }

            // Add id to x domain value
            _var.xDomain[d.x] = d;

            // Store name for futher use
            if(d.name != null && d.name !== "" && textValuesObj.x[d.name] == null) { textValuesObj.x[d.name] = gViz.shared.helpers.text.getSize(d.name); }

            // Iterate over values
            d.values.forEach(function(v) {

              // Set parent id
              v.parent = d.x;

              // Parse date value
              if(_var.xIsDate) {
                v.parsedName = d3.timeParse(_var.data.x.inFormat)(v.name);
                if(v.parsedName != null) { v.name = _var.xFormat(v.name); }
              }

              // Add id to xIn domain value
              _var.xInDomain[v.x] = v;

              // Store name for futher use
              if((d.name == null || d.name === "") && (v.name != null && v.name !== "" && textValuesObj.xIn[v.name] == null)) {
                textValuesObj.xIn[v.name] = gViz.shared.helpers.text.getSize(v.name);
              }

            });
          });

          // Initialize domains
          var xDomain = Object.keys(_var.xDomain), xInDomain = Object.keys(_var.xInDomain);

          // Set xLimits if not set
          _var.xBounds = [xDomain[0], xDomain[xDomain.length-1]];

          // Initialize count of elements
          var elementCount = Object.keys(_var.xInDomain).length, groupCount = Object.keys(_var.xDomain).length;

          // Store old calculed width
          var oldWidth = _var.x == null ? 0 : Math.abs(_var.x.range()[1] - _var.x.range()[0]);

          // Calculate height based on data elements
          _var.calcWidth = groupCount * _var.width;
          _var.calcWidth = _var.width > _var.calcWidth ? _var.width : _var.calcWidth;

          // Calculate ranges
          var minRange = _var.x == null ? 0 : _var.x.range()[0];
          var maxRange = _var.x == null ? _var.calcWidth : _var.x.range()[1];

          if(_var.x != null) {
            if(_var.zoom.direction === 'prev') { _var.zoom.transform.x -= _var.x.bandwidth(); }
            else if(_var.zoom.direction === 'next') {
              _var.zoom.transform.x += _var.x.bandwidth();
            }
          }

          // Set scales range and padding
          _var.x = d3.scaleBand().range([minRange, maxRange]).paddingInner(0).paddingOuter(0);
          _var.xIn = d3.scaleBand().paddingInner(0.26).paddingOuter(0.13);

          // Set x and xIn domain
          _var.x.domain(xDomain);
          _var.xIn.domain(xInDomain).range([0,_var.x.bandwidth()]);

          // Update barWidth if its not set by the user
          _var.barWidth = _var.data.attrs != null && _var.data.attrs.barWidth != null && !isNaN(+_var.data.attrs.barWidth) ? +_var.data.attrs.barWidth : _var.xIn.bandwidth();
          _var.barWidth = Math.min(_var.barWidth, 25);

          // Define axis
          _var.xAxis = d3.axisBottom(_var.x).tickPadding(17);

          // Set old last value
          _var.oldLastValue = _var.xBounds[1];

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','data'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};
