// Module declaration
gViz.viz.bubbleChart = function () {
  "use strict";

  // Get attributes values
  var _id = `vis-bubble-${Math.floor(Math.random() * ((1000000000 - 5) + 1)) + 5}`;
  var _var = null;
  var action = 'build';
  var animation = 900;
  var container = null;
  var colors = ["#ccc","#3a6bc6"];
  var data = [];
  var margin = { top: 22, right: 0, bottom: 45, left: 0 };

  // Event bindings
  var onClick = function(d) { console.log(d); };

  // Validate attributes
  var validate = function (step) {
    switch (step) {
      case 'build':      return (container != null) && (d3.selectAll(container).size() !== 0 || d3.select(container).size() !== 0);
      case 'initialize': return true;
      case 'create':     return data != null && data.data != null && data.data.length > 0;
      case 'setup':   return data != null && data.data != null && data.data.length > 0;
      case 'misc':       return true;
      case 'style':      return true;
      case 'size':       return true;
      case 'drag':       return true;
      default:           return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'build':

          main('initialize');
          main('drag');
          main('style');
          main('misc');
          main('create');
          main('size');
          main('setup');
          break;

        // Initialize visualization variable
        case 'initialize':

          // Initializing
          if (!_var) { _var = {};  }
          _var = gViz.viz.bubbleChart.initialize()
            ._var(_var)
            ._id((_var._id != null) ? _var._id : _id)
            .animation(animation)
            .container(container)
            .colors(colors)
            .data(data)
            .margin(margin)
            .onClick(onClick)
            .run();

          break;

        // Setup style functions
        case 'style':

          // Styling
          _var = gViz.viz.bubbleChart.style()
            ._var(_var)
            .run();
          break;

        // Create initial setup
        case 'create':

          // Creating wrappers
          _var = gViz.viz.bubbleChart.create()
            ._var(_var)
            .run();
          break;

        // Setup chart setup
        case 'setup':

          // Running
          _var = gViz.viz.bubbleChart.setup()
            ._var(_var)
            .components(gViz.viz.bubbleChart)
            .data(_var.data.data)
            .run();
          break;

        // Show misc
        case 'misc':

          // Running
          _var = gViz.viz.bubbleChart.misc()
            ._var(_var)
            .components(gViz.viz.bubbleChart)
            .run();
          break;

        // Show size
        case 'size':

          // Running
          _var = gViz.viz.bubbleChart.size()
            ._var(_var)
            .data(_var.data.data)
            .run();
          break;

        // Show drag
        case 'drag':

          // Running
          _var = gViz.viz.bubbleChart.drag()
            ._var(_var)
            .components(gViz.viz.bubbleChart)
            .run();
          break;


      }
    }

    return _var;
  };

  // Expose global variables
  ['_id','_var','action','animation','container','colors','data','margin','onClick'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return main;
    };
  });

  // Secondary functions
  main.build = function (_) { return main("build"); };

  // Execute the specific called function
  main.run = function (_) { return main(_); };

  return main;

}
