// Initialize the visualization class
gViz.viz.bubbleChart.setup = function () {
  "use strict";

  // Get attributes values
  var _var       = null;
  var components = null;
  var data       = null;

  // Validate attributes
  var validate = function (step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Element canvas
          var elements = _var.g.selectAll(".chart-elements").data(["chart-elements"], function(d) { return d; });
          elements.exit().remove();
          elements = elements.enter().insert("g", ":first-child").attr("class", "chart-elements").merge(elements);

          // Set data array
          var _data = (data == null ? _var.data.data : data);

          _var.simulation = d3.forceSimulation()
            .force("forceX", d3.forceX().strength(.1).x(_var.width * .5))
            .force("forceY", d3.forceY().strength(.1).y(_var.height * .5 + _var.margin.top))
            .force("center", d3.forceCenter().x(_var.width * .5).y(_var.height * .5 + _var.margin.top))
            .force("charge", d3.forceManyBody().strength(-15));

          //update the simulation based on the data
          _var.simulation
            .nodes(_data)
            .force("collide", d3.forceCollide().strength(.5).radius(function(d){ return _var.size(+d.value) + 1; }).iterations(1))
            .on("tick", function(d){
              elements.selectAll('.bubble-group')
                .attr('transform', function(d) { return 'translate('+d.x+','+d.y+')'; })
            });

          // Create groups
          var groups = elements.selectAll(".bubble-group").data(_data, function (d) { return d.id; });
          groups.exit().remove();
          groups = groups.enter().append("g").attr("class", "bubble-group").merge(groups);

          // For each element in group
          groups.each(function (e, i) {

            // Create bubble circles
            var circle = d3.select(this).selectAll('.bubble-circle').data([e], function (d) { return d.id; });
            circle.exit().remove();
            circle = circle.enter().append("circle").attr("class", 'bubble-circle').merge(circle);
            circle
              .attr('r', function(d) { return _var.size(+d.value); })
              .attr('x', function(d) { return -_var.size(+d.value)*0.5; })
              .attr('y', function(d) { return -_var.size(+d.value)*0.5; })
              .attr('fill', function(d) { return "#FFF"; })
              .attr('stroke', function(d) { return _var.colorScale(+d.value); })

            // Draw bubble texts
            var bubble = d3.select(this).selectAll('.bubble-text').data([e], function (d) { return d.id; });
            bubble.exit().remove();
            bubble = bubble.enter().append("text").attr("class", 'bubble-text').merge(bubble);
            bubble
              .attr('x', 0)
              .attr('y', function(d) { return d.img != null && d.img !== '' ? -_var.size(+d.value)*0.5 + 10 : 0; })
              .attr('text-anchor', 'middle')
              .attr('font-size', '11px')
              //.attr('fill', function(d) { return gViz.shared.helpers.colors.isDark(_var.colorScale(+d.value)) ? "#FFF" : "#454545"; })
              .attr('fill', function(d) { return _var.colorScale(+d.value); })
              .text(function(d) { return d.name; })
              .each(function(d) { gViz.shared.helpers.text.wrapBySize(d3.select(this), _var.size(+d.value)*1.4, 30, (d.img != null && d.img !== '' ? 1 : 3)); })

            // Insert / Update labels for the nodes
            var imageData = e.img != null && e.img !== '' ? [e] : [];
            var image = d3.select(this).selectAll('image.bubble-image').data(imageData, function (d) { return d.id; });
            image.exit().remove();
            image = image.enter().append('image').attr('class', 'bubble-image').merge(image);
            image
              .attr("xlink:href", function(d) { return d.img; })
              .attr('x', function(d) { return -_var.size(+d.value)*0.3; })
              .attr('y', function(d) { return 0; })
              .attr('width', function(d) { return _var.size(+d.value)*0.6; })
              .attr('height', function(d) { return _var.size(+d.value)*0.6; })

          });

          // Bind click event
          groups.on('click', function(d) {
            if(_var.onClick != null && typeof _var.onClick === "function") { _var.onClick(d); }
          });

          // Bind drag to groups
          groups.call(d3.drag()
            .on("start", _var.dragstarted)
            .on("drag", _var.dragging)
            .on("end", _var.dragended)
            .touchable(true));

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','components','data'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};
