// Module declaration
gViz.viz.bubbleChart.size = function() {
  "use strict";

  // Get attributes values
  var _var = null;
  var data = [];

  // Validate attributes
  var validate = function (step) {

    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Get initial set of colors
          var colors = _var.colors.map(function(d) { return d3.color(d); }).filter(function(d) { return d != null; });

          // Define scales
          if(colors.length === 0) { _var.bubbleColors = [d3.color("#ccc"),d3.color("#3a6bc6")]; }
          else if(colors.length === 1) { _var.bubbleColors = [colors[0], colors[0].darker(3)]; }
          else { _var.bubbleColors = colors; }

          // Get max and min sizes from attrs
          var attrs = _var.data != null && _var.data.attrs != null ? _var.data.attrs : {};
          var minSize = attrs.minSize != null && !isNaN(+attrs.minSize) ? +attrs.minSize : 30;
          var maxSize = attrs.maxSize != null && !isNaN(+attrs.maxSize) ? +attrs.maxSize : 60;

          // Initialize scale
          _var.colorScale = d3.scaleLinear().range(_var.bubbleColors);
          _var.size = d3.scaleLinear().range([d3.min([minSize,maxSize]), d3.max([minSize,maxSize])]);

          // Define aux variables
          var min = null,
              max = null,
              diff = null;

          // Get bounds
          data.forEach(function(d) {
            if(min == null || min > +d.value) { min = +d.value; }
            if(max == null || max < +d.value) { max = +d.value; }
          });

          // Check for default values
          if(isNaN(min)) { min = 0; }
          if(isNaN(max)) { max = 1; }

          // Store y bounds min from data
          _var.sizeBoundsData = [min,max];

          // Get mix and max values for axis set from json
          if(_var.data.size != null && _var.data.size.min != null && !isNaN(+_var.data.size.min) && +_var.data.size.min < min) { min = +_var.data.size.min; }
          if(_var.data.size != null && _var.data.size.max != null && !isNaN(+_var.data.size.max) && +_var.data.size.max > max) { max = +_var.data.size.max; }

          // Get diff
          var diff = Math.abs(max - min) === 0 ? Math.abs(max * 0.1) : Math.abs(max - min) * 0.05;

          // Set size domain
          _var.sizeBounds = [min, max];
          _var.size.domain(_var.sizeBounds);

          // Get color domain
          var colorDomain = d3.range(_var.colorScale.range().length)
            .map(function(d, i) {
              return _var.sizeBounds[0] + i * (Math.abs(_var.sizeBounds[1] - _var.sizeBounds[0]) / (_var.colorScale.range().length-1));
            });

          // Set color scale domain
          _var.colorScale.domain(colorDomain);

          // Set format
          _var.sizeFormat = gViz.shared.helpers.number.parseFormat(_var.data == null ? null : _var.data.size);

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','data'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) {
    return main('run');
  };

  return main;
};
