// Module declaration
gViz.viz.lineAreaChart = function() {
  "use strict";

  // Get attributes values
  var _id = "vis-line-area-chart-"+(Math.floor(Math.random() * ((1000000000 - 5) + 1)) + 5);
  var _var = null;
  var action = 'build';
  var animation = 200;
  var container = null;
  var colors = [{value:0, color:"#4ad2a1"},{value:150, color:"#4ad2a1"},{value:300, color:"#3d62cb"},{value:500, color:"#3d62cb"},{value:600, color:"#fa5050"}];
  var data = [];
  var margin = { top: 22, right: 0, bottom: 45, left: 0 };
  var opacity = 0.4;
  var isNow = true;
  var timespan = null;
  var touchEvents = null;
  var wheelZoom = {};
  var xZoom = {};
  var yZoom = {};
  var zoomMode = 'horizontal';
  var zoomLevel = 1;

  // Validate attributes
  var validate = function (step) {
    switch (step) {
      case 'build':      return (container != null) && (d3.selectAll(container).size() !== 0 || d3.select(container).size() !== 0);
      case 'initialize': return true;
      case 'axis':       return data != null && data.data != null && data.data.length > 0 && data.data[0].values.length > 0;
      case 'create':     return data != null && data.data != null && data.data.length > 0 && data.data[0].values.length > 0;
      case 'elements':   return data != null && data.data != null && data.data.length > 0 && data.data[0].values.length > 0;
      case 'misc':       return true;
      case 'style':      return true;
      case 'xScale':     return data != null && data.data != null && data.data.length > 0 && data.data[0].values.length > 0;
      case 'yScale':     return data != null && data.data != null && data.data.length > 0 && data.data[0].values.length > 0;
      case 'zoom':       return data != null && data.data != null && data.data.length > 0 && data.data[0].values.length > 0;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'build':

          main('initialize');
          main('style');
          main('misc');
          main('yScale');
          main('xScale');
          main('create');
          main('axis');
          main('elements');
          main('zoom');
          break;

        // Initialize visualization variable
        case 'initialize':

          // Initializing
          if (!_var) { _var = {};  }
          _var = gViz.viz.lineAreaChart.initialize()
            ._var(_var)
            ._id((_var._id != null) ? _var._id : _id)
            .animation(animation)
            .container(container)
            .colors(colors)
            .data(data)
            .isNow(isNow)
            .margin(margin)
            .opacity(opacity)
            .timespan(timespan)
            .touchEvents(touchEvents)
            .wheelZoom(wheelZoom)
            .xZoom(xZoom)
            .yZoom(yZoom)
            .zoomMode(zoomMode)
            .zoomLevel(zoomLevel)
            .run();
          break;

        // Setup style functions
        case 'style':

          // Setting styles
          _var = gViz.viz.lineAreaChart.style()
            ._var(_var)
            .run();
          break;

        // Create initial elements
        case 'create':

          // Creating wrappers
          _var = gViz.viz.lineAreaChart.create()
            ._var(_var)
            .run();
          break;

        // Setup X scale
        case 'xScale':

          // Creating
          _var = gViz.viz.lineAreaChart.xScale()
            ._var(_var)
            .data(_var.data.data)
            .run();
          break;

        // Setup Y scale
        case 'yScale':

          // Creating
          _var = gViz.viz.lineAreaChart.yScale()
            ._var(_var)
            .data(_var.data.data)
            .run();
          break;

        // Setup axis elements
        case 'axis':

          // Running
          _var = gViz.viz.lineAreaChart.axis()
            ._var(_var)
            .action('create')
            .run();
          break;

        // Setup elements
        case 'elements':

          // Running
          _var = gViz.viz.lineAreaChart.elements()
            ._var(_var)
            .components(gViz.viz.lineAreaChart)
            .run();
          break;

        // Show misc
        case 'misc':

          // Running
          _var = gViz.viz.lineAreaChart.misc()
            ._var(_var)
            .components(gViz.viz.lineAreaChart)
            .run();
          break;

        // Set zoom
        case 'zoom':

          // Add zoom horizontally
          _var = gViz.viz.lineAreaChart.xZoom()
            ._var(_var)
            .components(gViz.viz.lineAreaChart)
            .run();

          // Add zoom
          _var = gViz.viz.lineAreaChart.zoom()
            ._var(_var)
            .components(gViz.viz.lineAreaChart)
            .run();

          break;

      }
    }

    // Getters
    _var.get = {

      // Get most left point
      start: function() {
        return _var.x.invert(-_var.xZoom.transform.x / _var.xZoom.transform.k);
      },

      // Get most right point
      end: function() {
        return _var.x.invert((-_var.xZoom.transform.x + _var.width) / _var.xZoom.transform.k);
      },

      // Get min y value
      min: function() { return _var.y.invert(-_var.xZoom.transform.y + _var.height); },

      // Get max y value
      max: function() { return _var.y.invert(-_var.xZoom.transform.y); }
    }

    return _var;
  };

  // Expose global variables
  ['_id', '_var', 'action', 'animation', 'container', 'colors', 'data','isNow','margin','opacity','timespan','touchEvents','wheelZoom','xZoom','yZoom','zoomMode','zoomLevel'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return main;
    };
  });

  // Secondary functions
  main.build = function (_) { return main("build"); };

  // Execute the specific called function
  main.run = function (_) { return main(_); };

  return main;

}
