// Module declaration
gViz.viz.lineAreaChart.elements = function() {
  "use strict";

  // Get attributes values
  var _var       = null;
  var components = null;
  var data       = null;

  // Validate attributes
  var validate = function (step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          if(_var.xZoom.transform != null) {

            // Clean wrap gradients
            gViz.shared.visualComponents.gradient()
              .action('clean')
              .run();

            // Initialize line constructor
            _var.lineConstructor = d3.line()
              .x(function (d) { return _var._x(d.parsedX); })
              .y(function (d) { return _var._y(+d.y); })
              .curve(d3.curveLinear)

            // Initialize area constructor
            _var.areaConstructor = d3.area()
              .x(function (d) { return _var._x(d.parsedX); })
              .y0(function (d) { return _var.height; })
              .y1(function (d) { return _var._y(+d.y); })
              .curve(d3.curveLinear)

            // Set data array
            var _data = (data == null ? _var.data.data : data).sort(function(a,b) { return  d3.descending(a.mean, b.mean); });

            // Element canvas
            var elements = _var.g.selectAll(".chart-elements").data(["chart-elements"], function(d) { return d; });
            elements.exit().remove();
            elements = elements.enter().append("g").attr("class", "chart-elements").merge(elements);

            // Element canvas
            var elementsClip = _var.gClip.selectAll(".chart-elements-clip").data(["chart-elements-clip"], function(d) { return d; });
            elementsClip.exit().remove();
            elementsClip = elementsClip.enter().append("g").attr("class", "chart-elements-clip").merge(elementsClip);

            // Create groups
            var groups = elementsClip.selectAll(".element-group").data(_data, function(d) { return d.id; });
            groups.exit().remove();
            groups = groups.enter().append("g").attr("class", "element-group").merge(groups);
            groups.each(function(g) {

              // Store initial values for interpolation with y = 0
              var values = g.values;
              if(g._values == null) {
                g._values = values.map(function(d) { return { x: d.x, y: 0, parsedX: d.parsedX, formattedX: d.formattedX }; });
              }

              // // Update y bounds
              // _var.yBounds = d3.extent(values, function(d) { return +d.y; });
              // _var.y.domain(_var.yBounds);
              // _var._y.domain(_var.yBounds);

              // Update axis
              _var.updateXAxis()

              var last = { parsedX: g.values[g.values.length-1].parsedX, y: g.values[g.values.length-1].y };
              var first = { parsedX: g.values[0].parsedX, y: g.values[0].y };
              var values = [first].concat(g.values.concat([last]));

              // Create areas
              var hasArea = g.hasArea != null && g.hasArea === true;
              var areas = d3.select(this).selectAll(".area").data(hasArea ? [g] : [], function(d) { return d.id; });
              areas.exit().remove();
              areas = areas.enter().append("path").attr("class", "area")
                .attr("d", function (d) { return _var.areaConstructor(g._values); })
                .merge(areas);
              areas
                .attr("fill-opacity", _var.opacity)
                .attr("fill", 'url("#gViz-luko-main-gradient")')
                .transition()
                  .duration(_var.animation)
                  .ease(d3.easeLinear)
                  .attrTween("d", function (d) {
                    var previous = _var.rebuild.area && !_var.wheelZoom.enable ? _var.areaConstructor(g._values) : d3.select(this).attr('d');
                    var current = _var.areaConstructor(values);
                    _var.rebuild.area = false;
                    return d3.interpolatePath(previous, current);
                  });

              // Create lines
              var lines = d3.select(this).selectAll(".line").data([g], function(d) { return d.id; });
              lines.exit().remove();
              lines = lines.enter().append("path").attr("class", "line")
                .attr("d", function (d) { return _var.lineConstructor(g._values); })
                .merge(lines);
              lines
                .attr("fill", 'none')
                .attr("stroke-width", _var.lineWidth)
                .attr("stroke", 'url("#gViz-luko-main-gradient")')
                .transition()
                  .duration(_var.animation)
                  .ease(d3.easeLinear)
                  .attrTween("d", function (d) {
                    var previous = _var.rebuild.line && !_var.wheelZoom.enable ? _var.lineConstructor(g._values) : d3.select(this).attr('d');
                    var current = _var.lineConstructor(values);
                    _var.rebuild.line = false;
                    return d3.interpolatePath(previous, current);
                  });

                // Store previous values and set rebuild to false not to use g._values inside
                g._values = values.map(function(d) { return { x: d.x, y: +d.y, parsedX: d.parsedX, formattedX: d.formattedX }; });

                // Map colors
                var colors = _var.colors
                  .filter(function(d) { return _var._y(d.value) > 0; })
                  .map(function(d) {
                    return {
                      offset: (((_var._y(d.value) / (_var.height + _var.margin.bottom + _var.margin.top)) * 100).toFixed(2) + "%"),
                      color: d.color
                    };
                  });

                if(colors.length > 0) {

                  // Store initial and last colors
                  colors = [{ offset: "0%", color: colors[0].color }].concat(colors);
                  if(colors[colors.length-1].offset !== '100.00%') {
                    colors = colors.concat([{ offset:"100%", color: colors[colors.length-1].color }]);
                  }

                  // Set wrap gradient
                  gViz.shared.visualComponents.gradient()
                    ._var(_var)
                    .id("gViz-luko-main-gradient")
                    .colors(colors)
                    .direction('vertical')
                    .duration(_var.animation)
                    .y1(0)
                    .y2(_var.height)
                    .gradientUnits('userSpaceOnUse')
                    .gType('linear')
                    .wrap(_var.defs)
                    .run();

                }

            });

            // // Draw Background clip Path
            // _var.bgClip = _var.defs.selectAll(".bg-clip").data(["bg-clip"]);
            // _var.bgClip.exit().remove();
            // _var.bgClip = _var.bgClip.enter().insert('clipPath', ':first-child').attr("class", "bg-clip").merge(_var.bgClip);
            // _var.bgClip
            //   .attr("id", "clip-path-"+_var._id)
            //   .each(function() {

            //     // Draw Background rect
            //     _var.bgClipRect = d3.select(this).selectAll("rect.bg-rect").data(["bg-rect"]);
            //     _var.bgClipRect.exit().remove();
            //     _var.bgClipRect = _var.bgClipRect.enter().insert('rect', ':first-child').attr("class", "bg-rect").style('fill', 'transparent').merge(_var.bgClipRect);
            //     _var.bgClipRect
            //       .style('fill', 'transparent')
            //       .attr("x", 0)
            //       .attr('y', -_var.margin.top)
            //       .attr('width', _var.width)
            //       .attr("height", _var.height + _var.margin.top);

            //   });

            // Draw reference line
            var refLine = elements.selectAll("line.reference-line").data(["reference-line"], function(d) { return d; });
            refLine.exit().remove();
            refLine = refLine.enter().append('line').attr("class", "reference-line").merge(refLine);
            refLine
              .style('stroke-width', '1px')
              .style('stroke', function(d) { return _var.isReferenceLineNow() ? _var.referenceLineColor(d) : '#bbb'; })
              .attr("x1", function(d) { return _var.width * 5 / 6; })
              .attr("x2", function(d) { return _var.width * 5 / 6; })
              .attr('y1', 0)
              .attr('y2', _var.height + 5)

            // Get reference line values
            var bisector = d3.bisector(function (d) { return d.parsedX; }).left;
            var xValue = _var._x.invert(_var.width * 5 / 6);
            var referenceObj = {};

            // Get values from each line group to use on the tooltip
            _var.data.data.forEach(function(lineGroup, i) {

              // Get most close value to the mouse position
              var bisectIndex = bisector(lineGroup.values, xValue);
              if (bisectIndex > 0 && bisectIndex < lineGroup.values.length - 1) {
                var x0 = _var.x(lineGroup.values[bisectIndex - 1].parsedX);
                var x1 = _var.x(lineGroup.values[bisectIndex].parsedX);
                bisectIndex = _var.margin.left - x0 >= (x1 - x0) / 2 ? bisectIndex : bisectIndex - 1;
              }

              if(lineGroup.values[bisectIndex]) {
                // Set parent n attributes to tooltip obj
                Object.keys(lineGroup).forEach(function(k) { referenceObj[k] = lineGroup[k]; });

                // Set n attributes to tooltip obj
                Object.keys(lineGroup.values[bisectIndex]).forEach(function(k) { referenceObj[k] = lineGroup.values[bisectIndex][k]; });

                // Set x, color and z values with format
                referenceObj.x = lineGroup.values[bisectIndex].x;
                referenceObj.y = lineGroup.values[bisectIndex].y;
                referenceObj.color = _var.pointColor(lineGroup.values[bisectIndex]);
              }
            });

            // Draw reference line text
            var refLineText = elements.selectAll("text.reference-line-text").data(["reference-line-text"], function(d) { return d; });
            refLineText.exit().remove();
            refLineText = refLineText.enter().append('text').attr("class", "reference-line-text").merge(refLineText);
            refLineText
              .attr("x", function(d) { return _var.width * 5 / 6; })
              .attr("y", function(d) { return _var.height + (_var.isReferenceLineNow() ? 20 : 36); })
              .style("text-anchor", 'middle')
              .style("font-size", '14px')
              .style('fill', "#4568cc")
              .text(function(d) {
                var date = _var.isNow ? _var.xLast.y : referenceObj.y;
                return date == null ? '' : _var.yFormat(date);
              });

            // Draw reference line text
            var refLineHour = elements.selectAll("text.reference-line-hour").data(["reference-line-hour"], function(d) { return d; });
            refLineHour.exit().remove();
            refLineHour = refLineHour.enter().append('text').attr("class", "reference-line-hour").merge(refLineHour);
            refLineHour
              .attr("x", function(d) { return _var.width * 5 / 6; })
              .attr("y", function(d) { return _var.height + 18; })
              .style("text-anchor", 'middle')
              .style("font-size", '10px')
              .style('display', function(d) { return _var.isReferenceLineNow() ? 'none' : 'block'; })
              .style('fill', "#4568cc")
              .text(function(d) { return _var.xFormat(_var.isNow ? _var.xLast.x : referenceObj.x); });

            // Draw reference line bgRect
            var refLineBgRect = elements.selectAll(".reference-line-bg-rect").data(["reference-line-bg-rect"], function(d) { return d; });
            refLineBgRect.exit().remove();
            refLineBgRect = refLineBgRect.enter().insert('rect', ".reference-line-text").attr("class", "reference-line-bg-rect").merge(refLineBgRect);
            refLineBgRect
              .attr("x", function(d) { return _var.width * 5 / 6 - (Math.max(refLineText.node().getBBox().width, refLineHour.node().getBBox().width) + 20) / 2; })
              .attr("y", function(d) { return _var.height + 1; })
              .attr("width", function(d) { return Math.max(refLineText.node().getBBox().width, refLineHour.node().getBBox().width) + 20; })
              .attr("height", function(d) { return _var.margin.bottom; })
              .attr("fill", "#F7F7F7")

            // Update reference line with real time data
            var setTimeIntervalFunction = null;
            var stopNow = function() { clearInterval(setTimeIntervalFunction); }
            var startNow = function() {

              // Stop real time before starting the interval
              stopNow();

              // Set time interval
              setTimeIntervalFunction = setInterval(function() {

                // Draw reference line
                refLine.style('stroke', function(d) { return _var.isReferenceLineNow() ? _var.referenceLineColor(d) : '#bbb'; })

                // Draw reference line text
                refLineText
                  .attr("y", function(d) { return _var.height + (_var.isReferenceLineNow() ? 20 : 36); })
                  .style('fill', "#4568cc")

                // Draw reference line text
                refLineHour
                  .style('fill', "#4568cc")
                  .style('display', function(d) { return _var.isReferenceLineNow() ? 'none' : 'block'; })

              }, 1000);
            }

            // Start timer to color lines / text
            startNow();

          }

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','components','data'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function(_) { return main('run'); };

  return main;
};
