// Module declaration
gViz.viz.lineAreaChart.events = function() {
  "use strict";

  // Get attributes values
  var _var       = null;
  var action     = 'mouseover';
  var components = null;
  var node       = null;
  var mouse      = null;
  var origin     = 'node';
  var propAttrs  = ['title','body'];

  // Validate attributes
  var validate = function (step) {

    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Run code
        case 'run':

          // Set groups and bisect element
          var groups = _var.g.selectAll(".chart-elements").selectAll(".element-group");
          var points = _var.g.selectAll(".chart-elements").selectAll(".element-group").selectAll('.point');
          var lines  = _var.g.selectAll(".chart-elements").selectAll(".element-group").selectAll('.line');
          var bisector = d3.bisector(function (d) { return d.parsedX; }).left;

          switch (action) {

            case 'mouseover':

              // Fade other groups
              groups.transition()
                .style('opacity', function(g) { return g === node || g.id === node._parent.id ? 1 : 0.2; })

              // Add style to lines
              lines.transition()
                .style("filter", function(g) { return g === node || g.id === node._parent.id ? "url(#"+_var.shadowId+")" : ""; })
                .style('opacity', function(g) { return g === node || g.id === node._parent.id ? 1 : 0.2; })

              // Add style to points
              points.transition()
                .style("filter", function(g) { return g === node ? "url(#"+_var.shadowId+")" : ""; })
                .style('opacity', function(g) { return g.parsedX === node.parsedX || g === node || g._parent.id === node._parent.id ? 1 : 0.2; })

              // Get x and y values
              var x = _var.x(node.parsedX) + (_var.xIsDate || _var.xIsNumber ? 0 : _var.x.bandwidth()/2)
              var y = _var.y(+node.y);
              var z = _var.pointSize(node);

              // Get left and top positions
              var left = _var.wrap.node().getBoundingClientRect().left +_var.margin.left + x;
              var top  = _var.wrap.node().getBoundingClientRect().top + _var.margin.top + y - z - 5;

              // Initialize tooltip obj function
              var _getTooltipObj = function(n) {

                // Initialize tooltip object
                var tooltipObj = {};

                // Set parent n attributes to tooltip obj
                Object.keys(n._parent).forEach(function(k) { tooltipObj[k] = n._parent[k]; });

                // Set n attributes to tooltip obj
                Object.keys(n).forEach(function(k) { tooltipObj[k] = n[k]; });

                // Set x, y and z values with format
                tooltipObj.x = _var.xFormat(n.x);
                tooltipObj.y = _var.yFormat(n.y);
                tooltipObj.color = _var.pointColor(n);

                return tooltipObj;

              }

              // Set tooltip object
              var tooltipObj = _getTooltipObj(node);

              // Set tooltip header as title
              if(_var.data.tooltip.header != null) {
                _var.data.tooltip.header.forEach(function(p) {
                  _var.data.tooltip.title.unshift(gViz.shared.helpers.text.replaceVariables(p, {}));
                });
              }

              // Set tooltip component
              var tooltip = _var.data.tooltip == null ? {} : _var.data.tooltip;
              gViz.shared.visualComponents.tooltip()
                ._var(_var)
                .body(tooltip.body != null ? tooltip.body : "")
                .borderColor(tooltip.borderColor != null ? tooltip.borderColor : tooltipObj.color)
                .backgroundColor(tooltip.backgroundColor != null ? tooltip.backgroundColor : tooltipObj.color)
                .hasImg(tooltip.hasImg === true)
                .left(left)
                .muted(tooltip.muted != null && tooltip.muted === true)
                .obj(tooltipObj)
                .top(top)
                .title(tooltip.title != null ? tooltip.title : "")
                .run();

              break;

            case 'mouseout':

              // Fade other groups
              groups.transition().style('opacity', 1)

              // Add style to lines
              lines.transition()
                .style("filter", "")
                .style('opacity', 1);

              // Add style to points
              points.transition()
                .style("filter", '')
                .style('opacity', 1);

              // Set bars component
              gViz.shared.visualComponents.tooltip()
                ._var(_var)
                .action("hide")
                .run();

              break;

          }

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','action','components','node','mouse','origin'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) {
    return main('run');
  };

  return main;
};
