// Module declaration
gViz.viz.lineAreaChart.initialize = function() {
  "use strict";

  // Get attributes values
  var _id       = null;
  var _var      = null;
  var animation = 100;
  var container = null;
  var colors = [{value:0, color:"#4ad2a1"},{value:150, color:"#4ad2a1"},{value:300, color:"#3d62cb"},{value:500, color:"#3d62cb"},{value: 600, color:"#fa5050"}];
  var data      = [];
  var isNow     = true;
  var margin    = { top: 10, right: 10, bottom: 10, left: 10 };
  var opacity = 0.4;
  var timespan = 6000;
  var touchEvents = null;
  var wheelZoom = {};
  var xZoom = {};
  var yZoom = {};
  var zoomMode = 'horizontal';
  var zoomLevel = 1;

  // Validate attributes
  var validate = function(step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function(step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Initialize variables
          if (!_var) { _var = {}; }
          _var._id = _id;
          _var.animation = animation;
          _var.colors = (colors || [{value:0, color:"#4ad2a1"},{value:150, color:"#4ad2a1"},{value:300, color:"#3d62cb"},{value:500, color:"#3d62cb"},{value: 600, color:"#fa5050"}]).sort(function(a,b) { return d3.descending(+a.value, +b.value); });;
          _var.isNow = isNow;
          _var.margin = margin;
          _var.opacity = opacity || 0.4;
          _var.timespan = timespan;
          _var.touchEvents = touchEvents;
          _var.zoomMode = zoomMode;
          _var.zoomLevel = zoomLevel;
          _var.rebuild = { line: true, area: true };

          // Id for shadows
          _var.shadowId = "vis-shadow-"+(Math.floor(Math.random() * ((1000000000 - 5) + 1)) + 5);

           // Get container
          _var.container = {
            selector: container,
            d3: d3.select(container),
            el: ((typeof container === 'string' || container instanceof String) ? container : d3.select(container).node()),
            outerWrapper: d3.select(container).closest('.gViz-outer-wrapper'),
            outerWrapperClientRect: d3.select(container).closest('.gViz-outer-wrapper').node().getBoundingClientRect(),
            dimensions: {}
          };

          // Map data and get labels
          _var.data = _var._data = data;

          // Wheel Zoom
          if (_var.wheelZoom == null) { _var.wheelZoom = {}; }
          Object.keys(wheelZoom).forEach(function (key) { return _var.wheelZoom[key] = wheelZoom[key]; });

          // X Zoom
          if (_var.xZoom == null) { _var.xZoom = {}; }
          Object.keys(xZoom).forEach(function (key) { return _var.xZoom[key] = xZoom[key]; });

          // Y Zoom
          if (_var.yZoom == null) { _var.yZoom = {}; }
          Object.keys(yZoom).forEach(function (key) { return _var.yZoom[key] = yZoom[key]; });

          break;
      }
    }

    return _var;
  };

  // Expose global variables
  ['_id','_var','animation','container','colors','data','isNow','margin','opacity','timespan','touchEvents','wheelZoom','xZoom','yZoom','zoomMode','zoomLevel'].forEach(function(key) {

    // Attach variables to validation function
    validate[key] = function(_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return validate;
    };

    // Attach variables to main function
    return main[key] = function(_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return main;
    };
  });

  // Execute the specific called function
  main.run = function(_) { return main('run'); };

  return main;
};
