// Module declaration
gViz.viz.lineAreaChart.xScale = function() {
  "use strict";

  // Get attributes values
  var _var = undefined;
  var data = [];

  // Validate attributes
  var validate = function (step) {

    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Set format
          _var.xIsDate = (_var.data.x != null && _var.data.x.type === 'time' && _var.data.x.inFormat != null && _var.data.x.outFormat != null);
          _var.xIsNumber = (_var.data.x != null && _var.data.x.type === 'number' && _var.data.x.format != null);
          var xFmt = _var.xIsDate ? 'date' : 'number';
          _var.xFormat = gViz.shared.helpers[xFmt].parseFormat(_var.data == null ? null : _var.data.x);

          // Define scales
          var x = {};
          if(_var.xIsDate) { x = d3.scaleTime().range([0, _var.width]); }
          else { x = d3.scaleLinear().range([0, _var.width]); }

          // Define aux variables
          var min = null, max = null, diff = null;

          // Initialize domains
          _var.xDomain = {};
          var xDomain = [];
          var _data = {};

          // Set x last point
          _var.xLast = null;

          // Get domains
          data.forEach(function(d) {
            d.values.forEach(function(v,i) {

              // Set parent
              v._parent = d;

              // Date value
              if(_var.xIsDate) {

                // Parse values
                v.parsedX = d3.timeParse(_var.data.x.inFormat)(v.x);
                v.formattedX = _var.xFormat(v.x);

                // Add to _data hash
                _data[(+v.parsedX).toString()] = +v.parsedX;

                // Set domain
                if(min == null || min > +v.parsedX) { min = +v.parsedX; }
                if(max == null || max < +v.parsedX) { max = +v.parsedX; }

                // Set last x
                if(i === d.values.length-1 && (_var.xLast == null || _var.xLast.parsedX < v.parsedX)) { _var.xLast = v; }

              // Number values
              } else {

                // Parse values
                v.parsedX = +v.x;
                v.formattedX = _var.xFormat(v.x);

                // Add to _data hash
                _data[(+v.parsedX).toString()] = +v.parsedX;

                // Set domain
                if(min == null || min > +v.x) { min = +v.x; }
                if(max == null || max < +v.x) { max = +v.x; }

                // Set last x
                if(i === d.values.length-1 && (_var.xLast == null || _var.xLast.parsedX < v.parsedX)) { _var.xLast = v; }

              }

            });
          });

          // Sort values
          data.forEach(function(d) { d.values = d.values.sort(function(a,b) { return d3.ascending(a.parsedX, b.parsedX); }); });

          // Update _data hash to array
          _var.xData = Object.keys(_data).map(function(k) { return { parsedX: +k }; }).sort(function(a,b) { return d3.ascending(a.parsedX, b.parsedX); });

          // Check for default values
          if(isNaN(min)) { min = 0; }
          if(isNaN(max)) { max = 1; }

          // Get diff
          var diff = Math.abs(max - min) === 0 ? (max === 0 && min === 0 ? 0.1 : Math.abs(max * 0.1)) : 0;

          // Get x bounds
          _var.xBounds = _var.xIsDate ? [min,max] : [(min == 0 ? min : min - diff), max + diff];

          // Set xLimits if not set
          _var.xLimits = _var.xBounds;

          // Set x domain
          x.domain(_var.xBounds);

          // Copy scale
          if(_var.x == null || _var._x == null) { _var._x = _var.x = x; }

          // Set range
          _var.x.range([0, _var.width]);
          _var._x.range([0, _var.width]);

          // Get distance
          var zoomInfo = gViz.shared.data.lineChart.zoomLevelInfo(_var.zoomLevel);
          var hourInterval = 3600;
          _var.distance = zoomInfo.interval * _var.width / hourInterval;

          // Get x axis ticks
          var tickSpace = _var.data.x != null && _var.data.x.tickSpace != null && !isNaN(+_var.data.x.tickSpace) ? +_var.data.x.tickSpace : 100;
          var bins = d3.max([3, parseInt(_var.width / tickSpace, 10)]);

          // Define axis
          _var.xAxis = d3.axisBottom(_var._x).ticks(bins).tickPadding(10).tickFormat(gViz.shared.helpers.date.multiFormat);

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','data'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) {
    return main('run');
  };

  return main;
};
