// Module declaration
gViz.viz.lineAreaChart.yScale = function() {
  "use strict";

  // Get attributes values
  var _var = undefined;
  var data = [];

  // Validate attributes
  var validate = function (step) {

    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Initialize scale
          var y = d3.scaleLinear().range([_var.height, _var.margin.top]);

          // Define aux variables
          _var.yMax = null;
          var min = null,
              max = null,
              diff = null;

          // Get bounds
          data.forEach(function(d) {
            var mean = 0;
            d.values.forEach(function(v, i) {
              if(min == null || min > +v.y) { min = +v.y; }
              if(max == null || max < +v.y) { max = +v.y; }
              if(i === d.values.length-1 && (_var.yMax == null || _var.yMax < +v.y)) { _var.yMax = +v.y; }
              mean += +v.y;
            });
            d.mean = d.values.length === 0 ? 0 : mean / d.values.length;
          });

          // Check for default values
          if(isNaN(min)) { min = 0; }
          if(isNaN(max)) { max = 1; }

          // Store y bounds min from data
          _var.yBoundsData = [min,max];

          // Get mix and max values for axis set from json
          if(_var.data.y != null && _var.data.y.min != null && !isNaN(+_var.data.y.min) && +_var.data.y.min < min) { min = +_var.data.y.min; }
          if(_var.data.y != null && _var.data.y.max != null && !isNaN(+_var.data.y.max) && +_var.data.y.max > max) { max = +_var.data.y.max; }

          // Get diff
          var diff = Math.abs(max - min) === 0 ? (max === 0 && min === 0 ? 0.1 : Math.abs(max * 0.1)) : 0;

          // Set y domain
          var resetY = _var.yBounds != null && _var.yBounds[1] > (max + diff) && !(_var.wheelZoom.enable != null && _var.wheelZoom.enable === true);
          _var.yBounds = [min, resetY ? _var.yBounds[1] : (max + diff)];
          y.domain(_var.yBounds);

          // Set y scale
          if(_var.y == null || _var._y == null) { _var._y = _var.y = y; }

          // Set format
          _var.yFormat = gViz.shared.helpers.number.parseFormat(_var.data == null ? null : _var.data.y);

          // Get x axis ticks
          var bins = d3.max([2, parseInt(_var.height / 30, 10)]);

          // Define y axis
          _var.yAxis = d3.axisLeft(_var._y).ticks(bins).tickPadding(10).tickFormat(_var.yFormat);

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','data'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) {
    return main('run');
  };

  return main;
};
