var LukoGraph = function(params) {

  // Initialize consumption from Geraldo Franciscani Library
  return gViz.shared.modules.consumption()
    .container(params.container)
    .registrationToken(params.registration_token)
    .apiKey(params.api_key)
    .endPointing(params.api_endpoint)
    .language(params.language)
    .translate(params.translate)
    .menuItems(params.menuItems)
    .build();

}

// Initialize gViz global namespace
if (!window.gViz) {
  window.gViz = {};
}

// Create visualizations objects
if (!window.gViz.viz) {
  window.gViz.viz = {};
}

// Create shared objects
if (!window.gViz.shared) {
  window.gViz.shared = {};
}

if(!gViz.shared.helpers) { gViz.shared.helpers = {}; }
if(!gViz.shared.visualComponents) { gViz.shared.visualComponents = {}; }
if(!gViz.shared.data) { gViz.shared.data = {}; }
if(!gViz.shared.modules) { gViz.shared.modules = {}; }

// Module declaration
gViz.viz.bubbleChart = function () {
  "use strict";

  // Get attributes values
  var _id = `vis-bubble-${Math.floor(Math.random() * ((1000000000 - 5) + 1)) + 5}`;
  var _var = null;
  var action = 'build';
  var animation = 900;
  var container = null;
  var colors = ["#ccc","#3a6bc6"];
  var data = [];
  var margin = { top: 22, right: 0, bottom: 45, left: 0 };

  // Event bindings
  var onClick = function(d) { console.log(d); };

  // Validate attributes
  var validate = function (step) {
    switch (step) {
      case 'build':      return (container != null) && (d3.selectAll(container).size() !== 0 || d3.select(container).size() !== 0);
      case 'initialize': return true;
      case 'create':     return data != null && data.data != null && data.data.length > 0;
      case 'setup':   return data != null && data.data != null && data.data.length > 0;
      case 'misc':       return true;
      case 'style':      return true;
      case 'size':       return true;
      case 'drag':       return true;
      default:           return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'build':

          main('initialize');
          main('drag');
          main('style');
          main('misc');
          main('create');
          main('size');
          main('setup');
          break;

        // Initialize visualization variable
        case 'initialize':

          // Initializing
          if (!_var) { _var = {};  }
          _var = gViz.viz.bubbleChart.initialize()
            ._var(_var)
            ._id((_var._id != null) ? _var._id : _id)
            .animation(animation)
            .container(container)
            .colors(colors)
            .data(data)
            .margin(margin)
            .onClick(onClick)
            .run();

          break;

        // Setup style functions
        case 'style':

          // Styling
          _var = gViz.viz.bubbleChart.style()
            ._var(_var)
            .run();
          break;

        // Create initial setup
        case 'create':

          // Creating wrappers
          _var = gViz.viz.bubbleChart.create()
            ._var(_var)
            .run();
          break;

        // Setup chart setup
        case 'setup':

          // Running
          _var = gViz.viz.bubbleChart.setup()
            ._var(_var)
            .components(gViz.viz.bubbleChart)
            .data(_var.data.data)
            .run();
          break;

        // Show misc
        case 'misc':

          // Running
          _var = gViz.viz.bubbleChart.misc()
            ._var(_var)
            .components(gViz.viz.bubbleChart)
            .run();
          break;

        // Show size
        case 'size':

          // Running
          _var = gViz.viz.bubbleChart.size()
            ._var(_var)
            .data(_var.data.data)
            .run();
          break;

        // Show drag
        case 'drag':

          // Running
          _var = gViz.viz.bubbleChart.drag()
            ._var(_var)
            .components(gViz.viz.bubbleChart)
            .run();
          break;


      }
    }

    return _var;
  };

  // Expose global variables
  ['_id','_var','action','animation','container','colors','data','margin','onClick'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return main;
    };
  });

  // Secondary functions
  main.build = function (_) { return main("build"); };

  // Execute the specific called function
  main.run = function (_) { return main(_); };

  return main;

}

// Initialize the visualization class
gViz.viz.bubbleChart.create = function () {
  "use strict";

  // Get attributes values
  var _var = undefined;
  var animation = 900;

  // Validate attributes
  var validate = function (step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Draw svg
          _var.wrap = _var.container.d3.selectAll(`svg.chart-${_var._id}`).data(["chart-svg"], function(d) { return d; });
          _var.wrap.exit().remove();
          _var.wrap = _var.wrap.enter().append("svg").attr('class', `bubble-chart chart-${_var._id}`).merge(_var.wrap); // svg

          // Update outer dimensions
          _var.wrap
            .style('overflow','visible')
            .attr("width", _var.width + _var.margin.left + _var.margin.right)
            .attr("height", _var.height + _var.margin.top + _var.margin.bottom);

          // Draw g
          _var.g = _var.wrap.selectAll("g.chart-wrap").data(["chart-wrap"], function(d) { return d; }); // svg:g
          _var.g.exit().remove();
          _var.g = _var.g.enter().append('g').attr('class', "chart-wrap").merge(_var.g);

          // Update inner dimensions
          _var.g.attr("transform", `translate(${_var.margin.left},${_var.margin.top})`);

          // Draw defs
          _var.defs = _var.wrap.selectAll("defs.svg-defs").data(["svg-defs"], function(d) { return d; });
          _var.defs.exit().remove();
          _var.defs = _var.defs.enter().insert('defs',':first-child').attr("class", "svg-defs").merge(_var.defs);

          // Draw shadow
          gViz.shared.visualComponents.shadow()
            ._var(_var)
            .wrap(_var.wrap)
            .id(_var.shadowId)
            .run();

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var', 'animation'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};

// Initialize the visualization class
gViz.viz.bubbleChart.drag = function () {
  "use strict";

  // Get attributes values
  var _var       = null;
  var components = null;
  var data       = null;

  // Validate attributes
  var validate = function (step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Start the drag action
          _var.dragstarted = function (d) {
            if(!d3.event.active) { _var.simulation.alphaTarget(.03).restart(); }
            d.fx = d.x;
            d.fy = d.y;
          }

          // While dragging
          _var.dragging = function (d) {
            d.fx = d3.event.x;
            d.fy = d3.event.y;
          }

          // End the drag action
          _var.dragended = function (d) {
            if(!d3.event.active) { _var.simulation.alphaTarget(.03); }
            d.fx = null;
            d.fy = null;
          }

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','components','data'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};

// Initialize the visualization class
gViz.viz.bubbleChart.events = function () {
  "use strict";

  // Get attributes values
  var _var       = null;
  var action     = 'mouseover';
  var components = null;
  var node       = null;

  // Validate attributes
  var validate = function (step) {

    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Run code
        case 'run':

          // Set strokes and bubbles
          var strokes  = _var.g.select('.chart-setup').selectAll('.stroke')
          var bubbles   = _var.g.select('.chart-setup').selectAll('.bubble')

          switch (action) {

            case 'mouseover':

              // Fade strokes
              strokes.transition().style('opacity', function(g) { return g.x === node.x ? 1 : 0.2; })

              // Fade bubbles
              bubbles.transition()
                .style('opacity', function(g) { return g.x === node.x ? 1 : 0.2; })
                .style("filter", function(g) { return g === node ? "url(#"+_var.shadowId+")" : ""; })

              // Get x and y values
              var x = _var.hasWrapper(node.wrap) ? _var.x(node.x) + _var.x.bandwidth()/2 : _var.x(node.parent) + _var.xIn(node.x) + _var.xIn.bandwidth()/2;
              var y = _var.getY(node);

              // Get left and top positions
              var left = _var.wrap.node().getBoundingClientRect().left + x + _var.zoom.transform.x;
              var top  = _var.wrap.node().getBoundingClientRect().top + _var.margin.top + y;

              // Set node color
              var nodeColor = _var.getColor(node);

              // Initialize tooltip object
              var tooltipObj = { color: nodeColor  };

              // Set node attributes to tooltip obj
              Object.keys(node).forEach(function(k) { tooltipObj[k] = node[k]; });

              // Set x and y values with format
              tooltipObj.x = _var.xFormat(node.x);
              tooltipObj.y = _var.yFormat(node.y);

              // Add price function to body, if its not null
              var body = [];
              if(_var.data.tooltip.body != null) { _var.data.tooltip.body.forEach(function(d) { body.push(d); }); }
              if(_var.priceFunction != null) {
                body = ["<span class='metric'>"+_var.priceFunction(node)+"</span>"].concat(body);
              }

              // Set bubbles component
              gViz.shared.visualComponents.tooltip()
                ._var(_var)
                .body(body)
                .muted(_var.data.tooltip != null && _var.data.tooltip.muted != null && _var.data.tooltip.muted === true)
                .borderColor(nodeColor)
                .left(left)
                .hasImg(_var.data.tooltip != null && _var.data.tooltip.hasImg === true)
                .obj(tooltipObj)
                .side(x + _var.zoom.transform.x >= _var.x.bandwidth()/2 ? 'left' : 'right')
                .top(top)
                .title(_var.data.tooltip != null && _var.data.tooltip.title != null ? _var.data.tooltip.title : "")
                .run();

              break;

            case 'mouseout':

              // Reset opacity and filter
              strokes.transition().style('opacity', 1)
              bubbles.transition().style('opacity', 1).style("filter", "")

              // Set bubbles component
              gViz.shared.visualComponents.tooltip()
                ._var(_var)
                .action("hide")
                .run();

              break;
          }

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','action','components','node'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};

// Initialize the visualization class
gViz.viz.bubbleChart.initialize = function () {
  "use strict";

  // Get attributes values
  var _id = null;
  var _var = null;
  var animation = 900;
  var container = null;
  var colors = ["#ccc","#3a6bc6"];
  var data = [];
  var margin = { top: 10, right: 10, bottom: 10, left: 10 };

  // Events bindings
  var onClick = function(d) { console.log(d); };

  // Validate attributes
  var validate = function (step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Initialize variables
          if (!_var) { _var = {}; }
          _var._id = _id;
          _var.animation = animation;
          _var.colors = colors;
          _var.margin = margin;
          _var.onClick = onClick;

          // Id for shadows
          _var.shadowId = `vis-shadow-${Math.floor(Math.random() * ((1000000000 - 5) + 1)) + 5}`

           // Get container
          _var.container = {
            selector: container,
            d3: d3.select(container),
            el: ((typeof container === 'string' || container instanceof String) ? container : d3.select(container).node()),
            outerWrapper: d3.select(container).closest('.gViz-outer-wrapper'),
            outerWrapperClientRect: d3.select(container).closest('.gViz-outer-wrapper').node().getBoundingClientRect(),
            dimensions: {}
          };

          // Store data
          _var.data = data;

          break;
      }
    }

    return _var;
  };

  // Expose global variables
  ['_id', '_var', 'animation', 'container', 'colors', 'data', 'margin','onClick'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};

// Initialize the visualization class
gViz.viz.bubbleChart.misc = function() {
  "use strict";

  // Get attributes values
  var _var       = undefined;
  var action     = "start";
  var components = {};

  // Validate attributes
  var validate = function(step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function(step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          switch (action) {

            // First action to be done
            case 'start':

              /* -- Visualization Title -- */
              // Has title flag
              var hasTitle = _var.data.title != null && _var.data.title !== "";
              _var.container.dimensions.title = hasTitle ? 35 : 0;

              // Draw title wrapper
              var titleWrapper = _var.container.outerWrapper.selectAll(".gviz-title-wrapper").data(["gviz-title-wrapper"]);
              titleWrapper.exit().remove();
              titleWrapper = titleWrapper.enter().insert('div', ":first-child").attr('class', "gviz-title-wrapper").merge(titleWrapper);
              titleWrapper
                .style('width', '100%')
                .style('height', '30px')
                .style('line-height', '30px')
                .style('margin', '0px 0px 5px 0px')
                .style('padding', '0px 10px')
                .style('oveflow', 'hidden')
                .style('white-space', 'nowrap')
                .style('text-overflow', 'ellipsis')
                .style('background-color', '#eee')
                .style('color', '#666')
                .style('font-size', '15px')
                .style('font-weight', 'lighter')
                .style('display', hasTitle ? 'block' : 'none')
                .html(_var.data.title)

              /* -- Axis and Legend -- */
              // Set axis string
              var xTitle = (_var.data.x != null && _var.data.x.title != null && _var.data.x.title !== "" ? "<b>X - </b>"+_var.data.x.title : "");

              // Set axis title
              if(xTitle !== "") { _var.axisTitle = xTitle; }

              // Update container _id, height and client bound rect
              _var.container.d3
                .attr('data-vis-id', _var._id)
                .style('height', (_var.container.outerWrapperClientRect.height - _var.container.dimensions.title) + "px")
                .style('top', (_var.container.dimensions.title) + 'px')

              // Update grid background position
              _var.container.outerWrapper.selectAll('.grid-background')
                .style('top', (_var.container.dimensions.title) + 'px')

              // Define height and width
              var containerClientRect = _var.container.d3.node().getBoundingClientRect();
              _var.height = containerClientRect.height - (_var.margin.top + _var.margin.bottom);
              _var.width = containerClientRect.width - (_var.margin.left + _var.margin.right);

              // NO DATA AVAILABLE
              if (_var.data.data == null || _var.data.data.length === 0) {
                _var.container.d3.html("<h5 style='line-height: " + containerClientRect.height + "px; text-align: center;'>NO DATA AVAILABLE</h5>");
              } else {
                _var.container.d3.selectAll("h5").remove();
              }

              break;

          }
          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','action','components'].forEach(function(key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};

// Initialize the visualization class
gViz.viz.bubbleChart.setup = function () {
  "use strict";

  // Get attributes values
  var _var       = null;
  var components = null;
  var data       = null;

  // Validate attributes
  var validate = function (step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Element canvas
          var elements = _var.g.selectAll(".chart-elements").data(["chart-elements"], function(d) { return d; });
          elements.exit().remove();
          elements = elements.enter().insert("g", ":first-child").attr("class", "chart-elements").merge(elements);

          // Set data array
          var _data = (data == null ? _var.data.data : data);

          _var.simulation = d3.forceSimulation()
            .force("forceX", d3.forceX().strength(.1).x(_var.width * .5))
            .force("forceY", d3.forceY().strength(.1).y(_var.height * .5 + _var.margin.top))
            .force("center", d3.forceCenter().x(_var.width * .5).y(_var.height * .5 + _var.margin.top))
            .force("charge", d3.forceManyBody().strength(-15));

          //update the simulation based on the data
          _var.simulation
            .nodes(_data)
            .force("collide", d3.forceCollide().strength(.5).radius(function(d){ return _var.size(+d.value) + 1; }).iterations(1))
            .on("tick", function(d){
              elements.selectAll('.bubble-group')
                .attr('transform', function(d) { return 'translate('+d.x+','+d.y+')'; })
            });

          // Create groups
          var groups = elements.selectAll(".bubble-group").data(_data, function (d) { return d.id; });
          groups.exit().remove();
          groups = groups.enter().append("g").attr("class", "bubble-group").merge(groups);

          // For each element in group
          groups.each(function (e, i) {

            // Create bubble circles
            var circle = d3.select(this).selectAll('.bubble-circle').data([e], function (d) { return d.id; });
            circle.exit().remove();
            circle = circle.enter().append("circle").attr("class", 'bubble-circle').merge(circle);
            circle
              .attr('r', function(d) { return _var.size(+d.value); })
              .attr('x', function(d) { return -_var.size(+d.value)*0.5; })
              .attr('y', function(d) { return -_var.size(+d.value)*0.5; })
              .attr('fill', function(d) { return "#FFF"; })
              .attr('stroke', function(d) { return _var.colorScale(+d.value); })

            // Draw bubble texts
            var bubble = d3.select(this).selectAll('.bubble-text').data([e], function (d) { return d.id; });
            bubble.exit().remove();
            bubble = bubble.enter().append("text").attr("class", 'bubble-text').merge(bubble);
            bubble
              .attr('x', 0)
              .attr('y', function(d) { return d.img != null && d.img !== '' ? -_var.size(+d.value)*0.5 + 10 : 0; })
              .attr('text-anchor', 'middle')
              .attr('font-size', '11px')
              //.attr('fill', function(d) { return gViz.shared.helpers.colors.isDark(_var.colorScale(+d.value)) ? "#FFF" : "#454545"; })
              .attr('fill', function(d) { return _var.colorScale(+d.value); })
              .text(function(d) { return d.name; })
              .each(function(d) { gViz.shared.helpers.text.wrapBySize(d3.select(this), _var.size(+d.value)*1.4, 30, (d.img != null && d.img !== '' ? 1 : 3)); })

            // Insert / Update labels for the nodes
            var imageData = e.img != null && e.img !== '' ? [e] : [];
            var image = d3.select(this).selectAll('image.bubble-image').data(imageData, function (d) { return d.id; });
            image.exit().remove();
            image = image.enter().append('image').attr('class', 'bubble-image').merge(image);
            image
              .attr("xlink:href", function(d) { return d.img; })
              .attr('x', function(d) { return -_var.size(+d.value)*0.3; })
              .attr('y', function(d) { return 0; })
              .attr('width', function(d) { return _var.size(+d.value)*0.6; })
              .attr('height', function(d) { return _var.size(+d.value)*0.6; })

          });

          // Bind click event
          groups.on('click', function(d) {
            if(_var.onClick != null && typeof _var.onClick === "function") { _var.onClick(d); }
          });

          // Bind drag to groups
          groups.call(d3.drag()
            .on("start", _var.dragstarted)
            .on("drag", _var.dragging)
            .on("end", _var.dragended)
            .touchable(true));

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','components','data'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};

// Module declaration
gViz.viz.bubbleChart.size = function() {
  "use strict";

  // Get attributes values
  var _var = null;
  var data = [];

  // Validate attributes
  var validate = function (step) {

    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Get initial set of colors
          var colors = _var.colors.map(function(d) { return d3.color(d); }).filter(function(d) { return d != null; });

          // Define scales
          if(colors.length === 0) { _var.bubbleColors = [d3.color("#ccc"),d3.color("#3a6bc6")]; }
          else if(colors.length === 1) { _var.bubbleColors = [colors[0], colors[0].darker(3)]; }
          else { _var.bubbleColors = colors; }

          // Get max and min sizes from attrs
          var attrs = _var.data != null && _var.data.attrs != null ? _var.data.attrs : {};
          var minSize = attrs.minSize != null && !isNaN(+attrs.minSize) ? +attrs.minSize : 30;
          var maxSize = attrs.maxSize != null && !isNaN(+attrs.maxSize) ? +attrs.maxSize : 60;

          // Initialize scale
          _var.colorScale = d3.scaleLinear().range(_var.bubbleColors);
          _var.size = d3.scaleLinear().range([d3.min([minSize,maxSize]), d3.max([minSize,maxSize])]);

          // Define aux variables
          var min = null,
              max = null,
              diff = null;

          // Get bounds
          data.forEach(function(d) {
            if(min == null || min > +d.value) { min = +d.value; }
            if(max == null || max < +d.value) { max = +d.value; }
          });

          // Check for default values
          if(isNaN(min)) { min = 0; }
          if(isNaN(max)) { max = 1; }

          // Store y bounds min from data
          _var.sizeBoundsData = [min,max];

          // Get mix and max values for axis set from json
          if(_var.data.size != null && _var.data.size.min != null && !isNaN(+_var.data.size.min) && +_var.data.size.min < min) { min = +_var.data.size.min; }
          if(_var.data.size != null && _var.data.size.max != null && !isNaN(+_var.data.size.max) && +_var.data.size.max > max) { max = +_var.data.size.max; }

          // Get diff
          var diff = Math.abs(max - min) === 0 ? Math.abs(max * 0.1) : Math.abs(max - min) * 0.05;

          // Set size domain
          _var.sizeBounds = [min, max];
          _var.size.domain(_var.sizeBounds);

          // Get color domain
          var colorDomain = d3.range(_var.colorScale.range().length)
            .map(function(d, i) {
              return _var.sizeBounds[0] + i * (Math.abs(_var.sizeBounds[1] - _var.sizeBounds[0]) / (_var.colorScale.range().length-1));
            });

          // Set color scale domain
          _var.colorScale.domain(colorDomain);

          // Set format
          _var.sizeFormat = gViz.shared.helpers.number.parseFormat(_var.data == null ? null : _var.data.size);

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','data'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) {
    return main('run');
  };

  return main;
};

// Initialize the visualization class
gViz.viz.bubbleChart.style = function() {
  "use strict";

  // Get attributes values
  var _var      = undefined;
  var components = {};

  // Validate attributes
  var validate = function(step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function(step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','components'].forEach(function(key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};

// Module declaration
gViz.viz.barChart = function () {
  "use strict";

  // Get attributes values
  var _id = `vis-bar-${Math.floor(Math.random() * ((1000000000 - 5) + 1)) + 5}`;
  var _var = null;
  var action = 'build';
  var animation = 900;
  var container = null;
  var colors = { main: gViz.shared.helpers.colors.main };
  var data = [];
  var margin = { top: 22, right: 0, bottom: 45, left: 0 };
  var hover = null;
  var zoom = {};
  var priceFunction = null;;
  var xInDomain = null;;

  // Event bindings
  var onHover = function(d) { console.log(d); };
  var onClick = function(d) { console.log(d); };

  // Validate attributes
  var validate = function (step) {
    switch (step) {
      case 'build':      return (container != null) && (d3.selectAll(container).size() !== 0 || d3.select(container).size() !== 0);
      case 'initialize': return true;
      case 'axis':       return data != null && data.data != null && data.data.length > 0;
      case 'create':     return data != null && data.data != null && data.data.length > 0;
      case 'elements':   return data != null && data.data != null && data.data.length > 0;
      case 'misc':       return true;
      case 'style':      return true;
      case 'xScale':     return data != null && data.data != null && data.data.length > 0;
      case 'yScale':     return data != null && data.data != null && data.data.length > 0;
      case 'yScaleSize': return data != null && data.data != null && data.data.length > 0;
      case 'zoom':       return data != null && data.data != null && data.data.length > 0;
      default:           return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'build':

          main('initialize');
          main('style');
          main('misc');
          main('yScale');
          main('yScaleSize');
          main('xScale');
          main('yScaleSize');
          main('create');
          main('axis');
          main('zoom');
          main('elements');
          main('zoom');
          break;

        // Initialize visualization variable
        case 'initialize':

          // Initializing
          if (!_var) { _var = {};  }
          _var = gViz.viz.barChart.initialize()
            ._var(_var)
            ._id((_var._id != null) ? _var._id : _id)
            .animation(animation)
            .container(container)
            .colors(colors)
            .data(data)
            .hover(hover)
            .margin(margin)
            .zoom(zoom)
            .priceFunction(priceFunction)
            .onHover(onHover)
            .onClick(onClick)
            .xInDomain(xInDomain)
            .run();

          break;

        // Setup style functions
        case 'style':

          // Styling
          _var = gViz.viz.barChart.style()
            ._var(_var)
            .run();
          break;

        // Create initial elements
        case 'create':

          // Creating wrappers
          _var = gViz.viz.barChart.create()
            ._var(_var)
            .run();
          break;

        // Setup x scale
        case 'xScale':

          // Creating
          _var = gViz.viz.barChart.xScale()
            ._var(_var)
            .data(_var.data.data)
            .run();
          break;

        // Setup y scale
        case 'yScale':

          // Creating
          _var = gViz.viz.barChart.yScale()
            ._var(_var)
            .data(_var.data.data)
            .run();
          break;

        // Setup y scale size
        case 'yScaleSize':

          // Creating
          _var = gViz.viz.barChart.yScaleSize()
            ._var(_var)
            .data(_var.data.data)
            .run();
          break;

        // Setup axis elements
        case 'axis':

          // Running
          _var = gViz.viz.barChart.axis()
            ._var(_var)
            .action('create')
            .run();
          break;

        // Setup chart elements
        case 'elements':

          // Running
          _var = gViz.viz.barChart.elements()
            ._var(_var)
            .components(gViz.viz.barChart)
            .data(_var.data.data)
            .run();
          break;

        // Set zoom case
        case 'zoom':

          // Creating wrappers
          _var = gViz.viz.barChart.zoom()
            ._var(_var)
            .components(gViz.viz.barChart)
            .run();
          break;

        // Show misc
        case 'misc':

          // Running
          _var = gViz.viz.barChart.misc()
            ._var(_var)
            .components(gViz.viz.barChart)
            .run();
          break;

      }
    }

    // Getters
    _var.get = {

      // Get most left point
      start: function() {
        var start = null;
        _var.data.data.forEach(function(d) {
          if(_var.x(d.x) <= -_var.zoom.transform.x && _var.x(d.x) + _var.x.bandwidth() > -_var.zoom.transform.x) {
            start = d.values[0].date;
            var x = _var.x(d.x);
            d.values.forEach(function(v) {
              var xIn = _var.xIn(v.x);
              if(x + xIn <= -_var.zoom.transform.x && x + xIn + _var.xIn.bandwidth() > -_var.zoom.transform.x) {
                start = v.date;
              }
            });
          }
        });
        return start;
      },

      // Get most right point
      end: function() {
        var end = null;
        _var.data.data.forEach(function(d) {
          if(_var.x(d.x) <= -_var.zoom.transform.x+_var.width && _var.x(d.x)+_var.x.bandwidth() > -_var.zoom.transform.x+_var.width) {
            end = d.values[0].date;
            var x = _var.x(d.x);
            d.values.forEach(function(v) {
              var xIn = _var.xIn(v.x);
              if(x + xIn <= -_var.zoom.transform.x+_var.width && x + xIn + _var.xIn.bandwidth() > -_var.zoom.transform.x+_var.width) {
                end = v.date;
              }
            });
          }
        });
        return end;
      },

      // Get min y value
      min: function() { return _var.yBounds[0]; },

      // Get max y value
      max: function() { return _var.yBounds[1]; }

    }

    return _var;
  };

  // Expose global variables
  ['_id','_var','action','animation','container','colors','data','margin','hover','zoom','onHover','onClick','priceFunction','xInDomain'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return main;
    };
  });

  // Secondary functions
  main.build = function (_) { return main("build"); };

  // Execute the specific called function
  main.run = function (_) { return main(_); };

  return main;

}

// Module declaration
gViz.viz.barChart.axis = function() {
  "use strict";

  // Get attributes values
  var _var = undefined;
  var action = 'create';

  // Validate attributes
  var validate = function validate(step) {

    switch (step) {
      case 'run':
        return true;
      default:
        return false;
    }
  };

  // Main function
  var main = function main(step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          switch (action) {

            case 'create':

              // Create and update X axis
              _var.x_axis = _var.g.selectAll(".x.axis").data(['x']);
              _var.x_axis.exit().remove();
              _var.x_axis = _var.x_axis.enter().append('g').attr("class", "x axis").merge(_var.x_axis);
              _var.x_axis.call(_var.xAxis.tickSize(5)).attr("transform", 'translate(0,' + _var.height + ')')
              _var.x_axis.selectAll('.tick text')
                .attr('y', 26)
                .attr('x', function(d) { return -(_var.x.bandwidth() / 2) + Math.max((_var.xIn(_var.xIn.domain()[0]) + _var.xIn.bandwidth()/2 - _var.barWidth/2), 10); })
                .attr('text-anchor', 'start')
                .style('font-weight', 600)
                .style('font-size', '14px')
                .style('fill', '#9b9b9b')
                .text(function(d) { return _var.xDomain[d].name; })

              // Update x axis attributes and styles
              _var.updateXAxis = function() {

                // Draw Background rect
                var pathDomain = _var.x_axis.selectAll("path.domain-custom").data(["domain-custom"]);
                pathDomain.exit().remove();
                pathDomain = pathDomain.enter().append('path').attr("class", "domain-custom").merge(pathDomain);
                pathDomain.attr("d", "M 0,0 " + (_var.width + _var.margin.left + _var.margin.right) + ",0")

                _var.x_axis.selectAll('.tick text')
                  .attr('x', function(d) {

                    // Get domain range for specific date
                    var range = [_var.x(_var.xDomain[d].x), _var.x(_var.xDomain[d].x) + _var.x.bandwidth()];

                    // If the initial range is on the right of the most left bound
                    if(range[0] >= -_var.zoom.transform.x) {
                      return -(_var.x.bandwidth() / 2) + Math.max((_var.xIn(_var.xIn.domain()[0]) + _var.xIn.bandwidth()/2 - _var.barWidth/2), 10);

                    // If the initial range is on the left of the most left bound
                    } else {
                      var size  = this.getBBox().width;
                      if(range[1] + _var.zoom.transform.x > size + 20) {
                        return (_var.x.bandwidth() / 2) - (range[1] + _var.zoom.transform.x - 10);
                      } else {
                        return (_var.x.bandwidth() / 2) - (size + 10);
                      }
                    }
                  })

              }

              break;

          }
          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var', 'action'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) {
    return main('run');
  };

  return main;
};

function __range__(left, right, inclusive) {
  var range = [];
  var ascending = left < right;
  var end = !inclusive ? right : ascending ? right + 1 : right - 1;
  for (var i = left; ascending ? i < end : i > end; ascending ? i++ : i--) { range.push(i); }
  return range;
}

// Initialize the visualization class
gViz.viz.barChart.bars = function() {
  "use strict";

  // Get attributes values
  var _var       = null;
  var animation  = 900;
  var components = {};
  var nodeObj    = null;
  var nodeIndex  = 0;
  var node       = null;

  // Validate attributes
  var validate = function(step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function(step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Get parent selection
          var nodeSel = d3.select(nodeObj);

          // Clean wrap gradients
          gViz.shared.visualComponents.gradient()
            .action('clean')
            .run();

          // Draw grouping bar
          var wrapperBars = nodeSel.selectAll(".wrapper-bar").data([node], function(d) { return node.x + "--" + d.x; });
          wrapperBars.exit().remove();
          wrapperBars = wrapperBars.enter().append('rect').attr("class", "wrapper-bar").merge(wrapperBars);
          wrapperBars
            .style('fill', function(d) {

              // Find first node with color, if its available
              var colorIndex = _var.data.data.findIndex(function(d) { return d.color != null; });

              // Get color for group and set color
              if(d.color != null) { return d.color; }
              else if(_var.data.data.length === 1) { d.color = "transparent"; }
              else if(colorIndex === -1) { d.color = "#eaeaea"; }
              else {
                if(Math.abs(nodeIndex - colorIndex) % 2 === 0) { d.color = _var.data.data[colorIndex].color; }
                else { d.color = _var.data.data[colorIndex].color === "transparent" ? "#eaeaea" : "transparent"; }
              }

              return d.color;
            })
            .attr("x", 0)
            .attr('width', _var.x.bandwidth())
            .attr('y', -_var.margin.top)
            .attr('height',  _var.height + _var.margin.bottom + _var.margin.top)

          // Set bar width
          _var.barWidth = Math.min(_var.xIn.bandwidth(), _var.barWidth);

          // // Draw bars
          var bars = nodeSel.selectAll(".bar-rect").data(node.values, function(d) { return node.x + "--" + d.x + "--" + d.id; });
          bars.exit().remove();
          bars = bars.enter().append('rect').attr("class", "bar-rect")
            .attr('x', _var.getXIn)
            .attr('y', function(d) { return _var.getY(d) + _var.getHeight(d); })
            .attr('rx', _var.getRadius)
            .attr('ry', _var.getRadius)
            .attr('width', _var.barWidth)
            .attr('height', 0)
            .merge(bars);
          bars
            .attr('opacity', _var.getBarOpacity)
            .style('fill', function(d) { return _var.getBarColor(node, d); })
            .transition().duration(600)
              .attr('rx', _var.getRadius)
              .attr('ry', _var.getRadius)
              .attr('x', _var.getXIn)
              .attr('y', _var.getY)
              .attr('width', _var.barWidth.toFixed(2))
              .attr('height', function(d) { return (_var.getHeight(d) + _var.getRadius(d)).toFixed(2); })
              .attr('stroke', "transparent")
              .attr('stroke-width', "60px")
              .each(function(d) {
                var sel = d3.select(this);
                setTimeout(function() {
                  sel.style('stroke-dasharray', function(d) { return _var.getStrokeDasharray(d); })
                }, 100);
              });

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','animation','components','nodeObj','nodeIndex','node'].forEach(function(key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};

// Initialize the visualization class
gViz.viz.barChart.create = function () {
  "use strict";

  // Get attributes values
  var _var = undefined;
  var animation = 900;

  // Validate attributes
  var validate = function (step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Draw svg
          _var.wrap = _var.container.d3.selectAll(`svg.chart-${_var._id}`).data(["chart-svg"], function(d) { return d; });
          _var.wrap.exit().remove();
          _var.wrap = _var.wrap.enter().append("svg").attr('class', `bar-chart chart-${_var._id}`).merge(_var.wrap); // svg

          // Update outer dimensions
          _var.wrap
            .style('overflow','visible')
            .attr("width", _var.width + _var.margin.left + _var.margin.right)
            .attr("height", _var.height + _var.margin.top + _var.margin.bottom);

          // Get transformation
          var transform = {
            x: _var.zoom.transform != null && _var.zoom.transform.x != null ? _var.zoom.transform.x : _var.margin.left,
            y: _var.zoom.transform != null && _var.zoom.transform.y != null ? _var.zoom.transform.y : _var.margin.top
          };

          // Draw g
          _var.g = _var.wrap.selectAll("g.chart-wrap").data(["chart-wrap"], function(d) { return d; }); // svg:g
          _var.g.exit().remove();
          _var.g = _var.g.enter().append('g').attr('class', "chart-wrap").merge(_var.g);

          // Update inner dimensions
          _var.g.attr("transform", `translate(${transform.x},${transform.y})`).attr('clip-path', 'url(#clip-path-'+_var._id+')');

          // Draw g
          _var.gBars = _var.wrap.selectAll("g.chart-wrap-bars").data(["chart-wrap-bars"], function(d) { return d; }); // svg:g
          _var.gBars.exit().remove();
          _var.gBars = _var.gBars.enter().append('g').attr('class', "chart-wrap-bars").merge(_var.gBars);

          // Update inner dimensions
          _var.gBars.attr("transform", `translate(${transform.x},${transform.y})`).attr('clip-path', 'url(#bars-clip-path-'+_var._id+')');

          // Draw g
          _var.gClip = _var.wrap.selectAll("g.chart-wrap-clip").data(["chart-wrap-clip"], function(d) { return d; }); // svg:g
          _var.gClip.exit().remove();
          _var.gClip = _var.gClip.enter().insert('g',':first-child').attr('class', "chart-wrap-clip").merge(_var.gClip);

          // Update inner dimensions
          _var.gClip.attr("transform", `translate(${_var.margin.left},${_var.margin.top})`);

          // Draw defs
          _var.defs = _var.wrap.selectAll("defs.svg-defs").data(["svg-defs"], function(d) { return d; });
          _var.defs.exit().remove();
          _var.defs = _var.defs.enter().insert('defs',':first-child').attr("class", "svg-defs").merge(_var.defs);

          // Draw shadow
          gViz.shared.visualComponents.shadow()
            ._var(_var)
            .wrap(_var.wrap)
            .id(_var.shadowId)
            .run();

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var', 'animation'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};

// Initialize the visualization class
gViz.viz.barChart.elements = function () {
  "use strict";

  // Get attributes values
  var _var       = null;
  var components = null;
  var data       = null;

  // Validate attributes
  var validate = function (step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Set data array
          var _data = (data == null ? _var.data.data : data);

          // Element canvas
          var elements = _var.gBars.selectAll(".chart-elements").data(["chart-elements"], function(d) { return d; });
          elements.exit().remove();
          elements = elements.enter().insert("g", ":first-child").attr("class", "chart-elements").merge(elements);

          // Create groups
          var groups = elements.selectAll(".element-group").data(_data, function (d) { return d.x; });
          groups.exit().remove();
          groups = groups.enter().append("g").attr("class", "element-group").merge(groups);

          // For each element in group
          groups
            .attr('data-id', function(d) { return d.x; })
            .attr("transform", function (d) { return `translate(${_var.x(d.x)},0)`; })
            .each(function (e, i) {

              // Mouseover event
              components.bars()
                ._var(_var)
                .components(components)
                .nodeIndex(i)
                .nodeObj(this)
                .node(e)
                .run();

            });

          // Event bindings
          elements.selectAll('.bar-rect').on('mouseover', function(e) {

            // Set hovered node
            _var.hovered = e;

            // Mouseover event
            components.events()
              ._var(_var)
              .action("mouseover")
              .components(components)
              .node(e)
              .run();

          }).on('mouseout', function(e) {

            // Reset hovered node
            _var.hovered = null;

            // Mouseout event
            components.events()
              ._var(_var)
              .action("mouseout")
              .components(components)
              .run();

          });

          // Element canvas
          var tElements = _var.g.selectAll(".chart-t-elements").data(["chart-t-elements"], function(d) { return d; });
          tElements.exit().remove();
          tElements = tElements.enter().insert("g", ":first-child").attr("class", "chart-t-elements").merge(tElements);

          // Create tGroups
          var tGroups = tElements.selectAll(".element-t-group").data(_data, function (d) { return d.x; });
          tGroups.exit().remove();
          tGroups = tGroups.enter().append("g").attr("class", "element-t-group").merge(tGroups);

          // For each element in group
          tGroups
            .attr('data-id', function(d) { return d.x; })
            .attr("transform", function (d) { return `translate(${_var.x(d.x)},0)`; })
            .each(function (e, i) {

              // Get month days for month parse
              var monthDays = 31;
              if(e.dataType === 'month') { monthDays = d3.timeParse("%Y-%m-%d")(e.id).monthDays(); }

              // Get values for inside texts
              var textValuesObj = {}, index = -1;
              var textValues = e.values.filter(function(d, i) {
                var flag = textValuesObj[d.x] == null;
                textValuesObj[d.x] = true;
                if(e.dataType === 'month') { flag = flag && parseInt(d.x) <= monthDays; }
                return flag;
              }).filter(function(d) {
                index += 1;
                return  _var.xIn.bandwidth() >= 20 || (index%2 === 1);
              });

              // Draw X Inside Texts
              var texts = d3.select(this).selectAll("text.x-in-text").data(textValues, function(d) { return d.x; });
              texts.exit().remove();
              texts = texts.enter().append('text').attr("class", "x-in-text").merge(texts);
              texts
                .attr("x", function(d) {
                  return _var.xIn(d.x) + _var.xIn.bandwidth()/2;
                })
                .attr('y', _var.height + 17)
                .attr('text-anchor', 'middle')
                .text(function(d) { return d.name; })
                .each(function(d) { gViz.shared.helpers.text.wrapBySize(d3.select(this), 2*_var.xIn.bandwidth(), _var.margin.bottom, _var.xMaxLines); })

            });

          // Draw Background clip Path
          _var.bgClip = _var.defs.selectAll(".bg-clip").data(["bg-clip"]);
          _var.bgClip.exit().remove();
          _var.bgClip = _var.bgClip.enter().insert('clipPath', ':first-child').attr("class", "bg-clip").merge(_var.bgClip);
          _var.bgClip
            .attr("id", "clip-path-"+_var._id)
            .each(function() {

              // Draw Background rect
              _var.bgClipRect = d3.select(this).selectAll("rect.bg-rect").data(["bg-rect"]);
              _var.bgClipRect.exit().remove();
              _var.bgClipRect = _var.bgClipRect.enter().insert('rect', ':first-child').attr("class", "bg-rect").style('fill', 'transparent').merge(_var.bgClipRect);
              _var.bgClipRect
                .style('fill', 'transparent')
                .attr("x", 0)
                .attr('y', 0)
                .attr('width', _var.width + _var.margin.right + 1)
                .attr("height", _var.height + _var.margin.bottom + _var.margin.top);

            });

          // Draw Background clip Path
          _var.bgBarsClip = _var.defs.selectAll(".bars-bg-clip").data(["bg-clip"]);
          _var.bgBarsClip.exit().remove();
          _var.bgBarsClip = _var.bgBarsClip.enter().insert('clipPath', ':first-child').attr("class", "bars-bg-clip").merge(_var.bgBarsClip);
          _var.bgBarsClip
            .attr("id", "bars-clip-path-"+_var._id)
            .each(function() {

              // Draw Background rect
              _var.bgBarsClipRect = d3.select(this).selectAll("rect.bg-rect").data(["bg-rect"]);
              _var.bgBarsClipRect.exit().remove();
              _var.bgBarsClipRect = _var.bgBarsClipRect.enter().insert('rect', ':first-child').attr("class", "bg-rect").style('fill', 'transparent').merge(_var.bgBarsClipRect);
              _var.bgBarsClipRect
                .style('fill', 'transparent')
                .attr("x", 0)
                .attr('y', -_var.margin.top)
                .attr('width', _var.width + _var.margin.right + 1)
                .attr("height", _var.height + _var.margin.top);

            });


          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','components','data'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};

// Initialize the visualization class
gViz.viz.barChart.events = function () {
  "use strict";

  // Get attributes values
  var _var       = null;
  var action     = 'mouseover';
  var components = null;
  var node       = null;

  // Validate attributes
  var validate = function (step) {

    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Run code
        case 'run':

          // Set strokes and bars
          var strokes  = _var.g.select('.chart-elements').selectAll('.stroke')
          var bars   = _var.g.select('.chart-elements').selectAll('.bar')

          switch (action) {

            case 'mouseover':

              // Fade strokes
              strokes.transition().style('opacity', function(g) { return g.x === node.x ? 1 : 0.2; })

              // Fade bars
              bars.transition()
                .style('opacity', function(g) { return g.x === node.x ? 1 : 0.2; })
                .style("filter", function(g) { return g === node ? "url(#"+_var.shadowId+")" : ""; })

              // Get x and y values
              var x = _var.hasWrapper(node.wrap) ? _var.x(node.x) + _var.x.bandwidth()/2 : _var.x(node.parent) + _var.xIn(node.x) + _var.xIn.bandwidth()/2;
              var y = _var.getY(node);

              // Get left and top positions
              var left = _var.wrap.node().getBoundingClientRect().left + x + _var.zoom.transform.x;
              var top  = _var.wrap.node().getBoundingClientRect().top + _var.margin.top + y;

              // Set node color
              var nodeColor = _var.getColor(node);

              // Initialize tooltip object
              var tooltipObj = { color: nodeColor  };

              // Set node attributes to tooltip obj
              Object.keys(node).forEach(function(k) { tooltipObj[k] = node[k]; });

              // Set x and y values with format
              tooltipObj.x = _var.xFormat(node.x);
              tooltipObj.y = _var.yFormat(node.y);

              // Add price function to body, if its not null
              var body = [];
              if(_var.data.tooltip.body != null) { _var.data.tooltip.body.forEach(function(d) { 
                body.push(d); 
              }); }
              if(_var.priceFunction != null) {
                body = ["<span class='metric'>"+_var.priceFunction(node)+"</span>"].concat(body);
              }

              // Set bars component
              gViz.shared.visualComponents.tooltip()
                ._var(_var)
                .body(body)
                .muted(_var.data.tooltip != null && _var.data.tooltip.muted != null && _var.data.tooltip.muted === true)
                .borderColor(nodeColor)
                .left(left)
                .hasImg(_var.data.tooltip != null && _var.data.tooltip.hasImg === true)
                .obj(tooltipObj)
                .side(x + _var.zoom.transform.x >= _var.x.bandwidth()/2 ? 'left' : 'right')
                .top(top)
                .title(_var.data.tooltip != null && _var.data.tooltip.title != null ? _var.data.tooltip.title : "")
                .run();

              break;

            case 'mouseout':

              // Reset opacity and filter
              strokes.transition().style('opacity', 1)
              bars.transition().style('opacity', 1).style("filter", "")

              // Set bars component
              gViz.shared.visualComponents.tooltip()
                ._var(_var)
                .action("hide")
                .run();

              break;
          }

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','action','components','node'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};

// Initialize the visualization class
gViz.viz.barChart.initialize = function () {
  "use strict";

  // Get attributes values
  var _id = null;
  var _var = null;
  var animation = 900;
  var container = null;
  var colors = { main: gViz.shared.helpers.colors.main, aux: gViz.shared.helpers.colors.aux };
  var data = [];
  var hover = null;
  var margin = { top: 10, right: 10, bottom: 10, left: 10 };
  var zoom = {};
  var priceFunction = null;
  var xInDomain = null;

  // Events bindings
  var onHover = function(d) { console.log(d); };
  var onClick = function(d) { console.log(d); };

  // Validate attributes
  var validate = function (step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Initialize variables
          if (!_var) { _var = {}; }
          _var._id = _id;
          _var.animation = animation;
          _var.colors = colors;
          _var.margin = margin;
          _var.onHover = onHover;
          _var.onClick = onClick;
          _var.priceFunction = priceFunction;
          _var.userXInDomain = xInDomain;

          // Id for shadows
          _var.shadowId = `vis-shadow-${Math.floor(Math.random() * ((1000000000 - 5) + 1)) + 5}`

          // Zoom
          if (_var.zoom == null) { _var.zoom = { }; }
          Object.keys(zoom).forEach(function (key) { return _var.zoom[key] = zoom[key]; });

           // Get container
          _var.container = {
            selector: container,
            d3: d3.select(container),
            el: ((typeof container === 'string' || container instanceof String) ? container : d3.select(container).node()),
            outerWrapper: d3.select(container).closest('.gViz-outer-wrapper'),
            outerWrapperClientRect: d3.select(container).closest('.gViz-outer-wrapper').node().getBoundingClientRect(),
            dimensions: {}
          };

          // Store data
          _var.data = data;

          // Get bar width from the attrs
          _var.barWidth = _var.data.attrs != null && _var.data.attrs.barWidth != null && !isNaN(+_var.data.attrs.barWidth) ? +_var.data.attrs.barWidth : 10;

          // Map data and get labels
          _var.nodes = {};
          if(_var.data.data != null) { _var.data.data.forEach(function (d) { _var.nodes[d.x] = d; }); }

          break;
      }
    }

    return _var;
  };

  // Expose global variables
  ['_id', '_var', 'animation', 'container', 'colors', 'data', 'hover', 'margin','zoom','onClick','onHover','priceFunction','xInDomain'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};

// Initialize the visualization class
gViz.viz.barChart.misc = function() {
  "use strict";

  // Get attributes values
  var _var       = undefined;
  var action     = "start";
  var components = {};

  // Validate attributes
  var validate = function(step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function(step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          switch (action) {

            // First action to be done
            case 'start':

              /* -- Visualization Title -- */
              // Has title flag
              var hasTitle = _var.data.title != null && _var.data.title !== "";
              _var.container.dimensions.title = hasTitle ? 35 : 0;

              // Draw title wrapper
              var titleWrapper = _var.container.outerWrapper.selectAll(".gviz-title-wrapper").data(["gviz-title-wrapper"]);
              titleWrapper.exit().remove();
              titleWrapper = titleWrapper.enter().insert('div', ":first-child").attr('class', "gviz-title-wrapper").merge(titleWrapper);
              titleWrapper
                .style('width', '100%')
                .style('height', '30px')
                .style('line-height', '30px')
                .style('margin', '0px 0px 5px 0px')
                .style('padding', '0px 10px')
                .style('oveflow', 'hidden')
                .style('white-space', 'nowrap')
                .style('text-overflow', 'ellipsis')
                .style('background-color', '#eee')
                .style('color', '#666')
                .style('font-size', '15px')
                .style('font-weight', 'lighter')
                .style('display', hasTitle ? 'block' : 'none')
                .html(_var.data.title)

              /* -- Axis and Legend -- */
              // Set axis string
              var yTitle = (_var.data.y != null && _var.data.y.title != null && _var.data.y.title !== "" ? "<b>Y - </b>"+_var.data.y.title : "");
              var xTitle = (_var.data.x != null && _var.data.x.title != null && _var.data.x.title !== "" ? "<b>X - </b>"+_var.data.x.title : "");

              // Set axis title
              if(yTitle !== "" && xTitle !== "") { _var.axisTitle = yTitle+" / "+xTitle; }
              else if(yTitle !== "" && xTitle === "") { _var.axisTitle = yTitle; }
              else if(yTitle === "" && xTitle !== "") { _var.axisTitle = xTitle; }
              else { _var.axisTitle = ""; }

              // Iterate over data to get legends for positive and negative values
              var legendDomain = {};
              var legendDomainNeg = {};
              _var.data.data.forEach(function(d) {

                // Add legend
                if(_var.hasWrapper(d.wrap) && d.name != null && d.name !== "") {
                  if(+d.y >= 0) { legendDomain[d.x] = d; }
                  else { legendDomainNeg[d.x] = d; }
                }

                // Iterate over values
                d.values.forEach(function(v) {

                  // Add legend
                  if(+v.y >= 0) { legendDomain[v.x] = v; }
                  else { legendDomainNeg[v.x] = v; }

                });
              });

              // Initialize string
              var string = _var.axisTitle === "" ? "" : "<span class='axis-title'>" + _var.axisTitle + "</span>";
              var stringObj = {};

              // Iterate over legend domains
              var legendDomain = Object.keys(legendDomain).map(function(k) { return legendDomain[k]; })
              var legendDomainNeg = Object.keys(legendDomainNeg).map(function(k) { return legendDomainNeg[k]; })
              legendDomain.concat(legendDomainNeg).forEach(function(d) {

                // Get color
                var fillColor = _var.data.colors[d.x] == null || _var.data.colors[d.x].fill == null ? "#666" : _var.getColor(d);
                var strokeColor = _var.data.colors[d.x] == null || _var.data.colors[d.x].stroke == null ? "#333" : _var.getColor(d, 'stroke');
                var legend = _var.data.legend != null && _var.data.legend.text != null ? _var.data.legend.text : "{{name}}";
                var legendStr = "";

                // Add rect for obj
                legendStr += "<span class='legend-content' >";
                legendStr += "<span class='rect' style='background-color:"+fillColor+" ; border-top: 2px solid "+strokeColor+";'></span><span class='name'>";
                legendStr += gViz.shared.helpers.text.replaceVariables(legend, d);
                legendStr += "</span>";
                legendStr += "</span>";

                // If the legend str wasnt computed, add to legend
                if(stringObj[legendStr] == null) {
                  stringObj[legendStr] = true;
                  string += legendStr;
                }

              });

              // Set margin left and display style
              var hasLegend = _var.data.legend != null && _var.data.legend.isVisible != null && _var.data.legend.isVisible === true;
              var legendWrapper = _var.container.outerWrapper.selectAll(".gviz-legend-wrapper").data(["gviz-legend-wrapper"]);
              legendWrapper.exit().remove();
              legendWrapper = legendWrapper.enter().append("div").attr('class', "gviz-legend-wrapper").merge(legendWrapper);
              legendWrapper
                .style('width', '100%')
                .style('height', 'auto')
                .style('max-height', '60px')
                .style('line-height', '30px')
                .style('margin', '0px 0px 5px 0px')
                .style('padding', '0px 10px')
                .style('overflow', 'hidden')
                .style('color', '#666')
                .style('font-size', '13px')
                .style('z-index', "1005")
                .style('position', 'absolute')
                .style('top', hasTitle ? "35px" : "0px")
                .style('left', "0px")
                .style('display', hasLegend ? 'block' : 'none')
                .each(function(d) {

                  // Set margin left and display style
                  var legend = d3.select(this).selectAll(".gviz-legend").data(hasLegend ? ["gviz-legend"] : []);
                  legend.exit().remove();
                  legend = legend.enter().insert('div', ":first-child").attr('class', "gviz-legend").merge(legend);
                  legend
                    .html(string);

                });

              legendWrapper
                .on('mouseover', function() { d3.select(this).classed('gviz-legend-hover', true); })
                .on('mouseout', function() { d3.select(this).classed('gviz-legend-hover', false); });

              // Set legend dimension
              _var.container.dimensions.legend = hasLegend ? (legendWrapper.node().getBoundingClientRect().height + 5) : 0;

              // Update container _id, height and client bound rect
              _var.container.d3
                .attr('data-vis-id', _var._id)
                .style('height', (_var.container.outerWrapperClientRect.height - (_var.container.dimensions.title + _var.container.dimensions.legend)) + "px")
                .style('top', (_var.container.dimensions.title + _var.container.dimensions.legend) + 'px')

              // Update grid background position
              _var.container.outerWrapper.selectAll('.grid-background')
                .style('top', (_var.container.dimensions.title + _var.container.dimensions.legend) + 'px')

              // Define height and width
              var containerClientRect = _var.container.d3.node().getBoundingClientRect();
              _var.height = containerClientRect.height - (_var.margin.top + _var.margin.bottom);
              _var.width = containerClientRect.width - (_var.margin.left + _var.margin.right);

              // NO DATA AVAILABLE
              if (_var.data.data == null || _var.data.data.length === 0) {
                _var.container.d3.html("<h5 style='line-height: " + containerClientRect.height + "px; text-align: center;'>NO DATA AVAILABLE</h5>");
              } else {
                _var.container.d3.selectAll("h5").remove();
              }

              break;

            // Last action to be done
            case 'end':

              break;
          }
          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','action','components'].forEach(function(key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};

// Initialize the visualization class
gViz.viz.barChart.style = function() {
  "use strict";

  // Get attributes values
  var _var      = undefined;
  var components = {};

  // Validate attributes
  var validate = function(step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function(step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Is wrapper function
          _var.hasWrapper = function(wrap) {
            return ['sum','median','mean','max','min'].indexOf(wrap) !== -1 || (wrap != null && !isNaN(+wrap));
          }

          // Is wrapper function
          _var.wrapperType = function(wrap) {
            if(['sum','median','mean','max','min'].indexOf(wrap) !== -1) { return 'metric'; }
            else { return 'number'; };
          }

          // Get color function
          _var.getBarColor = function(parent, d, attr="fill") {
            var otherValues = parent.values != null ? parent.values.filter(function(f) { return f.x == d.x && f.id != d.id; }) : [];
            if(otherValues.length === 0) { return _var.getColor(d); }
            else {
              var other = otherValues[0];
              if(d.id === 'forecast' && +d.y > +other.y) { return _var.getColor(d); }
              else if(d.id === 'forecast' && +d.y <= +other.y) { return d.secondaryColor != null && d.secondaryColor !== '' ? d.secondaryColor : _var.getColor(d); }
              else if(d.id !== 'forecast' && +d.y < +other.y) { return _var.getColor(d); }
              else if(d.id !== 'forecast' && +d.y >= +other.y) { return d.secondaryColor != null && d.secondaryColor !== '' ? d.secondaryColor : _var.getColor(d); }
            }
          }

          // Get color function
          _var.getColor = function(d, attr="fill") {
            if(d.color != null && attr != 'stroke') { return d.color; }
            else if(_var.data.colors[d.x] == null) { return attr === 'stroke' ? "#333" : "#999"; }
            else if(+d.y >= 0 && _var.data.colors[d.x][attr] != null) { return _var.data.colors[d.x][attr]; }
            else if(+d.y >= 0 && _var.data.colors[d.x][attr+"-neg"] != null) { return _var.data.colors[d.x][attr+"-neg"]; }
            else if(+d.y < 0  && _var.data.colors[d.x][attr+"-neg"] != null) { return _var.data.colors[d.x][attr+"-neg"]; }
            else if(+d.y < 0  && _var.data.colors[d.x][attr] != null) { return _var.data.colors[d.x][attr]; }
            else { return attr === 'stroke' ? "#333" : "#999"; }
          }

          // Get bar opacity
          _var.getOpacity = function(d) {
            return d.opacity != null && !isNaN(+d.opacity) && +d.opacity > 0 && +d.opacity <= 1 ? +d.opacity : 1;
          }

          // Get stroke dasharray
          _var.getStrokeDasharray = function(d) {
            var height = _var.getHeight(d) + _var.getRadius(d);
            var width = _var.barWidth;
            return parseInt(width) + ' , ' + parseInt(2 * height);
          }


          // Get Y function
          _var.getStrokeY = function(d) {
            if(_var.yBounds[0] >= 0) { return _var.y(+d.y); }
            else if (_var.yBounds[1] < 0) { return _var.y(_var.yBounds[1]); }
            else { return _var.y(+d.y); }
          }

          // Get Height function
          _var.getHeight = function(d) {
            var value = 0;
            if(_var.yBounds[0] >= 0) { value = _var.height - _var.y(+d.y); }
            else if (_var.yBounds[1] < 0) { value = _var.y(+d.y); }
            else { value = +d.y >= 0 ? (_var.y(0) - _var.y(+d.y)) : (_var.y(+d.y) - _var.y(0)); }
            return value < 0 ? 0 : value;
          }

          // Get Y function
          _var.getY = function(d) {
            if(_var.yBounds[0] >= 0) { return _var.y(+d.y); }
            else if (_var.yBounds[1] < 0) { return _var.y(_var.yBounds[1]); }
            else { return +d.y >= 0 ? _var.y(+d.y) : _var.y(0); }
          }

          // Get X function
          _var.getXIn = function(d) {
            return _var.xIn(d.x) + _var.xIn.bandwidth()/2 - _var.barWidth/2;
          }


          // Get path for rounded corners
          _var.getRadius = function(d) {
            var radiusWidth = Math.min(16, _var.barWidth*0.7);
            var value = _var.getHeight(d) < radiusWidth ? _var.getHeight(d)/2 : radiusWidth/2;
            return value < 0 ? 0 : value;
          }

          // Get path for rounded corners
          _var.getPath = function(d) {

            // Get parameters
            var x = _var.getXIn(d);
            var y = _var.getY(d);
            var width = _var.barWidth;
            var height = _var.getHeight(d);
            var radiusWidth = Math.min(16, _var.barWidth*0.7);
            var radius = height < radiusWidth ? height/2 : radiusWidth/2;

            return "M" + x + "," + (y + height)
                 + "v-" + (height - 2 * radius)
                 + "a" + radius + "," + radius + " 0 0 1 " + radius + ", -" + radius
                 + "h" + (width - 2 * radius)
                 + "a" + radius + "," + radius + " 0 0 1 " + radius + "," + radius
                 + "v" + (height - 2 * radius)
                 + "z";
          };

          // Get x padding from scale
          _var.getXPadding = function() {
            return ((_var.calcWidth / Object.keys(_var.xDomain).length) - _var.x.bandwidth());
          }


          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','components'].forEach(function(key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};

// Initialize the visualization class
gViz.viz.barChart.xScale = function () {
  "use strict";

  // Get attributes values
  var _var = undefined;
  var data = [];

  // Validate attributes
  var validate = function (step) {

    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Set format
          _var.xIsDate = (_var.data.x != null && _var.data.x.type === 'time' && _var.data.x.inFormat != null && _var.data.x.outFormat != null);
          _var.xIsNumber = (_var.data.x != null && _var.data.x.type === 'number' && _var.data.x.format != null);
          var xFmt = _var.xIsDate ? 'date' : (_var.xIsNumber ? 'number' : 'text');
          _var.xFormat = gViz.shared.helpers[xFmt].parseFormat(_var.data == null ? null : _var.data.x);

          // Initialize domains
          _var.xDomain = {};
          _var.xInDomain = {};

          // Initialize text values objects for labels resizing
          var textValuesObj = { x: {}, xIn: {} };

          // Get domains
          data.forEach(function(d) {

            // Parse date value
            if(_var.xIsDate) {
              d.parsedName = d3.timeParse(_var.data.x.inFormat)(d.name);
              if(d.parsedName != null) { d.name = _var.xFormat(d.name); }
            }

            // Add id to x domain value
            _var.xDomain[d.x] = d;

            // Store name for futher use
            if(d.name != null && d.name !== "" && textValuesObj.x[d.name] == null) { textValuesObj.x[d.name] = gViz.shared.helpers.text.getSize(d.name); }

            // Iterate over values
            d.values.forEach(function(v) {

              // Set parent id
              v.parent = d.x;

              // Parse date value
              if(_var.xIsDate) {
                v.parsedName = d3.timeParse(_var.data.x.inFormat)(v.name);
                if(v.parsedName != null) { v.name = _var.xFormat(v.name); }
              }

              // Add id to xIn domain value
              _var.xInDomain[v.x] = v;

              // Store name for futher use
              if((d.name == null || d.name === "") && (v.name != null && v.name !== "" && textValuesObj.xIn[v.name] == null)) {
                textValuesObj.xIn[v.name] = gViz.shared.helpers.text.getSize(v.name);
              }

            });
          });

          // Initialize domains
          var xDomain = Object.keys(_var.xDomain), xInDomain = Object.keys(_var.xInDomain);

          // Set xLimits if not set
          _var.xBounds = [xDomain[0], xDomain[xDomain.length-1]];

          // Initialize count of elements
          var elementCount = Object.keys(_var.xInDomain).length, groupCount = Object.keys(_var.xDomain).length;

          // Store old calculed width
          var oldWidth = _var.x == null ? 0 : Math.abs(_var.x.range()[1] - _var.x.range()[0]);

          // Calculate height based on data elements
          _var.calcWidth = groupCount * _var.width;
          _var.calcWidth = _var.width > _var.calcWidth ? _var.width : _var.calcWidth;

          // Calculate ranges
          var minRange = _var.x == null ? 0 : _var.x.range()[0];
          var maxRange = _var.x == null ? _var.calcWidth : _var.x.range()[1];

          if(_var.x != null) {
            if(_var.zoom.direction === 'prev') { _var.zoom.transform.x -= _var.x.bandwidth(); }
            else if(_var.zoom.direction === 'next') {
              _var.zoom.transform.x += _var.x.bandwidth();
            }
          }

          // Set scales range and padding
          _var.x = d3.scaleBand().range([minRange, maxRange]).paddingInner(0).paddingOuter(0);
          _var.xIn = d3.scaleBand().paddingInner(0.26).paddingOuter(0.13);

          // Set x and xIn domain
          _var.x.domain(xDomain);
          _var.xIn.domain(xInDomain).range([0,_var.x.bandwidth()]);

          // Update barWidth if its not set by the user
          _var.barWidth = _var.data.attrs != null && _var.data.attrs.barWidth != null && !isNaN(+_var.data.attrs.barWidth) ? +_var.data.attrs.barWidth : _var.xIn.bandwidth();
          _var.barWidth = Math.min(_var.barWidth, 25);

          // Define axis
          _var.xAxis = d3.axisBottom(_var.x).tickPadding(17);

          // Set old last value
          _var.oldLastValue = _var.xBounds[1];

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','data'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};

// Initialize the visualization class
gViz.viz.barChart.yScaleSize = function () {
  "use strict";

  // Get attributes values
  var _var = undefined;
  var data = [];

  // Validate attributes
  var validate = function (step) {

    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Create scale
          _var.y = d3.scaleLinear().range([_var.height, _var.margin.top]);
          _var.y.domain(_var.yBounds).nice();

          // Set format
          _var.yFormat = gViz.shared.helpers.number.parseFormat(_var.data == null ? null : _var.data.y);

          // Get y axis ticks
          var bins = d3.max([3, parseInt(_var.height / 40, 10)]);

          // Define y axis
          _var.yAxis = d3.axisLeft(_var.y).ticks(bins).tickPadding(10).tickFormat(_var.yFormat);

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','data'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};

// Initialize the visualization class
gViz.viz.barChart.yScale = function () {
  "use strict";

  // Get attributes values
  var _var = undefined;
  var data = [];

  // Validate attributes
  var validate = function (step) {

    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Define aux variables
          var min = null,
              max = null,
              diff = null;

          // Get bounds
          data.forEach(function(d) {

            // Get wrap value, if exists, and update the domain
            var wrapValue = null;
            if(_var.hasWrapper(d.wrap)) {
              if(_var.wrapperType(d.wrap) === 'metric') { d.y = wrapValue = d3[d.wrap](d.values, function(v) { return +v.y; }); }
              else { d.y = wrapValue = +d.wrap; }
            }

            // Update domain
            if(wrapValue != null && (min == null || min > +wrapValue)) { min = +wrapValue; }
            if(wrapValue != null && (max == null || max < +wrapValue)) { max = +wrapValue; }

            // Set domain from xIn values
            d.values.forEach(function(v) {
              if(min == null || min > +v.y) { min = +v.y; }
              if(max == null || max < +v.y) { max = +v.y; }
            });
          });

          // Get axis target
          if(_var.data.y != null && _var.data.y.target != null && !isNaN(+_var.data.y.target)) {
            _var.yTarget = +_var.data.y.target;
            if(min == null || min > +_var.data.y.target) { min = +_var.data.y.target; }
            if(max == null || max < +_var.data.y.target) { max = +_var.data.y.target; }
          }

          // Check for default values
          if(isNaN(min)) { min = 0; }
          if(isNaN(max)) { max = 1; }

          // Get diff
          var diff = Math.abs(max - min) === 0 ? Math.abs(max * 0.1) : 0;

          // Get axis equal
          if(_var.data.y != null && _var.data.y.equal != null && _var.data.y.equal === true && max > 0 && min < 0) {
            if(Math.abs(max) > Math.abs(min)) { min = -max; }
            else { max = Math.abs(min); }
          }

          // Set y domain
          _var.yBounds = [(min == 0 ? min : min - diff), max + diff];

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','data'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};

// Initialize the visualization class
gViz.viz.barChart.zoom = function () {
  "use strict";

  // Get attributes values
  var _var = null;
  var components = {};

  // Validate attributes
  var validate = function (step) {

    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Zoom functions
          _var.zoom.callback = function(){

            if (d3.event != null && d3.event.transform != null) {

              // Store event
              var event = d3.event;

              // // Get bounds and domain
              // var _bounds = [_var.x(_var.xBounds[0]), _var.x(_var.xBounds[1])];

              // var min = null, max = null;
              // _var.data.data.forEach(function(d) {
              //   var xStart = _var.x(d.x);
              //   var xEnd = _var.x(d.x) + _var.x.bandwidth();
              //   var breakXStart = -d3.event.transform.x;
              //   var breakXEnd = -d3.event.transform.x + _var.width;
              //   if((xStart >= breakXStart && xStart < breakXEnd) || (xStart < breakXStart && xEnd > breakXStart)) {
              //     d.values.forEach(function(v) {
              //       var xInStart = xStart + _var.xIn(v.x);
              //       var xInEnd   = xStart + _var.xIn(v.x) + _var.xIn.bandwidth();
              //       if((xInStart >= breakXStart && xInStart < breakXEnd) || (xInStart < breakXStart && xInEnd > breakXStart)) {
              //         if(min == null || min > +v.y) { min = v.y; }
              //         if(max == null || max < +v.y) { max = v.y; }
              //       }
              //     });
              //   }
              // });

              // _var.y.domain([min,max]);

              // Set event transformation only vertically without zoom
              d3.event.transform.k = 1;
              d3.event.transform.y = _var.margin.top;

              // If there is new data to fetch and this function is enable ('next')
              if(d3.event.transform.x < -_var.calcWidth + _var.width + _var.margin.left && _var.zoom.enable) {

                // Bound zoom to not go over the right side
                d3.event.transform.x = -_var.x.range()[1] + _var.width;

                // Disable fetch new data
                _var.zoom.enable = false;

                // Disable zooming
                _var.zoom.behaviour.on("zoom", null);

                // Get point
                var t = _var.data.data[_var.data.data.length - 1];

                // Set direction to zoom
                _var.zoom.direction = 'next';

                // Run zoom function
                _var.zoom.fn(t, "next");

              // If there is new data to fetch and this function is enable ('prev')
              } else if(d3.event.transform.x > _var.x.range()[0] && _var.zoom.enable) {

                // Bound zoom to not go over the left side
                d3.event.transform.x = _var.x.range()[0];

                // Disable fetch new data
                _var.zoom.enable = false;

                // Disable zooming
                _var.zoom.behaviour.on("zoom", null);

                // Get point
                var _t = _var.data.data[0];

                // Set direction to zoom
                _var.zoom.direction = 'prev';

                // Run zoom function
                _var.zoom.fn(_t, "prev");

              }

              // Update bgClipRect
              _var.defs.selectAll(".bg-clip .bg-rect, .bars-bg-clip .bg-rect").attr('x', -d3.event.transform.x + _var.margin.left);

              // Transform outer g
              _var.g.attr("transform", d3.event.transform)
              _var.gBars
                .attr("transform", d3.event.transform)
                .selectAll(".bar-rect")
                  .attr('rx', _var.getRadius)
                  .attr('ry', _var.getRadius)
                  .transition().duration(600)
                    .attr('y', _var.getY)
                    .attr('height', function(d) { return _var.getHeight(d) + _var.getRadius(d); })

              // Set zoom transform
              _var.zoom.transform = d3.event.transform;

              // Update x axis values and ticks
              _var.updateXAxis();

              // Reset hovered node
              _var.hovered = null;

              // Mouseout event
              components.events()
                ._var(_var)
                .action("mouseout")
                .components(components)
                .run();

            }
          }

          // Set zoom transform initialy
          _var.zoom.transform = {
            x: _var.zoom.transform == null ? -((_var.x(_var.zoom.startPoint != null ? _var.zoom.startPoint : _var.xBounds[1]) + _var.x.bandwidth() + _var.getXPadding()) - _var.width) : _var.zoom.transform.x,
            y: _var.zoom.transform == null ? _var.margin.top : _var.zoom.transform.y,
            k: _var.zoom.transform == null ? 1 : _var.zoom.transform.k,
          }

          // Add zoom capabilities
          _var.zoom.behaviour = d3.zoom()
            .scaleExtent([1,1])
            .on("zoom", _var.zoom.callback)
            .on("start", function() { _var.wrap.classed('grabbing', true) })
            .on("end",   function() { _var.wrap.classed('grabbing', false) });

          // Bind zoom to svg
          _var.wrap
            .call(_var.zoom.behaviour)
            .call(_var.zoom.behaviour.transform, d3.zoomIdentity.translate(_var.zoom.transform.x, _var.zoom.transform.y).scale(_var.zoom.transform.k))
            .on("wheel.zoom", null)

          // Bind touch events to bar and trigger onClick attribute function
          var event, endEvent, moved = false;
          _var.wrap
            .selectAll(".bar-rect")
            .on("touchstart", function(e) {
              event = setTimeout(function() {
                if(_var.onClick != null && typeof _var.onClick === "function") { _var.onClick(e); }
              }, 500);
            }).on("touchmove", function(e) {
              clearTimeout(event);
            }).on("touchend", function(e) {
              clearTimeout(event);
            });

         break;
     }
   }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','components'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) { return main('run'); };

  return main;
};

// Initialize the helpers class¬
gViz.shared.data.api = {

  // Get data from API
  resolvedFetch: function(data) {
    return $.Deferred().resolve(data).promise();
  },

  // Get data from API
  fetch: function(appliance, startDate, endDate, precision, timeout, testAppliance, apiKey, registrationToken, endPointing) {

    var defer = $.Deferred();

    // console.log("API Dates: ", new Date(parseInt(`${startDate}000`)), new Date(parseInt(`${endDate}000`)));

    $.ajax({
      url: endPointing+"?start_date="+startDate+"&appliance="+appliance+"&end_date="+endDate+"&precision="+precision,
      beforeSend: function(xhr) {
        xhr.setRequestHeader("X-API-Key", apiKey);
        xhr.setRequestHeader("Authorization", registrationToken.apply());
      },
      type: 'GET',
      dataType: 'json',
      contentType: 'application/json',
      processData: false,
      timeout: timeout == null ? 15000 : timeout,
      success: function (data) {
        if(testAppliance != null) { data.appliance = testAppliance; }
        defer.resolve(data);
      },
      error: function(req, status, err) {
        console.log(err);
        defer.reject(err);
      }
    });

    return defer.promise();
  }

};

var getDataOrCache = function(sDay, eDay, startDate, endDate, appliance, apiKey, registrationToken, endPointing, isForecast) {

  // Parse
  var start = new Date(sDay.getTime());
  var end = new Date(eDay.getTime());

  if(appliance !== 'forecast') {
    start = new Date() < start ? new Date() : start;
    end = new Date() < end ? new Date() : end;
  }

  // Get parameters
  var cParam = {
    start: start.toISOString().split(":")[0] + ":00:00.000Z",
    end: end.toISOString().split(":")[0] + ":00:00.000Z",
    range: parseInt((end - start) / 3600000)
  }

  // Get initial and final data cached position
  cParam.cacheStart = gViz.shared.data.barChart.cached[appliance].findIndex(function(d) { return d.x == cParam.start; });
  cParam.cacheEnd   = gViz.shared.data.barChart.cached[appliance].findIndex(function(d) { return d.x == cParam.end; });

  if(cParam.cacheStart !== -1 && cParam.cacheEnd !== -1 && cParam.cacheEnd - cParam.cacheStart >= cParam.range) {

    // Get appliance data
    var aData = { appliance: appliance, consumption: { w: [], t: [] }, isCache: true };
    gViz.shared.data.barChart.cached[appliance].slice(cParam.cacheStart, cParam.cacheEnd+1).forEach(function(d) {
      aData.consumption.w.push(d.value);
      var date = `${d.date.getTime()}`;
      aData.consumption.t.push(date.substring(0,date.length-3));
    });;
    return window.gViz.shared.data.api.resolvedFetch(aData);

  } else {

    // Fetch data from api :: actual = "all" appliance
    return window.gViz.shared.data.api.fetch(appliance, startDate.substring(0,startDate.length-3), endDate.substring(0,endDate.length-3), "3600", 100000, null, apiKey, registrationToken, endPointing);

  }

}

// Store data in cache
var cacheData = function(rData) {

  // Apply cache only if the data wasn't fetched from cache
  if(rData.isCache == null || rData.isCache !== true) {

    // Cache data for future uses
    var keys = {};
    rData.consumption.t.forEach(function(d, i) {
      gViz.shared.data.barChart.cached[rData.appliance].push({
        x: (new Date(parseInt(`${d}000`))).toISOString(),
        date: new Date(parseInt(`${d}000`)),
        value: rData.consumption.w[i]
      });
    });

    // Unique keys
    var newCacheData = [], storedKeys = {};
    gViz.shared.data.barChart.cached[rData.appliance].reverse().forEach(function(d) {
      if(storedKeys[d.x] == null) {
        storedKeys[d.x] = true;
        newCacheData.push(d);
      }
    });

    // Sort new cache array and store data
    gViz.shared.data.barChart.cached[rData.appliance] = newCacheData.reverse().sort(function(a,b) { return d3.ascending(a.date.getTime(), b.date.getTime()); });

  }

}

// Get data considering interval, date point and direction
var getData = function(interval, x, direction, appliance, isForecast, apiKey, registrationToken, endPointing) {

  // Initialize data cache for appliance and forecast
  gViz.shared.data.barChart.cached[appliance] = gViz.shared.data.barChart.cached[appliance] || [];
  gViz.shared.data.barChart.cached["forecast"] = gViz.shared.data.barChart.cached["forecast"] || [];

  // Get interval
  if(interval == null) { interval = 'day'; }

  // Get data depending on cache
  var data = [];

  // Get other parameters
  if(direction == null) { direction = 'both'; }
  if(appliance == null) { appliance = 'all'; }
  if(isForecast == null) { isForecast = true; }

  // Get x
  if(x == null) { x = 'Today'; }
  else { x = d3.timeParse(interval === 'year' ? "%Y" : "%Y-%m-%d")(x); }

  switch(interval) {

    case 'week':

      // Set is now
      var lastMonday, sDay, eDay, startDate, endDate, fStartDate, fEndDate, sDate, eDate;

      // Set date format
      var dateOrder = gViz.shared.helpers.date.translated.DATEORDER['7DAYS'];

      // Get time for today
      if(x === 'Today' || direction === 'both') {

        sDate = x === 'Today' ? new Date() : new Date(x);
        eDate = x === 'Today' ? new Date() : new Date(x);

        // Get start date for actual and forecasted values
        sDay = gViz.shared.helpers.date.getLastMonday(sDate).addDays(-7);
        startDate =`${sDay.setHours(0,0,0,000)}`;
        fStartDate =`${sDay.setHours(0,0,0,000)}`;

        // Get end date for actual and forecasted values
        eDay = gViz.shared.helpers.date.getLastMonday(eDate).addDays(13);
        fEndDate = `${eDay.setHours(23,59,59,000)}`;
        endDate = `${eDate < new Date() ? eDay.getTime() : (new Date()).getTime()}`;

      // Get time for other periods
      } else {

        // Get start date for actual and forecasted values
        sDay = gViz.shared.helpers.date.getLastMonday((new Date(x)).addDays(direction === 'prev' ? -7 : 7));
        fStartDate = `${sDay.setHours(0,0,0,000)}`;
        startDate = `${sDay.setHours(0,0,0,000)}`;

        // Get end date for actual and forecasted values
        eDay = gViz.shared.helpers.date.getLastMonday((new Date(x)).addDays(direction === 'prev' ? -7 : 7)).addDays(6);
        fEndDate = `${eDay.setHours(23,59,59,999)}`;
        endDate = `${eDay.setHours(23,59,59,999)}`;

      }

      var parseWeekData = function(fData, aData) {

        // Initialize parents structures
        var parents = {}, itemNames = {}, dateValues = {};

        // Parse new data from api
        [fData, aData].forEach(function(rData) {

          // Store data in cache
          cacheData(rData);

          rData.consumption.t.forEach(function(d, i) {

            // Get date from data
            var days = d3.timeFormat("%a")(new Date(parseInt(`${d}000`)));
            var date = d3.timeFormat("%Y-%m-%d")(new Date(parseInt(`${d}000`)));
            var index = d3.timeFormat("%u")(new Date(parseInt(`${d}000`)));
            var weekStart = new Date(parseInt(`${d}000`)).addDays(1 - index);
            var isCurrentYear = weekStart.getFullYear() === (new Date()).getFullYear();

            // Fill dateValues with month date and aggregated values
            dateValues[rData.appliance] = dateValues[rData.appliance] || {};
            if(dateValues[rData.appliance][date] == null) { dateValues[rData.appliance][date] = { value: 0, weekStart: weekStart, days: days, index: index }; }
            dateValues[rData.appliance][date].value += +rData.consumption.w[i];

            // Initialize parent structure for date if not initialized
            if(parents[d3.timeFormat("%Y-%m-%d")(weekStart)] == null) {
              parents[d3.timeFormat("%Y-%m-%d")(weekStart)] = {
                "id": d3.timeFormat("%Y-%m-%d")(weekStart),
                "x": d3.timeFormat("%Y-%m-%d")(weekStart),
                "name": d3.timeFormat(dateOrder === 'default' ? ("%b %d" + (isCurrentYear ? "" : ", %Y")) : dateOrder)(weekStart),
                "values": [
                ]
              };
            }

            // Store item and parent item names
            if(itemNames[d3.timeFormat("%Y-%m-%d")(weekStart)] == null) { itemNames[d3.timeFormat("%Y-%m-%d")(weekStart)] = {} };

          });
        });

        Object.keys(dateValues).forEach(function(app) {
          Object.keys(dateValues[app]).sort(function(a,b) { return d3.ascending(d3.timeParse("%Y-%m-%d")(a),d3.timeParse("%Y-%m-%d")(b)); }).forEach(function(k) {

            // Store value on parent week
            if(dateValues[app][k].value !== 0) {
              parents[d3.timeFormat("%Y-%m-%d")(dateValues[app][k].weekStart)].values.push({
                "id": app,
                "x": dateValues[app][k].days,
                "name": dateValues[app][k].days,
                "date": d3.timeParse("%Y-%m-%d")(k),
                "y": dateValues[app][k].value,
                "index": dateValues[app][k].index,
                "color": app === 'forecast' ? "#ddd" : "#2566bc",
                "secondaryColor": app === 'forecast' ? "#233a71" : "rgba(36,121,219,0.7)",
                "opacity": 1
              });

              // Store item names
              itemNames[d3.timeFormat("%Y-%m-%d")(dateValues[app][k].weekStart)][dateValues[app][k].index.toString()] = true;
            }
          });
        });

        // Set missing months to 0 just to fit axis
        Object.keys(itemNames).forEach(function(k) {
          d3.range(7).forEach(function(d, i) {
            var date = d3.timeParse("%Y-%m-%d")(k);
            var index = d3.timeFormat("%u")(date.addDays(i)).toString();
            var days = d3.timeFormat("%a")(date);
            if(itemNames[k][index] == null) {
              parents[k].values.push({
                "id": isForecast ? "forecast" : "actual",
                "x": days.toString(),
                "name": days.toString(),
                "y": 0,
                "index": index,
                "color": "transparent",
                "secondaryColor": "transparent",
                "opacity": 0
              });
            }
          });
        });

        Object.keys(parents).sort(function(a,b) { return d3.ascending(a,b); }).forEach(function(k) {
          parents[k].values = parents[k].values.sort(function(a,b) {
            return d3.ascending(+a.index,+b.index) || d3.descending(a.id,b.id);
          });
          data.push(parents[k]);
        });

        // Store data to be used and also in cache
        this._data = data;

      };

      var req2 = getDataOrCache(sDay, eDay, startDate, endDate, appliance, apiKey, registrationToken, endPointing, isForecast);
      var req  = getDataOrCache(sDay, eDay, fStartDate, fEndDate, "forecast", apiKey, registrationToken, endPointing, isForecast);

      // Add or remove days depending on direction
      if(direction === 'prev') { x.addDays(-1); }
      else if(direction === 'next') { x.addDays(1); }

      return $.when(req, req2).done(parseWeekData);

      break;

    case 'month':

      // Set is now
      var now = new Date(), xDate = new Date(x);
      var sDay, eDay, startDate, endDate, fStartDate, fEndDate, eDate;

      // Set date format
      var dateOrder = gViz.shared.helpers.date.translated.DATEORDER['30DAYS'];

      // Get time for today
      if(x === 'Today' || direction === 'both') {

        xDate = x === 'Today' ? new Date() : new Date(x);

        // Get start date for actual and forecasted values
        sDay = (new Date(xDate.getFullYear(), xDate.getMonth(), 1)).addMonths(-1);
        fStartDate =`${sDay.setHours(0,0,0,000)}`;
        startDate =`${sDay.setHours(0,0,0,000)}`;

        // Get end date for actual and forecasted values
        eDate = (new Date(xDate.getFullYear(), xDate.getMonth(), 1)).addMonths(2).addDays(-1);
        eDay = eDate;
        fEndDate = `${eDay.setHours(23,59,59,000)}`;
        endDate = `${eDate < new Date() ? eDate : (new Date()).getTime()}`;

      // Get time for other periods
      } else {

        // Add or remove months from date, depending on direction
        xDate.addMonths(direction === 'prev' ? -1 : 1);

        // Get start date for actual and forecasted values
        sDay = new Date(xDate.getFullYear(), xDate.getMonth(), 1);
        fStartDate =`${sDay.setHours(0,0,0,000)}`;
        startDate =`${sDay.setHours(0,0,0,000)}`;

        // Get end date for actual and forecasted values
        eDay = new Date(xDate.getFullYear(), xDate.getMonth(), 1).addMonths(1).addDays(-1);
        fEndDate = `${eDay.setHours(23,59,59,000)}`;
        endDate = `${eDay.setHours(23,59,59,000)}`;

        // Fix start and end date not to go over now
        if(new Date(parseInt(startDate)) > new Date) { startDate = `${(new Date()).getTime()}`; }
        if(new Date(parseInt(endDate)) > new Date) { endDate = `${(new Date()).getTime()}`; }

      }

      var parseMonthData = function(fData, aData) {

        // Initialize parents structures
        var parents = {}, itemNames = {}, dateValues = {};

        // Parse new data from api
        [fData, aData].forEach(function(rData) {

          // Store data in cache
          cacheData(rData);

          rData.consumption.t.forEach(function(d, i) {

            // Get date from data
            var days = d3.timeFormat("%e")(new Date(parseInt(`${d}000`))).replace(' ','');
            var date = d3.timeFormat("%Y-%m-%d")(new Date(parseInt(`${d}000`)));
            var monthStart = new Date((new Date(parseInt(`${d}000`))).setDate(1));
            var isCurrentYear = monthStart.getFullYear() === (new Date()).getFullYear();

            // Fill dateValues with month date and aggregated values
            dateValues[rData.appliance] = dateValues[rData.appliance] || {};
            if(dateValues[rData.appliance][date] == null) { dateValues[rData.appliance][date] = { value: 0, monthStart: monthStart, days: days, index: parseInt(days) }; }
            dateValues[rData.appliance][date].value += +rData.consumption.w[i];

            // Initialize parent structure for date if not initialized
            if(parents[d3.timeFormat("%Y-%m-%d")(monthStart)] == null) {
              parents[d3.timeFormat("%Y-%m-%d")(monthStart)] = {
                "id": d3.timeFormat("%Y-%m-%d")(monthStart),
                "dataType": 'month',
                "x": d3.timeFormat("%Y-%m-%d")(monthStart),
                "name": d3.timeFormat(dateOrder === 'default' ? ("%b %d" + (isCurrentYear ? "" : ", %Y")) : dateOrder)(monthStart),
                "values": [
                ]
              };
            }

            // Store parent item names
            if(itemNames[d3.timeFormat("%Y-%m-%d")(monthStart)] == null) { itemNames[d3.timeFormat("%Y-%m-%d")(monthStart)] = {} };

          });
        });

        // Parse and store values
        Object.keys(dateValues).forEach(function(app) {
          Object.keys(dateValues[app]).sort(function(a,b) { return d3.ascending(d3.timeParse("%Y-%m-%d")(a),d3.timeParse("%Y-%m-%d")(b)); }).forEach(function(k) {

            // Store value on parent month
            if(dateValues[app][k].value !== 0) {
              parents[d3.timeFormat("%Y-%m-%d")(dateValues[app][k].monthStart)].values.push({
                "id": app,
                "x": dateValues[app][k].days,
                "name": dateValues[app][k].days,
                "date": d3.timeParse("%Y-%m-%d")(k),
                "y": dateValues[app][k].value,
                "index": dateValues[app][k].index,
                "color": app === 'forecast' ? "#ddd" : "#2566bc",
                "secondaryColor": app === 'forecast' ? "#233a71" : "rgba(36,121,219,0.7)",
                "opacity": 1
              });

              // Store item names
              itemNames[d3.timeFormat("%Y-%m-%d")(dateValues[app][k].monthStart)][dateValues[app][k].days.toString()] = true;
            }

          });
        });

        // Set missing months to 0 just to fit axis
        Object.keys(itemNames).forEach(function(k) {
          d3.range(31).forEach(function(d, i) {
            if(itemNames[k][(i+1).toString()] == null) {
              parents[k].values.push({
                "id": isForecast ? "forecast" : "actual",
                "x": (i+1).toString(),
                "name": (i+1).toString(),
                "y": 0,
                "index": i+1,
                "color": "transparent",
                "secondaryColor": "transparent",
                "opacity": 0
              });
            }
          });
        });

        Object.keys(parents).sort(function(a,b) { return d3.ascending(a,b); }).forEach(function(k) {
          parents[k].values = parents[k].values.sort(function(a,b) {
            return d3.ascending(+a.index,+b.index) || d3.descending(a.id,b.id);
          });
          data.push(parents[k]);
        });

        // Store data to be used and also in cache
        this._data = data;

      }

      // Add or remove days depending on direction
      if(direction === 'prev') { x.addDays(-1); }
      else if(direction === 'next') { x.addDays(1); }

      var req2 = getDataOrCache(sDay, eDay, startDate, endDate, appliance, apiKey, registrationToken, endPointing, isForecast);
      var req  = getDataOrCache(sDay, eDay, fStartDate, fEndDate, "forecast", apiKey, registrationToken, endPointing, isForecast);

      return $.when(req, req2).done(parseMonthData);

      break;

    case 'year':

      // Set is now
      var now = new Date(), xDate = new Date(x);
      var sDay, eDay, startDate, endDate, fStartDate, fEndDate, months = [];

      // Set date format
      var dateOrder = gViz.shared.helpers.date.translated.DATEORDER['THISYEAR'];

      // Get year
      var year = x === 'Today' ? now.getFullYear() : (xDate.getFullYear() + (direction === 'prev' ? - 1 : 1));

      sDay = new Date(year + (x === 'Today' ? -1 : 0), 0, 1);
      eDay = new Date(year + (x === 'Today' ? 1 : 0), 11, 31);
      fsDay = new Date(year + (x === 'Today' ? -1 : 0), 0, 1);
      feDay = new Date(year + (x === 'Today' ? 1 : 0), 11, 31);
      fStartDate = `${fsDay.setHours(0,0,0,0)}`;
      fEndDate = `${feDay.setHours(23,59,59,59)}`;
      startDate = `${sDay.setHours(0,0,0,0)}`;
      endDate = `${eDay.setHours(23,59,59,59)}`;

      var parseYearData = function(fData, aData) {

        // Initialize parents structures
        var parents = {}, itemNames = {}, monthValues = {};

        // Parse new data from api
        [aData, fData].forEach(function(rData) {

          // Store data in cache
          cacheData(rData);

          rData.consumption.t.forEach(function(d, i) {

            // Get date from data
            var date = new Date((new Date(parseInt(`${d}000`))).setDate(1));
            var months = d3.timeFormat("%b")(date);
            var isCurrentYear = date.getFullYear() === (new Date()).getFullYear();
            var year = date.getFullYear();

            // Only add the element if its on the searched year
            if(`${date.getFullYear()}` === `${year}`) {

              // Fill monthValues with month date and aggregated values
              monthValues[rData.appliance] = monthValues[rData.appliance] || {};
              monthValues[rData.appliance][year] = monthValues[rData.appliance][year] || {};
              if(monthValues[rData.appliance][year][months] == null) { monthValues[rData.appliance][year][months] = { value: 0, date: date, year: year }; }
              monthValues[rData.appliance][year][months].value += +rData.consumption.w[i];

              // Initialize parent structure for date if not initialized
              if(parents[d3.timeFormat("%Y")(date)] == null) {
                parents[d3.timeFormat("%Y")(date)] = {
                  "id": d3.timeFormat("%Y")(date),
                  "x": d3.timeFormat("%Y")(date),
                  "name": d3.timeFormat(dateOrder === 'default' ? "%Y" : dateOrder)(date),
                  "values": [
                  ]
                };
              }

              // Store parent item names
              if(itemNames[d3.timeFormat("%Y")(date)] == null) { itemNames[d3.timeFormat("%Y")(date)] = {}; }

            }
          });
        });

        // Push all month values to the year parent
        Object.keys(monthValues).forEach(function(rApp) {
          Object.keys(monthValues[rApp]).forEach(function(year) {
            Object.keys(monthValues[rApp][year]).forEach(function(months) {

              // Get attributes
              var date = monthValues[rApp][year][months].date;
              var value = monthValues[rApp][year][months].value;

              // Push month object
              if(value !== 0) {
                parents[d3.timeFormat("%Y")(date)].values.push({
                  "id": rApp,
                  "x": months,
                  "date": date,
                  "name": months,
                  "index": d3.timeFormat("%m")(date),
                  "y": value,
                  "color": rApp === 'forecast' ? "#ddd" : "#2566bc",
                  "secondaryColor": rApp === 'forecast' ? "#233a71" : "rgba(36,121,219,0.7)",
                  "opacity": 1
                });

                // Store item names
                itemNames[year][d3.timeFormat("%Y-%m-%d")(date)] = true;
              }
            });
          });
        });

        // Set missing months to 0 just to fit axis
        Object.keys(itemNames).forEach(function(k) {
          d3.range(12).forEach(function(d, i) {
            var date = new Date(parseInt(k), i, 1);
            var months = d3.timeFormat("%b")(date);
            if(itemNames[k][d3.timeFormat("%Y-%m-%d")(date)] == null) {
              parents[d3.timeFormat("%Y")(date)].values.push({
                "id": isForecast ? "forecast" : "actual",
                "x": months,
                "date": date,
                "name": months,
                "index": parseInt(d3.timeFormat("%m")(date)),
                "y": 0,
                "color": "transparent",
                "secondaryColor": "transparent",
                "opacity": 0
              });

            }
          });
        });

        Object.keys(parents).sort(function(a,b) { return d3.ascending(a,b); }).forEach(function(k) {
          parents[k].values = parents[k].values.sort(function(a,b) {
            return d3.ascending(+a.index,+b.index) || d3.descending(a.id,b.id);
          });
          data.push(parents[k]);
        });

        // Store data to be used
        this._data = data;

      }

      var req2 = getDataOrCache(sDay, eDay, startDate, endDate, appliance, apiKey, registrationToken, endPointing, isForecast);
      var req  = getDataOrCache(fsDay, feDay, fStartDate, fEndDate, "forecast", apiKey, registrationToken, endPointing, isForecast);

      return $.when(req, req2).done(parseYearData);

      break;

    default:

      // Set is now
      var now = new Date();
      var sDay, eDay, startDate, endDate, fStartDate, fEndDate, sDate, eDate;

      // Set date format
      var dateOrder = gViz.shared.helpers.date.translated.DATEORDER['TODAY'];

      // Get time for today
      if(x === 'Today' || direction === 'both') {

        sDate = x === 'Today' ? new Date() : new Date(x);
        eDate = x === 'Today' ? new Date() : new Date(x);

        // Get start date for actual and forecasted values
        sDay = (sDate).addDays(-1);
        startDate = `${sDay.setHours(0,0,0,0)}`;
        fStartDate = `${sDay.setHours(0,0,0,0)}`;

        // Get end date for actual and forecasted values
        eDay = (eDate).addDays(1);
        fEndDate = `${eDay.setHours(23,59,59,59)}`;
        endDate = `${eDay < new Date() ? eDay : (new Date()).getTime()}`;

      // Get time for other periods
      } else {

        // Get start date for actual and forecasted values
        sDay = (new Date(x)).addDays(direction === 'prev' ? -1 : 1);
        startDate = `${sDay.setHours(0,0,0,0)}`;
        fStartDate = `${sDay.setHours(0,0,0,0)}`;

        // Get end date for actual and forecasted values
        eDay = (new Date(x)).addDays(direction === 'prev' ? -1 : 1);
        fEndDate = `${eDay.setHours(23,59,59,59)}`;
        endDate = `${(direction === 'prev' ? eDay.setHours(23,59,59,59) : eDay.setHours(now.getHours(), now.getMinutes(), now.getSeconds(), now.getMilliseconds()))}`;

      }

      var parseTodayData = function(fData, aData) {

        // Store data in cache
        cacheData(fData);
        cacheData(aData);

        // Initialize parents structures
        var parents = {}, itemNames = {};

        // Parse new data from api
        fData.consumption.t.forEach(function(d, i) {

          // Get date from data
          var hours = (new Date(parseInt(`${d}000`))).getHours() + 1;
          var date = (new Date(parseInt(`${d}000`)));
          var isCurrentYear = date.getFullYear() === (new Date()).getFullYear();

          // Initialize parent structure for date if not initialized
          if(parents[d3.timeFormat("%Y-%m-%d")(date)] == null) {
            parents[d3.timeFormat("%Y-%m-%d")(date)] = {
              "id": d3.timeFormat("%Y-%m-%d")(date),
              "x": d3.timeFormat("%Y-%m-%d")(date),
              "name": d3.timeFormat(dateOrder === 'default' ? ("%A, %b %d" + (isCurrentYear ? "" : ", %Y")) : dateOrder)(date),
              "values": [
              ]
            };
          }
          if(itemNames[d3.timeFormat("%Y-%m-%d")(date)] == null) { itemNames[d3.timeFormat("%Y-%m-%d")(date)] = {} };

          if(fData.consumption.w[i] !== 0) {
            parents[d3.timeFormat("%Y-%m-%d")(date)].values.push({
              "id": "forecast",
              "x": hours === 0 ? 24 : hours,
              "name": hours === 0 ? 24 : hours,
              "date": date,
              "y": fData.consumption.w[i],
              "index": hours === 0 ? 24 : hours,
              "color": "#ddd",
              "secondaryColor": "#233a71",
              "opacity": 1
            });

            // Store item and parent item names
            itemNames[d3.timeFormat("%Y-%m-%d")(date)][(hours === 0 ? 24 : hours).toString()] = true;
          }
        });

        // Parse new data from api
        aData.consumption.t.forEach(function(d, i) {

          // Get date from data
          var hours = (new Date(parseInt(`${d}000`))).getHours() + 1;
          var date = (new Date(parseInt(`${d}000`)));
          var isCurrentYear = date.getFullYear() === (new Date()).getFullYear();

          // Initialize parent structure for date if not initialized
          if(parents[d3.timeFormat("%Y-%m-%d")(date)] == null) {
            parents[d3.timeFormat("%Y-%m-%d")(date)] = {
              "id": d3.timeFormat("%Y-%m-%d")(date),
              "x": d3.timeFormat("%Y-%m-%d")(date),
              "name": d3.timeFormat(dateOrder === 'default' ? ("%A, %b %d" + (isCurrentYear ? "" : ", %Y")) : dateOrder)(date),
              "values": [
              ]
            };
          }
          if(itemNames[d3.timeFormat("%Y-%m-%d")(date)] == null) { itemNames[d3.timeFormat("%Y-%m-%d")(date)] = {} };

          if(aData.consumption.w[i] !== 0) {
            parents[d3.timeFormat("%Y-%m-%d")(date)].values.push({
              "id": "actual",
              "x": hours === 0 ? 24 : hours,
              "name": hours === 0 ? 24 : hours,
              "date": date,
              "y": aData.consumption.w[i],
              "index": hours === 0 ? 24 : hours,
              "color": "#2566bc",
              "secondaryColor": "rgba(36,121,219,0.7)",
              "opacity": 1
            });

            // Store item and parent item names
            itemNames[d3.timeFormat("%Y-%m-%d")(date)][(hours === 0 ? 24 : hours).toString()] = true;
          }
        });

        // Set missing months to 0 just to fit axis
        Object.keys(itemNames).forEach(function(k) {
          d3.range(23).forEach(function(d, i) {
            if(itemNames[k][(i+1).toString()] == null) {
              parents[k].values.push({
                "id": isForecast ? "forecast" : "actual",
                "x": (i+1).toString(),
                "name": (i+1).toString(),
                "y": 0,
                "index": i,
                "color": "transparent",
                "secondaryColor": "transparent",
                "opacity": 0
              });
            }
          });
        });

        Object.keys(parents).sort(function(a,b) { return d3.ascending(a,b); }).forEach(function(k) {
          parents[k].values = parents[k].values.sort(function(a,b) {
            return d3.ascending(a.index,b.index) || d3.descending(a.id,b.id);
          });
          data.push(parents[k]);
        });

        // Store data to be used and also in cache
        this._data = data;

      }

      var req2 = getDataOrCache(sDay, eDay, startDate, endDate, appliance, apiKey, registrationToken, endPointing, isForecast);
      var req  = getDataOrCache(sDay, eDay, fStartDate, fEndDate, "forecast", apiKey, registrationToken, endPointing, isForecast);

      // Add or remove days depending on direction
      if(direction === 'prev') { x.addDays(-1); }
      else if(direction === 'next') { x.addDays(1); }

      return $.when(req, req2).done(parseTodayData);

      break;

  }

}

// Initialize the helpers class¬
gViz.shared.data.barChart = {

  // Store cached data for initial values
  cached: {},
  cachedKeys: {},

  // Get data
  fetch: function(interval, x, direction, appliance, isForecast, apiKey, registrationToken, endPointing) {
    return getData(interval, x, direction, appliance, isForecast, apiKey, registrationToken, endPointing);
  },

  // Replace text variables
  format: function() {

    return {
      "title": "",
      "legend": {
        "isVisible": false,
      },
      "tooltip": {
        "body": [ "<span class='metric'>{{y}}</span>" ]
      },
      "y": {
        "title": "",
        "prefix": "",
        "sufix": "Wh",
        "format": "",
        "min": 0
      },
      "x": {
        "title": "",
        "type": "text",
        "prefix": "",
        "sufix": "",
        "format": ""
      },
      "attrs": {
        "barWidth": 25
      },
      "colors": {
        "forecast": { "fill": "#ddd", "gradient": "#ddd", "stroke": "#ddd" },
        "actual": { "fill": "#3a6cc8", "gradient": "#3a6cc8", "stroke": "#3a6cc8" }
      },
      "data": []
    };
  },

  // Replace text variables
  getData: getData,

  // Set unit
  setUnit: function(data, unit) {
    if(unit != null) { data.y.sufix = unit; }
  }

}



var bubbleFakeData = [
  {
    "id": 'radiator',
    "name": 'Radiator',
    "value": 50,
    "img": "/img/event/HeatenB-01.png"
  },
  {
    "id": 'kitchen',
    "name": 'Kitchen',
    "value": 30,
    "img": "/img/event/Kitchenb-01.png"
  },
  {
    "id": 'ironing',
    "name": 'Ironing',
    "value": 10,
    "img": "/img/event/4.png"
  },
  {
    "id": 'freezer',
    "name": 'Freezer',
    "value": 60,
    "img": "/img/event/FridgeB-01.png"
  },
  {
    "id": 'loundry',
    "name": 'Loundry',
    "value": 60,
    "img": "/img/event/ACb-01.png"
  },
  {
    "id": 'Power',
    "name": 'Power',
    "value": 20,
    "img": "/img/event/powerb-01.png"
  }
];

// Get data considering interval, date point and direction
var getBubbleData = function(interval, x, direction, appliance, isForecast, apiKey, registrationToken, endPointing) {

  // Get interval
  if(interval == null) { interval = 'day'; }
  if(direction == null) { direction = 'both'; }
  if(appliance == null) { appliance = 'all'; }
  if(isForecast == null) { isForecast = true; }

  // Get x
  if(x == null) { x = 'Today'; }
  else { x = d3.timeParse(interval === 'year' ? "%Y" : "%Y-%m-%d")(x); }

  var data = [];

  switch(interval) {

    case 'week':

      // Set is now
      var lastMonday, sDay, eDay, startDate, endDate, fStartDate, fEndDate;

      // Get time for today
      if(x === 'Today') {

        // Get start date for actual and forecasted values
        sDay = gViz.shared.helpers.date.getLastMonday(new Date()).addDays(-6);
        startDate =`${sDay.setHours(0,0,1,000)}`;
        fStartDate =`${sDay.setHours(0,0,1,000)}`;

        // Get end date for actual and forecasted values
        eDay = gViz.shared.helpers.date.getLastMonday(new Date()).addDays(14);
        fEndDate = `${eDay.setHours(23,59,59,999)}`;
        endDate = `${(new Date()).getTime()}`;

      // Get time for other periods
      } else {

        // Get start date for actual and forecasted values
        sDay = gViz.shared.helpers.date.getLastMonday((new Date(x)).addDays(direction === 'prev' ? -7 : 7)).addDays(1);
        fStartDate = `${sDay.setHours(0,0,1,000)}`;
        startDate = `${sDay.setHours(0,0,1,000)}`;

        // Get end date for actual and forecasted values
        eDay = gViz.shared.helpers.date.getLastMonday((new Date(x)).addDays(direction === 'prev' ? -7 : 7)).addDays(6);
        fEndDate = `${eDay.setHours(23,59,59,999)}`;
        endDate = `${eDay.setHours(23,59,59,999)}`;

        // Fix start and end date not to go over now
        if(new Date(parseInt(startDate)) > new Date) { startDate = `${(new Date()).getTime()}`; }
        if(new Date(parseInt(endDate)) > new Date) { endDate = `${(new Date()).getTime()}`; }

      }

      // Remove isForecast
      if(isForecast != null && isForecast === false) { fStartDate = fEndDate = `${(new Date()).getTime()}`; }

      // Fetch data from api :: forecast = "forecast" appliance (change when API is done)
      var fReq = window.gViz.shared.data.api.fetch("forecast", fStartDate.substring(0,fStartDate.length-3), fEndDate.substring(0,fEndDate.length-3), "86400", 100000, null, apiKey, registrationToken, endPointing);

      // Fetch data from api :: actual = "all" appliance
      var aReq = window.gViz.shared.data.api.fetch(appliance, startDate.substring(0,startDate.length-3), endDate.substring(0,endDate.length-3), "86400", 100000, null, apiKey, registrationToken, endPointing);

      // Add or remove days depending on direction
      if(direction === 'prev') { x.addDays(-1); }
      else if(direction === 'next') { x.addDays(1); }

      return $.when(fReq, aReq).done(function(fData, aData) {

        // console.log("Fetched forecast new data", `${fData.consumption.t.length} points`);
        // console.log("Fetched actual new data", `${aData.consumption.t.length} points`);

        // Initialize parents structures
        var parents = {}, itemNames = {};

        // Parse new data from api
        fData.consumption.t.forEach(function(d, i) {

          // Get date from data
          var days = d3.timeFormat("%a")(new Date(parseInt(`${d}000`)));
          var index = d3.timeFormat("%u")(new Date(parseInt(`${d}000`)));
          var date = new Date(parseInt(`${d}000`)).addDays(1 - index);
          var isCurrentYear = date.getYear() === (new Date()).getYear();

          // Initialize parent structure for date if not initialized
          if(parents[d3.timeFormat("%Y-%m-%d")(date)] == null) {
            parents[d3.timeFormat("%Y-%m-%d")(date)] = {
              "x": d3.timeFormat("%Y-%m-%d")(date),
              "name": d3.timeFormat("%b %d" + (isCurrentYear ? "" : ", %Y"))(date),
              "values": [
              ]
            };
          }

          parents[d3.timeFormat("%Y-%m-%d")(date)].values.push({
            "id": "forecast",
            "x": days,
            "name": days,
            "date": date,
            "y": fData.consumption.w[i],
            "index": index,
            "color": "#ddd",
            "secondaryColor": "#233a71",
            "opacity": 1
          });

          // Store item and parent item names
          if(itemNames[d3.timeFormat("%Y-%m-%d")(date)] == null) { itemNames[d3.timeFormat("%Y-%m-%d")(date)] = {} };
          itemNames[d3.timeFormat("%Y-%m-%d")(date)][days.toString()] = true;
        });

        // Parse new data from api
        aData.consumption.t.forEach(function(d, i) {

          // Get date from data
          var days = d3.timeFormat("%a")(new Date(parseInt(`${d}000`)));
          var index = d3.timeFormat("%u")(new Date(parseInt(`${d}000`)));
          var date = new Date(parseInt(`${d}000`)).addDays(1 - index);
          var isCurrentYear = date.getYear() === (new Date()).getYear();

          // Initialize parent structure for date if not initialized
          if(parents[d3.timeFormat("%Y-%m-%d")(date)] == null) {
            parents[d3.timeFormat("%Y-%m-%d")(date)] = {
              "x": d3.timeFormat("%Y-%m-%d")(date),
              "name": d3.timeFormat("%b %d" + (isCurrentYear ? "" : ", %Y"))(date),
              "values": [
              ]
            };
          }

          parents[d3.timeFormat("%Y-%m-%d")(date)].values.push({
            "id": "actual",
            "x": days,
            "name": days,
            "date": date,
            "y": aData.consumption.w[i],
            "index": index,
            "color": "#2566bc",
            "secondaryColor": "rgba(36,121,219,0.7)",
            "opacity": 1
          });

          // Store item and parent item names
          if(itemNames[d3.timeFormat("%Y-%m-%d")(date)] == null) { itemNames[d3.timeFormat("%Y-%m-%d")(date)] = {} };
          itemNames[d3.timeFormat("%Y-%m-%d")(date)][days.toString()] = true;
        });

        // Set missing months to 0 just to fit axis
        Object.keys(itemNames).forEach(function(k) {
          d3.range(7).forEach(function(d, i) {
            var days = d3.timeFormat("%a")((new Date()).setDate(i+1));
            var index = d3.timeFormat("%u")((new Date()).setDate(i+1));
            if(itemNames[k][days.toString()] == null) {
              parents[k].values.push({
                "id": isForecast ? "forecast" : "actual",
                "x": days.toString(),
                "name": days.toString(),
                "y": 0,
                "index": index,
                "color": "transparent",
                "secondaryColor": "transparent",
                "opacity": 0
              });
            }
          });
        });

        Object.keys(parents).sort(function(a,b) { return d3.ascending(a,b); }).forEach(function(k) {
          parents[k].values = parents[k].values.sort(function(a,b) {
            return d3.descending(a.id,b.id) || d3.ascending(+a.index,+b.index);
          });
          data.push(parents[k]);
        });

        this._data = data;
        this._data = bubbleFakeData.slice(0,2);

      });

      break;

    case 'month':

      // Set is now
      var now = new Date(), xDate = new Date(x);
      var sDay, eDay, startDate, endDate, fStartDate, fEndDate;

      // Get time for today
      if(x === 'Today') {

        // Get start date for actual and forecasted values
        sDay = (new Date(now.getFullYear(), now.getMonth(), 2)).addMonths(-1);
        fStartDate =`${sDay.setHours(0,0,1,000)}`;
        startDate =`${sDay.setHours(0,0,1,000)}`;

        // Get end date for actual and forecasted values
        eDay = (new Date((new Date()).getFullYear(), now.getMonth() + 1, 1)).addMonths(+1);
        fEndDate = `${eDay.setHours(23,59,59,999)}`;
        endDate = `${(new Date()).getTime()}`;

      // Get time for other periods
      } else {

        // Add or remove months from date, depending on direction
        xDate.addMonths(direction === 'prev' ? -1 : 1);

        // Get start date for actual and forecasted values
        sDay = new Date(xDate.getFullYear(), xDate.getMonth(), 2);
        fStartDate =`${sDay.setHours(0,0,1,000)}`;
        startDate =`${sDay.setHours(0,0,1,000)}`;

        // Get end date for actual and forecasted values
        eDay = new Date(xDate.getFullYear(), xDate.getMonth() + 1, 1);
        fEndDate = `${eDay.setHours(0,59,59,999)}`;
        endDate = `${eDay.setHours(0,59,59,999)}`;

        // Fix start and end date not to go over now
        if(new Date(parseInt(startDate)) > new Date) { startDate = `${(new Date()).getTime()}`; }
        if(new Date(parseInt(endDate)) > new Date) { endDate = `${(new Date()).getTime()}`; }

      }

      // Remove isForecast
      if(isForecast != null && isForecast === false) { fStartDate = fEndDate = `${(new Date()).getTime()}`; }

      // Fetch data from api :: forecast = "forecast" appliance (change when API is done)
      var fReq = window.gViz.shared.data.api.fetch("forecast", fStartDate.substring(0,fStartDate.length-3), fEndDate.substring(0,fEndDate.length-3), "86400", 100000, null, apiKey, registrationToken, endPointing);

      // Fetch data from api :: actual = "all" appliance
      var aReq = window.gViz.shared.data.api.fetch(appliance, startDate.substring(0,startDate.length-3), endDate.substring(0,endDate.length-3), "86400", 100000, null, apiKey, registrationToken, endPointing);

      // Add or remove days depending on direction
      if(direction === 'prev') { x.addDays(-1); }
      else if(direction === 'next') { x.addDays(1); }

      return $.when(fReq, aReq).done(function(fData, aData) {

        // console.log("Fetched forecast new data", `${fData.consumption.t.length} points`);
        // console.log("Fetched actual new data", `${aData.consumption.t.length} points`);

        // Initialize parents structures
        var parents = {}, itemNames = {};

        // Parse new data from api
        fData.consumption.t.forEach(function(d, i) {

          // Get date from data
          var days = d3.timeFormat("%e")(new Date(parseInt(`${d}000`))).replace(' ','');
          var date = new Date((new Date(parseInt(`${d}000`))).setDate(1));
          var isCurrentYear = date.getYear() === (new Date()).getYear();

          // Initialize parent structure for date if not initialized
          if(parents[d3.timeFormat("%Y-%m-%d")(date)] == null) {
            parents[d3.timeFormat("%Y-%m-%d")(date)] = {
              "x": d3.timeFormat("%Y-%m-%d")(date),
              "name": d3.timeFormat("%B" + (isCurrentYear ? "" : ", %Y"))(date),
              "values": [
              ]
            };
          }

          parents[d3.timeFormat("%Y-%m-%d")(date)].values.push({
            "id": "forecast",
            "x": days,
            "name": days,
            "date": date,
            "y": fData.consumption.w[i],
            "index": +days,
            "color": "#ddd",
            "secondaryColor": "#233a71",
            "opacity": 1
          });

          // Store item and parent item names
          if(itemNames[d3.timeFormat("%Y-%m-%d")(date)] == null) { itemNames[d3.timeFormat("%Y-%m-%d")(date)] = {} };
          itemNames[d3.timeFormat("%Y-%m-%d")(date)][days.toString()] = true;
        });

        // Parse new data from api
        aData.consumption.t.forEach(function(d, i) {

          // Get date from data
          var days = d3.timeFormat("%e")(new Date(parseInt(`${d}000`))).replace(' ','');
          var date = new Date((new Date(parseInt(`${d}000`))).setDate(1));
          var isCurrentYear = date.getYear() === (new Date()).getYear();

          // Initialize parent structure for date if not initialized
          if(parents[d3.timeFormat("%Y-%m-%d")(date)] == null) {
            parents[d3.timeFormat("%Y-%m-%d")(date)] = {
              "x": d3.timeFormat("%Y-%m-%d")(date),
              "name": d3.timeFormat("%B" + (isCurrentYear ? "" : ", %Y"))(date),
              "values": [
              ]
            };
          }

          parents[d3.timeFormat("%Y-%m-%d")(date)].values.push({
            "id": "actual",
            "x": days,
            "name": days,
            "date": date,
            "y": aData.consumption.w[i],
            "index": +days,
            "color": "#2566bc",
            "secondaryColor": "rgba(36,121,219,0.7)",
            "opacity": 1
          });

          // Store item and parent item names
          if(itemNames[d3.timeFormat("%Y-%m-%d")(date)] == null) { itemNames[d3.timeFormat("%Y-%m-%d")(date)] = {} };
          itemNames[d3.timeFormat("%Y-%m-%d")(date)][days.toString()] = true;
        });

        // Set missing months to 0 just to fit axis
        Object.keys(itemNames).forEach(function(k) {
          d3.range(31).forEach(function(d, i) {
            if(itemNames[k][(i+1).toString()] == null) {
              parents[k].values.push({
                "id": isForecast ? "forecast" : "actual",
                "x": (i+1).toString(),
                "name": (i+1).toString(),
                "y": 0,
                "index": i+1,
                "color": "transparent",
                "secondaryColor": "transparent",
                "opacity": 0
              });
            }
          });
        });

        Object.keys(parents).sort(function(a,b) { return d3.ascending(a,b); }).forEach(function(k) {
          parents[k].values = parents[k].values.sort(function(a,b) {
            return d3.descending(a.id,b.id) || d3.ascending(+a.index,+b.index);
          });
          data.push(parents[k]);
        });

        this._data = data;
        this._data = bubbleFakeData.slice(0,3);

      });

      break;

    case 'year':

      // Set is now
      var now = new Date(), xDate = new Date(x);
      var sDay, eDay, startDate, endDate, fStartDate, fEndDate, months = [];

      // Get year
      var year = x === 'Today' ? now.getFullYear() : (xDate.getFullYear() + (direction === 'prev' ? - 1 : 1));

      // Iterate over months to join an year
      d3.range(12).forEach(function(d, i) {
        sDay = new Date(year, i, 2);
        eDay = new Date(year, i + 1, 1);
        fStartDate = `${sDay.setHours(0,0,1,0)}`;
        fEndDate = `${eDay.setHours(23,59,59,999)}`;
        startDate = `${sDay.setHours(0,0,1,0)}`;
        endDate = `${eDay.setHours(23,59,59,999)}`;

        // Remove isForecast
        if(isForecast != null && isForecast === false) { fStartDate = fEndDate = `${(new Date()).getTime()}`; }

        // Fetch data from api :: forecast = "forecast" appliance (change when API is done)
        months.push(window.gViz.shared.data.api.fetch("forecast", fStartDate.substring(0,fStartDate.length-3), fEndDate.substring(0,fEndDate.length-3),"86400", 100000, null, apiKey, registrationToken, endPointing));

        if((now.getMonth() >= i && now.getFullYear() === year) || year < now.getFullYear()) {

          // Reset endDate to now
          if(i === now.getMonth() && now.getFullYear() === year) { endDate = `${now.getTime()}`; }

          // Fetch data from api :: Actual = "all" appliance (change when API is done)
          months.push(window.gViz.shared.data.api.fetch(appliance, startDate.substring(0,startDate.length-3), endDate.substring(0,endDate.length-3),"86400", 100000, null, apiKey, registrationToken, endPointing));

        }

      });

      return $.when.apply($, months).done(function() {

        // Initialize parents structures
        var parents = {};

        // console.log("Fetched new data from year");

        // Iterate over responses of promises
        Array.from(arguments).forEach(function(data) {

          if(data.consumption.w.length !== 0) {

            // Parse new data from api
            var date = d3.min(data.consumption.t, function(d) { return new Date(parseInt(`${d}000`)); })
            var months = d3.timeFormat("%b")(date);
            var dataSum = d3.sum(data.consumption.w, function(d) { return d; })

            // Initialize parent structure for date if not initialized
            if(parents[d3.timeFormat("%Y")(date)] == null) {
              parents[d3.timeFormat("%Y")(date)] = {
                "x": d3.timeFormat("%Y")(date),
                "name": d3.timeFormat("%Y")(date),
                "values": [
                ]
              };
            }

            parents[d3.timeFormat("%Y")(date)].values.push({
              "id": data.appliance,
              "x": months,
              "date": date,
              "name": months,
              "y": dataSum,
              "color": data.appliance === 'forecast' ? "#ddd" : "#2566bc",
              "secondaryColor": data.appliance === 'forecast' ? "#233a71" : "rgba(36,121,219,0.7)",
              "opacity": 1
            });

          }

        });

        Object.keys(parents)
          .sort(function(a,b) { return d3.ascending(a,b); })
          .forEach(function(k) {
            parents[k].values = parents[k].values.sort(function(a,b) {
              return d3.ascending(a.date, b.date) || d3.descending(a.id, b.id);
            });
            data.push(parents[k]);
          });

        this._data = data;
        this._data = bubbleFakeData.slice(0,4);

      });

      break;

    default:

      // Set is now
      var now = new Date();
      var sDay, eDay, startDate, endDate, fStartDate, fEndDate;

      // Get time for today
      if(x === 'Today') {

        // Get start date for actual and forecasted values
        sDay = (new Date()).addDays(-1);
        startDate = `${sDay.setHours(1,0,0,0)}`;
        fStartDate = `${sDay.setHours(1,0,0,0)}`;

        // Get end date for actual and forecasted values
        eDay = (new Date()).addDays(2);
        fEndDate = `${eDay.setHours(1,0,0,0)}`;
        endDate = `${(new Date()).getTime()}`;

      // Get time for other periods
      } else {

        // Get start date for actual and forecasted values
        sDay = (new Date(x)).addDays(direction === 'prev' ? -1 : 1);
        startDate = `${sDay.setHours(1,0,0,0)}`;
        fStartDate = `${sDay.setHours(1,0,0,0)}`;

        // Get end date for actual and forecasted values
        eDay = (new Date(x)).addDays(direction === 'prev' ? 0 : 2);
        fEndDate = `${eDay.setHours(0,59,59,59)}`;
        endDate = `${direction === 'prev' ? eDay.setHours(0,59,59,59) : eDay.setHours(now.getHours(), now.getMinutes(), now.getSeconds(), now.getMilliseconds())}`;

        // Fix start and end date not to go over now
        if(new Date(parseInt(startDate)) > new Date) { startDate = `${(new Date()).getTime()}`; }
        if(new Date(parseInt(endDate)) > new Date) { endDate = `${(new Date()).getTime()}`; }

      }

      // Remove isForecast
      if(isForecast != null && isForecast === false) { fStartDate = fEndDate = `${(new Date()).getTime()}`; }

      // Fetch data from api :: forecast = "forecast" appliance (change when API is done)
      var fReq = window.gViz.shared.data.api.fetch("forecast", fStartDate.substring(0,fStartDate.length-3), fEndDate.substring(0,fEndDate.length-3), "3600", 100000, null, apiKey, registrationToken, endPointing);

      // Fetch data from api :: actual = "all" appliance
      var aReq = window.gViz.shared.data.api.fetch(appliance, startDate.substring(0,startDate.length-3), endDate.substring(0,endDate.length-3), "3600", 100000, null, apiKey, registrationToken, endPointing);

      // Add or remove days depending on direction
      if(direction === 'prev') { x.addDays(-1); }
      else if(direction === 'next') { x.addDays(1); }

      return $.when(fReq, aReq).done(function(fData, aData) {

        // console.log("Fetched forecast new data", `${fData.consumption.t.length} points`);
        // console.log("Fetched actual new data", `${aData.consumption.t.length} points`);

        // Initialize parents structures
        var parents = {}, itemNames = {};

        // Parse new data from api
        fData.consumption.t.forEach(function(d, i) {

          // Get date from data
          var hours = (new Date(parseInt(`${d}000`))).getHours();
          var date = hours === 0 ? (new Date(parseInt(`${d}000`))).addDays(-1) : (new Date(parseInt(`${d}000`)));
          var isCurrentYear = date.getYear() === (new Date()).getYear();

          // Initialize parent structure for date if not initialized
          if(parents[d3.timeFormat("%Y-%m-%d")(date)] == null) {
            parents[d3.timeFormat("%Y-%m-%d")(date)] = {
              "x": d3.timeFormat("%Y-%m-%d")(date),
              "name": d3.timeFormat("%A, %b %d" + (isCurrentYear ? "" : ", %Y"))(date),
              "values": [
              ]
            };
          }

          parents[d3.timeFormat("%Y-%m-%d")(date)].values.push({
            "id": "forecast",
            "x": hours === 0 ? 24 : hours,
            "name": hours === 0 ? 24 : hours,
            "date": date,
            "y": fData.consumption.w[i],
            "index": hours === 0 ? 24 : hours,
            "color": "#ddd",
            "secondaryColor": "#233a71",
            "opacity": 1
          });

          // Store item and parent item names
          if(itemNames[d3.timeFormat("%Y-%m-%d")(date)] == null) { itemNames[d3.timeFormat("%Y-%m-%d")(date)] = {} };
          itemNames[d3.timeFormat("%Y-%m-%d")(date)][(hours === 0 ? 24 : hours).toString()] = true;
        });

        // Parse new data from api
        aData.consumption.t.forEach(function(d, i) {

          // Get date from data
          var hours = (new Date(parseInt(`${d}000`))).getHours();
          var date = hours === 0 ? (new Date(parseInt(`${d}000`))).addDays(-1) : (new Date(parseInt(`${d}000`)));
          var isCurrentYear = date.getYear() === (new Date()).getYear();

          // Initialize parent structure for date if not initialized
          if(parents[d3.timeFormat("%Y-%m-%d")(date)] == null) {
            parents[d3.timeFormat("%Y-%m-%d")(date)] = {
              "x": d3.timeFormat("%Y-%m-%d")(date),
              "name": d3.timeFormat("%A, %b %d" + (isCurrentYear ? "" : ", %Y"))(date),
              "values": [
              ]
            };
          }

          parents[d3.timeFormat("%Y-%m-%d")(date)].values.push({
            "id": "actual",
            "x": hours === 0 ? 24 : hours,
            "name": hours === 0 ? 24 : hours,
            "date": date,
            "y": aData.consumption.w[i],
            "index": hours === 0 ? 24 : hours,
            "color": "#2566bc",
            "secondaryColor": "rgba(36,121,219,0.7)",
            "opacity": 1
          });

          // Store item and parent item names
          if(itemNames[d3.timeFormat("%Y-%m-%d")(date)] == null) { itemNames[d3.timeFormat("%Y-%m-%d")(date)] = {} };
          itemNames[d3.timeFormat("%Y-%m-%d")(date)][(hours === 0 ? 24 : hours).toString()] = true;
        });

        // Set missing months to 0 just to fit axis
        Object.keys(itemNames).forEach(function(k) {
          d3.range(23).forEach(function(d, i) {
            if(itemNames[k][(i+1).toString()] == null) {
              parents[k].values.push({
                "id": isForecast ? "forecast" : "actual",
                "x": (i+1).toString(),
                "name": (i+1).toString(),
                "y": 0,
                "index": i,
                "color": "transparent",
                "secondaryColor": "transparent",
                "opacity": 0
              });
            }
          });
        });

        Object.keys(parents).sort(function(a,b) { return d3.ascending(a,b); }).forEach(function(k) {
          parents[k].values = parents[k].values.sort(function(a,b) { return d3.descending(a.id,b.id) || d3.ascending(a.index,b.index); });
          data.push(parents[k]);
        });

        this._data = data;
        this._data = bubbleFakeData;

      });

      break;

  }

  return data;

}

// Initialize the helpers class¬
gViz.shared.data.bubbleChart = {

  // Get mock data
  fetch: function(interval, x, direction, appliance, isForecast, apiKey, registrationToken, endPointing) {
    return getBubbleData(interval, x, direction, appliance, isForecast, apiKey, registrationToken, endPointing);
  },

  // Replace text variables
  format: function() {

    return {
      "title": "",
      "size": {
        "title": "",
        "prefix": "",
        "sufix": "W",
        "format": ""
      },
      "data": []
    };
  },

  // Replace text variables
  getData: getBubbleData

}

// Initialize helpers array
gViz.shared.data.lineChart = {};

gViz.shared.data.lineChart.getDataOrCache = function(sDay, eDay, appliance, apiKey, registrationToken, endPointing, zoomLevel) {

  // Initialize dates
  var startDate = `${sDay.getTime()}`, endDate = `${eDay.getTime()}`;

  // Parse
  var start = new Date(sDay.getTime());
  var end = new Date(eDay.getTime());

  // Get cache interval
  var cacheInt = gViz.shared.data.lineChart.getCacheInterval(zoomLevel);

  // Fix start and end dates
  start = new Date() < start ? new Date() : start;
  end = new Date() < end ? new Date() : end;

  // Get parameters
  var cParam = {
    start: start.toISOString().split(".")[0] + ".000Z",
    end: end.toISOString().split(".")[0] + ".000Z",
    range: parseInt((end - start) / (cacheInt * 1000))
  }

  // Get initial and final data cached position
  cParam.cacheStart = gViz.shared.data.lineChart.cached[cacheInt][appliance].findIndex(function(d) { return d.x == cParam.start; });
  cParam.cacheEnd   = gViz.shared.data.lineChart.cached[cacheInt][appliance].findIndex(function(d) { return d.x == cParam.end; });

  if(cParam.cacheStart !== -1 && cParam.cacheEnd !== -1 && Math.abs(cParam.cacheEnd - cParam.cacheStart) >= cParam.range) {

    // Get appliance data
    var aData = { appliance: appliance, consumption: { w: [], t: [] }, isCache: true };
    gViz.shared.data.lineChart.cached[cacheInt][appliance].slice(cParam.cacheStart, cParam.cacheEnd+1).forEach(function(d) {
      aData.consumption.w.push(d.value);
      var date = `${d.date.getTime()}`;
      aData.consumption.t.push(date.substring(0,date.length-3));
    });

    return window.gViz.shared.data.api.resolvedFetch(aData);

  } else {

    // // Dev only
    //var aData = { appliance: appliance, consumption: { w: [], t: [] }, isCache: true };
    //return window.gViz.shared.data.api.resolvedFetch(aData);

    // // Fetch data from api :: actual = "all" appliance
    return window.gViz.shared.data.api.fetch(appliance, startDate.substring(0,startDate.length-3), endDate.substring(0,endDate.length-3), cacheInt, 100000, null, apiKey, registrationToken, endPointing);

  }

}

// Store data in cache
gViz.shared.data.lineChart.cacheData = function(rData, zoomLevel) {

  // Apply cache only if the data wasn't fetched from cache
  if(rData.isCache == null || rData.isCache !== true) {

    // Get cache interval
    var cacheInt = gViz.shared.data.lineChart.getCacheInterval(zoomLevel);

    // Cache data for future uses
    var keys = {};
    rData.consumption.t.forEach(function(d, i) {
      gViz.shared.data.lineChart.cached[cacheInt][rData.appliance].push({
        x: (new Date(parseInt(`${d}000`))).toISOString(),
        date: new Date(parseInt(`${d}000`)),
        value: rData.consumption.w[i]
      });
    });

    // Unique keys
    var newCacheData = [], storedKeys = {};
    gViz.shared.data.lineChart.cached[cacheInt][rData.appliance].forEach(function(d) {
      if(storedKeys[d.x] == null) {
        storedKeys[d.x] = true;
        newCacheData.push(d);
      }
    });

    // Sort new cache array and store data
    gViz.shared.data.lineChart.cached[cacheInt][rData.appliance] = newCacheData.sort(function(a,b) { return d3.ascending(a.date.getTime(), b.date.getTime()); });

  }

}

// Get data considering interval, date point and direction
gViz.shared.data.lineChart.getData = function(start, end, appliance, zoomLevel, apiKey, registrationToken, endPointing) {

  // Get cache interval
  var cacheInt = gViz.shared.data.lineChart.getCacheInterval(zoomLevel);

  // Initialize data cache for appliance and forecast
  gViz.shared.data.lineChart.cached[cacheInt] = gViz.shared.data.lineChart.cached[cacheInt] || {};
  gViz.shared.data.lineChart.cached[cacheInt][appliance] = gViz.shared.data.lineChart.cached[cacheInt][appliance] || [];

  // Get data depending on cache
  var data = [], req = null, mod = 0;

  if(appliance == null) { appliance = 'all'; }
  if(zoomLevel == null) { zoomLevel = 1; }

  if(zoomLevel >= 1) {
    if(start == null && end != null) {
      end = gViz.shared.data.lineChart.dateAdjust(end, zoomLevel);
      start = (new Date(end.getTime())).addDays(-30);
    } else if (start != null && end == null) {
      start = gViz.shared.data.lineChart.dateAdjust(start, zoomLevel);
      end = (new Date(start.getTime())).addDays(30);
      end = end > new Date() ? new Date() : end;
      end = gViz.shared.data.lineChart.dateAdjust(end, zoomLevel);
    } else if (start != null && end != null) {
      start = gViz.shared.data.lineChart.dateAdjust(start, zoomLevel);
      end = gViz.shared.data.lineChart.dateAdjust(end, zoomLevel);
    }
  }

  // Fetch data for adjusted dates
  req = gViz.shared.data.lineChart.getDataOrCache(start, end, appliance, apiKey, registrationToken, endPointing, zoomLevel);

  var parseData = function(rData) {

    console.log("parsedata : ", rData.consumption.t.length)

    // Fill rData consumption arrays with 0s if there is no data
    if(rData.consumption.t.length == 0) {
      console.log("parsedata lengtth 0")
      var zoomInfo = gViz.shared.data.lineChart.zoomLevelInfo(zoomLevel);
      var cacheInt = gViz.shared.data.lineChart.getCacheInterval(zoomLevel);
      var cDate = new Date(start.getTime()), cDateString;
      while(cDate <= end) {
        cDateString = `${cDate.getTime()}`;
        rData.consumption.t.push(cDateString.substring(0, cDateString.length-3));
        rData.consumption.w.push(0);
        cDate.addSeconds(cacheInt);
      }
    }
    if(parseInt(`${rData.consumption.t[rData.consumption.t.length-1]}000`) !== end.getTime()) {
        cDateString = `${end.getTime()}`;
        rData.consumption.t.push(cDateString.substring(0, cDateString.length-3));
        rData.consumption.w.push(0);
    }

    // Store data in cache
    gViz.shared.data.lineChart.cacheData(rData, zoomLevel);

    // Group data depending on the zoom level
    rData = gViz.shared.data.lineChart.groupData(rData, zoomLevel);

    // Store data to be used and also in cache
    this._data = rData;

  }

  return $.when(req).done(parseData);

}

// Group data depending on zoom level
gViz.shared.data.lineChart.groupData = function(rData, zoomLevel) {

  // Don't group for default level
  if(zoomLevel == 1 || zoomLevel == 5) { return rData; }

  // Get initial information to group data
  var zoomInfo = gViz.shared.data.lineChart.zoomLevelInfo(zoomLevel);
  var cacheInt = gViz.shared.data.lineChart.getCacheInterval(zoomLevel);
  var consT = [], consW = [], consValue = 0, consInterval = zoomInfo.seconds / cacheInt;

  // Group data
  rData.consumption.t.forEach(function(d, i) {
    if(i === 0) {
      consT.push(d);
      consW.push(rData.consumption.w[0]);
    } else {
      consValue += rData.consumption.w[i];
      if(i % consInterval === 0) {
        consT.push(d);
        consW.push(consValue);
        consValue = 0;
      }
    }
  });

  // Store new grouped values on rData
  rData.consumption.t = consT;
  rData.consumption.w = consW;

  return rData;
}

// Replace text variables
gViz.shared.data.lineChart.format = function(data) {

  // Set date format
  var dateOrder = gViz.shared.helpers.date.translated.DATEORDER['NOW'];

  var json = {
    "title": "",
    "legend": {
      "isVisible": false,
    },
    "tooltip": {
      "backgroundColor": "#FFF",
      "borderColor": "transparent",
      "title": [ "This is a title example" ],
      "body": [ "This is a body example" ]
    },
    "y": {
      "title": "",
      "prefix": "",
      "sufix": "W",
      "format": "",
      "min": 0,
      "grid": false
    },
    "x": {
      "title": "",
      "type": "time",
      "prefix": "",
      "sufix": "",
      "format": "",
      "inFormat": "%Y-%m-%d %H:%M:%S",
      "outFormat": dateOrder,
      "tickSpace": 100,
      "grid": false
    },
    "data": [
      {
        "id": "actual",
        "name": "Mês Atual",
        "color": "#3d62cc",
        "pointColor": "#267c56",
        "hasArea": true,
        "areaColor": "#e0e9f4",
        "strokeWidth": 2,
        "pointSize": 0,
        "values": []
      }
    ]
  };

  // Get in and Out format to parse dates
  var inFmt = function(d) { return new Date(d); };
  var outFmt = d3.timeFormat("%Y-%m-%d %H:%M:%S");

  // Sort elements and store it on the json
  data.sort(function(a,b) { return d3.ascending(+a.x, +b.x); }).forEach(function(d) {
    d.x = outFmt(inFmt(d.x));
    d.y = d.y;
    json.data[0].values.push(d);
  });

  return json;

};

gViz.shared.data.lineChart.parse =  function(data) {

  var _data = [];

  // Get in and out format to parse dates
  var inFmt = function(d) { return new Date(d); };
  var outFmt = d3.timeFormat("%Y-%m-%d %H:%M:%S");

  // sort elements and store it on the json
  data.sort(function(a,b) { return d3.ascending(+a.x, +b.x); }).forEach(function(d) {
    d.x = outFmt(inFmt(d.x));
    d.y = d.y;
    _data.push(d);
  });

  return _data;

};

// Store cached data for initial values
gViz.shared.data.lineChart.cached = {};

// Adjust data point based on zoom level
gViz.shared.data.lineChart.dateAdjust = function(date, zoomLevel) {
  var aDate, mod;
  if(zoomLevel <= 1) {
    aDate = new Date(date);
    mod = aDate.getMinutes() % 3;
    return new Date(aDate.setHours(aDate.getHours(), aDate.getMinutes() - mod, 0, 000));
  } else {
    aDate = new Date(date);
    mod = aDate.getSeconds() % 3;
    return new Date(aDate.setHours(aDate.getHours(), aDate.getMinutes(), aDate.getSeconds() - mod, 000));
  }
}

gViz.shared.data.lineChart.getCacheInterval = function(zoomLevel) {
  if(zoomLevel <= 1) { return 180; }
  else { return 3; }
}

gViz.shared.data.lineChart.zoomLevelInfo = function(zoomLevel) {
  switch(zoomLevel) {
    case -2:
      return { interval: 43200, max: 61, num: 20, seconds: 2160, zoom: 0.08333333333333333 };
    case -1:
      return { interval: 21600, max: 61, num: 20, seconds: 1080, zoom: 0.16666666666666666 };
    case 0:
      return { interval: 7200, max: 61, num: 20, seconds: 360, zoom: 0.5 };
    case 1:
      return { interval: 3600, max: 61, num: 20, seconds: 180, zoom: 1 };
    case 2:
      return { interval: 1800, max: 61, num: 20, seconds: 90, zoom: 2 };
    case 3:
      return { interval: 900, max: 61, num: 20, seconds: 45, zoom: 4 };
    case 4:
      return { interval: 300, max: 61, num: 20, seconds: 15, zoom: 12 };
    case 5:
      return { interval: 60, max: 61, num: 20, seconds: 3, zoom: 60 };
    default:
      return { interval: 3600, max: 61, num: 20, seconds: 180, zoom: 1 };
  }
}

// 12h, 24 points
// 6h, 24points
// 2h, 16 points
// 1h, start screen, 20 points
// 30min 30pts
// 15min 30pts
// 5min, 20pts
// 1min, 24pts

// Get data
gViz.shared.data.lineChart.fetch = function(start, end, appliance, zoomLevel, apiKey, registrationToken, endPointing) {
  return gViz.shared.data.lineChart.getData(start, end, appliance, zoomLevel, apiKey, registrationToken, endPointing);
}

// Set unit
gViz.shared.data.lineChart.setUnit = function(data, unit) {
  if(unit != null) { data.y.sufix = unit; }
}

// Module declaration
gViz.viz.lineAreaChart = function() {
  "use strict";

  // Get attributes values
  var _id = "vis-line-area-chart-"+(Math.floor(Math.random() * ((1000000000 - 5) + 1)) + 5);
  var _var = null;
  var action = 'build';
  var animation = 200;
  var container = null;
  var colors = [{value:0, color:"#4ad2a1"},{value:150, color:"#4ad2a1"},{value:300, color:"#3d62cb"},{value:500, color:"#3d62cb"},{value:600, color:"#fa5050"}];
  var data = [];
  var margin = { top: 22, right: 0, bottom: 45, left: 0 };
  var opacity = 0.4;
  var isNow = true;
  var timespan = null;
  var touchEvents = null;
  var wheelZoom = {};
  var xZoom = {};
  var yZoom = {};
  var zoomMode = 'horizontal';
  var zoomLevel = 1;

  // Validate attributes
  var validate = function (step) {
    switch (step) {
      case 'build':      return (container != null) && (d3.selectAll(container).size() !== 0 || d3.select(container).size() !== 0);
      case 'initialize': return true;
      case 'axis':       return data != null && data.data != null && data.data.length > 0 && data.data[0].values.length > 0;
      case 'create':     return data != null && data.data != null && data.data.length > 0 && data.data[0].values.length > 0;
      case 'elements':   return data != null && data.data != null && data.data.length > 0 && data.data[0].values.length > 0;
      case 'misc':       return true;
      case 'style':      return true;
      case 'xScale':     return data != null && data.data != null && data.data.length > 0 && data.data[0].values.length > 0;
      case 'yScale':     return data != null && data.data != null && data.data.length > 0 && data.data[0].values.length > 0;
      case 'zoom':       return data != null && data.data != null && data.data.length > 0 && data.data[0].values.length > 0;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'build':

          main('initialize');
          main('style');
          main('misc');
          main('yScale');
          main('xScale');
          main('create');
          main('axis');
          main('elements');
          main('zoom');
          break;

        // Initialize visualization variable
        case 'initialize':

          // Initializing
          if (!_var) { _var = {};  }
          _var = gViz.viz.lineAreaChart.initialize()
            ._var(_var)
            ._id((_var._id != null) ? _var._id : _id)
            .animation(animation)
            .container(container)
            .colors(colors)
            .data(data)
            .isNow(isNow)
            .margin(margin)
            .opacity(opacity)
            .timespan(timespan)
            .touchEvents(touchEvents)
            .wheelZoom(wheelZoom)
            .xZoom(xZoom)
            .yZoom(yZoom)
            .zoomMode(zoomMode)
            .zoomLevel(zoomLevel)
            .run();
          break;

        // Setup style functions
        case 'style':

          // Setting styles
          _var = gViz.viz.lineAreaChart.style()
            ._var(_var)
            .run();
          break;

        // Create initial elements
        case 'create':

          // Creating wrappers
          _var = gViz.viz.lineAreaChart.create()
            ._var(_var)
            .run();
          break;

        // Setup X scale
        case 'xScale':

          // Creating
          _var = gViz.viz.lineAreaChart.xScale()
            ._var(_var)
            .data(_var.data.data)
            .run();
          break;

        // Setup Y scale
        case 'yScale':

          // Creating
          _var = gViz.viz.lineAreaChart.yScale()
            ._var(_var)
            .data(_var.data.data)
            .run();
          break;

        // Setup axis elements
        case 'axis':

          // Running
          _var = gViz.viz.lineAreaChart.axis()
            ._var(_var)
            .action('create')
            .run();
          break;

        // Setup elements
        case 'elements':

          // Running
          _var = gViz.viz.lineAreaChart.elements()
            ._var(_var)
            .components(gViz.viz.lineAreaChart)
            .run();
          break;

        // Show misc
        case 'misc':

          // Running
          _var = gViz.viz.lineAreaChart.misc()
            ._var(_var)
            .components(gViz.viz.lineAreaChart)
            .run();
          break;

        // Set zoom
        case 'zoom':

          // Add zoom horizontally
          _var = gViz.viz.lineAreaChart.xZoom()
            ._var(_var)
            .components(gViz.viz.lineAreaChart)
            .run();

          // Add zoom
          _var = gViz.viz.lineAreaChart.zoom()
            ._var(_var)
            .components(gViz.viz.lineAreaChart)
            .run();

          break;

      }
    }

    // Getters
    _var.get = {

      // Get most left point
      start: function() {
        return _var.x.invert(-_var.xZoom.transform.x / _var.xZoom.transform.k);
      },

      // Get most right point
      end: function() {
        return _var.x.invert((-_var.xZoom.transform.x + _var.width) / _var.xZoom.transform.k);
      },

      // Get min y value
      min: function() { return _var.y.invert(-_var.xZoom.transform.y + _var.height); },

      // Get max y value
      max: function() { return _var.y.invert(-_var.xZoom.transform.y); }
    }

    return _var;
  };

  // Expose global variables
  ['_id', '_var', 'action', 'animation', 'container', 'colors', 'data','isNow','margin','opacity','timespan','touchEvents','wheelZoom','xZoom','yZoom','zoomMode','zoomLevel'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return main;
    };
  });

  // Secondary functions
  main.build = function (_) { return main("build"); };

  // Execute the specific called function
  main.run = function (_) { return main(_); };

  return main;

}

// Module declaration
gViz.viz.lineAreaChart.axis = function() {
  "use strict";

  // Get attributes values
  var _var = undefined;
  var action = 'create';

  // Validate attributes
  var validate = function validate(step) {

    switch (step) {
      case 'run':
        return true;
      default:
        return false;
    }
  };

  // Main function
  var main = function main(step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          switch (action) {

            case 'create':

              // Create and update X axis
              _var.x_axis = _var.g.selectAll(".x.axis").data(['x']);
              _var.x_axis.exit().remove();
              _var.x_axis = _var.x_axis.enter().append('g').attr("class", "x axis").merge(_var.x_axis);
              _var.x_axis.call(_var.xAxis.tickSize(5)).attr("transform", 'translate(0,' + _var.height + ')')

              // Update x axis attributes and styles
              _var.updateXAxis = function() {

                // Create and update Y axis
                _var.y_axis = _var.g.selectAll(".y.axis-text").data([_var.y.invert(_var.height), _var.y.invert(_var.margin.top)], function(d) { return d; });
                _var.y_axis.exit().remove();
                _var.y_axis = _var.y_axis.enter().append('text').attr("class", "y axis-text").merge(_var.y_axis);
                _var.y_axis
                  .attr('y', function(d,i) { return i === 0 ? _var.height - 10 : 30; })
                  .attr('x', 10)
                  .style('font-size', '12px')
                  .style('fill', '#9b9b9b')
                  .style('font-weight', '500')
                  .html(function(d) { return "– \&nbsp;\&nbsp;" + _var.yFormat(d); })

                _var.x_axis.selectAll("path.domain").remove();
                _var.x_axis.selectAll(".tick line").attr('y1', 0)
                _var.x_axis.selectAll(".tick text").attr('dy', 3)

                // Draw Background rect
                var pathDomain = _var.x_axis.selectAll("path.domain-custom").data(["domain-custom"]);
                pathDomain.exit().remove();
                pathDomain = pathDomain.enter().insert('path', ':first-child').attr("class", "domain-custom").merge(pathDomain);
                pathDomain.attr("d", "M 0,0 " + (_var.width + _var.margin.left + _var.margin.right) + ",0")

                // Draw x day text
                var xTitle = _var.g.selectAll("text.x-title").data(["x-title"]);
                xTitle.exit().remove();
                xTitle = xTitle.enter().append('text').attr("class", "x-title").merge(xTitle);
                xTitle
                  .attr('y', _var.height + 36)
                  .attr('x', 10)
                  .attr('text-anchor', 'start')
                  .style('font-weight', 600)
                  .style('font-size', '14px')
                  .text(function(d) {

                    // Today
                    var today = new Date((new Date()).setHours(0,0,0,0));

                    // Yesterday
                    var yesterday = new Date((new Date()).setHours(23,59,59,999));
                    yesterday.setDate(yesterday.getDate() - 2);

                    // Most left value
                    var leftValue = _var._x.invert(-_var.xZoom.transform.x + _var.distance - _var.width);

                    // Check if its the current Year
                    var isCurrentYear = leftValue.getYear() === today.getYear();

                    // Get intervals
                    if(leftValue >= today) { return gViz.shared.helpers.date.translated.today; }
                    else { return d3.timeFormat("%b %d" + (isCurrentYear ? "" : ", %Y"))(leftValue); }

                  });

              }


              break;

          }
          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var', 'action'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) {
    return main('run');
  };

  return main;
};

function __range__(left, right, inclusive) {
  var range = [];
  var ascending = left < right;
  var end = !inclusive ? right : ascending ? right + 1 : right - 1;
  for (var i = left; ascending ? i < end : i > end; ascending ? i++ : i--) { range.push(i); }
  return range;
}

// Module declaration
gViz.viz.lineAreaChart.create = function() {
  "use strict";

  // Get attributes values
  var _var = undefined;
  var animation = 900;

  // Validate attributes
  var validate = function (step) {

    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Draw svg
          _var.wrap = _var.container.d3.selectAll("svg.chart-"+_var._id).data(["chart-svg"], function(d) { return d; });
          _var.wrap.exit().remove();
          _var.wrap = _var.wrap.enter().append("svg").attr('class', "line-area-chart chart-"+_var._id).merge(_var.wrap); // svg

          // Update outer dimensions
          _var.wrap
            .attr("width", _var.width + _var.margin.left + _var.margin.right)
            .attr("height", _var.height + _var.margin.top + _var.margin.bottom)

          // Draw g
          _var.g = _var.wrap.selectAll("g.chart-wrap").data(["chart-wrap"], function(d) { return d; }); // svg:g
          _var.g.exit().remove();
          _var.g = _var.g.enter().append('g').attr('class', "chart-wrap").attr("transform", "translate("+_var.margin.left+","+_var.margin.top+")").merge(_var.g);

          // Draw g for clip path
          _var.gClip = _var.wrap.selectAll("g.chart-wrap-clip").data(["chart-wrap-clip"], function(d) { return d; }); // svg:g
          _var.gClip.exit().remove();
          _var.gClip = _var.gClip.enter().insert('g',':first-child').attr('class', "chart-wrap-clip").merge(_var.gClip);
          _var.gClip
            .attr('clip-path', 'url(#clip-path-'+_var._id+')');

          // Draw defs
          _var.defs = _var.wrap.selectAll("defs.svg-defs").data(["svg-defs"], function(d) { return d; });
          _var.defs.exit().remove();
          _var.defs = _var.defs.enter().insert('defs',':first-child').attr("class", "svg-defs").merge(_var.defs);

          // Draw shadow
          gViz.shared.visualComponents.shadow()
            ._var(_var)
            .wrap(_var.wrap)
            .id(_var.shadowId)
            .run();

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var', 'animation'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function(_) { return main('run'); };

  return main;
};

// Module declaration
gViz.viz.lineAreaChart.elements = function() {
  "use strict";

  // Get attributes values
  var _var       = null;
  var components = null;
  var data       = null;

  // Validate attributes
  var validate = function (step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          if(_var.xZoom.transform != null) {

            // Clean wrap gradients
            gViz.shared.visualComponents.gradient()
              .action('clean')
              .run();

            // Initialize line constructor
            _var.lineConstructor = d3.line()
              .x(function (d) { return _var._x(d.parsedX); })
              .y(function (d) { return _var._y(+d.y); })
              .curve(d3.curveLinear)

            // Initialize area constructor
            _var.areaConstructor = d3.area()
              .x(function (d) { return _var._x(d.parsedX); })
              .y0(function (d) { return _var.height; })
              .y1(function (d) { return _var._y(+d.y); })
              .curve(d3.curveLinear)

            // Set data array
            var _data = (data == null ? _var.data.data : data).sort(function(a,b) { return  d3.descending(a.mean, b.mean); });

            // Element canvas
            var elements = _var.g.selectAll(".chart-elements").data(["chart-elements"], function(d) { return d; });
            elements.exit().remove();
            elements = elements.enter().append("g").attr("class", "chart-elements").merge(elements);

            // Element canvas
            var elementsClip = _var.gClip.selectAll(".chart-elements-clip").data(["chart-elements-clip"], function(d) { return d; });
            elementsClip.exit().remove();
            elementsClip = elementsClip.enter().append("g").attr("class", "chart-elements-clip").merge(elementsClip);

            // Create groups
            var groups = elementsClip.selectAll(".element-group").data(_data, function(d) { return d.id; });
            groups.exit().remove();
            groups = groups.enter().append("g").attr("class", "element-group").merge(groups);
            groups.each(function(g) {

              // Store initial values for interpolation with y = 0
              var values = g.values;
              if(g._values == null) {
                g._values = values.map(function(d) { return { x: d.x, y: 0, parsedX: d.parsedX, formattedX: d.formattedX }; });
              }

              // // Update y bounds
              // _var.yBounds = d3.extent(values, function(d) { return +d.y; });
              // _var.y.domain(_var.yBounds);
              // _var._y.domain(_var.yBounds);

              // Update axis
              _var.updateXAxis()

              var last = { parsedX: g.values[g.values.length-1].parsedX, y: g.values[g.values.length-1].y };
              var first = { parsedX: g.values[0].parsedX, y: g.values[0].y };
              var values = [first].concat(g.values.concat([last]));

              // Create areas
              var hasArea = g.hasArea != null && g.hasArea === true;
              var areas = d3.select(this).selectAll(".area").data(hasArea ? [g] : [], function(d) { return d.id; });
              areas.exit().remove();
              areas = areas.enter().append("path").attr("class", "area")
                .attr("d", function (d) { return _var.areaConstructor(g._values); })
                .merge(areas);
              areas
                .attr("fill-opacity", _var.opacity)
                .attr("fill", 'url("#gViz-luko-main-gradient")')
                .transition()
                  .duration(_var.animation)
                  .ease(d3.easeLinear)
                  .attrTween("d", function (d) {
                    var previous = _var.rebuild.area && !_var.wheelZoom.enable ? _var.areaConstructor(g._values) : d3.select(this).attr('d');
                    var current = _var.areaConstructor(values);
                    _var.rebuild.area = false;
                    return d3.interpolatePath(previous, current);
                  });

              // Create lines
              var lines = d3.select(this).selectAll(".line").data([g], function(d) { return d.id; });
              lines.exit().remove();
              lines = lines.enter().append("path").attr("class", "line")
                .attr("d", function (d) { return _var.lineConstructor(g._values); })
                .merge(lines);
              lines
                .attr("fill", 'none')
                .attr("stroke-width", _var.lineWidth)
                .attr("stroke", 'url("#gViz-luko-main-gradient")')
                .transition()
                  .duration(_var.animation)
                  .ease(d3.easeLinear)
                  .attrTween("d", function (d) {
                    var previous = _var.rebuild.line && !_var.wheelZoom.enable ? _var.lineConstructor(g._values) : d3.select(this).attr('d');
                    var current = _var.lineConstructor(values);
                    _var.rebuild.line = false;
                    return d3.interpolatePath(previous, current);
                  });

                // Store previous values and set rebuild to false not to use g._values inside
                g._values = values.map(function(d) { return { x: d.x, y: +d.y, parsedX: d.parsedX, formattedX: d.formattedX }; });

                // Map colors
                var colors = _var.colors
                  .filter(function(d) { return _var._y(d.value) > 0; })
                  .map(function(d) {
                    return {
                      offset: (((_var._y(d.value) / (_var.height + _var.margin.bottom + _var.margin.top)) * 100).toFixed(2) + "%"),
                      color: d.color
                    };
                  });

                if(colors.length > 0) {

                  // Store initial and last colors
                  colors = [{ offset: "0%", color: colors[0].color }].concat(colors);
                  if(colors[colors.length-1].offset !== '100.00%') {
                    colors = colors.concat([{ offset:"100%", color: colors[colors.length-1].color }]);
                  }

                  // Set wrap gradient
                  gViz.shared.visualComponents.gradient()
                    ._var(_var)
                    .id("gViz-luko-main-gradient")
                    .colors(colors)
                    .direction('vertical')
                    .duration(_var.animation)
                    .y1(0)
                    .y2(_var.height)
                    .gradientUnits('userSpaceOnUse')
                    .gType('linear')
                    .wrap(_var.defs)
                    .run();

                }

            });

            // // Draw Background clip Path
            // _var.bgClip = _var.defs.selectAll(".bg-clip").data(["bg-clip"]);
            // _var.bgClip.exit().remove();
            // _var.bgClip = _var.bgClip.enter().insert('clipPath', ':first-child').attr("class", "bg-clip").merge(_var.bgClip);
            // _var.bgClip
            //   .attr("id", "clip-path-"+_var._id)
            //   .each(function() {

            //     // Draw Background rect
            //     _var.bgClipRect = d3.select(this).selectAll("rect.bg-rect").data(["bg-rect"]);
            //     _var.bgClipRect.exit().remove();
            //     _var.bgClipRect = _var.bgClipRect.enter().insert('rect', ':first-child').attr("class", "bg-rect").style('fill', 'transparent').merge(_var.bgClipRect);
            //     _var.bgClipRect
            //       .style('fill', 'transparent')
            //       .attr("x", 0)
            //       .attr('y', -_var.margin.top)
            //       .attr('width', _var.width)
            //       .attr("height", _var.height + _var.margin.top);

            //   });

            // Draw reference line
            var refLine = elements.selectAll("line.reference-line").data(["reference-line"], function(d) { return d; });
            refLine.exit().remove();
            refLine = refLine.enter().append('line').attr("class", "reference-line").merge(refLine);
            refLine
              .style('stroke-width', '1px')
              .style('stroke', function(d) { return _var.isReferenceLineNow() ? _var.referenceLineColor(d) : '#bbb'; })
              .attr("x1", function(d) { return _var.width * 5 / 6; })
              .attr("x2", function(d) { return _var.width * 5 / 6; })
              .attr('y1', 0)
              .attr('y2', _var.height + 5)

            // Get reference line values
            var bisector = d3.bisector(function (d) { return d.parsedX; }).left;
            var xValue = _var._x.invert(_var.width * 5 / 6);
            var referenceObj = {};

            // Get values from each line group to use on the tooltip
            _var.data.data.forEach(function(lineGroup, i) {

              // Get most close value to the mouse position
              var bisectIndex = bisector(lineGroup.values, xValue);
              if (bisectIndex > 0 && bisectIndex < lineGroup.values.length - 1) {
                var x0 = _var.x(lineGroup.values[bisectIndex - 1].parsedX);
                var x1 = _var.x(lineGroup.values[bisectIndex].parsedX);
                bisectIndex = _var.margin.left - x0 >= (x1 - x0) / 2 ? bisectIndex : bisectIndex - 1;
              }

              if(lineGroup.values[bisectIndex]) {
                // Set parent n attributes to tooltip obj
                Object.keys(lineGroup).forEach(function(k) { referenceObj[k] = lineGroup[k]; });

                // Set n attributes to tooltip obj
                Object.keys(lineGroup.values[bisectIndex]).forEach(function(k) { referenceObj[k] = lineGroup.values[bisectIndex][k]; });

                // Set x, color and z values with format
                referenceObj.x = lineGroup.values[bisectIndex].x;
                referenceObj.y = lineGroup.values[bisectIndex].y;
                referenceObj.color = _var.pointColor(lineGroup.values[bisectIndex]);
              }
            });

            // Draw reference line text
            var refLineText = elements.selectAll("text.reference-line-text").data(["reference-line-text"], function(d) { return d; });
            refLineText.exit().remove();
            refLineText = refLineText.enter().append('text').attr("class", "reference-line-text").merge(refLineText);
            refLineText
              .attr("x", function(d) { return _var.width * 5 / 6; })
              .attr("y", function(d) { return _var.height + (_var.isReferenceLineNow() ? 20 : 36); })
              .style("text-anchor", 'middle')
              .style("font-size", '14px')
              .style('fill', "#4568cc")
              .text(function(d) {
                var date = _var.isNow ? _var.xLast.y : referenceObj.y;
                return date == null ? '' : _var.yFormat(date);
              });

            // Draw reference line text
            var refLineHour = elements.selectAll("text.reference-line-hour").data(["reference-line-hour"], function(d) { return d; });
            refLineHour.exit().remove();
            refLineHour = refLineHour.enter().append('text').attr("class", "reference-line-hour").merge(refLineHour);
            refLineHour
              .attr("x", function(d) { return _var.width * 5 / 6; })
              .attr("y", function(d) { return _var.height + 18; })
              .style("text-anchor", 'middle')
              .style("font-size", '10px')
              .style('display', function(d) { return _var.isReferenceLineNow() ? 'none' : 'block'; })
              .style('fill', "#4568cc")
              .text(function(d) { return _var.xFormat(_var.isNow ? _var.xLast.x : referenceObj.x); });

            // Draw reference line bgRect
            var refLineBgRect = elements.selectAll(".reference-line-bg-rect").data(["reference-line-bg-rect"], function(d) { return d; });
            refLineBgRect.exit().remove();
            refLineBgRect = refLineBgRect.enter().insert('rect', ".reference-line-text").attr("class", "reference-line-bg-rect").merge(refLineBgRect);
            refLineBgRect
              .attr("x", function(d) { return _var.width * 5 / 6 - (Math.max(refLineText.node().getBBox().width, refLineHour.node().getBBox().width) + 20) / 2; })
              .attr("y", function(d) { return _var.height + 1; })
              .attr("width", function(d) { return Math.max(refLineText.node().getBBox().width, refLineHour.node().getBBox().width) + 20; })
              .attr("height", function(d) { return _var.margin.bottom; })
              .attr("fill", "#F7F7F7")

            // Update reference line with real time data
            var setTimeIntervalFunction = null;
            var stopNow = function() { clearInterval(setTimeIntervalFunction); }
            var startNow = function() {

              // Stop real time before starting the interval
              stopNow();

              // Set time interval
              setTimeIntervalFunction = setInterval(function() {

                // Draw reference line
                refLine.style('stroke', function(d) { return _var.isReferenceLineNow() ? _var.referenceLineColor(d) : '#bbb'; })

                // Draw reference line text
                refLineText
                  .attr("y", function(d) { return _var.height + (_var.isReferenceLineNow() ? 20 : 36); })
                  .style('fill', "#4568cc")

                // Draw reference line text
                refLineHour
                  .style('fill', "#4568cc")
                  .style('display', function(d) { return _var.isReferenceLineNow() ? 'none' : 'block'; })

              }, 1000);
            }

            // Start timer to color lines / text
            startNow();

          }

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','components','data'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function(_) { return main('run'); };

  return main;
};

// Module declaration
gViz.viz.lineAreaChart.events = function() {
  "use strict";

  // Get attributes values
  var _var       = null;
  var action     = 'mouseover';
  var components = null;
  var node       = null;
  var mouse      = null;
  var origin     = 'node';
  var propAttrs  = ['title','body'];

  // Validate attributes
  var validate = function (step) {

    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Run code
        case 'run':

          // Set groups and bisect element
          var groups = _var.g.selectAll(".chart-elements").selectAll(".element-group");
          var points = _var.g.selectAll(".chart-elements").selectAll(".element-group").selectAll('.point');
          var lines  = _var.g.selectAll(".chart-elements").selectAll(".element-group").selectAll('.line');
          var bisector = d3.bisector(function (d) { return d.parsedX; }).left;

          switch (action) {

            case 'mouseover':

              // Fade other groups
              groups.transition()
                .style('opacity', function(g) { return g === node || g.id === node._parent.id ? 1 : 0.2; })

              // Add style to lines
              lines.transition()
                .style("filter", function(g) { return g === node || g.id === node._parent.id ? "url(#"+_var.shadowId+")" : ""; })
                .style('opacity', function(g) { return g === node || g.id === node._parent.id ? 1 : 0.2; })

              // Add style to points
              points.transition()
                .style("filter", function(g) { return g === node ? "url(#"+_var.shadowId+")" : ""; })
                .style('opacity', function(g) { return g.parsedX === node.parsedX || g === node || g._parent.id === node._parent.id ? 1 : 0.2; })

              // Get x and y values
              var x = _var.x(node.parsedX) + (_var.xIsDate || _var.xIsNumber ? 0 : _var.x.bandwidth()/2)
              var y = _var.y(+node.y);
              var z = _var.pointSize(node);

              // Get left and top positions
              var left = _var.wrap.node().getBoundingClientRect().left +_var.margin.left + x;
              var top  = _var.wrap.node().getBoundingClientRect().top + _var.margin.top + y - z - 5;

              // Initialize tooltip obj function
              var _getTooltipObj = function(n) {

                // Initialize tooltip object
                var tooltipObj = {};

                // Set parent n attributes to tooltip obj
                Object.keys(n._parent).forEach(function(k) { tooltipObj[k] = n._parent[k]; });

                // Set n attributes to tooltip obj
                Object.keys(n).forEach(function(k) { tooltipObj[k] = n[k]; });

                // Set x, y and z values with format
                tooltipObj.x = _var.xFormat(n.x);
                tooltipObj.y = _var.yFormat(n.y);
                tooltipObj.color = _var.pointColor(n);

                return tooltipObj;

              }

              // Set tooltip object
              var tooltipObj = _getTooltipObj(node);

              // Set tooltip header as title
              if(_var.data.tooltip.header != null) {
                _var.data.tooltip.header.forEach(function(p) {
                  _var.data.tooltip.title.unshift(gViz.shared.helpers.text.replaceVariables(p, {}));
                });
              }

              // Set tooltip component
              var tooltip = _var.data.tooltip == null ? {} : _var.data.tooltip;
              gViz.shared.visualComponents.tooltip()
                ._var(_var)
                .body(tooltip.body != null ? tooltip.body : "")
                .borderColor(tooltip.borderColor != null ? tooltip.borderColor : tooltipObj.color)
                .backgroundColor(tooltip.backgroundColor != null ? tooltip.backgroundColor : tooltipObj.color)
                .hasImg(tooltip.hasImg === true)
                .left(left)
                .muted(tooltip.muted != null && tooltip.muted === true)
                .obj(tooltipObj)
                .top(top)
                .title(tooltip.title != null ? tooltip.title : "")
                .run();

              break;

            case 'mouseout':

              // Fade other groups
              groups.transition().style('opacity', 1)

              // Add style to lines
              lines.transition()
                .style("filter", "")
                .style('opacity', 1);

              // Add style to points
              points.transition()
                .style("filter", '')
                .style('opacity', 1);

              // Set bars component
              gViz.shared.visualComponents.tooltip()
                ._var(_var)
                .action("hide")
                .run();

              break;

          }

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','action','components','node','mouse','origin'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) {
    return main('run');
  };

  return main;
};

// Module declaration
gViz.viz.lineAreaChart.initialize = function() {
  "use strict";

  // Get attributes values
  var _id       = null;
  var _var      = null;
  var animation = 100;
  var container = null;
  var colors = [{value:0, color:"#4ad2a1"},{value:150, color:"#4ad2a1"},{value:300, color:"#3d62cb"},{value:500, color:"#3d62cb"},{value: 600, color:"#fa5050"}];
  var data      = [];
  var isNow     = true;
  var margin    = { top: 10, right: 10, bottom: 10, left: 10 };
  var opacity = 0.4;
  var timespan = 6000;
  var touchEvents = null;
  var wheelZoom = {};
  var xZoom = {};
  var yZoom = {};
  var zoomMode = 'horizontal';
  var zoomLevel = 1;

  // Validate attributes
  var validate = function(step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function(step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Initialize variables
          if (!_var) { _var = {}; }
          _var._id = _id;
          _var.animation = animation;
          _var.colors = (colors || [{value:0, color:"#4ad2a1"},{value:150, color:"#4ad2a1"},{value:300, color:"#3d62cb"},{value:500, color:"#3d62cb"},{value: 600, color:"#fa5050"}]).sort(function(a,b) { return d3.descending(+a.value, +b.value); });;
          _var.isNow = isNow;
          _var.margin = margin;
          _var.opacity = opacity || 0.4;
          _var.timespan = timespan;
          _var.touchEvents = touchEvents;
          _var.zoomMode = zoomMode;
          _var.zoomLevel = zoomLevel;
          _var.rebuild = { line: true, area: true };

          // Id for shadows
          _var.shadowId = "vis-shadow-"+(Math.floor(Math.random() * ((1000000000 - 5) + 1)) + 5);

           // Get container
          _var.container = {
            selector: container,
            d3: d3.select(container),
            el: ((typeof container === 'string' || container instanceof String) ? container : d3.select(container).node()),
            outerWrapper: d3.select(container).closest('.gViz-outer-wrapper'),
            outerWrapperClientRect: d3.select(container).closest('.gViz-outer-wrapper').node().getBoundingClientRect(),
            dimensions: {}
          };

          // Map data and get labels
          _var.data = _var._data = data;

          // Wheel Zoom
          if (_var.wheelZoom == null) { _var.wheelZoom = {}; }
          Object.keys(wheelZoom).forEach(function (key) { return _var.wheelZoom[key] = wheelZoom[key]; });

          // X Zoom
          if (_var.xZoom == null) { _var.xZoom = {}; }
          Object.keys(xZoom).forEach(function (key) { return _var.xZoom[key] = xZoom[key]; });

          // Y Zoom
          if (_var.yZoom == null) { _var.yZoom = {}; }
          Object.keys(yZoom).forEach(function (key) { return _var.yZoom[key] = yZoom[key]; });

          break;
      }
    }

    return _var;
  };

  // Expose global variables
  ['_id','_var','animation','container','colors','data','isNow','margin','opacity','timespan','touchEvents','wheelZoom','xZoom','yZoom','zoomMode','zoomLevel'].forEach(function(key) {

    // Attach variables to validation function
    validate[key] = function(_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return validate;
    };

    // Attach variables to main function
    return main[key] = function(_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return main;
    };
  });

  // Execute the specific called function
  main.run = function(_) { return main('run'); };

  return main;
};

// Module declaration
gViz.viz.lineAreaChart.misc = function() {
  "use strict";

  // Get attributes values
  var _var       = undefined;
  var action     = "start";
  var components = {};

  // Validate attributes
  var validate = function(step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function(step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          switch (action) {

            // First action to be done
            case 'start':

              /* -- Visualization Title -- */
              // Has title flag
              var hasTitle = _var.data.title != null && _var.data.title !== "";
              _var.container.dimensions.title = hasTitle ? 35 : 0;

              // Draw title wrapper
              var titleWrapper = _var.container.outerWrapper.selectAll(".gviz-title-wrapper").data(["gviz-title-wrapper"]);
              titleWrapper.exit().remove();
              titleWrapper = titleWrapper.enter().insert('div', ":first-child").attr('class', "gviz-title-wrapper").merge(titleWrapper);
              titleWrapper
                .style('width', '100%')
                .style('height', '30px')
                .style('line-height', '30px')
                .style('margin', '0px 0px 5px 0px')
                .style('padding', '0px 10px')
                .style('oveflow', 'hidden')
                .style('white-space', 'nowrap')
                .style('text-overflow', 'ellipsis')
                .style('background-color', '#eee')
                .style('color', '#666')
                .style('font-size', '15px')
                .style('font-weight', 'lighter')
                .style('display', hasTitle ? 'block' : 'none')
                .html(_var.data.title)

              /* -- Select -- */
              // Move select according to title values
              var hasSelect = _var.data.select != null && _var.data.select.isVisible != null && _var.data.select.isVisible === true;
              _var.container.outerWrapper.selectAll(".select-wrapper")
                .style('position', 'absolute')
                .style('top', _var.container.dimensions.title + 'px')
                .style('right', '0px')
                .style('z-index', '1006')
                .style('display', hasSelect ? 'block' : 'none')

              /* -- Axis and Legend -- */
              // Set axis string
              _var.yTitle = (_var.data.y != null && _var.data.y.title != null && _var.data.y.title !== "" ? _var.data.y.title : "");
              _var.xTitle = (_var.data.x != null && _var.data.x.title != null && _var.data.x.title !== "" ? _var.data.x.title : "");

              // Get hasAxisTitle flag and add size to dimensions hash
              var hasAxisTitle = false;
              _var.container.dimensions.axisTitle = hasAxisTitle && hasSelect ? 35 : 0;

              // Initialize string
              var string = "";
              var stringObj = {};

              // Iterate over data to get legend values
              _var.data.data.forEach(function(d, i) {

                // Get color
                var fillColor = d.color != null && d.color !== "" ? d.color : (d.stopColor != null && d.stopColor !== "" ? d.stopColor : "#FFF");
                var strokeColor = d.color != null && d.color !== "" ? d.color : (d.stopColor != null && d.stopColor !== "" ? d.stopColor : "#FFF");
                var legend = _var.data.legend != null && _var.data.legend.text != null ? _var.data.legend.text : "{{name}}";
                var legendStr = "";

                // Add rect for objw
                legendStr += "<span class='legend-content' >";
                legendStr += "<span class='rect' style='background-color:"+fillColor+" ; border-top: 2px solid "+strokeColor+";'></span><span class='name'>";
                legendStr += gViz.shared.helpers.text.replaceVariables(legend, d);
                legendStr += "</span>";
                legendStr += "</span>";

                // If the legend str wasnt computed, add to legend
                if(stringObj[legendStr] == null) {
                  stringObj[legendStr] = true;
                  string += legendStr;
                }

              });

              // Set margin left and display style
              var hasLegend = _var.data.legend != null && _var.data.legend.isVisible != null && _var.data.legend.isVisible === true;
              var legendWrapper = _var.container.outerWrapper.selectAll(".gviz-legend-wrapper").data(hasLegend ? [true] : []);
              legendWrapper.exit().remove();
              legendWrapper = legendWrapper.enter().append("div").attr('class', "gviz-legend-wrapper").merge(legendWrapper);
              legendWrapper
                .style('width', hasSelect && !hasAxisTitle ? 'calc(100% - 165px)' : '100%')
                .style('height', 'auto')
                .style('max-height', '60px')
                .style('line-height', '30px')
                .style('margin', '0px 0px 5px 0px')
                .style('padding', '0px 10px')
                .style('overflow', 'hidden')
                .style('color', '#666')
                .style('font-size', '13px')
                .style('z-index', "1005")
                .style('position', 'absolute')
                .style('top', (_var.container.dimensions.title + _var.container.dimensions.axisTitle) + "px")
                .style('left', "0px")
                .style('display', hasLegend ? 'block' : 'none')
                .each(function(d) {

                  // Set margin left and display style
                  var legend = d3.select(this).selectAll(".gviz-legend").data(hasLegend ? ["gviz-legend"] : []);
                  legend.exit().remove();
                  legend = legend.enter().insert('p', ":first-child").attr('class', "gviz-legend").merge(legend);
                  legend
                    .style('width', '100%')
                    .style('height', 'auto')
                    .html(string);

                });

              legendWrapper
                .on('mouseover', function() { d3.select(this).classed('gviz-legend-hover', true); })
                .on('mouseout', function() { d3.select(this).classed('gviz-legend-hover', false); });

              // Set legend dimension
              _var.container.dimensions.legend = hasLegend ? (legendWrapper.node().getBoundingClientRect().height + 5) : 0;

              // Update container _id, height and client bound rect
              _var.container.d3
                .attr('data-vis-id', _var._id)
                .style('height', (_var.container.outerWrapperClientRect.height - (_var.container.dimensions.title + _var.container.dimensions.axisTitle + _var.container.dimensions.legend)) + "px")
                .style('top', (_var.container.dimensions.title + _var.container.dimensions.axisTitle + _var.container.dimensions.legend) + 'px')

              // Update grid background position
              _var.container.outerWrapper.selectAll('.grid-background')
                .style('top', (_var.container.dimensions.title + _var.container.dimensions.axisTitle + _var.container.dimensions.legend) + 'px')

              // Define height and width
              var containerClientRect = _var.container.d3.node().getBoundingClientRect();
              _var.height = containerClientRect.height - (_var.margin.top + _var.margin.bottom);
              _var.width = containerClientRect.width - (_var.margin.left + _var.margin.right);

              // NO DATA AVAILABLE
              if (_var.data.data == null || _var.data.data.length === 0) {
                _var.container.d3.html("<h5 style='line-height: "+(containerClientRect.height)+"px; text-align: center;'>NO DATA AVAILABLE</h5>");
              } else {
                _var.container.d3.selectAll("h5").remove();
              }

              break;

            // Last action to be done
            case 'end':

              break;
          }
          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','action','components'].forEach(function(key) {

    // Attach variables to validation function
    validate[key] = function(_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return validate;
    };

    // Attach variables to main function
    return main[key] = function(_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function(_) { return main('run'); };

  return main;
};

// Module declaration
gViz.viz.lineAreaChart.style = function() {
  "use strict";

  // Get attributes values
  var _var = undefined;
  var animation = 900;

  // Validate attributes
  var validate = function (step) {

    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          _var.getZoomOffset = function() {
            if(_var.zoomLevel === 1) { return 3600000; }
            else { return 3600000; }
          }

          _var.isReferenceLineNow = function(d) {
            return _var.isNow && (new Date() - _var.xLast.parsedX) <= _var.getZoomOffset();
          }

          // Reference line color
          _var.referenceLineColor = function(d) {
            return "#4568cc";
            // if(_var.yMax >= 600) { return "#fa5050"; }
            // else if(_var.yMax < 600 && _var.yMax >= 500) { return d3.scaleLinear().domain([600,500]).range(["#fa5050","#3d62cb"])(_var.yMax); }
            // else if(_var.yMax < 500 && _var.yMax >= 300) { return "#3d62cb"; }
            // else if(_var.yMax < 300 && _var.yMax >= 100) { return d3.scaleLinear().domain([600,500]).range(["#3d62cb","#4ad2a1"])(_var.yMax); }
            // else { return "#4ad2a1"; }
          }

          _var.referenceLineX = function(d) {
            if(_var.referenceLine != null) {
              if(_var.referenceLine.status === 'now') { return _var._x(_var.xData[_var.xData.length-1].parsedX); }
              else if(_var.x(_var.referenceLine.value) != null && !isNaN(_var._x(_var.referenceLine.value))) { return _var._x(_var.referenceLine.value); }
              else { return 0; }
            } else { return 0; }
          }

          _var.referenceLineIsNow = function(d) {
            return _var.referenceLine != null && _var.referenceLine.status === 'now';
          }

          _var.referenceLineDisplay = function(d) {
            if(_var.referenceLine != null) {
              if(_var.referenceLine.status === 'now') { return true; }
              else if(_var._x(_var.referenceLine.value) != null && !isNaN(_var._x(_var.referenceLine.value))) { return true; }
              else { return false; }
            } else { return false; }
          }

          // Set area color
          _var.areaColor = function(d) {
            return d.areaColor != null && d.areaColor !== "" ? d.areaColor : _var.lineColor(d);
          }

          // Set line width
          _var.lineWidth = function(d) {
            return d.strokeWidth != null && !isNaN(d.strokeWidth) ? d.strokeWidth + "px" : "2px";
          }

          // Set line color
          _var.lineColor = function(d) {

            if(d.color != null && d.color !== "") { return d.color; }
            else if(d.startColor != null && d.startColor !== "" && d.stopColor != null && d.stopColor !== "") {

              // Set wrapper gradient
              gViz.shared.visualComponents.gradient()
                ._var(_var)
                .id("gradient-"+gViz.shared.helpers.text.removeSpecial(d.id+d.name))
                .colors([
                  { offset:"0%", color: d.startColor },
                  { offset:"100%", color: d.stopColor }
                ])
                .direction('horizontal')
                .gType('linear')
                .wrap(_var.defs)
                .run();

              return 'url("' + "#gradient-"+gViz.shared.helpers.text.removeSpecial(d.id+d.name) + '")';
            } else { return "#666"; }
          }

          // Set stroke style function
          _var.lineStyle = function(d) {
            var strokeStyle = d.strokeStyle != null ? d.strokeStyle : "solid";
            if(strokeStyle === "dotted") { return "2,2"; }
            else if(strokeStyle === "dashed") { return "7,3"; }
            else { return "0,0"; }
          }

          // Set point color
          _var.pointColor = function(d) {
            if(d.pointColor != null) { return d.pointColor; }
            else if(d._parent.pointColor != null) { return d._parent.pointColor; }
            else if(d._parent.color != null) { return d._parent.color; }
            else { return "#333"; }
          }

          // Set point color
          _var.pointSize = function(d) {
            return d.pointSize != null ? d.pointSize : (d._parent.pointSize != null ? d._parent.pointSize : 3);
          }

          _var.pointX = function(d) {
            return _var.x(d.parsedX) + (_var.xIsDate || _var.xIsNumber ? 0 : _var.x.bandwidth()/2);
          }

          _var.pointY = function(d) {
            return _var.y(+d.y);
          }

          // Set shape path for node
          _var.pointPath = function(d) {

            // Get radius
            var r = _var.pointSize(d);
            var dr = r*2;
            var x  = _var.pointX(d);
            var y  = _var.pointY(d);
            var shape = d.pointShape != null ? d.pointShape : (d._parent.pointShape != null ? d._parent.pointShape : "circle");

            // For each shape style
            switch(shape) {

              // Set rect shape
              case "rect":
                return "M " + ((x != null ? x : 0) - r) + " " + ((y != null ? y : 0) - r) + " " +
                       "l " + dr + ", 0 " +
                       "l 0 , " + dr + " " +
                       "l " + (-dr) + ", 0 " + "Z";
                break;

              // Set diamond shape
              case 'diamond':
                return "M " + (x != null ? x : 0) + " " + ((y != null ? y : 0) - r) + " " +
                       "l " + r + ", " + r + " " +
                       "l " + (-r) + ", " + r + " " +
                       "l " + (-r) + ", " + (-r) + " " + "Z";
                break;

              // Set triangle up shape
              case 'triangle-up':
                return "M " + (x != null ? x : 0) + " " + ((y != null ? y : 0) - r) + " " +
                       "l " + r + ", " + dr + " " +
                       "l " + (-dr) + ", " + 0 + " " + "Z";
                break;

              // Set triangle down shape
              case 'triangle-down':
                return "M " + ((x != null ? x : 0) - r) + " " + ((y != null ? y : 0) - r) + " " +
                       "l " + dr + ", 0 " +
                       "l " + (-r) + ", " + dr + " " + "Z";
                break;

              // Set circle shape
              default:
                return "M " + (x != null ? x : 0) + " " + (y != null ? y : 0) + " " +
                       "m -" + r + ", 0 " +
                       "a " + r + "," + r + " 0 1,0 " + ( r*2) + ",0 " +
                       "a " + r + "," + r + " 0 1,0 " + (-r*2) + ",0 ";
                break;
            }
          }

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var', 'animation'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function(_) { return main('run'); };

  return main;
};

// Module declaration
gViz.viz.lineAreaChart.xScale = function() {
  "use strict";

  // Get attributes values
  var _var = undefined;
  var data = [];

  // Validate attributes
  var validate = function (step) {

    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Set format
          _var.xIsDate = (_var.data.x != null && _var.data.x.type === 'time' && _var.data.x.inFormat != null && _var.data.x.outFormat != null);
          _var.xIsNumber = (_var.data.x != null && _var.data.x.type === 'number' && _var.data.x.format != null);
          var xFmt = _var.xIsDate ? 'date' : 'number';
          _var.xFormat = gViz.shared.helpers[xFmt].parseFormat(_var.data == null ? null : _var.data.x);

          // Define scales
          var x = {};
          if(_var.xIsDate) { x = d3.scaleTime().range([0, _var.width]); }
          else { x = d3.scaleLinear().range([0, _var.width]); }

          // Define aux variables
          var min = null, max = null, diff = null;

          // Initialize domains
          _var.xDomain = {};
          var xDomain = [];
          var _data = {};

          // Set x last point
          _var.xLast = null;

          // Get domains
          data.forEach(function(d) {
            d.values.forEach(function(v,i) {

              // Set parent
              v._parent = d;

              // Date value
              if(_var.xIsDate) {

                // Parse values
                v.parsedX = d3.timeParse(_var.data.x.inFormat)(v.x);
                v.formattedX = _var.xFormat(v.x);

                // Add to _data hash
                _data[(+v.parsedX).toString()] = +v.parsedX;

                // Set domain
                if(min == null || min > +v.parsedX) { min = +v.parsedX; }
                if(max == null || max < +v.parsedX) { max = +v.parsedX; }

                // Set last x
                if(i === d.values.length-1 && (_var.xLast == null || _var.xLast.parsedX < v.parsedX)) { _var.xLast = v; }

              // Number values
              } else {

                // Parse values
                v.parsedX = +v.x;
                v.formattedX = _var.xFormat(v.x);

                // Add to _data hash
                _data[(+v.parsedX).toString()] = +v.parsedX;

                // Set domain
                if(min == null || min > +v.x) { min = +v.x; }
                if(max == null || max < +v.x) { max = +v.x; }

                // Set last x
                if(i === d.values.length-1 && (_var.xLast == null || _var.xLast.parsedX < v.parsedX)) { _var.xLast = v; }

              }

            });
          });

          // Sort values
          data.forEach(function(d) { d.values = d.values.sort(function(a,b) { return d3.ascending(a.parsedX, b.parsedX); }); });

          // Update _data hash to array
          _var.xData = Object.keys(_data).map(function(k) { return { parsedX: +k }; }).sort(function(a,b) { return d3.ascending(a.parsedX, b.parsedX); });

          // Check for default values
          if(isNaN(min)) { min = 0; }
          if(isNaN(max)) { max = 1; }

          // Get diff
          var diff = Math.abs(max - min) === 0 ? (max === 0 && min === 0 ? 0.1 : Math.abs(max * 0.1)) : 0;

          // Get x bounds
          _var.xBounds = _var.xIsDate ? [min,max] : [(min == 0 ? min : min - diff), max + diff];

          // Set xLimits if not set
          _var.xLimits = _var.xBounds;

          // Set x domain
          x.domain(_var.xBounds);

          // Copy scale
          if(_var.x == null || _var._x == null) { _var._x = _var.x = x; }

          // Set range
          _var.x.range([0, _var.width]);
          _var._x.range([0, _var.width]);

          // Get distance
          var zoomInfo = gViz.shared.data.lineChart.zoomLevelInfo(_var.zoomLevel);
          var hourInterval = 3600;
          _var.distance = zoomInfo.interval * _var.width / hourInterval;

          // Get x axis ticks
          var tickSpace = _var.data.x != null && _var.data.x.tickSpace != null && !isNaN(+_var.data.x.tickSpace) ? +_var.data.x.tickSpace : 100;
          var bins = d3.max([3, parseInt(_var.width / tickSpace, 10)]);

          // Define axis
          _var.xAxis = d3.axisBottom(_var._x).ticks(bins).tickPadding(10).tickFormat(gViz.shared.helpers.date.multiFormat);

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','data'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) {
    return main('run');
  };

  return main;
};

// Initialize the visualization class
gViz.viz.lineAreaChart.xZoom = function() {
  "use strict";

  // Get attributes values
  var _var = null;
  var components = function (d) { return console.log("components"); };

  // Validate attributes
  var validate = function validate(step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function main(step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Run
        case 'run':

          // Set combined data from all series
          var data = _var.xData;
          var d3Zoom = null;

          // Update zoom function
          _var.xZoom.updateZoomMode = function() {

            // D3 zoom behaviour
            if(_var.xZoom.behaviour == null) {
              _var.xZoom.behaviour = d3.zoom()
                .duration(0)
                .on("start", _var.xZoom.start)
                .on("end", _var.xZoom.end)
                .on("zoom", _var.xZoom.callback);

              _var.wrap
                .on('.zoom', null)
                .call(_var.xZoom.behaviour)
                .on("dblclick.zoom", null)
            } else {
              _var.xZoom.behaviour
                .on("start", _var.xZoom.start)
                .on("end", _var.xZoom.end)
                .on("zoom", _var.xZoom.callback);
            }

            // Get scale
            var width = _var.width - _var.width/6;
            var _width = (_var.isNow ? width : _var.width);
            _var.xZoom.cached = _var.xZoom.cached || {};

            // Initialize scale and translate tranformations
            _var.xZoom.transform = _var.xZoom.transform || {};
            _var.xZoom.transform.k = _var.width / _var.distance;
            _var.xZoom.transform.y = _var.margin.top;

            // Apply x transformations
            if(_var.xZoom.transform.x == null || _var.isNow) {
              _var.xZoom.transform.x = -(_var.x(_var.xBounds[1]) * _var.xZoom.transform.k) + _width;
              _var.wrap
                .call(
                  _var.xZoom.behaviour.transform,
                  d3.zoomIdentity.scale(_var.xZoom.transform.k).translate(_var.xZoom.transform.x, _var.xZoom.transform.y)
                )
            } else if(_var.wheelZoom.enable != null) {

              // Get x transformation depending on zoom level
              _var.xZoom.transform.x = _var.xZoom.cached[_var.zoomLevel] != null ? _var.xZoom.cached[_var.zoomLevel].x : _var.xZoom.cached[1].x * _var.xZoom.transform.k + _width * (1 - _var.xZoom.transform.k) / 2;

              _var.wrap
                .call(
                  _var.xZoom.behaviour.transform,
                  d3.zoomIdentity.scale(_var.xZoom.transform.k).translate(_var.xZoom.transform.x, _var.xZoom.transform.y)
                )
            } else {
              _var.wrap.call(_var.xZoom.behaviour.translateTo, _var.xZoom.transform.x, _var.xZoom.transform.y)
            }
          }

          // On zoom start function
          _var.xZoom.start = function () {
            _var.wrap.classed("grabbing", true);
          };

          // On zoom end function
          _var.xZoom.end = function () {

            if(_var.xZoom.maxScaled == null) {

              // Prevent dragTimeout function to fire
              if(_var.xZoom.dragTimeout != null) { clearTimeout(_var.xZoom.dragTimeout); }

              // Remove grabbing cursor
              _var.wrap.classed("grabbing", false);

              // Store previous zoom
              _var.xPrevZoom = JSON.parse(JSON.stringify(_var.xZoom.transform));

              // Update clip rect position
              _var.defs.selectAll(".bg-clip").selectAll("rect.bg-rect").transition()
                .duration(_var.animation)
                .ease(d3.easeLinear)
                  .style('x', 0);

              // Update axis
              _var.x_axis.transition()
                .duration(_var.animation)
                .ease(d3.easeLinear)
                .call(_var.xAxis.scale(_var._x));

              // Update elements
              _var = gViz.viz.lineAreaChart.elements()
                ._var(_var)
                .components(gViz.viz.lineAreaChart)
                .run();

              _var.gClip.transition().duration(_var.animation).ease(d3.easeLinear)
                .attr('transform', "translate(0,"+_var.margin.top+")");

              if(d3Zoom != null) {
                if(d3Zoom.type === 'wheel') {

                  // Disable fetch new data
                  _var.xZoom.enable = false;
                  _var.wheelZoom.enable = true;
                  _var.wheelZoom.direction = d3Zoom.direction;

                  // Disable zooming
                  _var.xZoom.behaviour.on("zoom", null);

                  // Set can none to false
                  _var.canNone = false;

                  // Run zoom function
                  _var.wheelZoom.fn(_var.wheelZoom.x, d3Zoom.direction);

                } else {

                  if(d3Zoom.direction === 'prev') {

                    // Get point
                    var _t = data[0].parsedX;

                    // Set can none to false
                    _var.canNone = false;

                    // Run zoom function
                    _var.xZoom.fn(_t, "prev");

                  } else {

                    // Set can none to false
                    _var.canNone = false;

                    // Get point
                    var t = data[data.length - 1].parsedX;

                    // Run zoom function
                    _var.xZoom.fn(t, "next");

                  }
                }
              } else {

                // Remove zoom enable flag
                if(_var.wheelZoom.enable != null) {
                  setTimeout(function() {

                    // Remove zoom enable flag
                    _var.wheelZoom.enable = null;

                  }, _var.animation);
                }
              }
            } else {

              _var.xZoom.behaviour
                .on("end", _var.xZoom.endMaxScaled)
                .on("zoom", _var.xZoom.callbackMaxScaled);

              // Get x transformation depending on zoom level
              _var.wrap
                .call(
                  _var.xZoom.behaviour.transform,
                  d3.zoomIdentity.scale(_var.xZoom.transform.k).translate(_var.xZoom.transform.x / _var.xZoom.transform.k, _var.xZoom.transform.y / _var.xZoom.transform.k)
                )

            }

            // Reset control variables
            d3Zoom = null;
            _var.xZoom.maxScaled = null;

          };

          _var.xZoom.callbackMaxScaled = function () {
          }

          _var.xZoom.endMaxScaled = function () {
            _var.xZoom.prevTransform = JSON.parse(JSON.stringify(_var.xZoom.transform));
            _var.xZoom.behaviour
              .on("end", _var.xZoom.end)
              .on("zoom", _var.xZoom.callback);
          }

          // On zoom function
          _var.xZoom.callback = function () {

            if(_var.xZoom.maxScaled == null && d3Zoom == null && d3.event != null && d3.event.transform != null) {

              if(_var.wheelZoom.enable == null && ((_var.zoomLevel == -2 && _var.xZoom.prevTransform.k > d3.event.transform.k) || (_var.zoomLevel == 5 && _var.xZoom.prevTransform.k < d3.event.transform.k))) {
                _var.xZoom.maxScaled = true;
                return;
              }

              // Store event
              var event = d3.event;

              // Handle zoom
              if(event.sourceEvent != null && _var.xZoom.prevTransform.k !== event.transform.k) {

                // Cache transform for each scale
                if(_var.zoomLevel == 1) {
                  _var.xZoom.cached[_var.zoomLevel] = JSON.parse(JSON.stringify(_var.xZoom.transform));
                }

                // Get zooming direction
                var direction = d3.event.transform.k < _var.xZoom.prevTransform.k ? 'down' : 'up';

                // Set start and end points on screen on zoom start
                if(d3Zoom == null) {
                  _var.wheelZoom.x = {};
                  _var.wheelZoom.x.start = _var.x.invert(-d3.event.transform.x / d3.event.transform.k),
                  _var.wheelZoom.x.end = _var.xLast.parsedX < _var.x.invert(-d3.event.transform.x / d3.event.transform.k + _var.width / d3.event.transform.k) ? _var.xLast.parsedX : _var.x.invert(-d3.event.transform.x / d3.event.transform.k + _var.width / d3.event.transform.k);
                  _var.wheelZoom.x._end = _var.x.invert(-d3.event.transform.x + _var.width / d3.event.transform.k);
                }

                // Update zoom direction
                var zoomLevel = _var.zoomLevel + (direction === 'up' ? (1) : (-1));

                // Bounded zoom level
                if(zoomLevel <= -2) { zoomLevel = -2; }
                else if(zoomLevel >= 5) { zoomLevel = 5; }

                // Reset transform event
                var zoomInfo = gViz.shared.data.lineChart.zoomLevelInfo(zoomLevel);
                _var.xZoom.transform.k = zoomInfo.zoom;

                // Set zoom type and direction to be used on drag end
                d3Zoom = { type: 'wheel', direction: direction };

                // Show loading
                window.gViz.shared.helpers.loading.show('.loading-wrapper');

                return;

              }

              // Get scale from transformation
              _var._x = d3.event.transform.rescaleX(_var.x);

              // Set width
              var width = _var.width - _var.width/6;
              if(_var.isNow) {
                width = _var.width - _var.width/6;
              } else if(_var._x(_var.xLast.parsedX) > _var.width - _var.width/6 && _var._x(_var.xLast.parsedX) < _var.width) {
                width = _var._x(_var.xLast.parsedX);
              } else {
                width = _var.width;
              }

              // Get bounds and domain
              var _bounds = [_var._x(_var.xBounds[0]), _var._x(_var.xBounds[1])];
              var _domain = _var._x.domain();

              // Dragging behaviour
              if(event.sourceEvent != null && _var.xZoom.prevTransform.k === event.transform.k) {

                // Update cached with translations
                var deltaX = _var.xZoom.prevTransform.x - event.transform.x;
                Object.keys(_var.xZoom.cached).forEach(function(k) {
                  _var.xZoom.cached[k].x -= deltaX / event.transform.k;
                });

                // // Set is now
                if(parseInt(_var.x(_var.xBounds[1]) * _var.xZoom.transform.k + _var.xZoom.transform.x) !== parseInt(width) && _var.isNow && _var.canNone) {

                  // Run zoom function
                  _var.xZoom.fn((new Date()).getTime(), "none");
                  _var.isNow = false;
                  _var.canNone = false;

                } else if(_var.x(_var.xBounds[1]) * _var.xZoom.transform.k + _var.xZoom.transform.x === width && _var.isNow !== true) {

                  _var.isNow = true;
                  _var.canNone = false;

                }

                // If there is new data to fetch and this function is enable
                if (_domain[0] < data[0].parsedX && data[0].parsedX >= _var.xBounds[0] && _var.xZoom.enable && _domain[0] < _var.xLimits[0]) {

                  // Disable fetch new data
                  _var.xZoom.enable = false;

                  // Disable zooming
                  _var.xZoom.behaviour.on("zoom", null);

                  // Set zoom type and direction to be used on drag end
                  d3Zoom = { type: 'drag', direction: 'prev' };

                  // Show loading
                  window.gViz.shared.helpers.loading.show('.loading-wrapper');

                // If there is new data to fetch and this function is enable
                } else if (
                  (_var.isNow !== true || ((new Date()).getTime() - _var.xLast.parsedX) >= _var.getZoomOffset()) && _var.canNone === true &&
                  +_domain[1] > _var.xLast.parsedX &&
                  _var.xLast.parsedX <= _var.xBounds[1] &&
                  _var.xZoom.enable &&
                  +_domain[1] > _var.xLimits[1] &&
                  ((_var.lastXPosition != null && _var._x(_var.xLast.parsedX) < _var.lastXPosition && _var._x(_var.xLast.parsedX) < _var.width && _var.lastXPosition > _var.width) || (_var._x(_var.xLast.parsedX) < _var.width && _var._x(_var.xLast.parsedX) < width) )
                ) {

                  // Disable fetch new data
                  _var.xZoom.enable = false;

                  // Disable zooming
                  _var.xZoom.behaviour.on("zoom", null);

                  // Set zoom type and direction to be used on drag end
                  d3Zoom = { type: 'drag', direction: 'next' };

                  // Show loading
                  window.gViz.shared.helpers.loading.show('.loading-wrapper');

                }
              }

              if (_bounds[1] < width || _bounds[0] >= 0) {

                // Reset event transform
                event.transform.x = _var.xZoom.transform.x;

                // Get scale from transformation
                _var._x = event.transform.rescaleX(_var.x);

              }

              // Store previous transformation
              _var.xZoom.transform.k = event.transform.k;
              _var.xZoom.transform.x = event.transform.x;

              _var.canNone = true;
              _var.lastXPosition = _var._x(_var.xLast.parsedX);

              // Dragging behaviour
              if(event.sourceEvent != null && _var.xZoom.prevTransform.k === d3.event.transform.k) {

                // Update axis
                _var.x_axis.call(_var.xAxis.scale(_var._x));

                // Update transform only, not the elements
                var xTransform = _var.isNow ? 0 : _var.xZoom.transform.x - (_var.xPrevZoom == null ? _var.width/6 : _var.xPrevZoom.x);
                _var.gClip.attr('transform', 'translate('+(xTransform)+","+_var.margin.top+")");
                _var.defs.selectAll(".bg-clip").selectAll("rect.bg-rect").style('x', -xTransform);

                // If the drag stops after 500 seconds, fires the end zoom function
                if(_var.xZoom.dragTimeout != null) { clearTimeout(_var.xZoom.dragTimeout); }
                _var.xZoom.dragTimeout = setTimeout(function() {
                  _var.xZoom.end();
                }, 500);

              } else {

                // Update axis
                _var.x_axis.transition()
                  .duration(_var.animation)
                  .ease(d3.easeLinear)
                  .call(_var.xAxis.scale(_var._x));

                // Update elements
                _var = gViz.viz.lineAreaChart.elements()
                  ._var(_var)
                  .components(gViz.viz.lineAreaChart)
                  .run();

              }
            }

            // Store previous transformation
            _var.xZoom.prevTransform = JSON.parse(JSON.stringify(d3.event.transform));
          };

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var', "components"].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval("return " + key);
      }
      eval(key + " = _");
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval("return " + key);
      }
      eval(key + " = _");
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) {
    return main('run');
  };

  return main;
};

// Module declaration
gViz.viz.lineAreaChart.yScale = function() {
  "use strict";

  // Get attributes values
  var _var = undefined;
  var data = [];

  // Validate attributes
  var validate = function (step) {

    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Initialize scale
          var y = d3.scaleLinear().range([_var.height, _var.margin.top]);

          // Define aux variables
          _var.yMax = null;
          var min = null,
              max = null,
              diff = null;

          // Get bounds
          data.forEach(function(d) {
            var mean = 0;
            d.values.forEach(function(v, i) {
              if(min == null || min > +v.y) { min = +v.y; }
              if(max == null || max < +v.y) { max = +v.y; }
              if(i === d.values.length-1 && (_var.yMax == null || _var.yMax < +v.y)) { _var.yMax = +v.y; }
              mean += +v.y;
            });
            d.mean = d.values.length === 0 ? 0 : mean / d.values.length;
          });

          // Check for default values
          if(isNaN(min)) { min = 0; }
          if(isNaN(max)) { max = 1; }

          // Store y bounds min from data
          _var.yBoundsData = [min,max];

          // Get mix and max values for axis set from json
          if(_var.data.y != null && _var.data.y.min != null && !isNaN(+_var.data.y.min) && +_var.data.y.min < min) { min = +_var.data.y.min; }
          if(_var.data.y != null && _var.data.y.max != null && !isNaN(+_var.data.y.max) && +_var.data.y.max > max) { max = +_var.data.y.max; }

          // Get diff
          var diff = Math.abs(max - min) === 0 ? (max === 0 && min === 0 ? 0.1 : Math.abs(max * 0.1)) : 0;

          // Set y domain
          var resetY = _var.yBounds != null && _var.yBounds[1] > (max + diff) && !(_var.wheelZoom.enable != null && _var.wheelZoom.enable === true);
          _var.yBounds = [min, resetY ? _var.yBounds[1] : (max + diff)];
          y.domain(_var.yBounds);

          // Set y scale
          _var._y = _var.y = y;

          // Set format
          _var.yFormat = gViz.shared.helpers.number.parseFormat(_var.data == null ? null : _var.data.y);

          // Get x axis ticks
          var bins = d3.max([2, parseInt(_var.height / 30, 10)]);

          // Define y axis
          _var.yAxis = d3.axisLeft(_var._y).ticks(bins).tickPadding(10).tickFormat(_var.yFormat);

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','data'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) {
    return main('run');
  };

  return main;
};

// Initialize the visualization class
gViz.viz.lineAreaChart.zoom = function () {
  "use strict";

  // Get attributes values

  var _var = undefined;
  var components = function components(d) { return console.log("components"); };

  // Validate attributes
  var validate = function validate(step) {

    switch (step) {
      case 'run':
        return true;
      default:
        return false;
    }
  };

  // Main function
  var main = function main(step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Run
        case 'run':

          _var.xZoom.updateZoomMode();

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var', "components"].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval("return " + key);
      }
      eval(key + " = _");
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval("return " + key);
      }
      eval(key + " = _");
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) {
    return main('run');
  };

  return main;
};

// Initialize the helpers class
gViz.shared.helpers.array = {

  // Filter unique values
  unique: function(array, prop) {
    var uniqueValues = {};
    return array.filter(function(d) {
      var flag = uniqueValues[d[prop]] == null;
      uniqueValues[d[prop]] = true;
      return flag;
    });
  },

  // Sort values
  sorted: function(array, prop, order='ascending') {
    return array.sort(function(a,b) { return d3[order](a[prop], b[prop]); });
  }

};

//Function to convert hex format to a rgb color
function rgb2hex(rgb){
  rgb = rgb.match(/^rgba?[\s+]?\([\s+]?(\d+)[\s+]?,[\s+]?(\d+)[\s+]?,[\s+]?(\d+)[\s+]?/i);
  return (rgb && rgb.length === 4) ? "#" +
    ("0" + parseInt(rgb[1],10).toString(16)).slice(-2) +
    ("0" + parseInt(rgb[2],10).toString(16)).slice(-2) +
    ("0" + parseInt(rgb[3],10).toString(16)).slice(-2) : '';
}

// Initialize the helpers class
gViz.shared.helpers.colors = {

  // Initializer main color pallete
  main: d3.scaleOrdinal(["#ea5c84","#ac73dc"]),
  aux: d3.scaleOrdinal(["#FFFFFF","#ac73dc"]),

  // Initializer gray scale color pallete
  gray: d3.scaleOrdinal(["#434343","#666","#999","#aaa","#bbb","#ccc","#ddd","#eee"]),

  //Function to convert hex format to a rgb color
  rgb2hex: rgb2hex,

  // Is dark function
  isDark: function(c) {

    if(d3.color(c) != null) {

      c = rgb2hex(d3.color(c).toString());

      c = c.substring(1);
      // strip #
      var rgb = parseInt(c, 16);
      // convert rrggbb to decimal
      var r = (rgb >> 16) & 0xff;
      // extract red
      var g = (rgb >> 8) & 0xff;
      // extract green
      var b = (rgb >> 0) & 0xff;
      // extract blue
      var luma = (0.2126 * r) + (0.7152 * g) + (0.0722 * b);
      // per ITU-R BT.709
      return luma < 168;

    } else { return true; }
  },

  // Heat colors
  heat: function(d) {
    if(!d || d == null || isNaN(+d)) { return "#FFF"; }
    else if(+d < 30) { return "#10bbaa"; }
    else if(+d >= 30 && +d < 50) { return "#edaf5c"; }
    else if(+d >= 50 && +d < 75) { return "#e16b4d"; }
    else if(+d >= 75 && +d < 90) { return "#fc7676"; }
    else if(+d >= 90) { return "#d97055"; }
    else { return "#FFF"; }
  }

};

Date.prototype.monthDays = function(){
    var d= new Date(this.getFullYear(), this.getMonth()+1, 0);
    return d.getDate();
}

// Extend date object to increase number of years
Date.prototype.addYears = function(years) {
  this.setFullYear(this.getFullYear() + parseInt(years));
  return this;
};

// Extend date object to increase number of months
Date.prototype.addMonths = function(months) {
  this.setMonth(this.getMonth() + parseInt(months));
  return this;
};

// Extend date object to increase number of days
Date.prototype.addDays = function(days) {
  this.setDate(this.getDate() + parseInt(days));
  return this;
};

// Extend date object to increase number of minutes
Date.prototype.addMinutes = function(minutes) {
  this.setTime(this.getTime() + minutes*60000);
  return this;
}

// Extend date object to increase number of seconds
Date.prototype.addSeconds = function(seconds) {
  this.setTime(this.getTime() + seconds*1000);
  return this;
}

// Initialize the helpers class¬
gViz.shared.helpers.date = function() {

  // Multi Format
  var formatMillisecond = d3.timeFormat(".%L"),
      formatSecond = d3.timeFormat(":%S"),
      formatMinute = d3.timeFormat("%H:%M"),
      formatHour = d3.timeFormat("%H"),
      formatDay = d3.timeFormat("%a %d"),
      formatWeek = d3.timeFormat("%b %d"),
      formatMonth = d3.timeFormat("%B"),
      formatYear = d3.timeFormat("%Y");

  return {

    print: function(date) {
      return d3.timeFormat("%d/%m/%Y %H:%M:%S")(date);
    },

    translated: {
    },

    // Set locale from d3 time format
    setLocale: function(language) {
      d3.json("https://unpkg.com/d3-time-format@2/locale/"+language+".json", function(error, locale) {
        d3.timeFormatDefaultLocale(locale);
      });
    },

    getWeekDayNumberFromMonday: function(d) {
      if(d === 'Mon') { return 0; }
      else if(d === 'Tue') { return 1; }
      else if(d === 'Wed') { return 2; }
      else if(d === 'Thu') { return 3; }
      else if(d === 'Fri') { return 4; }
      else if(d === 'Sat') { return 5; }
      else if(d === 'Sun') { return 6; }
      else { return 0; }
    },

    getLastMonday: function (d) {
      var t = new Date(d);
      var day = t.getDay();
      t.setDate(t.getDate() - day);
      t.setHours(0, 0, 0);
      return t.addDays(day === 0 ? -6 : 1);
    },

    // Helpers
    helpers: {
      daysInMonth: function(month,year) {
        return new Date(year, month, 0).getDate();
      },
      dayDiff: function(to, from) {
        var n = 86400000;
        return Math.round(Math.abs((to - from)) / n);
      },
      diff: function(to, from, interval) {
        var n = 86400000;
        switch(interval) {
          case 'quarter': return Math.ceil((to.getMonth() - from.getMonth() + (12 * (to.getFullYear() - from.getFullYear())) + 1)/3); break;
          case 'month': return to.getMonth() - from.getMonth() + (12 * (to.getFullYear() - from.getFullYear())) + 1; break;
          case 'week': n = 60 * 60 * 1000 * 24 * 7; break;
          case 'day': n = 60 * 60 * 1000 * 24; break;
          case 'hour': n = 60 * 60 * 1000; break;
          case 'minute': n = 60 * 1000; break;
          case 'second': n = 1000; break;
          default: n = 86400000; break;
        }
        return Math.round(Math.abs((to - from)) / n) + 1;
      },
    },

    // Formats
    format: {
      hms: {
        parse:  function(d) { return d3.timeParse("%H:%M")(d); },
        format: function(d) { return d3.timeFormat("%H:%M")(d); },
      },
      en: {
        parse:  function(d) { return d3.timeParse("%Y-%m-%d")(d); },
        format: function(d) { return d3.timeFormat("%Y-%m-%d")(d); },
      },
      milliseconds: {
        parse:  function(d) { return d3.timeParse("%Y-%m-%d %H:%M:%S.%L")(d); },
        format: function(d) { return d3.timeFormat("%Y-%m-%d %H:%M:%S.%L")(d); },
      },
    },

    // Multi Formats
    multiFormat: function(date) {
      return (d3.timeSecond(date) < date ? formatMillisecond
          : d3.timeMinute(date) < date ? formatSecond
          : d3.timeHour(date) < date ? formatMinute
          : d3.timeDay(date) < date ? formatHour
          : d3.timeMonth(date) < date ? (d3.timeWeek(date) < date ? formatDay : formatWeek)
          : d3.timeYear(date) < date ? formatMonth
          : formatYear)(date);
    },

    // Get format for axis
    parseFormat: function(axis) {

      // Get axis format with prefix and suffix
      if(axis != null) {

        // Set prefix and suffix
        var prefix = axis.prefix != null ? axis.prefix : "";
        var suffix  = axis.suffix != null ? axis.suffix : (axis.sufix != null ? axis.sufix : "");

        // Get input format
        var inFmt = d3.timeParse("%Y-%m-%d");
        if(axis.inFormat != null && axis.inFormat != "") { inFmt = d3.timeParse(axis.inFormat); }

        // Get input format
        var outFmt = d3.timeFormat("%Y-%m-%d");
        if(axis.outFormat != null && axis.outFormat != "") { outFmt = d3.timeFormat(axis.outFormat); }

      } else {
        var prefix = "", suffix = "", inFmt = d3.timeParse("%Y-%m-%d"), outFmt = d3.timeFormat("%Y-%m-%d");
      }

      // Return format parsed
      return function(d) { return prefix + outFmt(inFmt(d)) + suffix; };

    }
  }
}();

var queue = [];

// Initialize the helpers class¬
gViz.shared.helpers.loading = {

  // Update loading div
  update: function(target) {
    if(target != null) {
      d3.select(target).selectAll('.loading-div .loading-div-inner')
        .style('height', $(target).outerHeight() + "px")
        .style('line-height', $(target).outerHeight() + "px")
        .style('width',  $(target).outerWidth() + "px");
    }
  },

  // Mostra loading div
  show: function(target) {

    if(target != null) {

     var string  = "<div class='showbox'>"
      string += "<div class='loader'>"
      string += "<svg class='circular' viewBox='25 25 50 50'>"
      string += "<circle class='path' cx='50' cy='50' r='20' fill='none' stroke-width='2' stroke-miterlimit='10'/>"
      string += "</svg>"
      string += "</div>"
      string += "</div>"

      // Create / Update loading div
      var loadingDiv = d3.select(target).selectAll('.loading-div').data(["loading-div"]);
      loadingDiv.exit().remove();
      loadingDiv = loadingDiv.enter().append("div").attr("class", 'loading-div').merge(loadingDiv);
      loadingDiv
        .style('display','block')
        .html("<div class='loading-div-inner'>"+string+"</div>");

       // Show loading div
      loadingDiv.selectAll(".loading-div-inner")
        .style('height', $(target).outerHeight() + "px")
        .style('line-height', $(target).outerHeight() + "px")
        .style('width',  $(target).outerWidth() + "px");

      // Remove scroll
      // d3.select(target).classed("no-scroll", true);

    }
  },

  // Some com loading div
  hide: function(target) {
    if(target != null) {
      d3.select(target).selectAll('.loading-div').remove();;
      // d3.select(target).classed("no-scroll", false);
    }
  }

};


// Initialize the helpers class¬
gViz.shared.helpers.number = function() {

  // Get user locale string or set default
  var getUserLocale = function() {

    var userLocale = 'en-US';

    // Get user locale
    userLocale = window.navigator.userLanguage || window.navigator.language;

    // Validate locale
    var number = 0;
    try {
      number.toLocaleString(userLocale);
    } catch (e) {
      return userLocale = 'en-US';
    }

    return userLocale;

  };

  // Set initial formats
  var numberFormat = {

    // Attrs
    userLocale: getUserLocale(),
    getUserLocale: getUserLocale,
    locale: function(d) { return (+d).toLocaleString(getUserLocale()); },
    localePercent: function(d){ return (+d).toFixed(1).toLocaleString(getUserLocale()) + "%"; },

    // Formats
    format: {
      s: d3.format(".2s"),
      perc: function(d, c) { if (c == null) { c = 2; } return d3.format("."+c+"%")(d); },
    }

  };

  // Diwo default
  numberFormat.diwo =  function(d) {
    var value = +d;
    if(d >= 1000000000 || d <= -1000000000) { return (d3.format(".2s")(d).replace('G',' G')); }
    else if(d >= 1000000 || d <= -1000000) { return (d3.format(".2s")(d).replace('M',' M')); }
    else if(d >= 1000 || d <= -1000) { return (d3.format(".2s")(d).replace('k',' k')); }
    else if(d >= 100 || d <= -100) { return (d.toFixed(0) + ' '); }
    else if(d >= 10 || d <= -10) { return ((d % 1 + '').length > 3 ? d.toFixed(1) : d) + ' '; }
    else if(d >= 1 || d <= -1) { return ((d % 1 + '').length > 1 ? d.toFixed(2) : d) + ' '; }
    else if(d < 1 && d > -1) { return ((d % 1 + '').length > 1 ? d.toFixed(2) : d) + ' '; }
  };

  // Get format for axis
  numberFormat.parseFormat =  function(axis) {

    // Get axis format with prefix and suffix
    if(axis != null) {

      // Set prefix and suffix
      var prefix = axis.prefix != null ? axis.prefix : "";
      var suffix  = axis.suffix != null ? axis.suffix : (axis.sufix != null ? axis.sufix : "");

      // Get format
      var fmt = numberFormat.diwo;
      if(axis.format === 'locale') { fmt = numberFormat.locale; }
      else if(axis.format != null && axis.format != "") { fmt = d3.format(axis.format); }

    } else {
      var prefix = "", suffix = "", fmt = numberFormat.diwo;
    }

    // Return format parsed
    return function(d) {
      return fmt(+d).toString().indexOf('-') !== -1 ? '-' + prefix + fmt(+d).toString().replace('-','') + suffix : prefix + fmt(+d) + suffix;
    };

  };

  // Return
  return  numberFormat;

}();

// https://github.com/wbkd/d3-extended
d3.selection.prototype.moveToFront = function () { return this.each(function () { return this.parentNode.appendChild(this); }); };

// Find closest parent with selector (http://stackoverflow.com/questions/18375761/does-d3-have-api-which-similar-with-jquery-closestselector)
d3.selection.prototype.closest = function (selector) {
  var closestMatch = undefined;
  var matchArr = [];
  this.each(function () {
    var elm = this;
    while (typeof elm.parentNode.matches === "function" && !closestMatch) {
      elm = elm.parentNode;
      if (elm.matches(selector)) {
        closestMatch = elm;
        matchArr.push(closestMatch);
      }
    }
    closestMatch = undefined;
  });
  return d3.selectAll(matchArr);
}

//enter exit update pattern principle
d3.selection.prototype.patternify = function (params) {
  var container = this;
  var selector = params.selector;
  var elementTag = params.elementTag;
  var data = params.data || [selector];
  // pattern in action
  var selection = container.selectAll('.' + selector).data(data)
  selection.exit().remove();
  selection = selection.enter().append(elementTag).merge(selection)
  selection.attr('class', selector);
  return selection;
}

// Initialize the helpers class¬
gViz.shared.helpers.selection = {

  getTransformation: function(transform) {
    // Create a dummy g for calculation purposes only. This will never
    // be appended to the DOM and will be discarded once this function
    // returns.
    var g = document.createElementNS("http://www.w3.org/2000/svg", "g");

    // Set the transform attribute to the provided string value.
    g.setAttributeNS(null, "transform", transform);

    // consolidate the SVGTransformList containing all transformations
    // to a single SVGTransform of type SVG_TRANSFORM_MATRIX and get
    // its SVGMatrix.
    var matrix = g.transform.baseVal.consolidate().matrix;

    // Below calculations are taken and adapted from the private function
    // transform/decompose.js of D3's module d3-interpolate.
    // var {a, b, c, d, e, f} = matrix;   // ES6, if this doesn't work, use below assignment
    var a=matrix.a, b=matrix.b, c=matrix.c, d=matrix.d, e=matrix.e, f=matrix.f; // ES5
    var scaleX, scaleY, skewX;
    if (scaleX = Math.sqrt(a * a + b * b)) a /= scaleX, b /= scaleX;
    if (skewX = a * c + b * d) c -= a * skewX, d -= b * skewX;
    if (scaleY = Math.sqrt(c * c + d * d)) c /= scaleY, d /= scaleY, skewX /= scaleY;
    if (a * d < b * c) a = -a, b = -b, skewX = -skewX, scaleX = -scaleX;
    return {
      translateX: e,
      translateY: f,
      rotate: Math.atan2(b, a) * 180 / Math.PI,
      skewX: Math.atan(skewX) * 180 / Math.PI,
      scaleX: scaleX,
      scaleY: scaleY
    };
  }
};

// Initialize the helpers class¬
gViz.shared.helpers.style = {

  // Get style from element
  get: function(oElm, strCssRule){
    var strValue = "";
    if(document.defaultView && document.defaultView.getComputedStyle){
        strValue = document.defaultView.getComputedStyle(oElm, "").getPropertyValue(strCssRule);
    }
    else if(oElm.currentStyle){
        strCssRule = strCssRule.replace(/\-(\w)/g, function (strMatch, p1){
            return p1.toUpperCase();
        });
        strValue = oElm.currentStyle[strCssRule];
    }
    return strValue;
  }

};

// Initialize the helpers class¬
gViz.shared.helpers.text = {

  // Replace text variables
  replaceVariables: function(string, obj) {
    return string.toString().split('{{').map(function(d) {
      if(d.indexOf("}}") !== -1) {
        var pieces = d.split('}}');
        var text = eval('obj.' + pieces[0]);
        d = (text == null ? "" : text) + pieces[1];
      }
      return d;
    }).join('');
  },

  // Remove special characters (id for gradients)
  removeSpecial: function(text) {
    return text == null ? '' : text.toString().replace(/[^\w]/gi, '-');
  },

  // Get size of text
  getSize: function(text, size, weight) {
    size = size == null ? '12px' : size;
    weight = weight == null ? 'bold' : weight;
    var t = d3.select('body').append('span')
      .style('font-size', size)
      .style('font-weight', weight)
      .style('white-space', 'nowrap')
      .html(text);
    var width = t.node().getBoundingClientRect().width;
    t.remove();
    return width + 10;
  },

  // Wrap function into width
  wrap: function(text, width) {
    text.each(function() {
      var text = d3.select(this),
        words = text.text().split(/\s+/).reverse(),
        word,
        line = [],
        lineNumber = 0,
        lineHeight = 1.1, // ems
        y = text.attr("y"),
        dy = isNaN(+text.attr("dy")) ? 0 : +text.attr("dy"),
        tspan = text.text(null).append("tspan").attr("x", 0).attr("y", y).attr("dy", dy + "em");
      while (word = words.pop()) {
        line.push(word);
        tspan.text(line.join(" "));
        if (tspan.node().getComputedTextLength() > width) {
          line.pop();
          tspan.text(line.join(" "));
          line = [word];
          tspan = text.append("tspan").attr("x", 0).attr("y", y).attr("dy", lineHeight + "em").text(word);
        }
      }
    });
  },

  // Wrap function into width
  wrapBySize: function(text, width, height, maxLines) {
    maxLines = maxLines == null ? 128932 : maxLines;
    text.each(function() {

      // Initial variables
      var text = d3.select(this);
      var words = text.text().trim().split('').reverse();
      var word;
      var line = [];
      var numLines = 0;
      var lineNumber = 0;
      var lineHeight = 1.1;
      var x = isNaN(+text.attr("x")) ? 0 : +text.attr("x");
      var y = isNaN(+text.attr("y")) ? 0 : +text.attr("y");
      var dy = isNaN(+text.attr("dy")) ? 0 : +text.attr("dy");
      var tspan = text.text(null).append("tspan").attr("x", x).attr("y", y).attr("dy", dy + "em");
      var bbox = null;

      // Populate words array incrementally iterating over characterers
      while (word = words.pop()) {
        line.push(word);
        tspan.text(line.join("").trim());
        bbox = text.node().getBBox();

        // If the word exceeds the width, draw tspan and start new one
        if (bbox.width > width) {
          line.pop();
          tspan.text(line.join("").trim());
          line = [word];
          tspan = text.append("tspan").attr("x", x).attr("y", y).attr("dy", ++lineNumber * lineHeight + dy + "em").text(word.trim());
          numLines += 1;

        // Otherwise, just add the text
        } else {
          tspan.text(line.join(""));
        }

        // Limit height or line number
        if(bbox.height > height || numLines >= maxLines) {
          if(numLines >= 1) { d3.select(text.node().lastChild).remove(); }
          if(words.length > 0) { d3.select(text.node().lastChild).text(d3.select(text.node().lastChild).text().slice(0,-1) + '...'); }
          words = [];
        }
      }
    });
  },

  // Get format for axis
  parseFormat: function(axis) {

    // Get format
    var fmt = function(d) { return d; };

    // Get axis format with prefix and suffix
    if(axis != null) {

      // Set prefix and suffix
      var prefix = axis.prefix != null ? axis.prefix : "";
      var suffix  = axis.suffix != null ? axis.suffix : (axis.sufix != null ? axis.sufix : "");

    } else {
      var prefix = "", suffix = "";
    }

    // Return format parsed
    return function(d) {
      return prefix + fmt(d) + suffix;
    };

  }

};

// Module declaration
gViz.shared.modules.consumption = function() {
  "use strict";

  // Get attributes values
  var _var = {};
  var translate = null;
  var registrationToken = "Token eyJhbGciOiJIUzI1NiIsImlhdCI6MTUyMDM1NDEwNywiZXhwIjoxNzQxMTA2MTA3fQ.eyJpZCI6NDQzfQ.toDSii_S1yX7zi1QgEJOKkeBilENLl_R40Z1-ZbtmOY";
  var apiKey = "af144b1eb71729a60a92b0eb7f3983d9fac7629421167d6c5bab8d7d7b64438f";
  var container = ".cons-graph";
  var endPointing = "https://app2.getluko.com/v1/consumption";
  var language = "en-US";
  var chartType = null;
  var appliance = 'all';
  var isForecast = true;
  var promises = [];
  var priceFunction = null;
  var lineColors = null;
  var lineTransparency = null;
  var timeActive = 1;
  var graphActive = 0;
  var zoomLevel = 1;
  var menuItems = null;

  // Actions
  var menuClick = function(id) { console.log('Clicked: ', id); }
  var onChange = function(change) { console.log(change); }

  // Validate attributes
  var validate = function (step) {
    switch (step) {
      case 'build': return true;
      default: return false;
    }
  };

  // Main function
  var main = function (step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'build':

          // Initialize _var
          if(_var == null) { _var = {}; }

          // Update translation language
          if(translate != null) {  gViz.shared.helpers.date.setLocale(translate.use()); }

           // Get container
          _var.container = {
            selector: container,
            d3: d3.select(container),
            el: ((typeof container === 'string' || container instanceof String) ? container : d3.select(container).node()),
            clientRect: d3.select(container).node().getBoundingClientRect(),
            dimensions: {}
          };

          // Set container position to relative
          _var.container.d3.style('position', 'relative');

          // Draw outer wrapper
          _var.outerWrap = _var.container.d3.selectAll('.gViz-outer-wrapper').data(["outer-wrapper"], d => d);
          _var.outerWrap.exit().remove();
          _var.outerWrap = _var.outerWrap.enter().append("div").attr('class', 'gViz-outer-wrapper').merge(_var.outerWrap);

          // Draw inner wrapper
          _var.innerWrap = _var.outerWrap.selectAll('.gViz-inner-wrapper').data(["inner-wrapper"], d => d);
          _var.innerWrap.exit().remove();
          _var.innerWrap = _var.innerWrap.enter().append("div").attr('class', 'gViz-inner-wrapper').merge(_var.innerWrap);

          // Loading
          _var.loading = _var.container.d3.selectAll('.loading-wrapper').data(["loading"], d => d);
          _var.loading.exit().remove();
          _var.loading = _var.loading.enter().append("div").attr('class', 'loading-wrapper').merge(_var.loading);

          // Transparent menu bottom
          _var.transMenu = _var.container.d3.selectAll('.time-selecter-scroller-bottom').data(["transparent-menu"], d => d);
          _var.transMenu.exit().remove();
          _var.transMenu = _var.transMenu.enter().append("div").attr('class', 'time-selecter-scroller-bottom').merge(_var.transMenu);

          // Set menu items
          var items = [
            { value: 0, id: 'NOW', translate: 'NOW', name: 'NOW' },
            { value: 1, id: 'TODAY', translate: 'TODAY', name: 'TODAY' },
            { value: 2, id: '7DAYS', translate: '7DAYS', name: '7 DAYS' },
            { value: 3, id: '30DAYS', translate: '30DAYS', name: '30 DAYS' },
            { value: 4, id: 'YEAR', translate: 'THISYEAR', name: 'THIS YEAR' }
          ];

          // Draw menu
          _var.menu = _var.container.d3.selectAll('.time-selecter-scroller').data(["menu"], d => d);
          _var.menu.exit().remove();
          _var.menu = _var.menu.enter().append("ul").attr('class', 'time-selecter-scroller').merge(_var.menu);
          _var.menu.each(function(d) {

            // Add translate for today date on d3 date
            gViz.shared.helpers.date.translated.today = translate != null ? (translate.instant(items[1].translate) || items[1].name) : items[1].name;

            // Add translation for DATEORDER on d3 date
            gViz.shared.helpers.date.translated.DATEORDER = {};
            items.forEach(function(d) {
              var t = translate != null ? translate.instant('DATEORDER-'+d.translate) : 'default';
              gViz.shared.helpers.date.translated.DATEORDER[d.translate] = t == null || t === '' ? 'default' : t;
            });

            // Get only active items
            var activeItems = items.filter(d => menuItems == null || (menuItems[d.id] != null && menuItems[d.id].active != null && menuItems[d.id].active == true))

            // Set timeActive depending on selection
            if(activeItems.length === 0) { timeActive = -2; }
            else if(activeItems.filter(d => d.value == 1).length === 0) { timeActive = activeItems[0].value; }

            // Draw menu items
            _var.menuItem = d3.select(this).selectAll('.time-selecter').data(activeItems, d => d.id);
            _var.menuItem.exit().remove();
            _var.menuItem = _var.menuItem.enter().append("div").attr('class', 'time-selecter').merge(_var.menuItem);
            _var.menuItem
              .attr('data-id', function(d) { return d.value; })
              .style('text-transform', 'uppercase')
              .text(function(d) { return translate != null ? (translate.instant(d.translate) || d.name) : d.name; })
              .on('click', function(d) {

                // Trigger on click function
                onChange({ type: 'menu', event: 'click', value: d.value, name: d.name, translate: d.translate });

                // Set time active to the clicked id
                timeActive = d.value;

                // Update Menu
                _var.update('menu', d.value);

                // Update charts
                if(graphActive == 0 || graphActive == 1){ _var.update('chart', d.value); }

                // Menu Click callback function
                menuClick(d.value);

              }).classed('time-selected', function(d) { return timeActive == d.value; })

          });

          // Initialize data
          _var.gVizBarData  = {};
          _var.gVizBubbleData  = {};

          // Get data from api
          _var.gVizLineData = window.gViz.shared.data.lineChart.format([]);
          _var.gVizBarData[timeActive.toString()] = window.gViz.shared.data.barChart.format();
          _var.gVizBubbleData  = window.gViz.shared.data.bubbleChart.format();

          // Initialize variables for charts
          _var.lineChart = null;
          _var.barChart = null;
          _var.bubbleChart = null;
          _var.setTimeIntervalFunction = null;
          _var.isNow = true;
          _var.barChartViewMode = "day";
          _var.bubbleChartViewMode = "day";

          // Promises
          var promises = [];

          // Add points to line chart function
          _var.barAddPoints = function(x, direction) {

            // Do nothing if no direction
            if(direction !== 'none') {

              // Trigger on click function
              onChange({ type: 'bar', event: 'drag', x: x, direction: direction });

              // Show loading
              window.gViz.shared.helpers.loading.show('.loading-wrapper');

              // Fetch data from API
              var timeDef = (new Date()).getTime();
              promises.push(timeDef);
              window.gViz.shared.data.barChart.fetch(_var.barChartViewMode, x.x, direction, appliance, isForecast, apiKey, registrationToken, endPointing)
              .then(function() {

                var timeDefIndex = promises.indexOf(timeDef);
                if(promises.length > 0 && timeDefIndex === promises.length-1) {

                  // Parse new data
                  if(direction === 'next') {
                    _var.gVizBarData[timeActive.toString()].data = _var.gVizBarData[timeActive.toString()].data.slice(1).concat(this._data[this._data.length-1] || []);
                  } else if(direction === 'prev') {
                    _var.gVizBarData[timeActive.toString()].data = (this._data || []).concat(_var.gVizBarData[timeActive.toString()].data);
                    _var.gVizBarData[timeActive.toString()].data.pop();
                  }

                  // Draw bar chart
                  drawBarChart();

                  // Clean promises array
                  promises = [];

                  // Hide loading
                  window.gViz.shared.helpers.loading.hide('.loading-wrapper');

                } else if(timeDefIndex !== -1) { promises.splice(timeDefIndex, 1); }

              }).fail(function(err) {

                var timeDefIndex = promises.indexOf(timeDef);
                if(promises.length > 0 && timeDefIndex === promises.length-1) {
                  drawBarChart();
                  promises = [];
                  window.gViz.shared.helpers.loading.hide('.loading-wrapper');
                } else if(timeDefIndex !== -1) { promises.splice(timeDefIndex, 1); }
              });

            }
          }

          // Add points to line chart function
          _var.lineAddPoints = function(x, direction) {

            setTimeout(function() {

              // // Do nothing if no direction
              if(direction !== 'none') {

                // Get informations
                var zoomInfo = gViz.shared.data.lineChart.zoomLevelInfo(zoomLevel);
                var now = (new Date()).getTime();

                // Trigger on click function
                onChange({ type: 'line', event: direction === 'zoom' ? 'zoom' : 'drag', x: x, direction: direction });

                // Reset dates for zoom direction
                if(direction === 'zoom') {

                  // Get dates
                  var midDate = new Date((x.end.getTime() + x.start.getTime())/2)
                  var startDate = new Date(midDate.getTime() - 1.5 * zoomInfo.interval * 1000);
                  var endDate = new Date() <= new Date(midDate.getTime() + 1.5 * zoomInfo.interval * 1000) ? new Date() : new Date(midDate.getTime() + 1.5 * zoomInfo.interval * 1000);
                  _var.gVizLineData.data[0].values = []

                  // Set is now
                  _var.isNow = now <= (new Date(endDate.getTime())).addSeconds(zoomInfo.seconds).getTime();

                  if(_var.isNow) { startDate = new Date(endDate.getTime() - 3 * zoomInfo.interval * 1000); }

                // Get time depending on direction
                } else {

                  // Get dates
                  var startDate = direction === 'prev' ? (new Date(x)).addSeconds(-zoomInfo.interval) : new Date(x);
                  var endDate = direction === 'prev' ? new Date(x) : new Date(Math.min(now, (new Date(x)).addSeconds(zoomInfo.interval).getTime()));

                  // Set is now
                  _var.isNow = now <= (new Date(endDate.getTime())).addSeconds(zoomInfo.seconds).getTime();
                }

                // Show loading
                window.gViz.shared.helpers.loading.show('.loading-wrapper');

                console.log(gViz.shared.helpers.date.print(x.start), gViz.shared.helpers.date.print(x.end));
                console.log(zoomLevel, gViz.shared.helpers.date.print(startDate), gViz.shared.helpers.date.print(midDate), gViz.shared.helpers.date.print(endDate));

                // Fetch data from api
                var timeDef = (new Date()).getTime();
                promises.push(timeDef);
                gViz.shared.data.lineChart.fetch(startDate, endDate, appliance, zoomLevel, apiKey, registrationToken, endPointing)
                .then(function(data) {

                  var timeDefIndex = promises.indexOf(timeDef);
                  if(promises.length > 0 && timeDefIndex === promises.length-1) {

                    // Parse new data from api
                    data.consumption.t.forEach(function(d, i) {
                      var point = window.gViz.shared.data.lineChart.parse([{ x: parseInt(`${d}000`), y: data.consumption.w[i] }])[0];;
                      if(direction === 'next' || direction === 'zoom') { _var.gVizLineData.data[0].values.push(point); }
                      else if(direction === 'prev') { _var.gVizLineData.data[0].values = [point].concat(_var.gVizLineData.data[0].values); }
                    });

                    // Filter unique values
                    _var.gVizLineData.data[0].values = gViz.shared.helpers.array.unique(_var.gVizLineData.data[0].values, 'x')
                    _var.gVizLineData.data[0].values = gViz.shared.helpers.array.sorted(_var.gVizLineData.data[0].values, 'x', 'ascending');

                    // Reduce number of points
                    if(direction !== 'zoom' && _var.gVizLineData.data[0].values.length > zoomInfo.max) {
                      if(direction==='next') {
                        var length = _var.gVizLineData.data[0].values.length;
                        _var.gVizLineData.data[0].values = _var.gVizLineData.data[0].values.slice(length-zoomInfo.max, length);
                      } else if(direction==='prev') {
                        _var.gVizLineData.data[0].values = _var.gVizLineData.data[0].values.slice(0,zoomInfo.max);
                      }
                    }

                    // Draw line chart
                    drawLineChart();

                    // Start updating real time
                    if(_var.isNow) { lineStartNow(_var.gVizLineData.data[0].values[_var.gVizLineData.data[0].values.length-1].x); }

                    // Clean promises array
                    promises = [];

                    // Hide loading
                    window.gViz.shared.helpers.loading.hide('.loading-wrapper');

                  }

                }).fail(function(err) {

                  var timeDefIndex = promises.indexOf(timeDef);
                  if(promises.length > 0 && timeDefIndex === promises.length-1) {
                    drawLineChart();
                    promises = [];
                    window.gViz.shared.helpers.loading.hide('.loading-wrapper');
                  } else if(timeDefIndex !== -1) { promises.splice(timeDefIndex, 1); }

                });

              }

            }, 300);
          }

          // Stop getting real time data
          var lineStopNow = function() { clearInterval(_var.setTimeIntervalFunction); }

          // Start getting real time data
          var lineStartNow = function(x) {

            // Stop real time before starting the interval
            lineStopNow();

            var getDataOnInterval = function() {

              // Get zoomLevelInfo
              var zoomInfo = gViz.shared.data.lineChart.zoomLevelInfo(zoomLevel);

              // If x is null, get time from 30 seconds before
              var startDate = x == null || new Date(x) >= new Date() ? (new Date()).addSeconds(-zoomInfo.interval) : new Date(x);
              var endDate = new Date();
              var lastPoint = _var.gVizLineData.data[0].values[_var.gVizLineData.data[0].values.length-1];

              // Fetch points only If the last point is desired seconds before now
              if(lastPoint == null || endDate.getTime() - lastPoint.parsedX.getTime() > zoomInfo.seconds * 1000) {

                // Trigger on click function
                onChange({ type: 'line', event: 'fetch-real-time-data', x: startDate, direction: 'next' });

                // Show loading
                window.gViz.shared.helpers.loading.show('.loading-wrapper');

                // Fetch data from api
                var timeDef = (new Date()).getTime();
                promises.push(timeDef);
                gViz.shared.data.lineChart.fetch(startDate, endDate, appliance, zoomLevel, apiKey, registrationToken, endPointing)
                .then(function(data) {

                  var timeDefIndex = promises.indexOf(timeDef);
                  if(promises.length > 0 && timeDefIndex === promises.length-1) {

                    // Parse new data from api
                    data.consumption.t.forEach(function(d, i) {
                      var point = window.gViz.shared.data.lineChart.parse([{ x: parseInt(`${d}000`), y: data.consumption.w[i] }])[0];;
                      _var.gVizLineData.data[0].values.push(point);
                    });

                    // Filter unique values
                    _var.gVizLineData.data[0].values = gViz.shared.helpers.array.unique(_var.gVizLineData.data[0].values, 'x');
                    _var.gVizLineData.data[0].values = gViz.shared.helpers.array.sorted(_var.gVizLineData.data[0].values, 'x', 'ascending');

                    // Reduce number of points
                    if(_var.gVizLineData.data[0].values.length > zoomInfo.max) {
                      var length = _var.gVizLineData.data[0].values.length;
                      _var.gVizLineData.data[0].values = _var.gVizLineData.data[0].values.slice(length-zoomInfo.max, length);
                    }

                    // Draw line chart
                    drawLineChart();

                    // Update x
                    var lastPoint = _var.gVizLineData.data[0].values[_var.gVizLineData.data[0].values.length-1];
                    x = lastPoint == null ? x : lastPoint.parsedX.getTime();

                    // Clean promises array
                    promises = [];

                    // Hide loading
                    window.gViz.shared.helpers.loading.hide('.loading-wrapper');

                  }

                }).fail(function(err) {

                  var timeDefIndex = promises.indexOf(timeDef);
                  if(promises.length > 0 && timeDefIndex === promises.length-1) {

                    // Update x
                    var lastPoint = _var.gVizLineData.data[0].values[_var.gVizLineData.data[0].values.length-1];
                    x = lastPoint == null ? x : lastPoint.parsedX.getTime();

                    // Draw line chart
                    drawLineChart();

                    // Clean promises array
                    promises = [];

                    // Hide loading
                    window.gViz.shared.helpers.loading.hide('.loading-wrapper');

                  } else if(timeDefIndex !== -1) { promises.splice(timeDefIndex, 1); }

                });
              }
            }

            // Set time interval
            getDataOnInterval();
            _var.setTimeIntervalFunction = setInterval(function() {
              getDataOnInterval();
            }, 3000);
          }

          _var.lineHandleZoom = function(x, direction) {

            lineStopNow();

            // Update zoom direction
            zoomLevel += direction === 'up' ? (1) : (-1);

            // Bounded zoom level
            if(zoomLevel <= -2) { zoomLevel = -2; }
            else if(zoomLevel >= 5) { zoomLevel = 5; }

            // Add points for zoom direction
            _var.lineAddPoints(x, 'zoom');

          }

          // Draw line chart
          var drawLineChart = function(){

            // Set unit for line chart data specified on the menuItems
            gViz.shared.data.lineChart.setUnit(_var.gVizLineData, (menuItems['NOW'] != null && menuItems['NOW'].unit != null && menuItems['NOW'].unit.trim().length != 0 ? menuItems['NOW'].unit : null));


            // Draw line chart
            _var.lineChart = gViz.viz.lineAreaChart()
              ._var(_var.lineChart)
              .container(d3.select(".cons-graph .gViz-outer-wrapper .gViz-inner-wrapper").node())
              .colors(lineColors)
              .data(_var.gVizLineData)
              .isNow(_var.isNow)
              .opacity(lineTransparency)
              .zoomMode('horizontal') // horizontal or vertical
              .zoomLevel(zoomLevel) // horizontal or vertical
              .yZoom({
                fn: function(x, direction) { },
                enable: true,
                transform: ( _var.lineChart == null ? null : _var.lineChart.yZoom.transform )
              })
              .xZoom({
                fn: function(x, direction) { _var.isNow = false; lineStopNow(); _var.lineAddPoints(x, direction); },
                enable: true,
                transform: ( _var.lineChart != null && _var.lineChart.xZoom != null ? _var.lineChart.xZoom.transform : null )
              })
              .wheelZoom({
                fn: function(x, direction) { _var.lineHandleZoom(x, direction); },
                x: {
                  start: _var.lineChart != null && _var.lineChart.wheelZoom != null && _var.lineChart.wheelZoom.x != null && _var.lineChart.wheelZoom.x.start != null ? _var.lineChart.wheelZoom.x.start : null,
                  end: _var.lineChart != null && _var.lineChart.wheelZoom != null && _var.lineChart.wheelZoom.x != null && _var.lineChart.wheelZoom.x.end != null ? _var.lineChart.wheelZoom.x.end : null
                }
              })
              .build()

          }

          // Handble bar chart click
          var handleBarChartClick = function(node){

            var xLabel, xValue, xSplit, weekValue;

            // Remove all inserted menu items
            d3.select('.time-selecter-scroller').selectAll(".time-selecter[data-mode='inserted']").remove()

            if(_var.barChartViewMode === 'day') {

              // Get x value for next draw
              xValue = d3.timeParse("%Y-%m-%d %H")(`${node.parent} ${parseInt(node.x) + 1}`);

              // Trigger on click function
              onChange({ type: 'bar', currentView: 'day', nextView: 'now',  event: 'bar-click', x: xValue });

              // Change functions
              _var.setLineChartView(xValue);
              menuClick(0);
              _var.update('menu', 0);

            } else if (_var.barChartViewMode === 'week') {

              // Get x value for next draw
              xSplit = node.parent.split('-');
              xValue = d3.timeParse("%Y-%m-%d")(node.parent);
              xLabel = d3.timeParse("%Y-%m-%d")(node.parent);
              weekValue = window.gViz.shared.helpers.date.getWeekDayNumberFromMonday(node.x);

              if(xValue != null) {

                var value = xValue.addDays(weekValue);
                var label = xLabel.addDays(weekValue);

                if(d3.timeFormat("%b %d, %Y")(label) === d3.timeFormat("%b %d, %Y")(new Date())) {

                  // Trigger on click function
                  onChange({ type: 'bar', currentView: 'week', nextView: 'day',  event: 'bar-click', x: 'today', translate: 'TODAY' });

                  // Update Menu
                  _var.update('menu', 1);

                  // Update charts
                  _var.update('chart', 1);

                  // Menu Click callback function
                  menuClick(1);

                } else {

                  // Trigger on click function
                  onChange({ type: 'bar', currentView: 'week', nextView: 'day',  event: 'bar-click', x: xValue });

                  // Initialize bar chart for TODAY perspective
                  _var.setBarChartView('day', d3.timeFormat("%Y-%m-%d")(value), 'both');

                  // Add item to menu
                  _var.update('menu', -1);
                  var isCurrentYear = label.getYear() === (new Date()).getYear();
                  d3.select('.time-selecter-scroller').insert('li', ".time-selecter[data-id='2']")
                    .attr('class', 'time-selecter time-selected')
                    .attr('data-mode', 'inserted')
                    .html(d3.timeFormat("%b %d" + (isCurrentYear ? "" : ", %Y"))(label))
                    .on('click', function() {
                      _var.update('menu', -1);
                      _var.setBarChartView('day', d3.timeFormat("%Y-%m-%d")(value), 'both');
                      d3.select(this).classed('time-selected', true);
                    })
                    .style('opacity', 0)
                    .transition().duration(700)
                      .style('opacity', 1)
                }
              }

            } else if (_var.barChartViewMode === 'month') {

              // Get x value for next draw
              xSplit = node.parent.split('-');
              xValue = d3.timeParse("%Y-%m-%d")(`${xSplit[0]}-${xSplit[1]}-${node.x}`);
              xLabel = d3.timeParse("%Y-%m-%d")(`${xSplit[0]}-${xSplit[1]}-${node.x}`);

              if(xValue != null) {

                var value = xValue;

                if(d3.timeFormat("%b %d, %Y")(xLabel) === d3.timeFormat("%b %d, %Y")(new Date())) {

                  // Trigger on click function
                  onChange({ type: 'bar', currentView: 'month', nextView: 'day',  event: 'bar-click', x: 'today', translate: 'TODAY' });

                  // Update Menu
                  _var.update('menu', 1);

                  // Update charts
                  _var.update('chart', 1);

                  // Menu Click callback function
                  menuClick(1);

                } else {

                  // Trigger on click function
                  onChange({ type: 'bar', currentView: 'month', nextView: 'day',  event: 'bar-click', x: xValue });

                  // Initialize bar chart for TODAY perspective
                  _var.setBarChartView('day',d3.timeFormat("%Y-%m-%d")(value), 'both');

                  // Add item to menu
                  _var.update('menu', -1);
                  var isCurrentYear = xLabel.getYear() === (new Date()).getYear();
                  d3.select('.time-selecter-scroller').insert('li', ".time-selecter[data-id='2']")
                    .attr('class', 'time-selecter time-selected')
                    .attr('data-mode', 'inserted')
                    .html(d3.timeFormat("%b %d" + (isCurrentYear ? "" : ", %Y"))(xLabel))
                    .on('click', function() {
                      _var.update('menu', -1);
                      _var.setBarChartView('week', d3.timeFormat("%Y-%m-%d")(value), 'both');
                      d3.select(this).classed('time-selected', true);
                    })
                    .style('opacity', 0)
                    .transition().duration(700)
                      .style('opacity', 1)
                }
              }

            } else if (_var.barChartViewMode === 'year') {

              // Get x value for next draw
              xValue = d3.timeParse("%b-%Y")(`${node.x}-${node.parent}`);
              xLabel = d3.timeParse("%b-%Y")(`${node.x}-${node.parent}`);

              if(xValue != null) {

                var value = xValue;

                if(d3.timeFormat("%b, %Y")(xLabel) === d3.timeFormat("%b, %Y")(new Date())) {

                  // Trigger on click function
                  onChange({ type:'bar', currentView:'year', nextView:'month',  event:'bar-click', x:xValue, translate:'THISMONTH' });

                  // Update Menu
                  _var.update('menu', 3);

                  // Update charts
                  _var.update('chart', 3);

                  // Menu Click callback function
                  menuClick(3);

                } else {

                  // Trigger on click function
                  onChange({ type: 'bar', currentView: 'year', nextView: 'month',  event: 'bar-click', x: xValue });

                  // Initialize bar chart for TODAY perspective
                  _var.setBarChartView('month',d3.timeFormat("%Y-%m-%d")(value), 'both');

                  // Add item to menu
                  _var.update('menu', -1);
                  var isCurrentYear = xLabel.getYear() === (new Date()).getYear();
                  d3.select('.time-selecter-scroller').insert('li', ".time-selecter[data-id='4']")
                    .attr('class', 'time-selecter time-selected')
                    .attr('data-mode', 'inserted')
                    .style('opacity', 0)
                    .html(d3.timeFormat("%b" + (isCurrentYear ? "" : ", %Y"))(xLabel))
                    .on('click', function() {
                      _var.update('menu', -1);
                      _var.setBarChartView('month', d3.timeFormat("%Y-%m-%d")(value), 'both');
                      d3.select(this).classed('time-selected', true);
                    })
                    .transition().duration(700)
                      .style('opacity', 1)
                }
              }

            }

            return;
          }

          // Draw bar chart
          var drawBarChart = function(){

            // Set unit for line chart data specified on the menuItems
            var menuItem = menuItems[items.filter(d => d.value === timeActive)[0].id];
            gViz.shared.data.barChart.setUnit(_var.gVizBarData[timeActive.toString()], (menuItem != null && menuItem.unit != null && menuItem.unit.trim().length != 0 ? menuItem.unit : null));

            // Draw bar chart
            _var.barChart = gViz.viz.barChart()
              ._var(_var.barChart)
              .container(d3.select(".cons-graph .gViz-outer-wrapper .gViz-inner-wrapper").node())
              .data(_var.gVizBarData[timeActive.toString()])
              .onClick(handleBarChartClick)
              .priceFunction(priceFunction)
              .zoom({
                fn: function(x, direction) { _var.barAddPoints(x, direction); },
                enable: true,
                transform: ( _var.barChart == null ? null : _var.barChart.zoom.transform ),
                startPoint: _var.gVizBarData[timeActive.toString()].data != null && _var.gVizBarData[timeActive.toString()].data.length === 3 ? _var.gVizBarData[timeActive.toString()].data[1].x : null
              })
              .build()

          }

          // Draw bar chart depending on option
          _var.setBarChartView = function(mode, x, direction) {

            // Set chart type
            chartType = 'bar';

            // Remove tooltip when view changed
            d3.selectAll('.gViz-tooltip').remove();

            _var.barChartViewMode = mode;

            // Stop line chart from getting real data
            lineStopNow();

            // Clear previous bar chart
            d3.selectAll(".cons-graph .gViz-outer-wrapper .gViz-inner-wrapper").selectAll("svg").remove();

            // Reset bar chart
            _var.barChart = null;
            _var.gVizBarData[timeActive.toString()] = window.gViz.shared.data.barChart.format();

            // Show loading
            window.gViz.shared.helpers.loading.show('.loading-wrapper');

            // Fetch data from API
            var timeDef = (new Date()).getTime();
            promises.push(timeDef);
            window.gViz.shared.data.barChart.fetch(_var.barChartViewMode, x, direction, appliance, isForecast, apiKey, registrationToken, endPointing).then(function() {

              var timeDefIndex = promises.indexOf(timeDef);
              if(promises.length > 0 && timeDefIndex === promises.length-1) {

                // Update barWidth
                var interval = _var.barChartViewMode === 'day' ? 24 : (_var.barChartViewMode === 'week' ? 7 : (_var.barChartViewMode === 'month' ? 31 : 12));
                var range = d3.select(".cons-graph .gViz-outer-wrapper .gViz-inner-wrapper").node().getBoundingClientRect().width;
                var step = range / (0.9 + interval);
                _var.gVizBarData[timeActive.toString()].attrs.barWidth = 0.9 * step;

                // Store data
                _var.gVizBarData[timeActive.toString()].data = this != null && this._data != null ? this._data : [];

                // Draw bar chart
                drawBarChart();

                // Clean promises array
                promises = [];

                // Hide loading
                window.gViz.shared.helpers.loading.hide('.loading-wrapper');

              }


            }).fail(function(err) {

              console.log(err);

              var timeDefIndex = promises.indexOf(timeDef);
              if(promises.length > 0 && timeDefIndex === promises.length-1) {
                drawBarChart();
                promises = [];
                window.gViz.shared.helpers.loading.hide('.loading-wrapper');
              } else if(timeDefIndex !== -1) { promises.splice(timeDefIndex, 1); }

            });

          }

          // Draw line chart depending on option
          _var.setLineChartView = function(x) {

            // Set chart type
            chartType = 'line';

            // Remove tooltip when view changed
            d3.selectAll('.gViz-tooltip').remove();

            // Clear previous bar chart
            d3.selectAll(".cons-graph .gViz-outer-wrapper .gViz-inner-wrapper").selectAll("svg").remove();

            // Reset line chart
            _var.lineChart = null;
            _var.gVizLineData = window.gViz.shared.data.lineChart.format([]);

            // Stop line real time
            lineStopNow();

            // If its now
            if(x == null || x >= new Date()) {

              _var.isNow = true;

              // Draw line chart
              drawLineChart();

              // Draw line chart on real time mode
              lineStartNow();

            } else {

              _var.isNow = false;

              // Add previous points
              _var.lineAddPoints(x, 'prev');

            }

          }

          // Draw bubble chart
          var drawBubbleChart = function(){

            // Draw bubble chart
            _var.bubbleChart = gViz.viz.bubbleChart()
              ._var(_var.bubbleChart)
              .container(d3.select(".cons-graph .gViz-outer-wrapper .gViz-inner-wrapper").node())
              .data(_var.gVizBubbleData)
              .onClick(handleBubbleChartClick)
              .build()

          }

          var handleBubbleChartClick = function(d) {

            // Trigger on click function
            onChange({ type: 'bubble', currentView: _var.bubbleChartViewMode, event: 'bubble-click', id: d.id, obj: d });

          }

          // Draw bar chart depending on option
          _var.setBubbleChartView = function(mode, x, direction) {

            // Set chart type
            chartType = 'bubble';

            // Remove tooltip when view changed
            d3.selectAll('.gViz-tooltip').remove();

            _var.bubbleChartViewMode = mode;

            // Stop line chart from getting real data
            lineStopNow();

            // Clear previous bubble chart
            d3.selectAll(".cons-graph .gViz-outer-wrapper .gViz-inner-wrapper").selectAll("svg:not(.bubble-chart)").remove();

            // Reset bubble chart
            //_var.bubbleChart = null;
            _var.gVizBubbleData = window.gViz.shared.data.bubbleChart.format();

            // Show loading
            window.gViz.shared.helpers.loading.show('.loading-wrapper');

            // Fetch data from API
            var timeDef = (new Date()).getTime();
            promises.push(timeDef);
            window.gViz.shared.data.bubbleChart.fetch(_var.bubbleChartViewMode, x, direction, appliance, isForecast, apiKey, registrationToken, endPointing).then(function() {

              var timeDefIndex = promises.indexOf(timeDef);
              if(promises.length > 0 && timeDefIndex === promises.length-1) {

                // Store data
                _var.gVizBubbleData.data = this._data || [];

                // Draw bubble chart
                drawBubbleChart();

                // Clean promises array
                promises = [];

                // Hide loading
                window.gViz.shared.helpers.loading.hide('.loading-wrapper');

              }

            }).fail(function(err) {

              var timeDefIndex = promises.indexOf(timeDef);
              if(promises.length > 0 && timeDefIndex === promises.length-1) {
                drawBubbleChart();
                promises = [];
                window.gViz.shared.helpers.loading.hide('.loading-wrapper');
              } else if(timeDefIndex !== -1) { promises.splice(timeDefIndex, 1); }

            });

          }

          // Set onChange function to be fired
          _var.on_change = function(value) {
            onChange = value;
          }

          // Reset and update menu items :: element = ['menu','chart']
          _var.update = function(element='chart', id=1) {

            // Update menu
            if(element == 'menu') {

              // Reset and Update menu for the correct time period
              d3.selectAll(".time-selecter-scroller .time-selecter").classed('time-selected', false);
              d3.selectAll(`.time-selecter-scroller .time-selecter[data-id='${id}']`).classed('time-selected', true);

              // Remove all inserted items on menu if the click is on the default items
              if(id !== -1) {

                d3.selectAll(".time-selecter-scroller .time-selecter[data-mode='inserted']").transition()
                  .duration(700)
                  .style('opacity', 0)
                  .on('end', function() { d3.select(this).remove(); });

              }

            // Update chart
            } else if(element == 'chart') {

              // NOW :: Draw line chart
              if(id==0 && (menuItems == null || (menuItems['NOW'] != null && menuItems['NOW'].active != null && menuItems['NOW'].active === true))) {

                // Fetch all data if not initialized
                if(_var.lineInitialized == null) {

                  // Show loading
                  window.gViz.shared.helpers.loading.show('.loading-wrapper');

                  // Fetch data from API
                  window.gViz.shared.data.lineChart
                    .fetch(null, new Date(), appliance, zoomLevel, apiKey, registrationToken,  endPointing)
                    .then(function() {

                      // Get today option
                      _var.setLineChartView(null);

                      // Update initialized flag to avoid future caching
                      _var.lineInitialized = true;

                      // Show loading
                      window.gViz.shared.helpers.loading.hide('.loading-wrapper');

                    });

                } else {

                  // Get today option
                  _var.setLineChartView(null);

                }

              // TODAY :: Draw bar chart and get other informations
              } else if (id==1 && (menuItems == null || (menuItems['TODAY'] != null && menuItems['TODAY'].active != null && menuItems['TODAY'].active === true))) {
                if(graphActive === 0) {

                  // Fetch all data if not initialized
                  if(_var.initialized == null) {

                    // Show loading
                    window.gViz.shared.helpers.loading.show('.loading-wrapper');

                    // Fetch data from API
                    window.gViz.shared.data.barChart
                      .fetch('year', null, null, appliance, isForecast, apiKey, registrationToken,  endPointing)
                      .then(function() {

                        // Get today option
                        _var.setBarChartView('day');

                        // Update initialized flag to avoid future caching
                        _var.initialized = true;

                        // Show loading
                        window.gViz.shared.helpers.loading.hide('.loading-wrapper');

                      });

                  } else {

                    // Get today option
                    _var.setBarChartView('day');

                  }

                }
                else if(graphActive === 1) { _var.setBubbleChartView('day'); }

              // WEEK :: Draw bar chart and get other informations
              } else if (id == 2 && (menuItems == null || (menuItems['7DAYS'] != null && menuItems['7DAYS'].active != null && menuItems['7DAYS'].active === true))) {
                if(graphActive === 0) { _var.setBarChartView('week'); }
                else if(graphActive === 1) { _var.setBubbleChartView('week'); }

              // MONTH :: Draw bar chart and get other informations
              } else if (id == 3 && (menuItems == null || (menuItems['30DAYS'] != null && menuItems['30DAYS'].active != null && menuItems['30DAYS'].active === true))) {
                if(graphActive === 0) { _var.setBarChartView('month'); }
                else if(graphActive === 1) { _var.setBubbleChartView('month'); }

              // YEAR :: Draw bar chart and get other informations
              } else if (id == 4 && (menuItems == null || (menuItems['YEAR'] != null && menuItems['YEAR'].active != null && menuItems['YEAR'].active === true))) {
                if(graphActive === 0) { _var.setBarChartView('year'); }
                else if(graphActive === 1) { _var.setBubbleChartView('year'); }
              }
            }

            // Update menu depending on view
            _var.menuItem.filter(d => d.value === 1)
              .text(function(d) {
                var value = graphActive === 1 || graphActive === 2 ? { value: 1, translate: 'YESTERDAY', name: 'YESTERDAY' } : d;
                return translate != null ? (translate.instant(value.translate) || value.name) : value.name;
              })

          }

          // Reset the object (back to Today)
          _var.reset = function() {

            // Update Menu
            _var.update('menu', 1);

            // Update charts
            _var.update('chart', 1);

            // Menu Click callback function
            menuClick(1);

          }

          // Delete the luko-graph object
          _var.delete = function() {
            _var.container.d3.html('');
            promises = [];
            _var = null;
          }

          // Get attributes from library
          _var.get = {

            // Get current chart type
            type: function() { return chartType; },

            // Get current chart left border timestamp
            start: function() {
              if(chartType == 'line' && _var.lineChart != null) { return _var.lineChart.get.start(); }
              else if(chartType == 'bar' && _var.barChart != null) { return _var.barChart.get.start(); }
              return null;
            },

            // Get current chart right border timestamp
            end: function() {
              if(chartType == 'line' && _var.lineChart != null) { return _var.lineChart.get.end(); }
              else if(chartType == 'bar' && _var.barChart != null) { return _var.barChart.get.end(); }
              return null;
            },

            // Get current chart right - left border timestamp
            zoom_h: function() {
              if(chartType == 'line' && _var.lineChart != null) { return _var.lineChart.get.end() - _var.lineChart.get.start(); }
              else if(chartType == 'bar' && _var.barChart != null) { return _var.barChart.get.end() - _var.barChart.get.start(); }
              return null;
            },

            // Get current chart min y value
            min: function() {
              if(chartType == 'line' && _var.lineChart != null) { return _var.lineChart.get.min(); }
              else if(chartType == 'bar' && _var.barChart != null) { return _var.barChart.get.min(); }
              return null;
            },

            // Get current chart max y value
            max: function() {
              if(chartType == 'line' && _var.lineChart != null) { return _var.lineChart.get.max(); }
              else if(chartType == 'bar' && _var.barChart != null) { return _var.barChart.get.max(); }
              return null;
            },

            // Get current chart max - min y value
            zoom_v: function() {
              if(chartType == 'line' && _var.lineChart != null) { return _var.lineChart.get.max() - _var.lineChart.get.min(); }
              else if(chartType == 'bar' && _var.barChart != null) { return _var.barChart.get.max() - _var.barChart.get.min(); }
              return null;
            },

            // Get current appliance value
            appliance: function() { return appliance; },

            // Get current isForecast value
            is_forecast: function() { return isForecast; },

          }

          // Set attributes from library
          _var.set = {

            // Set current appliance value
            appliance: function(value) { appliance = value; },

            // Set current isForecast value
            is_forecast: function(value) { return isForecast = value; },

            // Set current chart type
            type: function(value) {
              if(value === 'line' || value === 'bar') {

                // Update chart type
                chartType = value;

                // Set time active to the clicked id
                timeActive = value === 'line' ? 0 : 1;

                // Set chart view
                graphActive = 0;

                // Update Menu
                _var.update('menu', value === 'line' ? 0 : 1);

                // Update charts
                _var.update('chart', value === 'line' ? 0 : 1);

                // Toggle item on menu
                menuClick(value === 'line' ? 0 : 1);
              }
            },

            // Set specific day view
            day: function(value) {
              _var.setBarChartView('day', d3.timeFormat("%Y-%m-%d")(value.addDays(1)), 'prev');
            },

            // Set specific week view
            week: function(value) {
              _var.setBarChartView('week', d3.timeFormat("%Y-%m-%d")(value.addDays(7)), 'prev');
            },

            // Set specific month view
            month: function(value) {
              _var.setBarChartView('month', d3.timeFormat("%Y-%m-%d")(value.addMonths(1)), 'prev');
            },

            // Set specific year view
            year: function(value) {
              _var.setBarChartView('year', d3.timeFormat("%Y")(value.addYears(1)), 'prev');
            },

            // Set price function
            price_function: function(value) {
              priceFunction = value;
            },

            // Set line transparency
            line_transparency: function(value) {
              lineTransparency = value;
            },

            // Set line colors
            line_colors: function(value) {
              lineColors = value;
            },

            // Set time active value
            time_active: function(value) {
              timeActive = value;
            },

            // Set graph active value
            graph_active: function(value) {
              graphActive = value;
            },

            // Set menu click callback function
            menu_click: function(value) {
              menuClick = value;
            }

          };

          break;
      }
    }

    return _var;
  };

  // Expose global variables
  ['_var','translate','registrationToken','apiKey','container','endPointing','language','appliance','isForecast','promises','priceFunction','lineColors','lineTransparency','timeActive','graphActive','menuClick','onChange','menuItems'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return main;
    };
  });

  // Secondary functions
  main.build = function (_) { return main("build"); };

  // Execute the specific called function
  main.run = function (_) { return main(_); };

  return main;

}

// Initialize the visualization class
gViz.shared.visualComponents.axisGuide = function() {
  "use strict";

  // Get attributes values
  var _var       = undefined;
  var action     = "show";
  var color      = "#333";
  var height     = 0;
  var x          = 0;
  var y          = 0;
  var z          = 0;
  var left       = { x: 0, y: 0 };
  var top        = { x: 0, y: 0 };
  var value      = { x: 0, y: 0 };

  // Validate attributes
  var validate = function(step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function(step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Get window position
          var doc = document.documentElement;
          var offset = {
            left: (window.pageXOffset || doc.scrollLeft) - (doc.clientLeft || 0),
            top:  (window.pageYOffset || doc.scrollTop)  - (doc.clientTop || 0)
          };

          switch (action) {

            // Build entire visualizations
            case 'show':

              // Insert Path to axis
              var axisGuide = _var.g.selectAll(".axis-guide").data(['x','y']);
              axisGuide.exit().remove();
              axisGuide = axisGuide.enter().append('path').attr("class", "axis-guide").merge(axisGuide);
              axisGuide
                .attr('d', function(s) { return s === 'x' ? ("M "+x+" "+(y+z)+" "+x+" "+(y+z)) : ("M "+(x-z)+" "+y+" "+(x-z)+" "+y); })
                .style('stroke', color)
                .transition().duration(200)
                  .attr('d', function(s) { return s === 'x' ? ("M "+x+" "+(y+z)+" "+x+" "+height) : ("M "+(x-z)+" "+y+" 0 "+y); });

              // Insert axis guide text background rect
              var axisGuideRect = d3.select('body').selectAll(".axis-guide-rect").data(['x','y']);
              axisGuideRect.exit().remove();
              axisGuideRect = axisGuideRect.enter().append('div').attr("class", "axis-guide-rect").merge(axisGuideRect);
              axisGuideRect
                .style('opacity', 0)
                .style('background-color', color)
                .html(function(s) {
                  if(s === 'x') { return "<span style='color: " + (gViz.shared.helpers.colors.isDark(color) ? "#FFF" : "#434343") + ";'>" + value.x + "</span><span class='arrow up' style='color: " + color + ";'>▲</span>"; }
                  else { return "<span style='color: " + (gViz.shared.helpers.colors.isDark(color) ? "#FFF" : "#434343") + ";'>" + value.y + "</span><span class='arrow right' style='color: " + color + ";'>▶</span>"; }
                })
                .transition().delay(200).duration(0)
                  .style('opacity', 1)
                  .style('top',  function(s) {
                    var h = this.getBoundingClientRect().height;
                    return (offset.top + (s === 'x' ? top - y + z + height : top + z - h/2))+"px";
                  })
                  .style('left', function(s) {
                    var w = this.getBoundingClientRect().width;
                    return (offset.left + (s === 'x' ? left - w/2 : left - x - w)) + "px";
                  })

              break;

            // Build entire visualizations
            case 'hide':

              // Remove axis guide elements
              _var.g.selectAll(".axis-guide").remove();
              d3.selectAll(".axis-guide-rect").remove();

              break;

          }
          break;
      }
    }
  };

  // Exposicao de variaveis globais
  ['_var','action','color','height','left','top','value','x','y','z'].forEach(function(key) {

    // Attach variables to validation function
    validate[key] = function(_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return validate;
    };

    // Attach variables to main function
    return main[key] = function(_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function(_) { return main('run'); };

  return main;
};

// Module declaration
gViz.shared.visualComponents.backgroundGrid = function () {
  "use strict";

  // Get attributes values
  var _var       = null;
  var id         = "vis-shadow-"+(Math.floor(Math.random() * ((1000000000 - 5) + 1)) + 5);
  var height     = 100;
  var left       = 0;
  var top        = 0;
  var width      = 100;
  var wrap       = null;

  // Validate attributes
  var validate = function validate(step) {
    switch (step) {
      case 'run':
        return true;
      default:
        return false;
    }
  };

  // Main function
  var main = function main(step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Create background grid
          var gridBg = wrap.selectAll(".grid-"+id).data(["chart-svg"]);
          gridBg.exit().remove();
          gridBg = gridBg.enter().insert("div", ":first-child").attr('class', "grid-background grid-"+id).merge(gridBg); // svg

          // Update outer dimensions
          gridBg
            .style("top",  (top) + 'px')
            .style("left", (left) + 'px')
            .style("width",  (width) + 'px')
            .style("height", (height) + 'px');

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','id','height','left','width','top','wrap'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) {
    return main('run');
  };

  return main;
};

function __range__(left, right, inclusive) {
  var range = [];
  var ascending = left < right;
  var end = !inclusive ? right : ascending ? right + 1 : right - 1;
  for (var i = left; ascending ? i < end : i > end; ascending ? i++ : i--) { range.push(i); }
  return range;
}

// Initialize the visualization class
gViz.shared.visualComponents.gradient = function() {
  "use strict";

  // Get attributes values
  var _var       = undefined;
  var id         = "vis-gradient-"+(Math.floor(Math.random() * ((1000000000 - 5) + 1)) + 5);
  var action     = 'draw';
  var colors     = [{offset:"0%", color: "#333"},{offset:"100%", color: "#FFF"}];
  var direction  = "vertical";
  var duration   = 0;
  var gType      = "linear";
  var gradientUnits = "objectBoundingBox";
  var x1         = null;
  var x2         = null;
  var y1         = null;
  var y2         = null;
  var wrap       = null;

  // Validate attributes
  var validate = function(step) {
    switch (step) {
      case 'run': return wrap != null;
      default: return false;
    }
  };

  // Main function
  var main = function(step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          switch (action) {

            case 'draw':

              switch (gType) {

                // Build entire visualizations
                case 'linear':

                  // Set directions
                  if(direction === 'horizontal') {
                    x1 = "0%";
                    y1 = "0%";
                    x2 = "100%";
                    y2 = "0%";
                  } else {
                    x1 = "0%";
                    y1 = "0%";
                    x2 = "0%";
                    y2 = "100%";
                  }

                  // Remove special chars from id
                  id = gViz.shared.helpers.text.removeSpecial(id);

                  // Update gradients
                  var grad = wrap.selectAll("#"+id+".grad").data([id]);
                  grad.exit().remove();
                  grad = grad.enter().append("linearGradient").attr('id', function(d) { return id.toString(); }).attr("class", 'grad').merge(grad);
                  grad
                    .transition().duration(duration)
                    .attr('gradientUnits', gradientUnits)
                    .attr("x1", x1)
                    .attr("y1", y1)
                    .attr("x2", x2)
                    .attr("y2", y2)
                    .each(function(g) {

                      // Update gradient stops
                      var stop = d3.select(this).selectAll("stop").data(colors);
                      stop.exit().remove();
                      stop = stop.enter().append("stop").merge(stop);
                      stop
                        .attr("offset", function(d) { return d.offset; })
                        .attr("stop-color", function(d) { return d.color; });

                    });

                  break;

              }
              break;

            case 'clean':

              // Clean all gradients from wrap
              wrap.selectAll('.grad').remove();

              break;

          }
          break;
      }
    }
  };

  // Exposicao de variaveis globais
  ['_var','id','action','colors','direction','duration','gType','gradientUnits','x1','y1','x2','y2','wrap'].forEach(function(key) {

    // Attach variables to validation function
    validate[key] = function(_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return validate;
    };

    // Attach variables to main function
    return main[key] = function(_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function(_) { return main('run'); };

  return main;
};

// Initialize the visualization class
gViz.shared.visualComponents.image = function() {
  "use strict";

  // Get attributes values
  var _var       = undefined;
  var id         = "vis-image-"+(Math.floor(Math.random() * ((1000000000 - 5) + 1)) + 5);
  var action     = 'draw';
  var height     = 100;
  var imageUrl   = 'http://d2trtkcohkrm90.cloudfront.net/images/emoji/apple/ios-10/256/beer-mug.png';
  var wrap       = null;
  var width      = 100;

  // Validate attributes
  var validate = function(step) {
    switch (step) {
      case 'run': return wrap != null;
      default: return false;
    }
  };

  // Main function
  var main = function(step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          switch (action) {

            case 'draw':

              // Remove special chars from id
              id = gViz.shared.helpers.text.removeSpecial(id);

              // Update patterns
              var pattern = wrap.selectAll("#"+id+".image-pattern").data([id]);
              pattern.exit().remove();
              pattern = pattern.enter().append("pattern").attr('id', function(d) { return id.toString(); }).attr("class", 'image-pattern').merge(pattern);
              pattern
                .attr("x", "0%")
                .attr("y", "0%")
                .attr("height", "100%")
                .attr("width", "100%")
                .attr("viewBox", "0 0 "+width+" "+height)
                .each(function(g) {

                  // Update pattern images
                  var image = d3.select(this).selectAll("image").data([imageUrl]);
                  image.exit().remove();
                  image = image.enter().append("image").merge(image);
                  image
                    .attr("x", "0%")
                    .attr("y", "0%")
                    .attr("height", height)
                    .attr("width", width)
                    .attr("xlink:href", imageUrl)

                });

              break;

            case 'clean':

              // Clean all gradients from wrap
              wrap.selectAll('.grad').remove();

              break;

          }
          break;
      }
    }
  };

  // Exposicao de variaveis globais
  ['_var','id','action','height', 'imageUrl','width','wrap'].forEach(function(key) {

    // Attach variables to validation function
    validate[key] = function(_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return validate;
    };

    // Attach variables to main function
    return main[key] = function(_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function(_) { return main('run'); };

  return main;
};

// Module declaration
gViz.shared.visualComponents.shadow = function () {
  "use strict";

  // Get attributes values
  var _var         = null;
  var id           = "vis-shadow-"+(Math.floor(Math.random() * ((1000000000 - 5) + 1)) + 5);
  var filterUnits  = "userSpaceOnUse";
  var height       = "250%";
  var width        = "250%";
  var offsetHeight = "-100%";
  var offsetWidth  = "-100%";
  var stdDeviation = 2;
  var x            = -2;
  var y            = 2;
  var wrap         = null;

  // Validate attributes
  var validate = function validate(step) {
    switch (step) {
      case 'run':
        return true;
      default:
        return false;
    }
  };

  // Main function
  var main = function main(step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Create/Update defs
          var defs = wrap.selectAll("defs.shadow-defs").data(["shadow-defs"]);
          defs.exit().remove();
          defs = defs.enter().insert('defs',':first-child').attr("class", "shadow-defs").merge(defs);
          defs.each(function() {

            // Create filter with id #drop-shadow
            // height=130% so that the shadow is not clipped
            var filter = d3.select(this).selectAll(".shadow-filter").data(["shadow-filter"]);
            filter.exit().remove();
            filter = filter.enter().append('filter').attr("class", "shadow-filter").merge(filter);
            filter
              .attr("id", id)
              .attr('filterUnits', filterUnits)
              .attr("height", height)
              .attr("width", width)
              .attr("x", offsetHeight)
              .attr("y", offsetWidth)
              .each(function() {

                // SourceAlpha refers to opacity of graphic that this filter will be applied to
                // convolve that with a Gaussian with standard deviation 3 and store result
                // in blur
                var feGaussianBlur = d3.select(this).selectAll(".shadow-feGaussianBlur").data(["shadow-feGaussianBlur"]);
                feGaussianBlur.exit().remove();
                feGaussianBlur = feGaussianBlur.enter().append('feGaussianBlur').attr("class", "shadow-feGaussianBlur").merge(feGaussianBlur);
                feGaussianBlur
                  .attr("in", "SourceGraphic")
                  .attr("stdDeviation", stdDeviation)
                  .attr("result", "blur-out");

                // Source matrix
                var feColorMatrix = d3.select(this).selectAll(".shadow-feColorMatrix").data(["shadow-feColorMatrix"]);
                feColorMatrix.exit().remove();
                feColorMatrix = feColorMatrix.enter().append('feColorMatrix').attr("class", "shadow-feColorMatrix").merge(feColorMatrix);
                feColorMatrix
                  .attr("result", "bluralpha")
                  .attr("type", "matrix")
                  .attr("values", "0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0.7 0 ")

                // Translate output of Gaussian blur to the right and downwards with 2px
                // store result in offsetBlur
                var feOffset = d3.select(this).selectAll(".shadow-feOffset").data(["shadow-feOffset"]);
                feOffset.exit().remove();
                feOffset = feOffset.enter().append('feOffset').attr("class", "shadow-feOffset").merge(feOffset);
                feOffset
                //  .attr("in", "blurAlpha")
                  .attr("dx", x)
                  .attr("dy", y)
                  .attr("result", "the-shadow");

                // Translate output of Gaussian blur to the right and downwards with 2px
                // store result in offsetBlur
                var feBlend = d3.select(this).selectAll(".shadow-feBlend").data(["shadow-feBlend"]);
                feBlend.exit().remove();
                feBlend = feBlend.enter().append('feBlend').attr("class", "shadow-feBlend").merge(feBlend);
                feBlend
                  .attr('in', 'SourceGraphic')
                  .attr('in2', 'the-shadow')
                  .attr('mode', 'normal');

              });
          });

          break;
      }
    }

    return _var;
  };

  // Exposicao de variaveis globais
  ['_var','id','filterUnits','height','width','stdDeviation','x','y','wrap'].forEach(function (key) {

    // Attach variables to validation function
    validate[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return validate;
    };

    // Attach variables to main function
    return main[key] = function (_) {
      if (!arguments.length) {
        eval('return ' + key);
      }
      eval(key + ' = _');
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function (_) {
    return main('run');
  };

  return main;
};

function __range__(left, right, inclusive) {
  var range = [];
  var ascending = left < right;
  var end = !inclusive ? right : ascending ? right + 1 : right - 1;
  for (var i = left; ascending ? i < end : i > end; ascending ? i++ : i--) { range.push(i); }
  return range;
}

// Initialize the visualization class
gViz.shared.visualComponents.tooltipTable = function() {
  "use strict";

  // Get attributes values
  var _var        = undefined;
  var action      = "show";
  var body        = "#333";
  var borderColor = "#333";
  var backgroundColor = null;
  var color       = "#333";
  var content     = "";
  var hasImg      = false;
  var left        = 0;
  var muted       = false;
  var obj         = {};
  var target      = null;
  var title       = 0;
  var top         = null;
  var bottom      = null;

  // Validate attributes
  var validate = function(step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function(step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          switch (action) {

            // Show tooltip
            case 'show':
            case 'updateLocation':

              // If is muted
              if(muted) { obj.color = "#666"; color = "#666"; borderColor = "#666"; }

              // Set title content
              title = title == null || title.constructor !== Array ? [] : title;
              title = title.map(function(t, i) {

                return "<div class='node-header' style='color: {{color}}; background-color: " + (backgroundColor == null ? (gViz.shared.helpers.colors.isDark(obj.color) ? "#FFF" : "#434343") : backgroundColor) + "; "+( i !== 0 ? 'border-top: 1px solid {{color}}' : '')+"'>" + t + "</div>";

              }).join('');

              // Set body content
              body = body == null || body.constructor !== Array ? [] : body;
              body = body.map(function(d) { return "<div class='node-edge' style='border-top: 1px solid " + borderColor + "; background-color: " + (backgroundColor == null ? "{{color}}" : backgroundColor) + "; color: " + (backgroundColor == null ? (gViz.shared.helpers.colors.isDark(obj.color) ? "#FFF" : "#434343") : "{{color}}") + ";'>" + d + "</div>"; }).join('');

              // Join content
              content = gViz.shared.helpers.text.replaceVariables(title, obj) + gViz.shared.helpers.text.replaceVariables(body, obj);

              // Update tooltip content
              target
                .style("border", "1px solid "+borderColor)
                .style("left", left == null ? null : left + "px")
                .style("top", top == null ? null : top + "px")
                .style("bottom", bottom == null ? null : bottom + "px")
                .html(content)

              break;

            // Hide and remove tooltip
            case 'hide':

              // Hide & Remove
              target
                .style("border", "none")
                .html('');

              break;

          }
          break;
      }
    }
  };

  // Exposicao de variaveis globais
  ['_var','action','body','borderColor','backgroundColor','color','hasImg','left','muted','obj','target','top','bottom','title'].forEach(function(key) {

    // Attach variables to validation function
    validate[key] = function(_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return validate;
    };

    // Attach variables to main function
    return main[key] = function(_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function(_) { return main('run'); };

  return main;
};

// Initialize the visualization class
gViz.shared.visualComponents.tooltip = function() {
  "use strict";

  // Get attributes values
  var _var        = undefined;
  var action      = "show";
  var body        = "#000";
  var borderColor = "#000";
  var backgroundColor = null;
  var color       = "#000";
  var content     = "";
  var hasImg      = false;
  var left        = 0;
  var muted       = false;
  var obj         = {};
  var top         = 0;
  var title       = 0;
  var opacity     = 0.55;
  var side        = 'left';

  // Validate attributes
  var validate = function(step) {
    switch (step) {
      case 'run': return true;
      default: return false;
    }
  };

  // Main function
  var main = function(step) {

    // Validate attributes if necessary
    if (validate(step)) {

      switch (step) {

        // Build entire visualizations
        case 'run':

          // Get window position
          var doc = document.documentElement;
          var offset = {
            left: (window.pageXOffset || doc.scrollLeft) - (doc.clientLeft || 0),
            top:  (window.pageYOffset || doc.scrollTop)  - (doc.clientTop || 0)
          };

          switch (action) {

            // Show tooltip
            case 'show':
            case 'updateLocation':

              // If is muted
              obj.color = "#000"; color = "#000"; borderColor = "#000";

              // Set title content
              title = title == null || title.constructor !== Array ? [] : title;
              title = title.map(function(t, i) {

                return "<span class='title' style='color: #FFF; background-color: {{color}}; border-top: 1px solid "+(borderColor == null ? '{{color}}' : borderColor)+";'>" + t + "</span>";

              }).join('');

              // Set body content
              body = body == null || body.constructor !== Array ? [] : body;
              body = body.map(function(d) { return "<span class='text' style='background-color: #000; color: #FFF; border-top: 1px solid "+(borderColor == null ? '{{color}}' : borderColor)+";'>" + d + "</span>"; }).join('');

              // Join content
              content = gViz.shared.helpers.text.replaceVariables(title, obj) + gViz.shared.helpers.text.replaceVariables(body, obj);

              // Update tooltip content
              var tooltip = d3.selectAll('.gViz-tooltip').data(["gViz-tooltip"]);
              tooltip.exit().remove();
              tooltip = tooltip.enter().append("div").attr("class", 'gViz-tooltip').style('opacity', 0).merge(tooltip);
              tooltip.style('pointer-events', 'none')
                .each(function() {

                  // Update tooltip content
                  var ctn = d3.select(this).selectAll('.content').data(["gViz-content"]);
                  ctn.exit().remove();
                  ctn = ctn.enter().append("div").attr("class", 'content').merge(ctn);
                  ctn
                    .style("border", "1px solid "+borderColor)
                    .style("border-top", "none")
                    .style("border-bottom", "none")
                    .style("border-left", "none")
                    .style("border-right", "none")
                    .html(content)

                  // Get arrow styles
                  var arrowBGColor = backgroundColor != null ? backgroundColor : (body.length !== 0 ? gViz.shared.helpers.text.replaceVariables("{{color}}", obj) : "#FFF");
                  var arrowColor = borderColor != null ? borderColor : gViz.shared.helpers.text.replaceVariables("{{color}}", obj);

                  // Update tooltip content
                  var arrow = d3.select(this).selectAll('.arrow').data(["gViz-arrow"]);
                  arrow.exit().remove();
                  arrow = arrow.enter().append("div").attr("class", 'arrow ' + (body === '' ? 'no-body' : '')).merge(arrow);
                  arrow
                    .html("<span style= 'color: #000;'>"+(side === 'left' ? '▶' : '◀')+"</span><span class='arrow-bg' style='color:"+arrowColor+";'>"+(side === 'left' ? '▶' : '◀')+"</span>");

                });

                // Update tooltip position
                tooltip.transition().delay(10)
                  .on("end", function() {
                    var size = this.getBoundingClientRect();

                    // Tooltip
                    d3.select(this)
                      .style("left", function() { return (offset.left + left + (side === 'left' ? (-size.width - 8) : 8)) + "px"; })
                      .style("top", function() { return (offset.top + top - (size.height / 2) + (side === 'left' ? 0 : 2)) + "px"; })
                      .transition().duration(50)
                        .style("display", action === 'show' ? '' : 'block')
                        .style("opacity", opacity);

                    // Arrow
                    d3.select(this).selectAll('.arrow')
                      .style("top", function() { return ((size.height/2) - (side === 'left' ? 5 : 10)) + "px"; })
                      .style("left", function() { return side === 'left' ? "calc(100% - 3px)" : "-8px"; })
                  })

              break;

            // Hide only
            case 'hideSoft':

              // Hide only
              d3.selectAll('.gViz-tooltip').transition().delay(250).duration(10)
                .style("opacity", 0)
                .style('display', 'none');

              break;

            // Hide and remove tooltip
            case 'hide':

              // Hide & Remove
              var tooltip = d3.selectAll('.gViz-tooltip').data(["gViz-tooltip"]);
              tooltip.exit().remove();
              tooltip = tooltip.enter().append("div").attr("class", 'gViz-tooltip').merge(tooltip);
              tooltip.style("opacity", 0).remove()

              break;

          }
          break;
      }
    }
  };

  // Exposicao de variaveis globais
  ['_var','action','body','borderColor','backgroundColor','color','hasImg','left','muted','obj','side','top','title'].forEach(function(key) {

    // Attach variables to validation function
    validate[key] = function(_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return validate;
    };

    // Attach variables to main function
    return main[key] = function(_) {
      if (!arguments.length) { eval("return "+key); }
      eval(key + " = _");
      return main;
    };
  });

  // Executa a funcao chamando o parametro de step
  main.run = function(_) { return main('run'); };

  return main;
};
